/**
 * This software is copyright (c) 2009 by
 *  - TextGrid Consortion (http://www.textgrid.de)
 *  - Worms University of Applied Sciences (http://www.fh-worms.de)
 *
 * This is free software. You can redistribute it
 * and/or modify it under the terms described in
 * the GNU Lesser General Public License v3 of which you
 * should have received a copy. Otherwise you can download
 * it from
 *
 *   http://www.gnu.org/licenses/lgpl-3.0.txt
 *
 * @copyright TextGrid Consortion (http://www.textgrid.de)
 * @copyright Worms University of Applied Sciences (http://www.fh-worms.de)
 * @license http://www.gnu.org/licenses/lgpl-3.0.txt GNU Lesser General Public License v3
 * @author  Yahya Al-Hajj (alhajj@fh-worms.de)
 */
package info.textgrid.lab.core.metadataeditor.autoCompletion;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * A help class for the auto-completion functionality in the metadata editor.
 * 
 * @author Yahya Al-Hajj <alhajj@fh-worms.de>
 * 
 * 
 */
public class AutoCompletionUtil {

	private static AutoCompletionUtil instance = new AutoCompletionUtil();

	private final List<String> langList = Collections
			.synchronizedList(new ArrayList<String>());
	private Map<String, String> langMap = Collections
			.synchronizedMap(new HashMap<String, String>());

	public static AutoCompletionUtil getInstance() {
		return instance;
	}

	public List<String> getLanguagesList() {
		if (langList.isEmpty()) {
			createLanguagesList1();
			createLanguagesList2();
		}

		return langList;
	}

	public Map<String, String> getLanguagesMap() {
		if (langMap.isEmpty()) {
			createLanguagesMap1();
			createLanguagesMap2();
			createLanguagesMap3();
			createLanguagesMap4();
			createLanguagesMap5();
		}

		return langMap;
	}

	private void createLanguagesList1() {

		langList.add("!O!ung"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("!Xóõ"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("'Are'are"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("'Auhelawa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("//Ani"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("//Gana"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("//Xegwi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("/Gwi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("/Xam"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Aralle-Tabulahan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Arifama-Miniafia"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bainouk-Gunyaamolo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cashibo-Cacataibo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chichimeca-Jonaz"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chichonyi-Chidzihana-Chikauma"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chonyi-Dzihana-Kauma"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cinda-Regi-Tiyal"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cocama-Cocamilla"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Columbia-Wenatchi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gabrielino-Fernandeño"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Havasupai-Walapai-Yavapai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Idakho-Isukha-Tiriki"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ikoma-Nata-Isenye"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Izi-Ezaa-Ikwo-Mgbo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kag-Fer-Jiir-Koor-Ror-Us-Zuksun"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kaningdon-Nindem"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Katcha-Kadugli-Miri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kuikúro-Kalapálo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kwalhioqua-Tlatskanai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lamja-Dengsa-Tola"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lavatbura-Lamusong"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lisabata-Nuniali"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Luidakho-Luisukha-Lutirichi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Makhuwa-Marrevone"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Malecite-Passamaquoddy"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mator-Taygi-Karagas"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mbariman-Gudhinma"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Namosi-Naitasiri-Serua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nangikurrunggurr"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nottoway-Meherrin"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Numana-Nunku-Gbantu-Numbu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pitcairn-Norfolk"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Rakahanga-Manihiki"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tanimuca-Retuarã"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ukwuani-Aboh-Ndoni"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Uripiv-Wala-Rano-Atchin"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Vaghat-Ya-Bijim-Legeri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wangaaybuwan-Ngiyambaa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wotapuri-Katarqalai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("=/Hua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("=/Kx'au//'ein"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("A'tong"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("A-Pucikwar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("A-hmaos"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Aari"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Aasáx"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Abadi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Abaga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Abai Sungai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Abanyom"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Abar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Abau"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Abaza"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Abellen Ayta"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Abidji"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Abinomn"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Abipon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Abishira"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Abkhazian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Abom"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Abon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Abron"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Abu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Abu' Arapesh"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Abua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Abui"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Abun"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Abure"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Abureni"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Abé"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Acatepec Me'phaa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Acatepec Tlapanec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Achagua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Achang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ache"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Acheron"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Achi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Achinese"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Achterhoeks"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Achuar-Shiwiar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Achumawi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Aché"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Acoli"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Acroá"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Adabe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Adai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Adamawa Fulfulde"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Adamorobe Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Adang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Adangbe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Adangme"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Adap"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Adasen"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Adele"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Adhola"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Adi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Adioukrou"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Adivasi Oriya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Adiwasi Garasia"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Adnyamathanha"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Adonara"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Aduge"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Adygei"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Adyghe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Adzera"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Aeka"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Aekyom"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Aequian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Aer"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Afade"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Afar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Afghan Persian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Afghan Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Afitti"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Afra"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Afrihili"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Afrikaans"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Afro-Seminole Creole"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Agarabi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Agariya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Agatu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Agavotaguerra"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Aghem"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Aghu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Aghu Tharnggalu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Aghul"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Aghwan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Agi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Agob"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Agoi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Aguacateco"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Aguano"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Aguaruna"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Aguna"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Agusan Manobo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Agutaynen"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Agwagwune"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ahanta"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Aheu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ahirani"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ahom"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ahtena"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ahwai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ai-Cham"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Aighon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Aikanã"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Aiklep"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Aimaq"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Aimele"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Aimol"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ainbai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ainu (China)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ainu (Japan)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Aiome"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Airoran"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Aiton"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Aja (Benin)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Aja (Sudan)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ajawa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ajië"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ajyíninka Apurucayali"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ak"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Aka"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Aka-Bea"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Aka-Bo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Aka-Cari"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Aka-Jeru"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Aka-Kede"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Aka-Kol"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Aka-Kora"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Akan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Akar-Bale"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Akaselem"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Akawaio"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ake"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Akebu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Akei"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Akeu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Akha"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Akhvakh"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Akkadian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Akkala Sami"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Aklanon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Akolet"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Akoose"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Akoye"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Akpa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Akpes"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Akrukay"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Akuku"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Akum"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Akuntsu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Akurio"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Akwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Al-Sayyid Bedouin Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Alaba-K’abeena"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Alabama"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Alabat Island Agta"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Alacatlatzala Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Alago"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Alagwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Alak"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Alamblak"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Alangan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Alanic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Alapmunte"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Alawa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Albanian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Alcozauca Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Alege"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Alekano"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Alemannic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Aleut"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Algerian Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Algerian Saharan Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Algerian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Algonquin"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ali"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Alladian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Allar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Alngith"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Alo Phola"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Alor"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Aloápam Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Alsatian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Alsea"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Alu Kurumba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Alugu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Alumu-Tesu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Alune"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Aluo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Alur"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Alutor"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Alviri-Vidari"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Alyawarr"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ama (Papua New Guinea)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ama (Sudan)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Amahai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Amahuaca"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Amaimon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Amal"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Amanab"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Amanayé"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Amara"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Amarag"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Amarakaeri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Amarasi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Amatlán Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Amba (Solomon Islands)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Amba (Uganda)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ambai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ambakich"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ambala Ayta"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ambelau"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ambele"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Amblong"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ambo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ambo-Pasco Quechua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ambonese Malay"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ambrak"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ambulas"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Amdang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Amdo Tibetan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Amele"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("American Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Amganad Ifugao"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Amharic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ami"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Amis"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Amo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Amol"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Amoltepec Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ampanang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Amri Karbi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Amto"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Amundava"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ana Tinga Dogon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Anaang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Anakalangu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Anal"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Anam"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Anambé"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Anamgura"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Anasi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ancient Aramaic (up to 700 BCE)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ancient Greek (to 1453)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ancient Hebrew"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ancient Macedonian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ancient North Arabian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ancient Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Andai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Andalusian Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Andaman Creole Hindi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Andaqui"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Andarum"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Andegerebinha"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Andh"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Andi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Andio"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Andoa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Andoque"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Andra-Hus"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Aneityum"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Anem"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Aneme Wake"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Anfillo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Angaataha"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Angal"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Angal Enen"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Angal Heneng"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Angami Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Angguruk Yali"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Angika"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Anglo-Norman"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Angloromani"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Angolar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Angor"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Angoram"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Angosturas Tunebo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ani Phowa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Anii"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Animere"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Anindilyakwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Anjam"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ankave"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Anmatyerre"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Anor"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Anserma"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ansus"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Antakarinya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Antankarana Malagasy"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Antigua and Barbuda Creole English"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Anu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Anuak"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Anufo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Anuki"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Anus"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Anuta"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Anyin"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Anyin Morofo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ao Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Aoheng"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Aore"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ap Ma"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Apalachee"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Apalaí"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Apali"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Apalik"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Apasco-Apoala Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Apatani"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Apiaká"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Apinayé"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Apma"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Aproumu Aizi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Apurinã"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Aputai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Aquitanian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Arabana"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Arabela"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Aragonese"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Araki"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Aramanik"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Arammba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Aranadan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Aranama-Tamique"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Arandai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Araona"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Arapaho"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Arapaso"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Arawak"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Araweté"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Arawum"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Arawá"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Arbore"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Arbëreshë Albanian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Archi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ardhamāgadhī Prākrit"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Are"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Areba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Arem"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Arequipa-La Unión Quechua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Argentine Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Argobba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Arguni"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Arhuaco"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Arhâ"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Arhö"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ari"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Aribwatsa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Aribwaung"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Arigidi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Arikapú"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Arikara"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Arikem"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Arin"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Aringa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Arma"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Armazic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Armenian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Armenian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Aromanian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Arop-Lokep"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Arop-Sissano"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Arosi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Arpitan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Arta"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Aruamu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Aruek"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Arumanian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Aruop"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Arutani"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Aruá (Amazonas State)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Aruá (Rodonia State)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Arvanitika Albanian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("As"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Asaro'o"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Asas"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ashe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ashkun"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Asho Chin"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ashtiani"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Asháninka"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ashéninka Pajonal"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ashéninka Perené"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Asilulu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Askopan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Asoa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Assamese"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Assan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Assangori"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Assiniboine"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Assyrian Neo-Aramaic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Asturian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Asturleonese"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Asu (Nigeria)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Asu (Tanzania)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Asue Awyu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Asumboa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Asunción Mixtepec Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Asuri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ata"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ata Manobo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Atakapa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Atampaya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Atatláhuca Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Atayal"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Atemble"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Athpariya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ati"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Atikamekw"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Atohwaim"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Atong"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Atorada"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Atsahuaca"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Atsam"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Atsugewi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Attapady Kurumba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Attié"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Atzingo Matlatzinca"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Au"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Aulua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Aurá"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Aushi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Aushiri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Austral"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Australian Aborigines Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Australian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Austrian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Auwe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Auye"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Auyokawa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ava Guaraní"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Avaric"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Avatime"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Avau"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Avava"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Avestan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Avikam"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Avokaya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Avá-Canoeiro"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Awa (China)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Awa (Papua New Guinea)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Awa-Cuaiquer"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Awabakal"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Awad Bing"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Awadhi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Awak"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Awar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Awara"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Awbono"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Aweer"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Awera"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Awetí"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Awing"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Awiyaana"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Awjilah"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Awngi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Awtuw"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Awu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Awun"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Awutu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Awyi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Axamb"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Axi Yi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ayabadhu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ayacucho Quechua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ayangan Ifugao"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ayautla Mazatec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ayere"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ayi (China)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ayi (Papua New Guinea)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ayiwo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ayizi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ayizo Gbe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Aymara"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ayoquesco Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ayoreo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ayu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ayutla Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Azerbaijani"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Azha"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Azhe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Azoyú Me'phaa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Azoyú Tlapanec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Baan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Baangi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Baatonum"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Baba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Baba Malay"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Babalia Creole Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Babango"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Babanki"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Babatana"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Babine"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bable"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Babuza"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bacama"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bacanese Malay"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bactrian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bada (Indonesia)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bada (Nigeria)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Badaga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bade"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Badeshi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Badimaya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Badui"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Badyara"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Baeggu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Baelelea"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Baetora"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bafanji"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bafaw-Balong"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bafia"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bafut"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Baga Binari"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Baga Kaloum"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Baga Koga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Baga Manduri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Baga Mboteni"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Baga Sitemu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Baga Sobané"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bagheli"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bagirmi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bagirmi Fulfulde"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bago-Kusuntu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bagri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bagupi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bagusa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bagvalal"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Baha Buyang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Baham"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bahamas Creole English"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Baharna Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bahau"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bahinemo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bahing"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bahnar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bahonsuai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Baibai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Baikeno"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Baima"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Baimak"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bainouk-Gunyuño"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bainouk-Samik"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Baiso"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Baissa Fali"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bajan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bajelani"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Baka (Cameroon)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Baka (Sudan)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bakairí"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bakaka"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bakhtiari"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Baki"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bakoko"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bakole"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bakpinka"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bakumpai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bakwé"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Balaesang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Balangao"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Balangingi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Balanta-Ganja"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Balanta-Kentohe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Balantak"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Balau"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Baldemu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bali (Democratic Republic of Congo)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bali (Nigeria)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Balinese"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Balinese Malay"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Balkan Gagauz Turkish"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Balkan Romani"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Balo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Baloi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Balti"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Baltic Romani"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Baluan-Pam"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Baluchi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bamako Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bamali"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bambalang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bambam"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bambara"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bambassi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bambili-Bambui"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bamenyam"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bamu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bamukumbit"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bamun"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bamunka"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bamwe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ban Khor Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bana"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Banam Bay"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Banao Itneg"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Banaro"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Banda (Indonesia)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Banda Malay"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Banda-Bambari"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Banda-Banda"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Banda-Mbrès"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Banda-Ndélé"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Banda-Yangere"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bandi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bandial"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bandjalang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bandjigali"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bangala"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bangandu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bangba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Banggai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Banggarla"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bangi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bangi Me"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bangingih Sama"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bangka"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bangolan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bangubangu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bangwinji"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Baniva"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Baniwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Banjar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bankagooma"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bankon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bannoni"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bantawa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bantayanon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bantik"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bantoanon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Baoulé"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bara Malagasy"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Baraamu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Barai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Barakai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Baram Kayan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Barama"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Barambu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Baramu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Barapasi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Baras"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Barasana-Eduria"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Barbacoas"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Barbareño"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Barclayville Grebo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bardi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Barein"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bargam"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bari"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bariai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bariji"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Barikanchi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Barok"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Barombi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Barro Negro Tunebo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Barrow Point"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Baruga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Baruya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Barwe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Barzani Jewish Neo-Aramaic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Baré"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Barí"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Basa (Cameroon)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Basa (Nigeria)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Basa-Gumna"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Basa-Gurmana"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Basap"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Basay"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bashkardi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bashkir"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Basketo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Basque"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bassa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bassa-Kontagora"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bassari"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bassossi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bata"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Batad Ifugao"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Batak"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Batak Alas-Kluet"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Batak Angkola"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Batak Dairi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Batak Karo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Batak Mandailing"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Batak Simalungun"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Batak Toba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Batanga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Batek"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bateri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bathari"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bati (Cameroon)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bati (Indonesia)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bats"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Batu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Batui"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Batuley"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bau"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bau Bidayuh"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bauchi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Baure"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bauria"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bauro"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bauwaki"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bauzi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bavarian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bawm Chin"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bay Miwok"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bayali"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Baybayanon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Baygo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bayono"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bayot"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bayungu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bazigar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Beami"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Beaver"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Beba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bebe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bebele"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bebeli"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bebil"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bedawiyet"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bedik"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bedjond"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bedoanas"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Beeke"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Beele"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Beembe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Beezen"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Befang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Begbere-Ejar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Beja"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bekati'"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bekwarra"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bekwil"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Belait"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Belanda Bor"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Belanda Viri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Belarusian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Belhariya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Beli (Papua New Guinea)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Beli (Sudan)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Belize Kriol English"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bella Coola"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bellari"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bemba (Democratic Republic of Congo)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bemba (Zambia)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bembe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bena (Nigeria)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bena (Tanzania)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Benabena"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bench"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bende"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bendi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Beneraf"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Beng"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Benga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bengali"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Benggoi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bengkala Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bentong"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Benyadu'"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Beothuk"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bepour"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bera"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Berakou"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Berau Malay"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Berbice Creole Dutch"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Berik"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Berinomo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Berom"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Berta"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Berti"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Besisi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Besme"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Besoa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Betaf"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Betawi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bete"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bete-Bendi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Beti (Côte d'Ivoire)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Betta Kurumba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bezhta"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bhadrawahi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bhalay"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bharia"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bhatola"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bhatri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bhattiyali"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bhaya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bhele"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bhilali"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bhili"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bhojpuri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bhoti Kinnauri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bhujel"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bhunjia"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Biafada"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Biage"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Biak"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Biali"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bian Marind"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Biangai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Biao"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Biao Mon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Biao-Jiao Mien"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Biatah Bidayuh"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bidiyo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bidyara"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bidyogo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Biem"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bierebo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bieria"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Biete"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Big Nambas"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Biga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bijori"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bikaru"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bikol"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bikya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bila"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bilakura"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bilaspuri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bilba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bilbil"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bile"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bilin"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bilma Kanuri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Biloxi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bilua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bilur"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bima"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bimin"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bimoba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bina (Nigeria)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bina (Papua New Guinea)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Binahari"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Binandere"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bine"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bini"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Binji"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Binongan Itneg"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bintauna"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bintulu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Binukid"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Binumarien"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bipi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Birale"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Birao"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Birgit"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Birhor"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Biri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Biritai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Birked"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Birri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Birwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Biseni"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bishnupriya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bishuo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bisis"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bislama"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bisorio"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bissa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bisu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bit"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bitare"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bitur"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Biwat"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Biyo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Biyom"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Blablanga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Blafe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Blagar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Blang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Blin"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bliss"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Blissymbolics"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Blissymbols"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bo (Laos)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bo (Papua New Guinea)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bo-Rukul"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bo-Ung"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Boano (Maluku)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Boano (Sulawesi)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bobongko"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bobot"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bodo (Central African Republic)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bodo (India)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bodo Gadaba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bodo Parja"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bofi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Boga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bogaya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Boghom"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Boguru"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bohtan Neo-Aramaic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Boikin"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bokha"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Boko (Benin)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Boko (Democratic Republic of Congo)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bokobaru"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bokoto"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bokyi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bola"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bolango"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bole"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bolgarian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bolgo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bolia"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bolinao"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bolivian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bolo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Boloki"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bolon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bolondo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bolongan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bolyu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bom"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Boma"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bomboli"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bomboma"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bomitaba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bomu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bomwali"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bon Gula"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bonan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bondei"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bondo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bondoukou Kulango"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bondum Dom Dogon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bonerate"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bonerif"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bonggi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bonggo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bongili"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bongo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bongu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bonjo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bonkeng"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bonkiman"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bontok"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bookan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Boon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Boor"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bora"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Borana-Arsi-Guji Oromo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Border Kuna"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Borei"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Borgu Fulfulde"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Borna (Democratic Republic of Congo)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Borna (Ethiopia)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Boro (Ethiopia)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Boro (Ghana)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Borong"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Boruca"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Borôro"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Boselewa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bosngun"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bosnian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bote-Majhi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Botlikh"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Botolan Sambal"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bouna Kulango"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bouyei"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bozaba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bragat"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Brahui"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Braj"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Brazilian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Brek Karen"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Brem"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Breri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Breton"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bribri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Brithenig"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("British Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Brokkat"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Brokpake"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Brokskat"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Brooke's Point Palawano"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Broome Pearling Lugger Pidgin"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Brunei"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Brunei Bisaya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bu-Nao Bunu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bualkhaw Chin"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Buamu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bube"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bubi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bubia"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Budeh Stieng"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Budibud"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Budong-Budong"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Budu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Budukh"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Buduma"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Budza"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bugan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bugawac"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bughotu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Buginese"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Buglere"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bugun"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Buhi'non Bikol"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Buhid"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Buhutu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bukar-Sadung Bidayuh"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bukat"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bukharic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bukit Malay"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bukitan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bukiyip"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Buksa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bukusu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bukwen"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bulgarian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bulgarian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bulgebi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Buli (Ghana)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Buli (Indonesia)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bullom So"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bulo Stieng"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bulu (Cameroon)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bulu (Papua New Guinea)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bum"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bumaji"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bumbita Arapesh"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bumthangkha"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bun"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Buna"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bunaba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bunak"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bunama"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bundeli"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bung"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bungain"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bungku"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bungu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bunoge Dogon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bunun"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Buol"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bura-Pabir"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Burak"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Buraka"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Burarra"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Burate"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Burduna"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bure"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Buriat"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Burji"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Burmbar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Burmese"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Burmeso"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Buru (Indonesia)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Buru (Nigeria)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Burui"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Burum-Mindik"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Burumakok"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Burun"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Burunge"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Burushaski"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Burusu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Buruwai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Busa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Busam"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Busami"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Busang Kayan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bushi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bushoong"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Businenge Tongo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Buso"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Busoa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bussa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Busuu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Butbut Kalinga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Butmas-Tur"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Butuanon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Buwal"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Buxinhua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Buya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Buyu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Buyuan Jinuo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bwaidoka"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bwanabwana"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bwatoo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bwe Karen"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bwela"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bwile"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bwisi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Byangsi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Byep"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Bädi Kanum"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("C'lela"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Caac"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cabiyarí"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cabécar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cacaloxtepec Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cacaopera"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cacgia Roglai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cacua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Caddo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cafundo Creole"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cagua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cahuarano"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cahuilla"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cajamarca Quechua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cajatambo North Lima Quechua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cajonos Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cajun French"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Caka"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cakchiquel"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cakchiquel-Quiché Mixed Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cakfem-Mushere"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Calamian Tagbanwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Calderón Highland Quichua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Callawalla"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Caluyanun"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Caló"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cameroon Mambila"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cameroon Pidgin"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Camling"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Campalagian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Campidanese Sardinian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Camsá"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Camtho"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Camunic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Candoshi-Shapra"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Canela"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Canichana"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cao Lan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cao Miao"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Capanahua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Capiznon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cappadocian Greek"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Caquinte"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Car Nicobarese"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cara"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Carabayo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Caramanta"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Carapana"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Carian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Caribbean Hindustani"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Caribbean Javanese"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Carijona"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Carolina Algonquian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Carolinian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Carpathian Romani"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Carrier"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cashinahua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Casiguran Dumagat Agta"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Castilian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Casuarina Coast Asmat"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Catalan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Catalan Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Catawba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cauca"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cavineña"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cayubaba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cayuga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cayuse"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cañar Highland Quichua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cebaara Senoufo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cebuano"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Celtiberian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cemuhî"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cen"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Central Asmat"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Central Atlas Tamazight"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Central Awyu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Central Aymara"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Central Bai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Central Berawan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Central Bicolano"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Central Bontok"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Central Cagayan Agta"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Central Dusun"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Central Grebo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Central Hongshuihe Zhuang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Central Huasteca Nahuatl"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Central Huishui Hmong"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Central Huishui Miao"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Central Kanuri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Central Khmer"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Central Kurdish"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Central Maewo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Central Malay"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Central Masela"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Central Mashan Hmong"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Central Mashan Miao"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Central Mazahua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Central Melanau"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Central Mnong"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Central Nahuatl"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Central Nicobarese"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Central Ojibwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Central Okinawan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Central Palawano"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Central Pame"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Central Pashto"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Central Pomo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Central Puebla Nahuatl"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Central Sama"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Central Siberian Yupik"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Central Sierra Miwok"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Central Subanen"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Central Tagbanwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Central Tarahumara"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Central Tunebo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Central Yupik"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Central-Eastern Niger Fulfulde"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Centúúm"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cerma"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chachapoyas Quechua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chachi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chadian Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chadian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chadong"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chagatai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chaima"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chak"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chakali"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chakma"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chala"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chaldean Neo-Aramaic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chalikha"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chamacoco"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chamalal"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chamari"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chambeali"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chambri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chamicuro"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chamorro"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chang Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Changriwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Changthang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chantyal"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chané"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chara"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chaudangsi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chaungtha"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chaura"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chavacano"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chayahuita"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chayuco Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chazumba Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Che"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chechen"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cheke Holo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chemakum"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chenapian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chenchu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chenoua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chepang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chepya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cheq Wong"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cherepon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cherokee"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chesu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chetco"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chewa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chewong"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cheyenne"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chhattisgarhi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chhintange"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chhulung"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chiangmai Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chiapanec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chibcha"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chicahuaxtla Triqui"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chichewa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chichicapan Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chickasaw"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chicomuceltec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chidigo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chiduruma"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chiga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chigmecatitlán Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chilcotin"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chilean Quechua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chilean Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chilisso"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chiltepec Chinantec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chimakum"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chimalapa Zoque"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chimariko"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chimborazo Highland Quichua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chimila"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("China Buriat"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chinali"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chinbon Chin"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chincha Quechua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chinese"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chinese Pidgin English"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chinese Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chinook"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chinook jargon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chipaya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chipewyan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chipiajes"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chippewa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chiquihuitlán Mazatec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chiquitano"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chiquián Ancash Quechua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chiripá"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chiru"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chitimacha"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chitkuli Kinnauri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chittagonian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chitwania Tharu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Choapan Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chocangacakha"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chochotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Choctaw"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chodri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chokri Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chokwe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chol"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cholón"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chong"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Choni"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chopi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chorasmian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chortí"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chothe Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chrau"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chru"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chuang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chuanqiandian Cluster Miao"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chuave"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chug"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chuj"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chuka"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chukot"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chukwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chulym"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chumburung"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Churahi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Church Slavic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Church Slavonic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chut"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chuukese"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chuvantsy"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chuvash"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chuwabu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Chácobo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ci Gbe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cia-Cia"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cibak"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cicipu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cimbrian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cineni"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cinta Larga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cisalpine Gaulish"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cishingini"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Citak"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ciwogai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Clallam"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Classical Armenian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Classical Mandaic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Classical Mongolian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Classical Nahuatl"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Classical Nepal Bhasa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Classical Newari"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Classical Quechua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Classical Syriac"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Classical Tibetan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Coahuilteco"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Coast Miwok"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Coastal Kadazan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Coastal Konjo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Coatecas Altas Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Coatepec Nahuatl"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Coatlán Mixe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Coatlán Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Coatzospan Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cochimi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cocopa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cocos Islands Malay"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Coeur d'Alene"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cofán"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cogui"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Col"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Colombian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Colonia Tovar German"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Colorado"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Comaltepec Chinantec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Comanche"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Comecrudo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Como Karim"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Comox"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Con"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Congo Swahili"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cook Islands Maori"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Coos"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Copainalá Zoque"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Copala Triqui"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Coptic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Coquille"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cori"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cornish"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Corongo Ancash Quechua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Corsican"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Costa Rican Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cotabato Manobo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cotoname"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cowlitz"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Coxima"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Coyaima"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Coyotepec Popoloca"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Coyutla Totonac"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cree"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Creek"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Crimean Tatar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Crimean Turkish"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Croatia Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Croatian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cross River Mbembe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Crow"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cruzeño"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cuba Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cubeo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cuiba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Culina"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cumanagoto"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cumbric"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cumeral"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cun"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cuneiform Luwian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cung"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cupeño"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Curonian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Curripaco"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cusco Quechua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cutchi-Swahili"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cuvok"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cuyamecalco Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cuyonon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cwi Bwamu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Cypriot Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Czech"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Czech Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Côông"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Da'a Kaili"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Da-Hua Miao"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Daai Chin"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Daantanai'"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Daasanach"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Daba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dabarre"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dabe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dacian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dadibi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dadiya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Daga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dagaari Dioula"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dagba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dagbani"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dagik"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dagoman"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dahalo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Daho-Doo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dai Zhuang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dair"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dakaka"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dakka"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dakota"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dakpakha"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dalmatian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Daloa Bété"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dama"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Damakawa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Damal"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dambi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dameli"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dampelas"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Danaru"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Danau"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dandami Maria"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dangaléat"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dangaura Tharu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Danish"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Danish Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dano"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dao"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Daonda"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dar Daju Daju"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dar Fur Daju"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dar Sila Daju"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Darai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Darang Deng"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dargwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dari"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Darling"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Darlong"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Darmiya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Daro-Matu Melanau"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dass"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Datooga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Daur"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Davawenyo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dawawa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dawera-Daweloor"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dawida"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dawro"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Day"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dayi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Daza"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dazaga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Deccan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dedua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Defaka"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Defi Gbe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Deg"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Degaru"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Degema"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Degenan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Degexit'an"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dehu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dehwari"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dek"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dela-Oenale"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Delaware"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Delo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dem"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dema"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Demisa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Demta"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dendi (Benin)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dendi (Central African Republic)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dene Suline"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dengese"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dengka"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Deno"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Denya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dení"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Deori"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dera (Indonesia)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dera (Nigeria)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Desano"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Desiya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dewoin"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dezfuli"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dghwede"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dhaiso"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dhalandji"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dhangu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dhanki"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dhanwar (India)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dhanwar (Nepal)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dhao"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dhargari"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dhatki"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dhimal"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dhivehi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dhodia"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dhofari Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dholuo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dhundari"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dhurga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dhuwal"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dia"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dibabawon Manobo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dibiyaso"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dibo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dibole"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dicamay Agta"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Didinga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dido"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Diebroud"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dieri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Digaro-Mishmi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Digo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dii"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dijim-Bwilim"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dilling"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dima"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dimasa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dimbong"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dime"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dimili"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dimir"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dimli (individual language)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dimli (macrolanguage)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dineor"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ding"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dinka"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dirari"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dirasha"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Diri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Diriku"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dirim"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Disa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ditammari"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Diuwe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Diuxi-Tilantongo Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Divehi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dixon Reef"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dizin"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Djambarrpuyngu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Djamindjung"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Djangun"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Djauan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Djawi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Djeebbana"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Djimini Senoufo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Djinang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Djinba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Djingili"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Djiwarli"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dobel"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dobu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Doe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Doga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Doghoro"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dogoso"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dogosé"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dogri (individual language)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dogri (macrolanguage)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dogrib"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dogul Dom Dogon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Doka"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Doko-Uyanga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dolgan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dolpo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dom"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Domaaki"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Domari"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dombe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dominican Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dompo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Domu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Domung"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dondo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dong"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dongo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dongotono"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dongshanba Lalo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dongxiang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Donno So Dogon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Doondo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dori'o"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dorig"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Doromu-Koki"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dororo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dorze"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Doso"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Doutai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Doyayo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Drents"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Drung"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Duala"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Duano"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Duau"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dubli"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dubu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Duduela"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dugun"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Duguri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dugwor"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Duhwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Duke"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dulbu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Duli"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Duma"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dumbea"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dumi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dumpas"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dumpu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dumun"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Duna"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dungan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dungmali"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dungra Bhil"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dungu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dupaninan Agta"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dura"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Durango Nahuatl"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Duri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Duriankere"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Duruma"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Duruwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dusner"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dusun Deyah"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dusun Malang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dusun Witu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dutch"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dutch Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dutton World Speedwords"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Duungooma"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Duupa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Duvle"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Duwai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Duwet"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dwang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dyaabugay"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dyaberdyaber"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dyan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dyangadi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dyirbal"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dyugun"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dyula"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dza"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dzalakha"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dzando"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dzao Min"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dzodinka"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dzongkha"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dzùùngoo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Dâw"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("E"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("E'ma Buyang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("E'ñapa Woromaipu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Early Tripuri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("East Ambae"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("East Berawan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("East Damar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("East Futuna"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("East Kewa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("East Limba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("East Makian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("East Masela"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("East Nyala"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("East Tarangan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("East Yugur"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Eastern Abnaki"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Eastern Acipa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Eastern Apurímac Quechua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Eastern Arrernte"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Eastern Balochi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Eastern Bolivian Guaraní"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Eastern Bontok"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Eastern Bru"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Eastern Canadian Inuktitut"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Eastern Cham"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Eastern Egyptian Bedawi Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Eastern Frisian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Eastern Gorkha Tamang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Eastern Gurung"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Eastern Highland Chatino"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Eastern Highland Otomi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Eastern Hongshuihe Zhuang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Eastern Huasteca Nahuatl"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Eastern Huishui Hmong"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Eastern Huishui Miao"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Eastern Karaboro"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Eastern Katu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Eastern Kayah"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Eastern Keres"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Eastern Krahn"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Eastern Lalu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Eastern Lawa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Eastern Magar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Eastern Maninkakan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Eastern Mari"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Eastern Maroon Creole"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Eastern Meohang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Eastern Mnong"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Eastern Muria"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Eastern Ngad'a"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Eastern Nisu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Eastern Ojibwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Eastern Oromo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Eastern Parbate Kham"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Eastern Penan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Eastern Pomo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Eastern Qiandong Miao"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Eastern Tamang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Eastern Tawbuid"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Eastern Xiangxi Miao"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Eastern Xwla Gbe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Eastern Yiddish"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ebira"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Eblan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ebrié"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ebughu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ecuadorian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ede Cabe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ede Ica"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ede Idaca"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ede Ije"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Edera Awyu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Edo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Edolo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Edomite"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Edopi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Edwas"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Efai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Efe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Efik"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Efutop"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ega"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Eggon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Egypt Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Egyptian (Ancient)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Egyptian Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ehueun"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Eipomek"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Eitiep"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ejagham"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ejamat"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ekajuk"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ekari"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ekegusii"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Eki"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ekit"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ekpeye"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("El Alto Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("El Hugeirat"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("El Molo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("El Nayar Cora"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Elamite"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Eleme"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Elepi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Elip"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Elkei"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Elotepec Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Eloyi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Elpaputih"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Elseng"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Elu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Elymian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Emae"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Emai-Iuleha-Ora"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Eman"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Embaloh"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Emberá-Baudó"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Emberá-Catío"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Emberá-Chamí"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Emberá-Tadó"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Embu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Emem"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Emerillon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Emilian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Emok"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Emplawas"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Emumu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("En"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Enawené-Nawé"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ende"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Enga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Engenni"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Enggano"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("English"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Enrekang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Enu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Enwan (Akwa Ibom State)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Enwan (Edu State)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Enya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Epena"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Epi-Olmec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Epie"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Epigraphic Mayan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Eravallan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Erave"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ere"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Eritai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Erokwanas"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Erre"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Erromintxela"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ersu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Eruwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Erzya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Esan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ese"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ese Ejja"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Eshtehardi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Esimbi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Esperanto"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Esselen"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Estado de México Otomi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Estonian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Estonian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Esuma"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Etchemin"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Etebi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Eten"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Eteocretan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Eteocypriot"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ethiopian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Etkywan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Eton (Cameroon)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Eton (Vanuatu)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Etruscan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Etulo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Evant"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Even"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Evenki"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ewage-Notu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ewe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ewondo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Extremaduran"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Eyak"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Fa D'ambu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Fagani"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Faire Atta"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Faita"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Faiwol"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Fala"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Falam Chin"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Fali"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Faliscan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Fam"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Fanagalo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Fang (Cameroon)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Fang (Equatorial Guinea)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Fania"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Fanti"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Far Western Muria"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Farefare"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Faroese"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Fas"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Fasu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Fataleka"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Fataluku"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Fayu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Fe'fe'"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Fedan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Fembe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Fernando Po Creole English"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Feroge"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Fiji Hindi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Fijian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Filipino"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Filomena Mata-Coahuitlán Totonac"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Finland-Swedish Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Finnish"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Finnish Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Finongan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Fipa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Firan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Fiwaga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Flemish"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Flemish Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Flinders Island"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Foau"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Foi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Foia Foia"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Folopa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Foma"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Fon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Fongoro"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Foodo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Forak"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Fordata"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Fore"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Forest Enets"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Forest Maninka"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Fortsenal"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Francisco León Zoque"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Francoprovençal"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Frankish"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("French"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("French Belgian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("French Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Friulian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Fulah"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Fuliiru"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Fulniô"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Fum"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Fungwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Fur"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Furu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Futuna-Aniwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Fuyug"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Fwe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Fwâi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Fyam"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Fyer"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ga'anda"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ga'dang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gaa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gaam"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gabri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gadang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gaddang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gaddi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gade"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gade Lohar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gadjerawang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gadsup"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gaelic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gafat"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gagadu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gagauz"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gagnoa Bété"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gagu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gahri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gaikundi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gail"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gaina"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gal"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Galambu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Galatian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Galela"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Galeya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Galibi Carib"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Galice"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Galician"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Galindan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gallurese Sardinian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Galo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Galoli"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gamale Kham"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gambera"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gambian Wolof"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gamilaraay"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gamit"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gamkonora"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gamo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gamo-Ningi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gan Chinese"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gana"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ganang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ganda"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gane"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ganggalida"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ganglau"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gangte"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gangulu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gants"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ganza"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ganzi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gao"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gapapaiwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Garawa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Garhwali"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Garifuna"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Garig-Ilgar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Garo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Garre"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Garus"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Garza"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gata'"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gavar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gavião Do Jiparaná"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gawar-Bati"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gawwada"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gayil"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gayo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gazi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gbagyi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gbanu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gbanziri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gbari"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gbati-ri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gbaya (Central African Republic)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gbaya (Sudan)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gbaya-Bossangoa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gbaya-Bozoum"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gbaya-Mbodomo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gbayi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gbesi Gbe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gbii"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gbiri-Niragu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gboloo Grebo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ge"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Geba Karen"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gebe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gedaged"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gedeo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Geez"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Geji"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gejia"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Geko Karen"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gela"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gelao"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Geman Deng"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Geme"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gen"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gende"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gengle"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Georgian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gepo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gera"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("German"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("German Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Geruma"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Geser-Gorom"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gey"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ghadamès"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ghanaian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ghanongga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ghari"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ghayavi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gheg Albanian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ghera"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ghodoberi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ghomara"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ghomálá'"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ghotuo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ghulfan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Giangan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gibanawa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gichuka"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gidar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Giiwo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gikuyu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gikyode"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gilaki"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gilbertese"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gilima"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gilyak"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gimi (Eastern Highlands)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gimi (West New Britain)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gimme"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gimnime"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ginuman"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ginyanga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Girawa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Giryama"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gitonga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gitua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gitxsan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Giyug"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gizrra"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Glaro-Twabo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Glavda"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Glio-Oubi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gnau"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Goan Konkani"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Goaria"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gobasi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gobu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Godié"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Godwari"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Goemai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gofa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gogo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gogodala"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gokana"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gola"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Golin"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gondi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gone Dau"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gongduk"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gonja"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gooniyandi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gor"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gorakor"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gorap"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gorontalo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gorovu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gorowa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gothic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Goundo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gourmanchéma"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gowlan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gowli"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gowro"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gozarkhani"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Grangali"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Grass Koiari"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Grebo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Greek Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Green Gelao"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Greenlandic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Grenadian Creole English"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gresi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Groma"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gronings"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gros Ventre"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Guadeloupean Creole French"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Guahibo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Guajajára"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Guajá"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Guambiano"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Guana (Brazil)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Guana (Paraguay)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Guanano"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Guanche"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Guanyinqiao"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Guarani"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Guarayu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Guarequena"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Guatemalan Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Guató"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Guayabero"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gudanji"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gude"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gudu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Guduf-Gava"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Guerrero Amuzgo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Guerrero Nahuatl"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Guevea De Humboldt Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gugadj"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gugu Badhun"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gugu Warra"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gugubera"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Guguyimidjir"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Guhu-Samane"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Guianese Creole French"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Guibei Zhuang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Guiberoua Béte"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Guibian Zhuang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Guinea Kpelle"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Guinean Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Guiqiong"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gujarati"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gujari"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gula (Central African Republic)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gula (Chad)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gula Iro"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gula'alaa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gulay"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gule"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gulf Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Guliguli"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gumalu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gumatj"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gumawana"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gumuz"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gun"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gundi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gungabula"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gungu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Guntai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gunwinggu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gunya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gupa-Abawa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gupapuyngu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Guragone"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Guramalum"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gurani"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gurdjar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gureng Gureng"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gurgula"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Guriaso"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gurinji"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gurmana"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Guro"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Guruntum-Mbaaru"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gusan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gusii"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gusilay"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Guwamu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Guya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Guyanese Creole English"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Guyani"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gvoko"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gwahatike"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gwamhi-Wuri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gwandara"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gweda"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gweno"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gwere"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gwichʼin"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gyele"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Gyem"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Güilá Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ha"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Habu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hadiyya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hadothi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hadrami"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hadrami Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hadza"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Haeke"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hahon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hai//om"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Haida"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Haigwai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hainyaxo Bozo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Haiphong Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Haisla"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Haitian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Haitian Creole"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Haitian Vodoun Culture Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Haji"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hajong"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Haka Chin"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hakka Chinese"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hakö"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Halang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Halang Doan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Halbi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Halh Mongolian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Halia"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Halkomelem"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hamap"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hamba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hamer-Banna"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hamtai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Han"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hanga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hanga Hundi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hangaza"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hani"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hano"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hanoi Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hanunoo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Harami"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Harari"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Harijan Kinnauri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Haroi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Harsusi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Haruai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Haruku"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Haryanvi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Harzani"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hasha"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hassaniyya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hatam"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hattic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hausa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hausa Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Haveke"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Havu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hawai'i Creole English"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hawai'i Pidgin Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hawaiian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Haya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hazaragi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hdi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hebrew"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hehe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Heiban"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Heiltsuk"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Helambu Sherpa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Helong"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hema"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hemba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Herdé"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Herero"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hermit"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hernican"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Heung Kong Sau Yue"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hewa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Heyo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hiberno-Scottish Gaelic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hibito"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hidatsa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hieroglyphic Luwian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Higaonon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Highland Konjo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Highland Oaxaca Chontal"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Highland Popoluca"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Highland Puebla Nahuatl"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Highland Totonac"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hijazi Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hijuk"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hiligaynon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Himarimã"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hindi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hinduri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hinukh"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hiri Motu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hittite"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hitu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hiw"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hixkaryána"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hlai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hlepho Phowa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hlersu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hmar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hmong"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hmong Daw"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hmong Don"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hmong Dô"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hmong Njua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hmong Shua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hmwaveke"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ho"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ho Chi Minh City Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ho-Chunk"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hoava"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hobyót"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hoia Hoia"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Holikachuk"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Holiya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Holma"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Holoholo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Holu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Homa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Honduras Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hong Kong Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Honi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hopi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Horned Miao"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Horo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Horom"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Horpa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Horuru"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hote"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hoti"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hovongan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hoyahoya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hozo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hpon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hrangkhol"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hre"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hruso"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Huachipaeri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Huallaga Huánuco Quechua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Huamalíes-Dos de Mayo Huánuco Quechua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Huambisa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Huarijio"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Huastec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Huaulu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Huautla Mazatec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Huaxcaleca Nahuatl"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Huaylas Ancash Quechua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Huaylla Wanca Quechua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Huba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Huehuetla Tepehua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Huichol"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Huilliche"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Huitepec Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Huizhou Chinese"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hukumina"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hula"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hulaulá"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Huli"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hulung"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Humburi Senni Songhay"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Humene"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Humla"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hun-Saare"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hunde"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hung"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hungana"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hungarian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hungarian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hungworo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hunjara-Kaina Ke"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hunnic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hunsrik"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hunzib"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hupa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hupdë"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hupla"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hurrian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hutterisch"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hutterite German"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hwana"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hyam"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hértevin"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Hõne"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("I'saka"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("I-Wak"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Iaai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Iamalele"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Iapama"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Iatmul"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Iau"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ibali Teke"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ibaloi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Iban"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ibanag"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ibani"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ibatan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Iberian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ibibio"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ibilo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ibino"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ibu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ibuoro"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Icelandic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Icelandic Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Iceve-Maci"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ida'an"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Idaté"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Idere"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Idesa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Idi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ido"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Idoma"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Idon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Idu-Mishmi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Idun"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Iduna"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ifo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ifè"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Igala"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Igana"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Igbo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Igede"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ignaciano"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Igo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Iguta"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Igwe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Iha"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Iha Based Pidgin"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ihievbe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ija-Zuba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ik"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ika"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ikizu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Iko"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ikobi-Mena"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ikpeng"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ikpeshi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ikposo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Iku-Gora-Ankwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ikulu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ikwere"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ila"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ile Ape"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ili Turki"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ili'uun"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ilianen Manobo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Illyrian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Iloko"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ilongot"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ilue"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ilwana"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Imbabura Highland Quichua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Imbongu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Imeraguen"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Imonda"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Imperial Aramaic (700-300 BCE)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Imroing"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Inabaknon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Inapang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Inari Sami"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Indian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Indo-Portuguese"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Indonesian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Indonesian Bajau"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Indonesian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Indri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Indus Kohistani"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Indus Valley Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Inebu One"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ineseño"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Inga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ingrian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ingush"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Inlaod Itneg"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Inoke-Yate"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Inonhan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Inor"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Inpui Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Interglossa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Interlingua (International Auxiliary Language Association)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Interlingue"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("International Sign"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Intha"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Inuktitut"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Inupiaq"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Iowa-Oto"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ipalapa Amuzgo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ipiko"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ipili"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ipulo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Iquito"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ir"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Iranian Persian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Iranun"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Iraqw"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Irarutu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Iraya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Iresim"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Irigwe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Irish"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Irish Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Irula"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Irántxe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Isabi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Isanzu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Isarog Agta"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Isconahua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Isebe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Isekiri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ishkashimi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Isinai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Isirawa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Island Carib"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Islander Creole English"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Isnag"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Isoko"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Israeli Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Isthmus Mixe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Isthmus Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Isthmus-Cosoleacaque Nahuatl"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Isthmus-Mecayapan Nahuatl"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Isthmus-Pajapan Nahuatl"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Istriot"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Istro Romanian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Isu (Fako Division)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Isu (Menchum Division)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Italian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Italian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Itawit"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Itelmen"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Itene"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Iteri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Itik"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ito"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Itonama"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Itu Mbon Uzo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Itundujia Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Itzá"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Iu Mien"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ivatan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ivbie North-Okpela-Arhe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Iwaidja"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Iwal"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Iwam"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Iwur"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ixcatec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ixcatlán Mazatec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ixil"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ixtayutla Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ixtenco Otomi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Iyayu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Iyive"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Iyo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Iyo'wujwa Chorote"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Iyojwa'ja Chorote"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Izere"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Izon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Izora"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Iñapari"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jabutí"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jad"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jadgali"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jah Hut"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jahanka"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jair Awyu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jakalteko"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jakati"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jakun"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jalapa De Díaz Mazatec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jalkunan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jamaican Country Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jamaican Creole English"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jamaican Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jamamadí"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jambi Malay"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jamiltepec Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jamsay Dogon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jandavra"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jangkang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jangshung"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Janji"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Japanese"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Japanese Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Japrería"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jaqaru"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jara"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jarai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jarawa (India)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jarawa (Nigeria)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jaru"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jauja Wanca Quechua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jaunsari"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Javanese"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Javindo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jawe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jaya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jebero"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jeh"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jehai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jemez"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jenaama Bozo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jeng"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jennu Kurumba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jere"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jeri Kuo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jerung"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jewish Babylonian Aramaic (ca. 200-1200 CE)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jewish Palestinian Aramaic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jhankot Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jiamao"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jiarong"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jiba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jibu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jicarilla Apache"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jiiddu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jilbe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jilim"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jimi (Cameroon)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jimi (Nigeria)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jina"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jingpho"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jinyu Chinese"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jiongnai Bunu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jirel"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jiru"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jita"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jju"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Joba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jofotek-Bromnya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jola-Fonyi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jola-Kasa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jonkor Bourmataguil"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jordanian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jorto"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jorá"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jowulu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ju"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ju/'hoan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Juang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Judeo-Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Judeo-Berber"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Judeo-Georgian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Judeo-Iraqi Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Judeo-Italian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Judeo-Moroccan Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Judeo-Persian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Judeo-Tat"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Judeo-Tripolitanian Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Judeo-Tunisian Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Judeo-Yemeni Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jukun Takum"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jumjum"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jumla Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jumli"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jungle Inga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Juquila Mixe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jur Modo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Juray"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jurchen"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jurúna"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jutish"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Juwal"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Juxtlahuaca Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Jwira-Pepesa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Júma"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("K'iche'"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kaamba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kaan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kaansa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kaapor"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kaapor Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kaba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kabalai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kabardian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kabatei"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kabixí"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kabiyè"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kabola"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kabore One"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kabras"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kaburi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kabutra"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kabuverdianu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kabwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kabwari"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kabyle"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kachama-Ganjule"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kachari"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kachchi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kachi Koli"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kachin"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kacipo-Balesi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kaco'"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kadai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kadar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kadara"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kadaru"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kadiwéu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kado"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kaduo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kafa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kafoa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kagan Kalagan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kagate"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kagayanen"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kagoma"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kagoro"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kagulu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kahe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kahua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kaian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kaibobo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kaidipang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kaiep"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kaikadi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kaike"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kaiku"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kaimbulawa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kaimbé"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kaingang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kairak"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kairiru"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kairui-Midiki"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kais"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kaivi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kaiwá"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kaiy"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kajakse"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kajali"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kajaman"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kakabai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kakabe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kakanda"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kakauhua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kaki Ae"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kakihum"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kako"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kakwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kala Lagaw Ya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kalaallisut"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kalabakan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kalabari"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kalabra"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kalagan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kalaktang Monpa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kalam"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kalami"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kalamsé"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kalanadi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kalanga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kalao"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kalapuya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kalarko"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kalasha"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kalenjin"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kalispel-Pend d'Oreille"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kalkoti"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kalkutung"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kalmyk"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kalo Finnish Romani"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kalou"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kaluli"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kalumpang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kam"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kamakan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kamang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kamano"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kamantan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kamar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kamara"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kamarian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kamaru"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kamas"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kamasa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kamasau"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kamayo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kamayurá"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kamba (Brazil)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kamba (Kenya)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kambaata"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kambaira"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kambera"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kamberau"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kambiwá"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kami (Nigeria)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kami (Tanzania)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kamo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kamoro"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kamta"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kamu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kamula"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kamviri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kamwe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kanakanabu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kanamarí"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kanashi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kanasi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kanauji"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kandas"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kandawo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kande"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kanembu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kanga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kangean"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kanggape"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kangjia"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kango (Bas-Uélé District)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kango (Tshopo District)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kangri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kaniet"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kanikkaran"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kaningi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kaningra"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kaninuwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kanite"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kanjari"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kanjobal"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kanju"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kankanaey"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kannada"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kannada Kurumba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kanowit-Tanjong Melanau"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kanoé"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kansa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kantosi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kanu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kanufi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kanuri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kanyok"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kao"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kaonde"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kap"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kapampangan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kapauri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kapin"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kapinawá"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kapingamarangi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kapori"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kapriman"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kaptiau"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kapya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kaqchikel"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kara (Central African Republic)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kara (Korea)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kara (Papua New Guinea)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kara (Tanzania)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kara-Kalpak"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Karachay-Balkar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Karadjeri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Karagas"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Karahawyana"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Karaim"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Karajá"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Karakhanid"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Karami"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Karamojong"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Karang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Karanga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Karankawa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Karao"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Karas"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Karata"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Karawa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Karbi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kare (Central African Republic)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kare (Papua New Guinea)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Karekare"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Karelian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Karey"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kari"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Karingani"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Karipuna"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Karipúna"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Karipúna Creole French"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Karirí-Xocó"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Karitiâna"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kariya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kariyarra"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Karkar-Yuri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Karkin"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Karko"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Karnai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Karo (Brazil)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Karo (Ethiopia)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Karok"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Karon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Karon Dori"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Karore"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kasanga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kasem"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kashaya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kashmiri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kashubian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kasiguranin"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kaska"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kaskean"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kasseng"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kasua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kataang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Katabaga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Katawixi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Katbol"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kathoriya Tharu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kathu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kati"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Katkari"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Katla"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kato"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Katso"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Katua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Katukína"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kaulong"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kaur"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kaure"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kaurna"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kauwera"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kavalan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kavet"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kawacha"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kawaiisu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kawe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kawi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kaxararí"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kaxuiâna"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kayabí"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kayagar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kayan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kayan Mahakam"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kayan River Kayan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kayapa Kallahan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kayapó"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kayardild"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kayeli"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kayong"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kayort"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kaytetye"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kayupulau"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kazakh"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kazukuru"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ke'o"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Keak"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Keapara"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kedah Malay"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kedang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Keder"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kehu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kei"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Keiga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Keijar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kein"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Keiyo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kekchí"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kela (Democratic Republic of Congo)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kela (Papua New Guinea)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kelabit"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kele (Democratic Republic of Congo)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kele (Papua New Guinea)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Keley-I Kallahan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Keliko"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kelo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kelon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kemak"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kembayan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kemberano"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kembra"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kemezung"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kemi Sami"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kemiehua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kemtuik"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kenaboi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kenati"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kendayan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kendeje"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kendem"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kenga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Keningau Murut"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Keninjal"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kensiu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kenswei Nsei"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kenuzi-Dongola"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kenyan Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kenyang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kenyi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Keoru-Ahia"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kepkiriwát"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kepo'"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kera"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kerak"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kereho"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kerek"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kerewe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kerewo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kerinci"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kesawai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ket"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ketangalan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kete"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ketengban"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ketum"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Keyagana"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kgalagadi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Khakas"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Khalaj"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Khaling"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Khamba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Khams Tibetan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Khamti"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Khamyang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Khana"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Khandesi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Khanty"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Khao"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kharam Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kharia"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kharia Thar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Khasi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Khayo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Khazar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Khe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Khehek"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Khengkha"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Khetrani"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Khezha Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Khiamniungan Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Khinalugh"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Khirwar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Khisa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Khlor"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Khlula"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Khmu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kho'ini"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Khoibu Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kholok"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Khonso"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Khorasani Turkish"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Khorezmian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Khotanese"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Khowar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Khua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Khuen"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Khumi Awa Chin"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Khumi Chin"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Khunsari"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Khvarshi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kháng"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Khün"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kibet"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kibiri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kickapoo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kiembu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kigiryama"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kikai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kikuyu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kildin Sami"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kilivila"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kiliwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kilmeri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kim"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kim Mun"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kimaama"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kimaghima"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kimaragang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kimbu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kimbundu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kimki"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kimré"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kinabalian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kinalakna"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kinamiging Manobo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kinaray-A"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kinga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kinnauri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kintaq"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kinuku"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kinyarwanda"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kioko"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kiong"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kiorr"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kiowa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kiowa Apache"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kipfokomo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kipsigis"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kiput"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kir-Balar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kirdki"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kire"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kirghiz"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kirike"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kirikiri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kirmanjki (individual language)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kirmanjki (macrolanguage)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kis"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kisa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kisankasa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kisar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kisi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kistane"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kiswahili"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kita Maninkakan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kitan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kitharaka"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kitja"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kitsai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kituba (Congo)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kituba (Democratic Republic of Congo)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kiwilwana"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Klamath-Modoc"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Klao"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Klias River Kadazan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Klingon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Knaanic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ko"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Koalib"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Koasati"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Koba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kobiana"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kobol"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kobon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Koch"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kochila Tharu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Koda"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kodaku"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kodava"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kodeoha"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kodi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kodia"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Koenoem"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kofa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kofei"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kofyar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kogi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Koguryo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kohin"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kohistani Shina"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Koho"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kohumono"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Koi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Koibal"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Koireng"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Koitabu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Koiwat"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kok Borok"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kokata"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Koke"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kokoda"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kokola"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kokota"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kol"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kol (Cameroon)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kol (Papua New Guinea)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kola"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kolbila"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kolibugan Subanon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kolum So Dogon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Koluwawa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kom (Cameroon)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kom (India)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Koma"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Komba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kombai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kombio"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Komering"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Komi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Komi-Permyak"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Komi-Zyrian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kominimung"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Komo (Democratic Republic of Congo)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Komo (Sudan)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Komodo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kompane"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Komyandaret"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kon Keu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Konabéré"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Konai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Konda"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Konda-Dora"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Koneraw"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kongo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Konkani (individual language)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Konkani (macrolanguage)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Konkomba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Konni"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kono (Guinea)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kono (Nigeria)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kono (Sierra Leone)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Konomala"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Konongo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Konso"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Konyak Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Konyanka Maninka"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Konzo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Koongo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Koonzime"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Koorete"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kopar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kopkaka"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Korafe-Yegha"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Korak"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Korana"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Korandje"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Korean"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Korean Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Koreguaje"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Koresh-e Rostam"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Korku"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Korlai Creole Portuguese"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Koro (Côte d'Ivoire)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Koro (Papua New Guinea)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Koro (Vanuatu)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Koromfé"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Koromira"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Koronadal Blaan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Koroni"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Korop"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Koropó"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Koroshi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Korowai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Korra Koraga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Korubo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Korupun-Sela"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Korur"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Korwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Koryak"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kosadle"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kosare"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kosarek Yale"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kosena"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Koshin"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kosraean"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kota (Gabon)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kota (India)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kota Bangun Kutai Malay"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kota Marudu Talantang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kota Marudu Tinagas"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kotafon Gbe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kotava"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Koti"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kott"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kouya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kovai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kove"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kowaki"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kowiai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Koy Sanjaq Surat"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Koya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Koyaga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Koyo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Koyra Chiini Songhay"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Koyraboro Senni Songhai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Koyukon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kpagua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kpala"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kpan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kpasam"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kpati"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kpatili"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kpelle"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kpessi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kplang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Krache"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Krahô"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kraol"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Krenak"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Krevinian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kreye"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Krikati-Timbira"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Krim"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Krio"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kriol"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Krisa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Krobu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Krongo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kru'ng 2"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Krymchak"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kryts"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kuan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kuanhua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kuanua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kuanyama"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kube"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kubi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kubo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kubu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kucong"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kudiya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kudmali"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kudu-Camo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kugama"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kugbo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kui (India)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kui (Indonesia)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kuijau"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kujarge"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kuk"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kukatja"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kukele"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kukna"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kuku-Mangk"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kuku-Mu'inh"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kuku-Muminh"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kuku-Ugbanh"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kuku-Uwanh"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kuku-Yalanji"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kula"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kulere"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kulfa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kulina"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kulina Pano"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kulisusu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kullu Pahari"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kulon-Pazeh"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kulung (Nepal)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kulung (Nigeria)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kumak"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kumalu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kumam"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kuman"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kuman (Russia)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kumaoni"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kumarbhag Paharia"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kumba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kumbainggar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kumbaran"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kumbewaha"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kumhali"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kumiai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kumukio"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kumyk"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kumzari"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kunama"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kunbarlang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kunda"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kundal Shahi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kunduvadi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kung"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kung-Ekoka"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kungarakany"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kunggara"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kunggari"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kuni"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kuni-Boazi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kunigami"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kunimaipa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kunja"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kunjen"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kunyi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kunza"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kuo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kuot"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kupa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kupang Malay"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kupia"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kupsabiny"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kur"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kura Ede Nago"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kurama"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kuranko"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kurdish"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kuri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kuria"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kurichiya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kurmukar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kurrama"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kurti"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kurtokha"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kurudu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kurukh"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kuruáya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kusaal"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kusaghe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kushi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kusu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kusunda"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kutang Ghale"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kutenai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kutep"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kuthant"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kutto"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kutu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kuturmi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kuuku-Ya'u"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kuvi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kuwaa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kuwaataay"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kuy"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kven Finnish"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kw'adza"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kwa'"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kwaami"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kwadi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kwaio"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kwaja"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kwak"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kwakiutl"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kwakum"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kwama"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kwambi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kwamera"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kwami"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kwamtim One"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kwang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kwanga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kwangali"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kwanja"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kwanyama"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kwara'ae"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kwasio"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kwato"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kwaya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kwaza"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kwegu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kwer"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kwerba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kwerba Mamberamo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kwere"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kwerisa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kwese"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kwesten"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kwini"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kwinsu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kwinti"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kwoma"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kwomtari"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kxoe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kyak"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kyaka"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kyenele"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kyenga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kyerung"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kyrgyz"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kâte"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kélé"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Kölsch"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("La'bi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Laal"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Laalaa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Laari"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Laba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Label"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Labir"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Labo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Labo Phowa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Labu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Labuk-Kinabatangan Kadazan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lacandon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lachi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lachiguiri Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lachixío Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ladakhi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ladin"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ladino"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Laeko-Libuat"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lafofa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Laghu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Laghuu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lagwan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Laha (Indonesia)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Laha (Viet Nam)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lahanan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lahnda"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lahta Karen"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lahu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lahu Shi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lahul Lohar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Laimbue"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Laiyolo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lak"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Laka (Chad)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Laka (Nigeria)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lakalei"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lake Miwok"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lakha"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Laki"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lakkia"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lakon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lakondê"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lakota"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lakota Dida"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lala"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lala-Bisa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lala-Roba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lalana Chinantec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lalia"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lama (Myanmar)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lama (Togo)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lamaholot"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lamalera"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lamam"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lamang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lamatuka"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lamba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lambadi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lambayeque Quechua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lambichhong"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lamboya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lambya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lame"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lamenu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lamet"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lamkang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lamma"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lamnso'"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lamogai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lampung Api"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lampung Nyo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lamu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lamu-Lamu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lanas Lobu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Landoma"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lang'e"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Langam"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Langbashe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Langi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Langnian Buyang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lango (Sudan)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lango (Uganda)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Langobardic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Langue des signes de Belgique Francophone"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lanoh"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lao"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Laomian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Laopang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Laos Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lapaguía-Guivini Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Laragia"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Larantuka Malay"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lardil"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Larevat"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Large Flowery Miao"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lari"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Larike-Wakasihu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Laro"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Larteh"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Laru"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lasalimu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lasgerdi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lashi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lasi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Late Middle Chinese"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Latgalian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Latin"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Latu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Latundê"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Latvian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Latvian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lau"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Laua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lauan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lauje"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Laura"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Laurentian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lave"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Laven"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lavukaleve"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lawangan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lawunuia"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Layakha"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Laz"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lealao Chinantec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Leco"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ledo Kaili"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Leelau"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lefa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lega-Mwenga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lega-Shabunda"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Legbo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Legenyem"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lehali"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lehalurup"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lehar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Leinong Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Leipon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lelak"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lele (Chad)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lele (Democratic Republic of Congo)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lele (Guinea)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lele (Papua New Guinea)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lelemi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lelepa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lembena"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lemerig"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lemio"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lemnian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lemolang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lemoro"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lenakel"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lenca"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lendu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lengilu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lengo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lengola"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lengua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lengua de Señas Panameñas"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lengua de Señas del Paraguay"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lengua de señas catalana"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Leningitij"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lenje"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lenkau"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lenyima"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Leonese"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lepcha"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lepki"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lepontic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lere"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lese"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lesing-Gelimi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Letemboi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Leti (Cameroon)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Leti (Indonesia)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Letzeburgesch"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Levuka"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lewo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lewo Eleng"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lewotobi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Leyigha"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lezghian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lhaovo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lhokpu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lhomi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Li'o"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Liabuku"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Liana-Seti"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Liangmai Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lianshan Zhuang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Liberia Kpelle"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Liberian English"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Libido"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Libinza"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Libon Bikol"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Liburnian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Libyan Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Libyan Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ligbi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ligenza"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ligurian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ligurian (Ancient)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lihir"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lijili"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lika"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Liki"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Likila"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Likuba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Likum"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Likwala"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lilau"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lillooet"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Limassa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Limbu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Limbum"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Limburgan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Limburger"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Limburgish"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Limi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Limilngan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Limos Kalinga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lindu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Linear A"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lingala"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lingao"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lingarak"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lingkhim"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lingua Franca"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lingua Franca Nova"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lipan Apache"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lipo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lisela"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lish"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lishana Deni"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lishanid Noshan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lishán Didán"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lisu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Literary Chinese"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lithuanian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lithuanian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Litzlitz"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Liujiang Zhuang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Liuqian Zhuang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Liv"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Livvi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Llengua de Signes Catalana"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Llengua de signes valenciana"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lo-Toga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Loarki"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lobala"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lobi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lodhi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Logba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Logo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Logol"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Logooli"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Logorik"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Logudorese Sardinian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Loja Highland Quichua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lojban"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lokaa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Loko"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lokoya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lola"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lolak"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lole"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lolo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Loloda"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lolopo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Loma (Côte d'Ivoire)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Loma (Liberia)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lomaiviti"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lomavren"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lombard"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lombi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lombo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lomwe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Loncong"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Long Phuri Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Long Wat"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Longgu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Longto"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Longuda"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Loniu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lonwolwol"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lonzo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Loo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lopa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lopi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lopit"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lorang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lorediakarkar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Loreto-Ucayali Spanish"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lote"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lotha Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lotud"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lou"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Louisiana Creole French"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Loun"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Loup A"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Loup B"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lovono"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Low German"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Low Saxon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lowa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lower Burdekin"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lower Chehalis"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lower Grand Valley Dani"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lower Silesian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lower Sorbian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lower Ta'oih"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lower Tanana"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lower Tanudan Kalinga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lowland Oaxaca Chontal"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lowland Tarahumara"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Loxicha Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lozi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lua'"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Luang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Luba-Katanga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Luba-Lulua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lubila"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lubu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lubuagan Kalinga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lubukusu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Luchazi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lucumi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ludian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lufu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lugbara"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Luguru"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Luhu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lui"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Luimbi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Luiseno"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lukabaras"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lukpa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lule Sami"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lulogooli"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lumba-Yakkha"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lumbee"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lumbu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lumun"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lun Bawang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Luna"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lunanakha"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lunda"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lundayeh"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lungga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Luo (Cameroon)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Luo (Kenya and Tanzania)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Luopohe Hmong"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Luopohe Miao"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Luri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lusengo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lushai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lushootseed"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lusi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lusitanian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lutachoni"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lutos"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Luvale"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Luwati"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Luwo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Luxembourgish"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Luyana"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Luyia"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lwalu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lycian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lydian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lyngngam"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lyons Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lyélé"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Láadan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Láá Láá Bwamu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Löyöp"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Lü"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ma (Democratic Republic of Congo)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ma (Papua New Guinea)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ma'anyan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ma'di"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ma'ya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Maa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Maaka"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Maasina Fulfulde"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Maay"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Maba (Chad)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Maba (Indonesia)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mabaale"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mabaan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mabaka Valley Kalinga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mabire"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Maca"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Macaguaje"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Macaguán"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Macanese"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Macedo-Romanian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Macedonian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Machame"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Machiguenga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Machinere"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Machinga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Maco"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Macuna"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Macushi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mada (Cameroon)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mada (Nigeria)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Madagascar Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Madak"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Maden"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Madi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Madngele"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Madurese"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mae"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Maek"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Maeng Itneg"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mafa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mafea"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mag-Indi Ayta"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mag-antsi Ayta"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Magahi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Magbukun Ayta"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Magdalena Peñasco Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Magoma"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Magori"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Maguindanaon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mah Meri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mahali"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mahasu Pahari"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mahei"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mahican"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mahongwe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mahou"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mai Brat"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Maia"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Maiadomu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Maiani"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Maii"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mailu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Maindo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mainfränkisch"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mainstream Kenyah"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mairasi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Maisin"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Maithili"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Maiwa (Indonesia)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Maiwa (Papua New Guinea)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Maiwala"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Majang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Majera"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Majhi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Majhwar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Majukayang Kalinga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mak (China)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mak (Nigeria)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Makaa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Makah"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Makasae"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Makasar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Makassar Malay"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Makayam"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Makhuwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Makhuwa-Meetto"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Makhuwa-Moniga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Makhuwa-Saka"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Makhuwa-Shirima"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Maklew"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Makolkol"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Makonde"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Maku'a"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Makuri Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Makuráp"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Makwe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Makyan Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mal"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mal Paharia"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mala (Nigeria)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mala (Papua New Guinea)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mala Malasar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Malaccan Creole Malay"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Malaccan Creole Portuguese"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Malagasy"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Malakhel"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Malalamai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Malango"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Malankuravan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Malapandaram"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Malaryan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Malas"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Malasanga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Malasar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Malavedan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Malawi Lomwe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Malawi Sena"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Malay (individual language)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Malay (macrolanguage)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Malayalam"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Malayic Dayak"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Malaynon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Malayo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Malaysian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Malba Birifor"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Maldivian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Male (Ethiopia)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Male (Papua New Guinea)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Maleng"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Maleu-Kilenge"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Malfaxal"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Malgana"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Malgbe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mali"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Maligo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Malila"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Malimba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Malimpung"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Malinaltepec Me'phaa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Malinaltepec Tlapanec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Malo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Malol"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Maltese"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Maltese Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Malua Bay"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Malvi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Maléku Jaíka"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Malê"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mam"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mama"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mamaa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mamaindé"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mamanwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mamara Senoufo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mamasa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mambae"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mambai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mamboru"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mambwe-Lungu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mampruli"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mamuju"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mamulique"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mamusi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mamvu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Man Met"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Manado Malay"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Manam"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Manambu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Manangba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Manangkari"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Manchu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Manda (Australia)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Manda (India)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Manda (Tanzania)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mandahuaca"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mandaic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mandan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mandandanyi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mandar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mandara"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mandari"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mandarin Chinese"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mandaya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mandeali"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mander"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mandingo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mandinka"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mandjak"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mandobo Atas"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mandobo Bawah"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Manem"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Manga Kanuri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mangala"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mangarayi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mangareva"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mangas"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mangayat"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mangbetu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mangbutu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mangerr"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mangga Buang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Manggarai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mango"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mangole"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mangseng"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mangue"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Manichaean Middle Persian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Manide"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Manikion"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Manipa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Manipuri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mankanya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mann"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Manna-Dora"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mannan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Manombai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mansaka"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mansi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mansoanka"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Manta"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mantsi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Manumanaw Karen"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Manusela"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Manx"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Manya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Manyawa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Manyika"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Manza"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mao Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Maonan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Maore Comorian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Maori"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mape"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mapena"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mapia"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mapidian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mapos Buang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mapoyo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mapuche"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mapudungun"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mapun"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Maquiritari"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mara"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mara Chin"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Marachi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Maraghei"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Maragus"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Maram Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Marama"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Maramba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Maranao"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Maranunggu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mararit"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Marathi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Marau"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Marba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Maremgi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Marenje"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Marfa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Margany"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Marghi Central"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Marghi South"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Margos-Yarowilca-Lauricocha Quechua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Margu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mari (East Sepik Province)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mari (Madang Province)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mari (Russia)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Maria (India)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Maria (Papua New Guinea)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Maricopa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Maridan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Maridjabin"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Marik"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Marimanindji"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Marind"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Maring"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Maring Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Maringarr"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Marino"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mariri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Marithiel"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Maritime Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Maritsauá"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mariyedi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Marka"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Markweeta"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Marma"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Marovo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Marriammu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Marrucinian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Marshallese"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Marsian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Martha's Vineyard Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Marti Ke"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Martu Wangka"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Martuyhunira"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Maru"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Marwari"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Marwari (India)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Marwari (Pakistan)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Marúbo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Masaaba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Masadiit Itneg"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Masai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Masalit"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Masana"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Masbatenyo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mashco Piro"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mashi (Nigeria)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mashi (Zambia)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Masikoro Malagasy"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Masimasi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Masiwang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Maskelynes"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Maskoy Pidgin"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Maslam"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Masmaje"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Massalat"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Massep"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Matagalpa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Matal"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Matbat"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Matengo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Matepi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Matigsalug Manobo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Matipuhy"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mato"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mato Grosso Arára"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mator"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Matsés"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mattole"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Matukar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Matumbi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Matya Samo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Matís"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Maung"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mauritian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mauwake"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mawa (Chad)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mawa (Nigeria)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mawak"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mawan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mawayana"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mawchi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mawes"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mawukakan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Maxakalí"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Maxi Gbe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Maya Samo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mayaguduna"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mayangna"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mayeka"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Maykulan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mayo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mayogo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mayoyao Ifugao"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mazagway"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mazaltepec Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mazanderani"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mazatlán Mazatec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mazatlán Mixe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mbabaram"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mbala"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mbalanhu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mbandja"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mbangala"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mbangi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mbangwe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mbara (Australia)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mbara (Chad)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mbati"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mbato"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mbay"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mbe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mbe'"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mbedam"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mbelime"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mbere"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mbesa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mbo (Cameroon)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mbo (Democratic Republic of Congo)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mboi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mboko"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mbole"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mbonga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mbongno"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mbosi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mbowe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mbre"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mbu'"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mbugu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mbugwe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mbuko"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mbukushu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mbula"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mbula-Bwazza"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mbule"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mbulungish"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mbum"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mbunda"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mbunga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mburku"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mbwela"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mbyá Guaraní"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Me'en"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mea"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Medebur"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Media Lengua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mediak"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Median"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mednyj Aleut"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Medumba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mefele"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Megam"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Megleno Romanian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mehek"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mehináku"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mehri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mekeo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mekmek"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mekwei"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mele-Fila"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Melo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Melpa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Memoni"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mendalam Kayan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mendankwe-Nkwen"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mende (Papua New Guinea)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mende (Sierra Leone)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mengaka"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mengen"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mengisa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Menka"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Menominee"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mentawai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Menya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Meoswar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mer"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Meramera"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Merei"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Merey"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Meriam"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Merlav"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Meroitic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Meru"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Merwari"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mesaka"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mescalero-Chiricahua Apache"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mese"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Meskwaki"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mesme"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mesmes"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mesopotamian Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mesqan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Messapic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Meta'"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Metlatónoc Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mewari"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mewati"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mexican Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Meyah"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mezontla Popoloca"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mezquital Otomi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mfinu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mfumte"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mi'kmaq"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Miahuatlán Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Miami"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Miani"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Michif"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Michigamea"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Michoacán Mazahua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Michoacán Nahuatl"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Micmac"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mid Grand Valley Dani"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mid-Southern Banda"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Middle Armenian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Middle Breton"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Middle Cornish"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Middle Dutch (ca. 1050-1350)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Middle English (ca. 1100-1500)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Middle French (ca. 1400-1600)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Middle High German (ca. 1050-1350)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Middle Hittite"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Middle Irish (ca. 900-1200)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Middle Korean (10th-16th cent.)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Middle Low German (ca. 1100-1600)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Middle Mongolian (13th to the early 15th cent.)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Middle Newar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Middle Watut"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Middle Welsh (12th to 14th cent.)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Midob"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Migaama"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Migabac"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Migum"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Miji"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Miju-Mishmi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mikasuki"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mili"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Miltu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Miluk"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Milyan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Min Bei Chinese"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Min Dong Chinese"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Min Nan Chinese"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Min Zhong Chinese"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mina (Cameroon)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mina (India)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Minaean"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Minangkabau"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Minanibai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Minaveha"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mindiri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mingang Doso"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mingrelian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Minica Huitoto"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Minidien"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Minigir"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Minoan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Minokok"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Minriq"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mintil"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Minz Zhuang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Miqie"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mirandese"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Miraya Bikol"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mire"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mirgan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Miri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Miriti"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Miriwung"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mirpur Panjabi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Miship"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Misima-Paneati"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mitla Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mitlatongo Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mittu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mituku"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Miu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Miwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mixed Great Andamanese"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mixifore"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mixtepec Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mixtepec Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Miya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Miyako"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Miyobe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mlabri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mlahsö"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mlap"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mlomp"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mmaala"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mmen"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mo'da"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Moabite"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Moba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mobilian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mobumrin Aizi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mochi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mochica"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mocho"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mocoví"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Modang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Modern Greek (1453-)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Modole"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Moere"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mofu-Gudur"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mogholi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mogum"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mohave"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mohawk"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mohegan-Pequot"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Moi (Congo)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Moi (Indonesia)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Moikodi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Moingi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Moji"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mok"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Moken"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mokerang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mokilese"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Moklen"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mokole"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mokpwe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Moksela"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Moksha"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Molale"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Molbog"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Moldavian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Moldova Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Moldovan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Molengue"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Molima"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Molmo One"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Molo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Molof"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Moloko"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mom Jango"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Moma"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Momare"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mombum"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Momina"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Momuna"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Monastic Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mondropolon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mondé"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mong"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mong Leng"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mong Njua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mongo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mongol"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mongolia Buriat"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mongolian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mongolian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mongondow"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Moni"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mono (Cameroon)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mono (Democratic Republic of Congo)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mono (Solomon Islands)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mono (USA)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Monom"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Monsang Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Montagnais"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Montol"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Monumbo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Monzombo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Moo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Moose Cree"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mopán Maya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mor (Bomberai Peninsula)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mor (Mor Islands)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Moraid"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Morawa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Morelos Nahuatl"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Morerebi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Moresada"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mori Atas"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mori Bawah"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Morigi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Morisyen"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Moro"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Moroccan Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Moroccan Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Morokodo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Morom"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Moronene"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Morori"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Morouas"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mortlockese"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Moru"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mosimo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mosiro"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Moskona"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mossi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mota"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Motlav"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Motu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mouk-Aria"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mountain Koiali"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Movima"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Moyadan Itneg"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Moyon Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mozambican Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mozarabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mpade"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mpi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mpiemo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mpongmpong"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mpoto"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mpotovoro"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mpuono"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mpur"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mro Chin"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mru"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mser"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mt. Iraya Agta"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mt. Iriga Agta"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mualang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mubami"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mubi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Muda"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mudbura"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mudhili Gadaba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mudu Koraga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Muduapa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Muduga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mufian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mugom"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Muinane"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mukha-Dora"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mukulu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mulaha"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mulam"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mullu Kurumba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mullukmulluk"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Multiple languages"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Muluridyi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mum"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mumuye"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Muna"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Munda"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mundabli"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mundang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mundani"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mundari"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mundat"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mundurukú"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mungaka"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Munggui"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mungkip"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Muniche"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Munit"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Munji"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Munsee"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Muong"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Muratayak"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Murik (Malaysia)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Murik (Papua New Guinea)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Murkim"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Murle"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Murrinh-Patha"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mursi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Murui Huitoto"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Murupi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Muruwari"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Musak"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Musar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Musasa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Musey"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Musgu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mushungulu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Musi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Muskum"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Muslim Tat"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Musom"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mussau-Emira"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Muthuvan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mutu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Muya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Muyang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Muyuw"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Muzi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mvanip"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mvuba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mwaghavul"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mwali Comorian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mwan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mwani"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mwatebu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mwera (Chimwera)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mwera (Nyasa)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mwerlap"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mwimbi-Muthambi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mwotlap"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mycenaean Greek"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Myene"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mysian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mzieme Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mághdì"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mískito"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mócheno"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mün Chin"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Mündü"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Māhārāṣṭri Prākrit"); //$NON-NLS-1$ //$NON-NLS-2$

	}

	private void createLanguagesList2() {

		langList.add("N'Ko"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("N/u"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Na"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Naaba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Naasioi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Naba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nabak"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nabi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nachering"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nadruvian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nadëb"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nafaanra"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nafi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nafri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nafusi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Naga Pidgin"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nagarchal"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nage"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nagu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nagumi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Naha'ai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nahali"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nahari"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nahavaq"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Najdi Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Naka'ela"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nakai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nakama"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nakanai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nakara"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nake"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Naki"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nakwi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nalca"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nali"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nalik"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nalu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Naluo Yi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nalögo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nama (Namibia)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nama (Papua New Guinea)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Namakura"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Namat"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nambo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nambya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Namia"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Namiae"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Namibian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Namla"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Namo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Namonuito"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Namuyi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nanai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nancere"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nande"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nandi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nanerigé Sénoufo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nanga Dama Dogon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nanggu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nankina"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nanti"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nanticoke"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nanubae"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Napo Lowland Quechua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Napu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nar Phu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nara"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Narak"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Narango"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Narau"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Narim"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Naro"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Narom"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Narragansett"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Narrinyeri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Narungga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nasal"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nasarian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Naskapi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Natagaimas"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Natanzi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nataoran Amis"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Natchez"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nateni"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nathembo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Natioro"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Natügu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nauete"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Naukan Yupik"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nauna"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nauru"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Navaho"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Navajo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Navut"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nawaru"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nawathinehena"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nawdm"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nawuri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Naxi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nayi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nayini"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ncane"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nchumbulu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nda'nda'"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ndai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ndaka"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ndaktup"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ndali"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ndam"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ndamba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ndasa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ndau"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nde-Gbite"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nde-Nsele-Nta"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ndemli"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ndendeule"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ndengereko"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nding"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ndo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ndobo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ndoe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ndogo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ndolo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ndom"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ndombe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ndonde Hamba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ndonga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ndoola"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nduga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ndumu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ndunda"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ndunga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ndut"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ndyuka-Trio Pidgin"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ndzwani Comorian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Neapolitan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nedebang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nefamese"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Negerhollands"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Negeri Sembilan Malay"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Negidal"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nehan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nek"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nekgini"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Neko"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Neku"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Neme"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nemi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nen"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nend"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nenets"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nenge"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nengone"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Neo-Hittite"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nepal Bhasa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nepalese Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nepali"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nepali Kurux"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nete"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Neve'ei"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Neverver"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("New Caledonian Javanese"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("New Zealand Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Newari"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Neyo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nez Perce"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nga La"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ngaanyatjarra"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ngad'a"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ngadjunmaya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ngaing"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ngaju"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ngala"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ngalakan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ngalkbun"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ngalum"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ngam"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ngamambo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ngambay"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ngamini"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ngamo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nganasan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ngandi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ngando (Central African Republic)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ngando (Democratic Republic of Congo)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ngandyera"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ngangam"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nganyaywana"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ngarinman"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ngarinyin"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ngarla"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ngarluma"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ngas"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ngasa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ngatik Men's Creole"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ngawn Chin"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ngawun"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ngazidja Comorian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ngbaka"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ngbaka Ma'bo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ngbaka Manza"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ngbee"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ngbinda"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ngbundu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ngelima"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ngemba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ngeq"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ngete"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nggem"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nggwahyi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ngie"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ngiemboon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ngile"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ngindo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ngiti"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ngizim"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ngkâlmpw Kanum"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ngom"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ngomba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ngombale"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ngombe (Central African Republic)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ngombe (Democratic Republic of Congo)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ngong"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ngongo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ngoni"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ngoreme"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ngoshie"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ngul"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ngulu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nguluwan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ngumbi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ngundi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ngundu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ngungwel"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ngura"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ngurimi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ngurmbur"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nguôn"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ngwaba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ngwe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ngwo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ngäbere"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nhanda"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nhengatu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nhuwala"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nias"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nicaragua Creole English"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nicaraguan Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nicarao"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Niellim"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nigeria Mambila"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nigerian Fulfulde"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nigerian Pidgin"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nigerian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nihali"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nii"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nijadali"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Niksek"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nila"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nilamba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nimadi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nimanbur"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nimbari"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nimboran"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nimi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nimo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nimoa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ninam"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ninde"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nindi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ningera"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ninggerum"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ningil"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ningye"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ninia Yali"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ninzo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nipsan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nisa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nisenan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nisga'a"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nisi (China)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nisi (India)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Niuafo'ou"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Niuatoputapu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Niuean"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nivaclé"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Njalgulgule"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Njebi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Njen"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Njerep"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Njyem"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nkami"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nkangala"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nkari"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nkem-Nkum"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nkhumbi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nkongho"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nkonya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nkoroo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nkoya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nkukoli"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nkutu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nnam"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("No linguistic content"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nobiin"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nobonob"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nocamán"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nocte Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nogai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Noiri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nokuku"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nomaande"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nomane"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nomatsiguenga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nomu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nong Zhuang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nonuya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nooksack"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Noon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Noone"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nootka"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nopala Chatino"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Noric"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Norn"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Norra"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("North Alaskan Inupiatun"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("North Ambrym"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("North Asmat"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("North Awyu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("North Azerbaijani"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("North Babar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("North Bolivian Quechua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("North Central Mixe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("North Efate"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("North Fali"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("North Giziga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("North Junín Quechua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("North Levantine Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("North Marquesan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("North Mesopotamian Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("North Mofu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("North Moluccan Malay"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("North Muyu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("North Ndebele"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("North Nuaulu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("North Picene"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("North Slavey"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("North Tairora"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("North Tanna"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("North Wahgi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("North Watut"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("North Wemale"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northeast Kiwai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northeast Maidu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northeast Pashayi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northeastern Dinka"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northeastern Pomo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northeastern Thai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northern Alta"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northern Altai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northern Amami-Oshima"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northern Bai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northern Betsimisaraka Malagasy"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northern Binukidnon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northern Bobo Madaré"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northern Bontok"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northern Catanduanes Bicolano"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northern Conchucos Ancash Quechua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northern Dagara"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northern Dong"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northern East Cree"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northern Emberá"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northern Frisian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northern Ghale"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northern Gondi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northern Grebo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northern Guiyang Hmong"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northern Guiyang Miao"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northern Haida"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northern Hindko"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northern Huishui Hmong"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northern Huishui Miao"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northern Kalapuya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northern Kankanay"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northern Khmer"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northern Kissi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northern Kurdish"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northern Lorung"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northern Luri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northern Mashan Hmong"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northern Mashan Miao"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northern Muji"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northern Nago"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northern Ngbandi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northern Nisu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northern Nuni"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northern Oaxaca Nahuatl"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northern Ohlone"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northern One"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northern Paiute"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northern Pame"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northern Pashto"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northern Pastaza Quichua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northern Pomo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northern Puebla Nahuatl"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northern Pumi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northern Qiandong Miao"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northern Qiang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northern Rengma Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northern Roglai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northern Sami"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northern Sierra Miwok"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northern Sorsoganon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northern Sotho"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northern Subanen"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northern Tarahumara"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northern Tepehuan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northern Thai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northern Tiwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northern Tlaxiaco Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northern Toussian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northern Tujia"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northern Tutchone"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northern Uzbek"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northern Yukaghir"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northwest Alaska Inupiatun"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northwest Gbaya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northwest Maidu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northwest Oaxaca Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northwest Pashayi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northwestern Dinka"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northwestern Fars"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northwestern Kolami"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northwestern Ojibwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Northwestern Tamang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Norwegian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Norwegian Bokmål"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Norwegian Nynorsk"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Norwegian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Not applicable"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Notre"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Notsi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nottoway"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Novial"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Noy"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nsari"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nsenga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nshi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nsongo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ntcham"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ntomba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nubaca"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nubi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nubri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nuer"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nugunu (Australia)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nugunu (Cameroon)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nuk"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nukak Makú"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nukna"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nukuini"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nukumanu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nukuoro"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nukuria"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Numanggang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Numbami"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nume"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Numee"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Numidian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nung (Myanmar)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nung (Viet Nam)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nungali"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nunggubuyu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nungu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nuosu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nupbikha"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nupe-Nupe-Tako"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nusa Laut"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nusu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nyabwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nyaheun"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nyahkur"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nyakyusa-Ngonde"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nyali"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nyam"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nyamal"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nyambo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nyamusa-Molo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nyamwanga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nyamwezi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nyaneka"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nyang'i"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nyanga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nyanga-li"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nyangatom"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nyangbo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nyangga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nyangumarta"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nyanja"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nyankole"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nyarafolo Senoufo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nyaturu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nyaw"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nyawaygi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nyemba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nyengo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nyenkha"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nyeu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nyigina"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nyiha (Malawi)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nyiha (Tanzania)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nyika (Malawi and Zambia)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nyika (Tanzania)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nyindrou"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nyindu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nyole"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nyong"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nyore"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nyoro"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nyulnyul"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nyunga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nyungwe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nyâlayu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nzakambay"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nzakara"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nzanyi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nzima"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ná-Meo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Nüpode Huitoto"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("O'chi'chi'"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("O'du"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Obanliku"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Obispeño"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Oblo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Obo Manobo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Obokuitai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Obolo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Obulom"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ocaina"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Occidental"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Occitan (post 1500)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ocotepec Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ocotlán Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Od"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Odiai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Odoodee"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Odual"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Odut"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ofayé"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Official Aramaic (700-300 BCE)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ofo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ogbah"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ogbia"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ogbogolo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ogbronuagum"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ogea"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Oirat"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Oirata"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ojibwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ojitlán Chinantec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Okanagan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Oki-No-Erabu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Okiek"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Oko-Eni-Osayen"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Oko-Juwoi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Okobo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Okodia"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Okolod"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Okpamheri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Okpe (Northwestern Edo)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Okpe (Southwestern Edo)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Oksapmin"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Oku"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Old Aramaic (up to 700 BCE)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Old Avar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Old Breton"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Old Bulgarian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Old Burmese"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Old Chinese"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Old Church Slavonic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Old Cornish"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Old Dutch"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Old English (ca. 450-1100)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Old French (842-ca. 1400)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Old Frisian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Old Georgian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Old High German (ca. 750-1050)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Old Hittite"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Old Hungarian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Old Irish (to 900)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Old Japanese"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Old Kentish Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Old Korean (3rd-9th cent.)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Old Manipuri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Old Marathi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Old Mon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Old Newari"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Old Norse"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Old Nubian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Old Occitan (to 1500)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Old Ossetic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Old Persian (ca. 600-400 B.C.)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Old Provençal (to 1500)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Old Russian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Old Saxon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Old Slavonic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Old Spanish"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Old Tamil"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Old Tibetan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Old Turkish"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Old Uighur"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Old Welsh"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Olekha"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Olo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Oloma"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Olrat"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Olu'bo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Olukhayo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Olulumo-Ikom"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Oluluyia"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Olumarachi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Olumarama"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Olunyole"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Olusamia"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Olushisa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Oluta Popoluca"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Olutsotso"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Oluwanga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Omagua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Omaha-Ponca"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Omani Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Omati"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ombamba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ombo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Omejes"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ometepec Nahuatl"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Omi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Omok"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Omotik"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Omurano"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ona"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Oneida"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ong"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Onin"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Onin Based Pidgin"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Oniyan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Onjob"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ono"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Onobasulu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Onondaga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ontenu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ontong Java"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Oorlams"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Opao"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Opata"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Opuuo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Orang Kanaq"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Orang Seletar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Oraon Sadri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Orejón"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Oring"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Oriya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Orizaba Nahuatl"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Orma"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ormu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ormuri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Oro"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Oro Win"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Oroch"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Oroha"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Orok"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Orokaiva"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Oroko"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Orokolo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Oromo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Oroqen"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Orowe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Oruma"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Orya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Osage"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Osatu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Oscan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Osing"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ososo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ossetian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ossetic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ot Danum"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Otank"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Oti"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Otoro"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ottawa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ottoman Turkish (1500-1928)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Otuho"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Otuke"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ouma"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ounge"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Owa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Owenia"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Owiniga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Oy"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Oya'oya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Oyda"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ozolotepec Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ozumacín Chinantec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pa Di"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pa'a"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pa'o Karen"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pa-Hng"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Paama"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Paasaal"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pacahuara"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pacaraos Quechua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pacific Gulf Yupik"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pacoh"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Padoe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Paekche"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Paelignian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pagi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pagibete"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pagu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pahanan Agta"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pahari-Potwari"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pahi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pahlavani"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pahlavi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pai Tavytera"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Paicî"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Paipai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Paite Chin"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Paiwan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pak-Tong"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pakanha"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pakaásnovos"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pakistan Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Paku"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Paku Karen"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pal"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Palaic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Palaka Senoufo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Palantla Chinantec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Palauan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Palenquero"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pali"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Palikúr"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Paliyan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Palor"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Palpa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Palu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Palu'e"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Paluan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Palula"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Palumata"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Palya Bareli"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pam"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pambia"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pamlico"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pamona"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pamosu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pampanga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pamplona Atta"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pana (Burkina Faso)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pana (Central African Republic)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Panamanian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Panamint"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Panang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Panao Huánuco Quechua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Panará"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Panasuan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Panawa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pancana"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Panchpargania"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pande"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pangasinan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pangseng"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pangutaran Sama"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pangwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pangwali"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Panim"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Paniya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Panjabi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pankararé"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pankararú"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pankhu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pannei"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Panoan Katukína"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Panobo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Panytyima"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pao"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Papantla Totonac"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Papapana"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Papar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Papasena"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Papel"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Papi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Papiamento"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Papitalai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Papora"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Papuan Malay"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Papuma"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Para Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Parachi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Paraguayan Guaraní"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Paraguayan Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Parakanã"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Paranan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Paranawát"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Paraujano"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Parauk"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Parawen"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pardhan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pardhi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pare"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Parecís"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Parenga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Parkari Koli"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Parkwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Parsi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Parsi-Dari"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Parthian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Parya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pará Arára"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pará Gavião"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pashto"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pasi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pass Valley Yali"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Patamona"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Patani"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pataxó Hã-Ha-Hãe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Patep"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pathiya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Patpatar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pattani"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pattani Malay"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Paulohi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Paumarí"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pauri Bareli"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pauserna"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pawaia"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pawnee"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Paynamar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pear"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pech"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pecheneg"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pedi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Peere"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pei"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pekal"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pela"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pele-Ata"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pelende"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pemon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Penang Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Penchal"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pendau"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pengo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pennsylvania German"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Penrhyn"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pentlatch"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Perai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Peranakan Indonesian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Peripheral Mongolian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pero"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Persian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Persian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Peruvian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Petapa Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Petats"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Petjo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Peñoles Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pfaelzisch"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Phai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Phake"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Phala"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Phalura"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Phana'"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Phangduwali"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Phende"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Philippine Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Phimbi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Phoenician"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Phola"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pholo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Phom Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Phong-Kniang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Phrae Pwo Karen"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Phrygian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Phu Thai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Phuan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Phudagi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Phuie"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Phukha"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Phuma"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Phunoi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Phuong"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Phupa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Phupha"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Phuza"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Piamatsina"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Piame"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Piapoco"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Piaroa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Picard"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pichis Ashéninka"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pictish"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pidgin Delaware"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Piemontese"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pijao"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pije"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pijin"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pilagá"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pileni"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pilipino"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pima Bajo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pimbwe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pinai-Hagahai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pingelapese"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pini"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pinigura"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pinji"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pinotepa Nacional Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pintiini"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pintupi-Luritja"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pinyin"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pipil"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pirahã"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Piratapuyo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pirlatapa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Piro"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Piru"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pisabo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pisaflores Tepehua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Piscataway"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pisidian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pite Sami"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Piti"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pitjantjatjara"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pitta Pitta"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Piu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Piya-Kwonci"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Plains Cree"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Plains Indian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Plains Miwok"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Plapo Krumen"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Plateau Malagasy"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Plautdietsch"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Playero"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pnar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pochuri Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pochutec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Podena"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pogolo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pohnpeian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pokangá"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Poke"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pokomo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Polabian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Polari"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Polci"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Polish"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Polish Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Polonombauk"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pom"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pomo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ponam"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ponares"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pongu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pongyong"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ponosakan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pontic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Popti'"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Poqomam"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Poqomchi'"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Porohanon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Port Sandwich"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Port Vato"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Portuguese"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Portuguese Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Potawatomi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Potiguára"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pottangi Ollar Gadaba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Poule"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Poumei Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pouye"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Powari"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Powhatan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Poyanáwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Prasuni"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pray 3"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Principense"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Providencia Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Prussian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Psikye"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pu Ko"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pu-Xian Chinese"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Puare"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pudtol Atta"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Puelche"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Puerto Rican Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Puimei Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Puinave"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pukapuka"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pulaar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pulabu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pular"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Puluwatese"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Puma"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pumpokol"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pumé"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Punan Aput"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Punan Bah-Biau"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Punan Batu 1"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Punan Merah"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Punan Merap"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Punan Tubu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Punic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Punjabi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Puno Quechua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Punu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Puoc"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Puquina"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Puragi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Purari"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Purepecha"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Puri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Purik"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Purisimeño"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Puruborá"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Purum"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Purum Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pushto"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Putai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Putoh"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Putukwam"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Puyo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Puyo-Paekche"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Puyuma"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pwaamei"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pwapwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pwo Eastern Karen"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pwo Northern Karen"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pwo Western Karen"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pyapun"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pye Krumen"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pyen"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pyu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pyu (Myanmar)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Páez"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pááfang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Päri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pémono"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pévé"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Pökoot"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Q'anjob'al"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Qabiao"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Qaqet"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Qashqa'i"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Qatabanian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Qawasqar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Qila Muji"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Qimant"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Qiubei Zhuang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Quapaw"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Quebec Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Quechan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Quechua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Quenya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Querétaro Otomi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Quetzaltepec Mixe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Queyu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Quiavicuzas Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Quiché"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Quileute"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Quinault"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Quinqui"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Quioquitani-Quierí Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Quiotepec Chinantec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Quiripi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Rabha"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Rade"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Raetic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Rahambuu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Rajah Kabunsuwan Manobo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Rajasthani"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Rajbanshi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Raji"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Rajong"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Rajput Garasia"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Rakhine"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ralte"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Rama"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ramoaaina"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ramopa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Rampi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Rana Tharu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Rang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Rangkas"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ranglong"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Rangpuri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Rao"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Rapa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Rapanui"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Rapoisi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Rapting"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Rara Bakati'"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Rarotongan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Rasawa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ratagnon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ratahan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Rathawi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Rathwi Bareli"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Raute"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ravula"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Rawa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Rawang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Rawat"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Rawo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Rayón Zoque"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Razajerdi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Red Gelao"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Reel"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Rejang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Rejang Kayan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Reli"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Rema"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Rembarunga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Rembong"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Remo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Remontado Dumagat"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Rempi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Remun"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Rendille"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Rengao"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Rennell-Bellona"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Rennellese Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Repanbitip"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Rer Bare"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Rerau"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Rerep"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Reshe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Resígaro"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Retta"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Reyesano"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Riang (India)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Riang (Myanmar)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Riantana"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ribun"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Rien"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Rikbaktsa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Rinconada Bikol"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Rincón Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ringgou"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ririo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ritarungo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Riung"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Riverain Sango"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Rogo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Rohingya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Roma"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Romagnol"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Romam"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Romanian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Romanian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Romano-Greek"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Romano-Serbian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Romanova"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Romansh"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Romany"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Romblomanon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Rombo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Romkun"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ron"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ronga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Rongga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Rongmei Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Rongpo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ronji"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Roon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Roria"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Rotokas"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Rotuman"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Roviana"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ruching Palaung"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Rudbari"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Rufiji"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ruga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Rukai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ruma"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Rumai Palaung"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Rumu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Runa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Rundi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Runga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Rungus"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Rungwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Russia Buriat"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Russian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Russian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Rusyn"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Rutul"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ruuli"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ruund"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Rwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Réunion Creole French"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("S'gaw Karen"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sa'a"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sa'ban"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sa'och"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Saafi-Saafi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Saam"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Saamia"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Saaroa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Saba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sabaean"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sabah Bisaya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sabah Malay"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sabanê"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sabaot"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sabine"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sabu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sabüm"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sacapulteco"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sadri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Saek"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Saep"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Safaliba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Safeyoka"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Safwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sagala"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sagalla"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Saho"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sahu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Saidi Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Saint Lucian Creole French"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Saisiyat"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sajalong"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sajau Basap"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sakachep"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sakalava Malagasy"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sakam"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sakan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sakao"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sakata"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sake"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sakirabiá"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sala"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Salako"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Salampasu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Salar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Salas"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Salasaca Highland Quichua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Salchuq"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Saleman"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Saliba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Salinan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sallands"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Salt-Yui"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Saluan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Salumá"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Salvadoran Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sam"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sama"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Samaritan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Samaritan Aramaic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Samarokena"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Samatao"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Samba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Samba Daka"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Samba Leko"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sambe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Samberigi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Samburu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Samei"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Samo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Samoan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Samogitian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Samosa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sampang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Samre"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Samtao"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Samvedi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("San Agustín Mixtepec Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("San Baltazar Loxicha Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("San Blas Kuna"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("San Dionisio Del Mar Huave"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("San Felipe Otlaltepec Popoloca"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("San Francisco Del Mar Huave"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("San Francisco Matlatzinca"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("San Jerónimo Tecóatl Mazatec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("San Juan Atzingo Popoloca"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("San Juan Colorado Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("San Juan Guelavía Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("San Juan Teita Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("San Luís Temalacayuca Popoloca"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("San Marcos Tlalcoyalco Popoloca"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("San Martín Itunyoso Triqui"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("San Martín Quechua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("San Mateo Del Mar Huave"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("San Miguel Creole French"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("San Miguel El Grande Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("San Miguel Piedras Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("San Pedro Amuzgos Amuzgo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("San Pedro Quiatoni Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("San Salvador Kongo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("San Vicente Coatlán Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sanaani Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sanapaná"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sandawe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sanga (Democratic Republic of Congo)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sanga (Nigeria)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sanggau"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sangil"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sangir"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sangisari"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sangkong"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sanglechi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sango"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sangtam Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sangu (Gabon)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sangu (Tanzania)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sani"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sanie"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Saniyo-Hiyewe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sankaran Maninka"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sansi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sanskrit"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sansu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Santa Ana de Tusi Pasco Quechua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Santa Catarina Albarradas Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Santa Inés Ahuatempan Popoloca"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Santa Inés Yatzechi Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Santa Lucía Monteverde Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Santa María Del Mar Huave"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Santa María La Alta Nahuatl"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Santa María Quiegolani Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Santa María Zacatepec Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Santa Teresa Cora"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Santali"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Santiago Xanica Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Santiago del Estero Quichua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Santo Domingo Albarradas Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sanumá"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Saparua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sapo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Saponi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Saposa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sapuan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sapé"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sara"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sara Dunjo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sara Kaba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sara Kaba Deme"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sara Kaba Náà"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Saramaccan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sarangani Blaan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sarangani Manobo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sarasira"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Saraveca"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sardinian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sarikoli"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sarli"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sarsi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sartang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sarua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sarudu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Saruga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sasak"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sasaru"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sassarese Sardinian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Satawalese"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Saterfriesisch"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sateré-Mawé"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Saudi Arabian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sauk"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sauraseni Prākrit"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Saurashtra"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sauri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sauria Paharia"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sause"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sausi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Savara"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Savi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Savosavo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sawai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Saweru"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sawi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sawila"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sawknah"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Saxwe Gbe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Saya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sayula Popoluca"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Scots"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Scottish Gaelic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Scythian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sea Island Creole English"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Seba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sebat Bet Gurage"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Seberuang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sebop"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sebuyau"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sechelt"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Secoya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sedang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sedoa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Seeku"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Segai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Segeju"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Seget"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sehwi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Seimat"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Seit-Kaitetu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sekani"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sekapan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sekar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Seke (Nepal)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Seke (Vanuatu)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Seki"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Seko Padang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Seko Tengah"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sekpele"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Selangor Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Selaru"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Selayar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Selee"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Selepet"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Selian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Selkup"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Selungai Murut"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Seluwasan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Semai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Semandang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Semaq Beri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sembakung Murut"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Semelai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Semimi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Semnam"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Semnani"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sempan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sena"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Senara Sénoufo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Senaya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sene"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Seneca"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sened"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sengele"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Senggi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sengo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sengseng"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Senhaja De Srair"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sensi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sentani"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Senthang Chin"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sentinel"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sepa (Indonesia)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sepa (Papua New Guinea)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sepedi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sepen"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sepik Iwam"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sera"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Seraiki"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Serbian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Serbo-Croatian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sere"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Serer"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Seri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Serili"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Seroa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Serrano"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Seru"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Serua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Serudung Murut"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Serui-Laut"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Seselwa Creole French"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Seta"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Setaman"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Seti"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Settla"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Severn Ojibwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sewa Bay"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Seze"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sha"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Shabak"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Shabo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Shahmirzadi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Shahrudi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Shall-Zwall"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Shama-Sambuga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Shamang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Shambala"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Shan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Shanenawa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Shanga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Shangzhai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sharanahua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Shark Bay"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sharwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Shasta"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Shatt"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Shau"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Shawnee"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("She"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Shehri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Shekhawati"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Shekkacho"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sheko"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Shelta"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Shempire Senoufo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Shendu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sheni"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sherbro"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sherdukpen"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sherpa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sheshi Kham"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Shi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Shihhi Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Shiki"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Shilluk"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Shina"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Shinabo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Shipibo-Conibo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Shixing"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sholaga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Shom Peng"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Shona"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Shoo-Minda-Nye"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Shor"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Shoshoni"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Shua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Shuadit"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Shuar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Shubi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Shughni"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Shumashti"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Shumcho"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Shuswap"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Shuwa-Zamani"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Shwai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Shwe Palaung"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sialum"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Siamou"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Siane"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Siang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Siar-Lak"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Siawi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sibe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sibu Melanau"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sicanian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sicel"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sichuan Yi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sicilian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Siculo Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sidamo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sidetic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sie"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sierra Leone Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sierra Negra Nahuatl"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sierra de Juárez Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sighu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sihan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sihuas Ancash Quechua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sika"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sikaiana"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sikaritai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sikiana"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sikkimese"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Siksika"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sikule"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sila"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Silacayoapan Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sileibi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Silesian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Silimo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Siliput"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Silopi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Silt'e"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Simaa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Simba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Simbali"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Simbari"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Simbo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Simeku"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Simeulue"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Simte"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sinagen"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sinasina"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sinaugoro"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sindarin"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sindhi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sindhi Bhil"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sindihui Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Singa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Singapore Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Singpho"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sinhala"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sinhalese"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sinicahua Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sinicized Miao"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sininkere"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sinsauru"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sinte Romani"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sinyar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sio"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Siona"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sipacapense"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sira"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Siraya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sirenik Yupik"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Siri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Siriano"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sirionó"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sirmauri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Siroi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sissala"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sissano"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Siuslaw"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sivandi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Siwai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Siwi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Siwu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Siyin Chin"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Skagit"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Skalvian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Skepi Creole Dutch"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Skolt Sami"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Skou"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Slave (Athapascan)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Slovak"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Slovakian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Slovenian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Small Flowery Miao"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Smärky Kanum"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Snohomish"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("So (Cameroon)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("So (Democratic Republic of Congo)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("So'a"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sobei"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sochiapam Chinantec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sochiapan Chinantec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Soga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sogdian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Soi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sok"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sokoro"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Solano"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Soli"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Solong"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Solos"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Som"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Somali"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Somrai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Somray"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Somyev"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sonde"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Songa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Songe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Songo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Songomeno"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Songoora"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sonha"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sonia"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Soninke"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sonsorol"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Soo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sop"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Soqotri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sora"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sori-Harengan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sorkhei"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sorothaptic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sorsogon Ayta"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sos Kundi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sota Kanum"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sou"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("South African Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("South Awyu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("South Azerbaijani"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("South Bolivian Quechua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("South Central Banda"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("South Central Dinka"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("South Efate"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("South Fali"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("South Giziga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("South Lembata"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("South Levantine Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("South Marquesan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("South Muyu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("South Ndebele"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("South Nuaulu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("South Picene"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("South Slavey"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("South Tairora"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("South Ucayali Ashéninka"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("South Watut"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("South Wemale"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("South West Bay"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southeast Ambrym"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southeast Babar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southeast Ijo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southeast Pashayi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southeastern Dinka"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southeastern Ixtlán Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southeastern Kolami"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southeastern Nochixtlán Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southeastern Pomo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southeastern Puebla Nahuatl"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southeastern Tarahumara"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southeastern Tepehuan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southern Alta"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southern Altai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southern Amami-Oshima"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southern Aymara"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southern Bai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southern Balochi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southern Betsimisaraka Malagasy"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southern Binukidnon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southern Birifor"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southern Bobo Madaré"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southern Bontok"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southern Carrier"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southern Catanduanes Bicolano"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southern Conchucos Ancash Quechua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southern Dagaare"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southern Dong"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southern East Cree"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southern Ghale"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southern Gondi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southern Grebo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southern Guiyang Hmong"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southern Guiyang Miao"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southern Haida"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southern Hindko"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southern Kalapuya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southern Kalinga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southern Kisi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southern Kiwai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southern Kurdish"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southern Lolopo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southern Lorung"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southern Luri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southern Ma'di"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southern Mashan Hmong"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southern Mashan Miao"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southern Mnong"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southern Muji"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southern Nago"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southern Nambikuára"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southern Ngbandi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southern Nicobarese"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southern Nisu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southern Nuni"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southern Ohlone"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southern One"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southern Pame"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southern Pashto"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southern Pastaza Quechua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southern Pomo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southern Puebla Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southern Puget Sound Salish"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southern Pumi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southern Qiandong Miao"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southern Qiang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southern Rengma Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southern Rincon Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southern Roglai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southern Sama"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southern Sami"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southern Samo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southern Sierra Miwok"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southern Sorsoganon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southern Sotho"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southern Subanen"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southern Thai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southern Tiwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southern Toussian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southern Tujia"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southern Tutchone"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southern Uzbek"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southern Yukaghir"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southwest Gbaya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southwest Palawano"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southwest Pashayi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southwest Tanna"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southwestern Bontok"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southwestern Dinka"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southwestern Fars"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southwestern Guiyang Hmong"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southwestern Huishui Hmong"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southwestern Huishui Miao"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southwestern Nisu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southwestern Tamang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southwestern Tarahumara"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southwestern Tepehuan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Southwestern Tlaxiaco Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sowa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sowanda"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sowari"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Soyaltepec Mazatec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Soyaltepec Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Spanish"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Spanish Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Spiti Bhoti"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Spokane"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Squamish"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sranan Tongo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sri Lankan Creole Malay"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sri Lankan Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Standard Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Standard Estonian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Standard Latvian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Standard Malay"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Stellingwerfs"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Stod Bhoti"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Stoney"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Straits Salish"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Suabo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Suarmin"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Suau"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Suba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Suba-Simbiti"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Subi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Subiya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Subtiaba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sudanese Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sudanese Creole Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sudest"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sudovian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Suena"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Suga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Suganga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sugut Dusun"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sui"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Suki"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Suku"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sukuma"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sukur"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sukurum"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sula"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sulka"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sulod"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sulung"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Suma"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sumariup"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sumau"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sumbawa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sumbwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sumerian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sumi Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sunam"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sundanese"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sunum"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sunwar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Suoy"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Supyire Senoufo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sur"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Surbakhal"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Suri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Surigaonon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Surjapuri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sursurunga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Suruahá"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Surubu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Suruí"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Suruí Do Pará"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Susquehannock"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Susu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Susuami"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Suundi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Suwawa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Suyá"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Svan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Swabian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Swahili (individual language)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Swahili (macrolanguage)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Swampy Cree"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Swati"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Swedish"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Swedish Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Swiss German"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Swiss-French Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Swiss-German Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Swiss-Italian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Syenara Senoufo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sylheti"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Syriac"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sáliba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("São Paulo Kaingáng"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sãotomense"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sìcìté Sénoufo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Sô"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("T'en"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ta'izzi-Adeni Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Taabwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tabaa Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tabaru"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tabasco Chontal"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tabasco Nahuatl"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tabasco Zoque"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tabassaran"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tabla"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tabo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tabriak"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tacahua Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tacana"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tachawit"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tachelhit"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tachoni"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tadaksahak"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tadyawan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tae'"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tafi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tagabawa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tagakaulo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tagal Murut"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tagalog"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tagargrent"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tagbanwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tagbu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tagdal"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tagish"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tagoi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tagwana Senoufo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tahaggart Tamahaq"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tahitian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tahltan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tai Daeng"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tai Dam"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tai Do"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tai Dón"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tai Hang Tong"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tai Hongjin"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tai Loi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tai Long"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tai Mène"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tai Nüa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tai Pao"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tai Thanh"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tai Ya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Taiap"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Taikat"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tainae"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Taino"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tairuma"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Taita"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Taiwan Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Taje"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tajik"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tajiki Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tajio"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tajuasohn"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Takelma"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Takestani"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Takia"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Takpa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Takua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Takuu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Takwane"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tal"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tala"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Talaandig"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Talaud"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Taliabu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Talieng"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Talinga-Bwisi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Talise"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Talodi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Taloki"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Talondo'"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Talu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Talur"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Talysh"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tama (Chad)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tama (Colombia)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tamagario"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Taman (Indonesia)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Taman (Myanmar)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tamanaku"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tamashek"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tamasheq"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tamazola Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tambas"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tambora"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tambotalo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tambunan Dusun"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tami"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tamil"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tamki"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tamnim Citak"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tampias Lobu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tampuan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tampulma"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tanacross"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tanahmerah"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tanaina"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tanapag"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tandaganon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tandia"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tandroy-Mahafaly Malagasy"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tanema"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tangale"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tangchangya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tangga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tanggu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tangkhul Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tangko"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tanglang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tangoa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tanguat"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tangut"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tanimbili"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tanjijili"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tanosy Malagasy"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tanzanian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tapeba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tapei"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tapieté"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tapirapé"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tarao Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tareng"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tariana"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tarifit"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tarok"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Taroko"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tarpia"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tartessian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tasawaq"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tase Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tasmanian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tasmate"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tataltepec Chatino"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tatana"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tatar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tatuyo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tauade"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Taulil"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Taungyo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Taupota"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tause"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Taushiro"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tausug"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tauya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Taveta"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tavoyan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tavringer Romani"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tawala"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tawallammat Tamajaq"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tawandê"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tawang Monpa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tawara"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Taworta"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tawoyan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tawr Chin"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tay Boi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tay Khang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tayabas Ayta"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tayart Tamajeq"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tayo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Taznatit"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tboli"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tchitchege"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tchumbuli"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Te'un"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Teanu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tebi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tebul Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tebul Ure Dogon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tecpatlán Totonac"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tedaga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tedim Chin"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tee"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tefaro"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tegali"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tehit"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tehuelche"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tejalapan Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Teke-Ebo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Teke-Fuumu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Teke-Kukuya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Teke-Laali"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Teke-Nzikou"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Teke-Tege"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Teke-Tsaayi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Teke-Tyee"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tektiteko"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tela-Masbuar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Telefol"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Telugu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Teluti"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tem"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Temacine Tamazight"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Temascaltepec Nahuatl"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tembo (Kitembo)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tembo (Motembo)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tembé"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Teme"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Temein"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Temi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Temiar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Temoaya Otomi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Temoq"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tempasuk Dusun"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Temuan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tena"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tena Lowland Quichua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tenango Otomi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tene Kan Dogon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tenggarong Kutai Malay"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tengger"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tenharim"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tenino"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tenis"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tennet"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Teop"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Teor"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tepecano"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tepetotutla Chinantec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tepeuxila Cuicatec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tepinapa Chinantec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tepo Krumen"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ter Sami"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tera"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Terebu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Terei"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tereno"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Teressa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tereweng"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Teribe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Terik"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Termanu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ternate"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ternateño"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tese"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Teshenawa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Teso"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tetela"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tetelcingo Nahuatl"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tetete"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tetum"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tetun Dili"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Teutila Cuicatec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tewa (Indonesia)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tewa (USA)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tewe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Texcatepec Otomi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Texistepec Popoluca"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Texmelucan Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tezoatlán Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tha"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Thachanadan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Thado Chin"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Thai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Thai Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Thai Song"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Thakali"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Thangal Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Thangmi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Thao"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tharaka"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Thayore"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Thaypan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("The"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tho"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Thompson"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Thopho"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Thracian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Thu Lao"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Thudam"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Thulung"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Thurawal"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Thuri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tiagbamrin Aizi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tiale"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tiang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tibea"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tibetan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tichurong"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ticuna"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tidaá Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tidikelt Tamazight"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tidong"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tidore"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tiemacèwè Bozo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tiene"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tifal"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tigak"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tigon Mbembe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tigre"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tigrinya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tii"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tijaltepec Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tikar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tikopia"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tilapa Otomi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tillamook"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tilquiapan Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tilung"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tima"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Timbe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Timbisha"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Timne"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Timor Pidgin"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Timucua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Timugon Murut"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tinani"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tindi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tingal"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tingui-Boto"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tinigua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tinoc Kallahan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tinputz"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tinà Sambal"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tippera"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tira"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tirahi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tiranige Diga Dogon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tiri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tiruray"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tita"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Titan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tiv"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tiwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tiwi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tiéfo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tiéyaxo Bozo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tjurruru"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tlachichilco Tepehua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tlacoapa Me'phaa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tlacoapa Tlapanec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tlacoatzintepec Chinantec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tlacolulita Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tlahuitoltepec Mixe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tlamacazapa Nahuatl"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tlazoyaltepec Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tlingit"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("To"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("To'abaita"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Toaripi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Toba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Toba-Maskoy"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tobagonian Creole English"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tobanga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tobati"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tobelo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tobian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tobilung"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tobo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tocantins Asurini"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tocho"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Toda"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Todrah"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tofanma"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tofin Gbe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Togbo-Vara Banda"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Togoyo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tohono O'odham"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tojolabal"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tok Pisin"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tokano"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tokelau"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tokharian A"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tokharian B"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Toku-No-Shima"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tol"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tolaki"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tolomako"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tolowa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Toma"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tomadino"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tombelala"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tombonuo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tombulu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tomedes"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tomini"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tomo Kan Dogon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tomoip"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tondano"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tonga (Nyasa)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tonga (Thailand)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tonga (Tonga Islands)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tonga (Zambia)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tongwe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tonjon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tonkawa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tonsawang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tonsea"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tontemboan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tooro"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Topoiyo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Toposa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Toraja-Sa'dan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Toram"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Torau"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tornedalen Finnish"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Toro"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Toro So Dogon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Toro Tegu Dogon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Toromono"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Torona"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Torres Strait Creole"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Torricelli"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Torwali"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Torá"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tosk Albanian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Totela"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Toto"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Totoli"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Totomachapan Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Totontepec Mixe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Totoro"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Touo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Toura (Côte d'Ivoire)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Toura (Papua New Guinea)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Towei"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Transalpine Gaulish"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Traveller Danish"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Traveller Norwegian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Traveller Scottish"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tregami"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tremembé"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Trieng"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Trimuris"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tring"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tringgus-Sembaan Bidayuh"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Trinidad and Tobago Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Trinidadian Creole English"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Trinitario"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Trió"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Truká"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Trumai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ts'ün-Lao"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tsaangi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tsakhur"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tsakonian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tsakwambo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tsamai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tsat"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tsaukambo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tseku"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tsetsaut"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tshangla"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tsikimba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tsimané"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tsimihety Malagasy"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tsimshian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tsishingini"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tso"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tsoa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tsogo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tsonga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tsotsitaal"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tsotso"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tsou"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tsum"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tsuvadi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tsuvan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tswa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tswana"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tswapong"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tuamotuan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tubar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tucano"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tugen"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tugun"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tugutil"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tukang Besi North"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tukang Besi South"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tuki"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tukpa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tukudede"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tukumanféd"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tula"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tulehu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tulishi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tulu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tulu-Bohuai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tuma-Irumu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tumak"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tumari Kanuri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tumbuka"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tumi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tumleo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tumshuqese"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tumtum"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tumulung Sisaala"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tumzabt"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tundra Enets"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tunen"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tungag"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tunggare"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tunia"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tunica"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tunisian Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tunisian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tunjung"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tunni"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tunzu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tuotomb"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tuparí"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tupinambá"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tupinikin"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tupuri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tupí"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Turaka"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Turi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Turiwára"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Turka"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Turkana"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Turkic Khalaj"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Turkish"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Turkish Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Turkmen"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Turks And Caicos Creole English"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Turoyo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Turumsa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Turung"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tuscarora"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tutelo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tutong"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tutsa Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tutuba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tututepec Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tututni"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tuvalu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tuvinian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tuwali Ifugao"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tuwari"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tuwuli"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tuxináwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tuxá"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tuyuca"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Twana"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Twendi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Twents"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Twi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tyap"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tyaraity"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tz'utujil"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tzeltal"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tzotzil"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tày"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tày Sa Pa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tày Tac"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Téén"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Tübatulabal"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Türk İşaret Dili"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("U"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Uab Meto"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Uamué"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Uare"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ubaghara"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ubang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ubi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ubir"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ubykh"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ucayali-Yurúa Ashéninka"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Uda"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Udi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Udihe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Udmurt"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Uduk"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ufim"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ugandan Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ugaritic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ughele"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ugong"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Uhami"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Uighur"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Uisai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ujir"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ukaan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ukhwejo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ukit"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ukpe-Bayobiri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ukpet-Ehom"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ukrainian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ukrainian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ukue"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ukuriguma"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ukwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ulau-Suain"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ulch"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ulithian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ullatan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ulukwumi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ulumanda'"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ulwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Uma"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Uma' Lasan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Uma' Lung"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Umanakaina"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Umatilla"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Umbindhamu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Umbrian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Umbu-Ungu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Umbugarla"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Umbundu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Umbuygamu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ume Sami"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Umeda"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Umiray Dumaget Agta"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Umon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Umotína"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Umpila"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Una"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Unami"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Uncoded languages"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Unde Kaili"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Undetermined"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Uneapa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Uneme"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Unserdeutsch"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Unua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Uokha"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Upper Chehalis"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Upper Grand Valley Dani"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Upper Guinea Crioulo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Upper Kinabatangan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Upper Kuskokwim"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Upper Necaxa Totonac"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Upper Saxon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Upper Sorbian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Upper Ta'oih"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Upper Tanana"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Upper Tanudan Kalinga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Upper Taromi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Upper Umpqua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ura (Papua New Guinea)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ura (Vanuatu)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Uradhi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Urak Lawoi'"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Urali"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Urapmin"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Urarina"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Urartian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Urat"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Urdu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Urhobo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Uri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Urigina"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Urim"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Urimo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Urningangg"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Uru"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Uru-Eu-Wau-Wau"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Uru-Pa-In"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Uruangnirin"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Uruava"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Urubú-Kaapor"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Urubú-Kaapor Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Uruguayan Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Urum"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Urumi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Usaghade"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Usan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Usarufa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ushojo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Usila Chinantec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Usku"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Uspanteco"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Usui"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Utarmbung"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ute-Southern Paiute"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Utu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Uvbie"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Uya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Uyghur"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Uzbek"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Uzbeki Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Uzekwe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("V'ënen Taut"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Vaagri Booli"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Vaeakau-Taumako"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Vafsi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Vaghri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Vaghua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Vagla"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Vai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Vaiphei"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Vale"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Valencian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Valencian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Valle Nacional Chinantec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Valley Maidu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Valman"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Valpei"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Vamale"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Vame"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Vandalic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Vangunu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Vanimo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Vano"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Vanuma"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Vao"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Varhadi-Nagpuri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Varisi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Varli"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Vasavi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Vasekela Bushman"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Veddah"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Vehes"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Veluws"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Vemgo-Mabas"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Venda"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Venetian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Venetic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Venezuelan Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Vengo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ventureño"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Veps"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Vera'a"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Vestinian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Vidunda"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Viemo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Vietnamese"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Viid"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Vilela"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Vili"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Villa Viciosa Agta"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Vincentian Creole English"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Vinmavis"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Vinza"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Virgin Islands Creole English"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Vishavan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Viti"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Vitou"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Vlaams"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Vlaamse Gebarentaal"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Vlax Romani"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Volapük"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Volscian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Vono"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Voro"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Votic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Vumbu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Vunapu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Vunjo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Vure"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Vurës"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Vute"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Vwanji"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Võro"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wa'ema"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Waama"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Waamwang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Waata"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wab"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wabo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Waboda"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Waci Gbe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wadaginam"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Waddar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wadiyara Koli"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wadjiginy"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wadjigu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wae Rana"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Waffa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wagawaga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wagaya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wagdi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wageman"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wagi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wahau Kayan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wahau Kenyah"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wahgi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Waigali"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Waigeo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wailaki"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wailapa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Waima"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Waima'a"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Waimaha"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Waimiri-Atroari"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Waioli"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Waiwai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Waja"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wajarri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Waka"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wakawaka"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wakde"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wakhi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wakoná"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wala"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Walak"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wali (Ghana)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wali (Sudan)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Waling"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Walio"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Walla Walla"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wallisian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Walloon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Walmajarri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Walo Kumbe Dogon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Walser"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Walungge"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wamas"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wambaya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wambon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wambule"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wamey"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wamin"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wampanoag"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wampar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wampur"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wanambre"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wanap"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wanbasana"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wancho Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wanda"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wandala"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wandamen"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wandarang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wandji"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Waneci"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wanga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wanggamala"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wangganguru"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wanggom"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wanman"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wannu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wano"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wantoat"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wanukaka"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wané"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Waorani"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wapan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wapishana"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wappo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("War-Jaintia"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wara"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Warao"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Warapu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Waray (Australia)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Waray (Philippines)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wardaman"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Warduji"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Warembori"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wares"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Waris"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Waritai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wariyangga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Warji"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Warkay-Bipim"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Warlmanpa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Warlpiri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Warluwara"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Warnang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Waropen"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Warrgamay"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Warrwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Waru"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Warumungu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Waruna"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Warungu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wasa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wasco-Wishram"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wasembo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Washo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Waskia"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wasu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Watakataui"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Watam"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Watubela"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Waurá"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wauyai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wawa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wawonii"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Waxianghua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wayampi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wayana"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wayanad Chetti"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wayoró"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wayu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wayuu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wedau"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Weh"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wejewa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Welaung"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Weliki"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Welsh"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Welsh Romani"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Weme Gbe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Were"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Weri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wersing"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("West Albay Bikol"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("West Ambae"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("West Berawan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("West Central Banda"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("West Central Oromo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("West Coast Bajau"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("West Damar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("West Goodenough"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("West Kewa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("West Lembata"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("West Makian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("West Masela"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("West Tarangan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("West Uvean"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("West Yugur"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("West-Central Limba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Western Abnaki"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Western Apache"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Western Arrarnta"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Western Balochi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Western Bolivian Guaraní"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Western Bru"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Western Bukidnon Manobo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Western Canadian Inuktitut"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Western Cham"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Western Dani"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Western Fijian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Western Frisian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Western Gurung"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Western Highland Chatino"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Western Highland Purepecha"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Western Huasteca Nahuatl"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Western Juxtlahuaca Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Western Kanjobal"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Western Karaboro"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Western Katu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Western Kayah"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Western Keres"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Western Krahn"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Western Lalu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Western Lawa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Western Magar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Western Maninkakan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Western Mari"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Western Mashan Hmong"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Western Mashan Miao"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Western Meohang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Western Muria"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Western Neo-Aramaic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Western Niger Fulfulde"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Western Ojibwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Western Panjabi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Western Parbate Kham"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Western Penan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Western Sisaala"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Western Subanon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Western Tamang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Western Tawbuid"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Western Totonac"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Western Tunebo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Western Xiangxi Miao"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Western Xwla Gbe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Western Yiddish"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Westphalien"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wetamut"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wewaw"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Weyto"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("White Gelao"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("White Lachi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Whitesands"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wiarumus"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wichita"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wichí Lhamtés Güisnay"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wichí Lhamtés Nocten"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wichí Lhamtés Vejoz"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wik-Epa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wik-Iiyanh"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wik-Keyangan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wik-Me'anha"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wik-Mungkan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wik-Ngathana"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wikalkan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wikngenchera"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wilawila"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wintu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Winyé"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wipi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wiradhuri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wiraféd"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wirangu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wiru"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wiyot"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Woccon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wogamusin"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wogeo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Woi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wojenaka"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wolaitta"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wolane"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wolani"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wolaytta"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Woleaian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wolio"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wolof"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wom (Nigeria)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wom (Papua New Guinea)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Womo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wongo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Woods Cree"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Woria"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Worimi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Worodougou"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Worora"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wotu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Woun Meu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Written Oirat"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wu Chinese"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wuding-Luquan Yi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wudu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wuliwuli"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wulna"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wumboko"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wumbvu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wumeng Nasu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wunai Bunu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wunambal"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wurrugu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wusa Nasu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wushi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wusi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wutung"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wutunhua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wuvulu-Aua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wuzlam"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wyandot"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wymysorys"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wára"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wãpha"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wè Northern"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wè Southern"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Wè Western"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Xaasongaxango"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Xadani Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Xakriabá"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Xamtanga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Xanaguía Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Xaragure"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Xavánte"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Xerénte"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Xetá"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Xhosa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Xiandao"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Xiang Chinese"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Xibe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Xicotepec De Juárez Totonac"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Xinca"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Xingú Asuriní"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Xipaya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Xipináwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Xiri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Xiriâna"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Xishanba Lalo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Xokleng"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Xukurú"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Xwela Gbe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Xârâcùù"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yaaku"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yabarana"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yabaâna"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yabem"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yaben"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yabong"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yace"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yaeyama"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yafi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yagaria"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yagnobi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yagomi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yagua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yagwoia"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yahadian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yahang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yahuna"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yaka (Central African Republic)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yaka (Congo)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yaka (Democratic Republic of Congo)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yakaikeke"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yakama"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yakan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yakha"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yakoma"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yakut"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yala"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yalahatan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yalakalore"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yalarnnga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yale"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yaleba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yalunka"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yalálag Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yamap"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yamba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yambes"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yambeta"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yamdena"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yameo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yami"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yaminahua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yamna"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yamongeri"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yamphe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yamphu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yan-nhangu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yana"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yanahuanca Pasco Quechua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yanda Dom Dogon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yandruwandha"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yanesha'"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yang Zhuang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yangben"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yangbye"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yangho"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yangkam"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yangman"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yango"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yangulam"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yangum Dey"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yangum Gel"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yangum Mon"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yankunytjatjara"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yanomamö"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yanomámi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yansi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yanyuwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yao"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yaosakor Asmat"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yaouré"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yapese"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yapunda"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yaqay"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yaqui"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yarawata"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yareba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yareni Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yarsun"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yarí"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yasa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yassic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yatee Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yatzachi Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yau (Morobe Province)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yau (Sandaun Province)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yaul"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yauma"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yaur"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yautepec Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yauyos Quechua"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yavitero"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yawa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yawalapití"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yawanawa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yawarawarga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yaweyuha"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yawiyo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yawuru"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yazgulyam"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yecuatla Totonac"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yei"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yekhee"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yekora"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yela"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yele"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yelmek"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yelogu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yemba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yemsa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yendang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yeni"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yeniche"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yerakai"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yeretuar"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yerong"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yerukula"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yeskwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yessan-Mayo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yetfa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yevanic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yeyi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yiddish"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yiddish Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yidgha"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yidiny"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yil"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yimas"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yimchungru Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yinbaw Karen"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yinchia"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yindjibarndi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yindjilandji"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yine"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yinggarda"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yintale Karen"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yir Yoront"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yis"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yiwom"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yoba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yocoboué Dida"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yogad"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yoidik"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yoke"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yokuts"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yola"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yoloxochitl Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yom"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yombe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yonaguni"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yong"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yongbei Zhuang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yonggom"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yongnan Zhuang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yopno"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yora"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yoron"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yoruba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yos"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yosondúa Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Youjiang Zhuang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Youle Jinuo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Younuo Bunu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yout Wam"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yoy"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yuaga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yucatec Maya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yucatec Maya Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yucateco"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yuchi"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yucuañe Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yucuna"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yue Chinese"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yug"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yugambal"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yugh"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yugoslavian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yuhup"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yuki"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yukpa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yukuben"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yulu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yuqui"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yuracare"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yurok"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yurutí"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yutanduchi Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yuwana"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Yámana"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Zaachila Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Zabana"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Zacatepec Chatino"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Zacatlán-Ahuacatlán-Tepetzintla Nahuatl"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Zaghawa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Zaiwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Zakhring"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Zambian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Zan Gula"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Zanaki"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Zande (individual language)"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Zangskari"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Zangwal"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Zaniza Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Zaramo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Zari"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Zarma"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Zarphatic"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Zauzou"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Zay"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Zayein Karen"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Zayse-Zergulla"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Zaysete"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Zaza"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Zazaki"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Zazao"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Zeem"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Zeeuws"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Zemba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Zeme Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Zemgalian"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Zenag"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Zenaga"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Zenzontepec Chatino"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Zerenkel"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Zhaba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Zhang-Zhung"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Zhire"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Zhoa"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Zhuang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Zia"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Zigula"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Zimakani"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Zimba"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Zimbabwe Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Zinza"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Zire"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ziriya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Zizilivakan"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Zo'é"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Zokhuo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Zoogocho Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Zoroastrian Dari"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Zorop"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Zotung Chin"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Zou"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Zulgo-Gemzek"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Zulu"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Zumaya"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Zumbun"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Zuni"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Zuojiang Zhuang"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Zyphe"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Záparo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("finlandssvenskt teckenspråk"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("suomenruotsalainen viittomakieli"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("tlhIngan-Hol"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Àhàn"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Áncá"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Äiwoo"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Ömie"); //$NON-NLS-1$ //$NON-NLS-2$

		langList.add("Önge"); //$NON-NLS-1$ //$NON-NLS-2$
	}

	private void createLanguagesMap1() {

		langMap.put("!O!ung", "oun"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("oun", "!O!ung"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("!Xóõ", "nmn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nmn", "!Xóõ"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("'Are'are", "alu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("alu", "'Are'are"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("'Auhelawa", "kud"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kud", "'Auhelawa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("//Ani", "hnh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hnh", "//Ani"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("//Gana", "gnk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gnk", "//Gana"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("//Xegwi", "xeg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xeg", "//Xegwi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("/Gwi", "gwj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gwj", "/Gwi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("/Xam", "xam"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xam", "/Xam"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Aralle-Tabulahan", "atq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("atq", "Aralle-Tabulahan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Arifama-Miniafia", "aai"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aai", "Arifama-Miniafia"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bainouk-Gunyaamolo", "bcz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bcz", "Bainouk-Gunyaamolo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cashibo-Cacataibo", "cbr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cbr", "Cashibo-Cacataibo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chichimeca-Jonaz", "pei"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pei", "Chichimeca-Jonaz"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chichonyi-Chidzihana-Chikauma", "coh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("coh", "Chichonyi-Chidzihana-Chikauma"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chonyi-Dzihana-Kauma", "coh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("coh", "Chonyi-Dzihana-Kauma"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cinda-Regi-Tiyal", "cdr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cdr", "Cinda-Regi-Tiyal"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cocama-Cocamilla", "cod"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cod", "Cocama-Cocamilla"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Columbia-Wenatchi", "col"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("col", "Columbia-Wenatchi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gabrielino-Fernandeño", "xgf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xgf", "Gabrielino-Fernandeño"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Havasupai-Walapai-Yavapai", "yuf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yuf", "Havasupai-Walapai-Yavapai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Idakho-Isukha-Tiriki", "ida"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ida", "Idakho-Isukha-Tiriki"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ikoma-Nata-Isenye", "ntk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ntk", "Ikoma-Nata-Isenye"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Izi-Ezaa-Ikwo-Mgbo", "izi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("izi", "Izi-Ezaa-Ikwo-Mgbo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kag-Fer-Jiir-Koor-Ror-Us-Zuksun", "gel"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gel", "Kag-Fer-Jiir-Koor-Ror-Us-Zuksun"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kaningdon-Nindem", "kdp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kdp", "Kaningdon-Nindem"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Katcha-Kadugli-Miri", "xtc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xtc", "Katcha-Kadugli-Miri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kuikúro-Kalapálo", "kui"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kui", "Kuikúro-Kalapálo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kwalhioqua-Tlatskanai", "qwt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("qwt", "Kwalhioqua-Tlatskanai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lamja-Dengsa-Tola", "ldh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ldh", "Lamja-Dengsa-Tola"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lavatbura-Lamusong", "lbv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lbv", "Lavatbura-Lamusong"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lisabata-Nuniali", "lcs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lcs", "Lisabata-Nuniali"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Luidakho-Luisukha-Lutirichi", "ida"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ida", "Luidakho-Luisukha-Lutirichi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Makhuwa-Marrevone", "xmc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xmc", "Makhuwa-Marrevone"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Malecite-Passamaquoddy", "pqm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pqm", "Malecite-Passamaquoddy"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mator-Taygi-Karagas", "ymt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ymt", "Mator-Taygi-Karagas"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mbariman-Gudhinma", "zmv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zmv", "Mbariman-Gudhinma"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Namosi-Naitasiri-Serua", "bwb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bwb", "Namosi-Naitasiri-Serua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nangikurrunggurr", "nam"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nam", "Nangikurrunggurr"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nottoway-Meherrin", "nwy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nwy", "Nottoway-Meherrin"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Numana-Nunku-Gbantu-Numbu", "nbr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nbr", "Numana-Nunku-Gbantu-Numbu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pitcairn-Norfolk", "pih"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pih", "Pitcairn-Norfolk"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Rakahanga-Manihiki", "rkh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rkh", "Rakahanga-Manihiki"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tanimuca-Retuarã", "tnc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tnc", "Tanimuca-Retuarã"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ukwuani-Aboh-Ndoni", "ukw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ukw", "Ukwuani-Aboh-Ndoni"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Uripiv-Wala-Rano-Atchin", "upv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("upv", "Uripiv-Wala-Rano-Atchin"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Vaghat-Ya-Bijim-Legeri", "bij"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bij", "Vaghat-Ya-Bijim-Legeri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wangaaybuwan-Ngiyambaa", "wyb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wyb", "Wangaaybuwan-Ngiyambaa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wotapuri-Katarqalai", "wsv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wsv", "Wotapuri-Katarqalai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("=/Hua", "huc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("huc", "=/Hua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("=/Kx'au//'ein", "aue"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aue", "=/Kx'au//'ein"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("A'tong", "aot"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aot", "A'tong"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("A-Pucikwar", "apq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("apq", "A-Pucikwar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("A-hmaos", "hmd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hmd", "A-hmaos"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Aari", "aiw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aiw", "Aari"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Aasáx", "aas"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aas", "Aasáx"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Abadi", "kbt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kbt", "Abadi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Abaga", "abg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("abg", "Abaga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Abai Sungai", "abf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("abf", "Abai Sungai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Abanyom", "abm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("abm", "Abanyom"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Abar", "mij"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mij", "Abar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Abau", "aau"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aau", "Abau"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Abaza", "abq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("abq", "Abaza"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Abellen Ayta", "abp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("abp", "Abellen Ayta"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Abidji", "abi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("abi", "Abidji"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Abinomn", "bsa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bsa", "Abinomn"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Abipon", "axb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("axb", "Abipon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Abishira", "ash"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ash", "Abishira"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Abkhazian", "abk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("abk", "Abkhazian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Abom", "aob"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aob", "Abom"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Abon", "abo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("abo", "Abon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Abron", "abr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("abr", "Abron"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Abu", "ado"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ado", "Abu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Abu' Arapesh", "aah"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aah", "Abu' Arapesh"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Abua", "abn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("abn", "Abua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Abui", "abz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("abz", "Abui"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Abun", "kgr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kgr", "Abun"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Abure", "abu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("abu", "Abure"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Abureni", "mgj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mgj", "Abureni"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Abé", "aba"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aba", "Abé"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Acatepec Me'phaa", "tpx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tpx", "Acatepec Me'phaa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Acatepec Tlapanec", "tpx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tpx", "Acatepec Tlapanec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Achagua", "aca"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aca", "Achagua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Achang", "acn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("acn", "Achang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ache", "yif"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yif", "Ache"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Acheron", "acz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("acz", "Acheron"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Achi", "acr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("acr", "Achi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Achinese", "ace"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ace", "Achinese"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Achterhoeks", "act"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("act", "Achterhoeks"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Achuar-Shiwiar", "acu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("acu", "Achuar-Shiwiar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Achumawi", "acv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("acv", "Achumawi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Aché", "guq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("guq", "Aché"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Acoli", "ach"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ach", "Acoli"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Acroá", "acs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("acs", "Acroá"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Adabe", "adb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("adb", "Adabe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Adai", "xad"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xad", "Adai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Adamawa Fulfulde", "fub"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("fub", "Adamawa Fulfulde"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Adamorobe Sign Language", "ads"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ads", "Adamorobe Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Adang", "adn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("adn", "Adang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Adangbe", "adq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("adq", "Adangbe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Adangme", "ada"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ada", "Adangme"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Adap", "adp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("adp", "Adap"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Adasen", "tiu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tiu", "Adasen"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Adele", "ade"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ade", "Adele"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Adhola", "adh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("adh", "Adhola"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Adi", "adi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("adi", "Adi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Adioukrou", "adj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("adj", "Adioukrou"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Adivasi Oriya", "ort"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ort", "Adivasi Oriya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Adiwasi Garasia", "gas"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gas", "Adiwasi Garasia"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Adnyamathanha", "adt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("adt", "Adnyamathanha"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Adonara", "adr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("adr", "Adonara"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Aduge", "adu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("adu", "Aduge"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Adygei", "ady"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ady", "Adygei"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Adyghe", "ady"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ady", "Adyghe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Adzera", "adz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("adz", "Adzera"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Aeka", "aez"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aez", "Aeka"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Aekyom", "awi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("awi", "Aekyom"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Aequian", "xae"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xae", "Aequian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Aer", "aeq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aeq", "Aer"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Afade", "aal"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aal", "Afade"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Afar", "aar"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aar", "Afar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Afghan Persian", "prs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("prs", "Afghan Persian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Afghan Sign Language", "afg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("afg", "Afghan Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Afitti", "aft"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aft", "Afitti"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Afra", "ulf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ulf", "Afra"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Afrihili", "afh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("afh", "Afrihili"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Afrikaans", "afr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("afr", "Afrikaans"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Afro-Seminole Creole", "afs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("afs", "Afro-Seminole Creole"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Agarabi", "agd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("agd", "Agarabi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Agariya", "agi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("agi", "Agariya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Agatu", "agc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("agc", "Agatu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Agavotaguerra", "avo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("avo", "Agavotaguerra"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Aghem", "agq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("agq", "Aghem"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Aghu", "ahh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ahh", "Aghu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Aghu Tharnggalu", "ggr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ggr", "Aghu Tharnggalu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Aghul", "agx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("agx", "Aghul"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Aghwan", "xag"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xag", "Aghwan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Agi", "aif"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aif", "Agi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Agob", "kit"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kit", "Agob"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Agoi", "ibm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ibm", "Agoi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Aguacateco", "agu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("agu", "Aguacateco"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Aguano", "aga"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aga", "Aguano"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Aguaruna", "agr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("agr", "Aguaruna"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Aguna", "aug"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aug", "Aguna"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Agusan Manobo", "msm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("msm", "Agusan Manobo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Agutaynen", "agn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("agn", "Agutaynen"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Agwagwune", "yay"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yay", "Agwagwune"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ahanta", "aha"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aha", "Ahanta"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Aheu", "thm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("thm", "Aheu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ahirani", "ahr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ahr", "Ahirani"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ahom", "aho"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aho", "Ahom"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ahtena", "aht"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aht", "Ahtena"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ahwai", "nfd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nfd", "Ahwai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ai-Cham", "aih"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aih", "Ai-Cham"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Aighon", "aix"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aix", "Aighon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Aikanã", "tba"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tba", "Aikanã"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Aiklep", "mwg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mwg", "Aiklep"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Aimaq", "aiq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aiq", "Aimaq"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Aimele", "ail"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ail", "Aimele"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Aimol", "aim"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aim", "Aimol"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ainbai", "aic"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aic", "Ainbai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ainu (China)", "aib"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aib", "Ainu (China)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ainu (Japan)", "ain"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ain", "Ainu (Japan)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Aiome", "aki"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aki", "Aiome"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Airoran", "air"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("air", "Airoran"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Aiton", "aio"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aio", "Aiton"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Aja (Benin)", "ajg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ajg", "Aja (Benin)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Aja (Sudan)", "aja"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aja", "Aja (Sudan)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ajawa", "ajw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ajw", "Ajawa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ajië", "aji"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aji", "Ajië"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ajyíninka Apurucayali", "cpc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cpc", "Ajyíninka Apurucayali"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ak", "akq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("akq", "Ak"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Aka", "soh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("soh", "Aka"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Aka-Bea", "abj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("abj", "Aka-Bea"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Aka-Bo", "akm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("akm", "Aka-Bo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Aka-Cari", "aci"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aci", "Aka-Cari"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Aka-Jeru", "akj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("akj", "Aka-Jeru"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Aka-Kede", "akx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("akx", "Aka-Kede"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Aka-Kol", "aky"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aky", "Aka-Kol"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Aka-Kora", "ack"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ack", "Aka-Kora"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Akan", "aka"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aka", "Akan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Akar-Bale", "acl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("acl", "Akar-Bale"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Akaselem", "aks"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aks", "Akaselem"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Akawaio", "ake"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ake", "Akawaio"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ake", "aik"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aik", "Ake"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Akebu", "keu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("keu", "Akebu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Akei", "tsr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tsr", "Akei"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Akeu", "aeu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aeu", "Akeu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Akha", "ahk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ahk", "Akha"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Akhvakh", "akv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("akv", "Akhvakh"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Akkadian", "akk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("akk", "Akkadian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Akkala Sami", "sia"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sia", "Akkala Sami"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Aklanon", "akl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("akl", "Aklanon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Akolet", "akt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("akt", "Akolet"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Akoose", "bss"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bss", "Akoose"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Akoye", "miw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("miw", "Akoye"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Akpa", "akf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("akf", "Akpa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Akpes", "ibe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ibe", "Akpes"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Akrukay", "afi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("afi", "Akrukay"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Akuku", "ayk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ayk", "Akuku"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Akum", "aku"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aku", "Akum"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Akuntsu", "aqz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aqz", "Akuntsu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Akurio", "ako"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ako", "Akurio"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Akwa", "akw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("akw", "Akwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Al-Sayyid Bedouin Sign Language", "syy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("syy", "Al-Sayyid Bedouin Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Alaba-K’abeena", "alw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("alw", "Alaba-K’abeena"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Alabama", "akz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("akz", "Alabama"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Alabat Island Agta", "dul"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dul", "Alabat Island Agta"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Alacatlatzala Mixtec", "mim"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mim", "Alacatlatzala Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Alago", "ala"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ala", "Alago"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Alagwa", "wbj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wbj", "Alagwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Alak", "alk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("alk", "Alak"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Alamblak", "amp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("amp", "Alamblak"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Alangan", "alj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("alj", "Alangan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Alanic", "xln"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xln", "Alanic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Alapmunte", "apv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("apv", "Alapmunte"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Alawa", "alh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("alh", "Alawa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Albanian", "sqi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sqi", "Albanian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Alcozauca Mixtec", "xta"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xta", "Alcozauca Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Alege", "alf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("alf", "Alege"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Alekano", "gah"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gah", "Alekano"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Alemannic", "gsw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gsw", "Alemannic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Aleut", "ale"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ale", "Aleut"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Algerian Arabic", "arq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("arq", "Algerian Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Algerian Saharan Arabic", "aao"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aao", "Algerian Saharan Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Algerian Sign Language", "asp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("asp", "Algerian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Algonquin", "alq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("alq", "Algonquin"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ali", "aiy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aiy", "Ali"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Alladian", "ald"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ald", "Alladian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Allar", "all"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("all", "Allar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Alngith", "aid"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aid", "Alngith"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Alo Phola", "ypo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ypo", "Alo Phola"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Alor", "aol"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aol", "Alor"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Aloápam Zapotec", "zaq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zaq", "Aloápam Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Alsatian", "gsw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gsw", "Alsatian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Alsea", "aes"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aes", "Alsea"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Alu Kurumba", "xua"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xua", "Alu Kurumba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Alugu", "aub"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aub", "Alugu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Alumu-Tesu", "aab"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aab", "Alumu-Tesu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Alune", "alp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("alp", "Alune"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Aluo", "yna"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yna", "Aluo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Alur", "alz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("alz", "Alur"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Alutor", "alr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("alr", "Alutor"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Alviri-Vidari", "avd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("avd", "Alviri-Vidari"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Alyawarr", "aly"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aly", "Alyawarr"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ama (Papua New Guinea)", "amm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("amm", "Ama (Papua New Guinea)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ama (Sudan)", "nyi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nyi", "Ama (Sudan)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Amahai", "amq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("amq", "Amahai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Amahuaca", "amc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("amc", "Amahuaca"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Amaimon", "ali"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ali", "Amaimon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Amal", "aad"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aad", "Amal"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Amanab", "amn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("amn", "Amanab"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Amanayé", "ama"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ama", "Amanayé"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Amara", "aie"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aie", "Amara"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Amarag", "amg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("amg", "Amarag"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Amarakaeri", "amr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("amr", "Amarakaeri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Amarasi", "aaz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aaz", "Amarasi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Amatlán Zapotec", "zpo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zpo", "Amatlán Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Amba (Solomon Islands)", "utp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("utp", "Amba (Solomon Islands)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Amba (Uganda)", "rwm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rwm", "Amba (Uganda)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ambai", "amk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("amk", "Ambai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ambakich", "aew"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aew", "Ambakich"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ambala Ayta", "abc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("abc", "Ambala Ayta"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ambelau", "amv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("amv", "Ambelau"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ambele", "ael"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ael", "Ambele"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Amblong", "alm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("alm", "Amblong"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ambo", "amb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("amb", "Ambo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ambo-Pasco Quechua", "qva"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("qva", "Ambo-Pasco Quechua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ambonese Malay", "abs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("abs", "Ambonese Malay"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ambrak", "aag"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aag", "Ambrak"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ambulas", "abt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("abt", "Ambulas"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Amdang", "amj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("amj", "Amdang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Amdo Tibetan", "adx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("adx", "Amdo Tibetan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Amele", "aey"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aey", "Amele"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("American Sign Language", "ase"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ase", "American Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Amganad Ifugao", "ifa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ifa", "Amganad Ifugao"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Amharic", "amh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("amh", "Amharic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ami", "amy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("amy", "Ami"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Amis", "ami"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ami", "Amis"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Amo", "amo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("amo", "Amo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Amol", "alx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("alx", "Amol"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Amoltepec Mixtec", "mbz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mbz", "Amoltepec Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ampanang", "apg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("apg", "Ampanang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Amri Karbi", "ajz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ajz", "Amri Karbi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Amto", "amt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("amt", "Amto"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Amundava", "adw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("adw", "Amundava"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ana Tinga Dogon", "dti"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dti", "Ana Tinga Dogon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Anaang", "anw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("anw", "Anaang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Anakalangu", "akg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("akg", "Anakalangu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Anal", "anm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("anm", "Anal"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Anam", "pda"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pda", "Anam"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Anambé", "aan"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aan", "Anambé"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Anamgura", "imi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("imi", "Anamgura"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Anasi", "bpo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bpo", "Anasi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ancient Aramaic (up to 700 BCE)", "oar"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("oar", "Ancient Aramaic (up to 700 BCE)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ancient Greek (to 1453)", "grc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("grc", "Ancient Greek (to 1453)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ancient Hebrew", "hbo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hbo", "Ancient Hebrew"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ancient Macedonian", "xmk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xmk", "Ancient Macedonian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ancient North Arabian", "xna"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xna", "Ancient North Arabian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ancient Zapotec", "xzp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xzp", "Ancient Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Andai", "afd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("afd", "Andai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Andalusian Arabic", "xaa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xaa", "Andalusian Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Andaman Creole Hindi", "hca"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hca", "Andaman Creole Hindi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Andaqui", "ana"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ana", "Andaqui"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Andarum", "aod"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aod", "Andarum"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Andegerebinha", "adg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("adg", "Andegerebinha"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Andh", "anr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("anr", "Andh"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Andi", "ani"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ani", "Andi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Andio", "bzb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bzb", "Andio"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Andoa", "anb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("anb", "Andoa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Andoque", "ano"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ano", "Andoque"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Andra-Hus", "anx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("anx", "Andra-Hus"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Aneityum", "aty"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aty", "Aneityum"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Anem", "anz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("anz", "Anem"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Aneme Wake", "aby"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aby", "Aneme Wake"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Anfillo", "myo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("myo", "Anfillo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Angaataha", "agm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("agm", "Angaataha"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Angal", "age"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("age", "Angal"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Angal Enen", "aoe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aoe", "Angal Enen"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Angal Heneng", "akh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("akh", "Angal Heneng"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Angami Naga", "njm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("njm", "Angami Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Angguruk Yali", "yli"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yli", "Angguruk Yali"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Angika", "anp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("anp", "Angika"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Anglo-Norman", "xno"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xno", "Anglo-Norman"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Angloromani", "rme"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rme", "Angloromani"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Angolar", "aoa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aoa", "Angolar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Angor", "agg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("agg", "Angor"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Angoram", "aog"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aog", "Angoram"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Angosturas Tunebo", "tnd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tnd", "Angosturas Tunebo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ani Phowa", "ypn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ypn", "Ani Phowa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Anii", "blo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("blo", "Anii"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Animere", "anf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("anf", "Animere"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Anindilyakwa", "aoi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aoi", "Anindilyakwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Anjam", "boj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("boj", "Anjam"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ankave", "aak"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aak", "Ankave"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Anmatyerre", "amx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("amx", "Anmatyerre"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Anor", "anj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("anj", "Anor"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Anserma", "ans"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ans", "Anserma"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ansus", "and"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("and", "Ansus"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Antakarinya", "ant"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ant", "Antakarinya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Antankarana Malagasy", "xmv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xmv", "Antankarana Malagasy"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Antigua and Barbuda Creole English", "aig"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aig", "Antigua and Barbuda Creole English"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Anu", "anl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("anl", "Anu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Anuak", "anu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("anu", "Anuak"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Anufo", "cko"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cko", "Anufo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Anuki", "aui"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aui", "Anuki"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Anus", "auq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("auq", "Anus"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Anuta", "aud"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aud", "Anuta"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Anyin", "any"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("any", "Anyin"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Anyin Morofo", "mtb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mtb", "Anyin Morofo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ao Naga", "njo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("njo", "Ao Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Aoheng", "pni"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pni", "Aoheng"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Aore", "aor"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aor", "Aore"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ap Ma", "kbx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kbx", "Ap Ma"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Apalachee", "xap"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xap", "Apalachee"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Apalaí", "apy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("apy", "Apalaí"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Apali", "ena"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ena", "Apali"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Apalik", "apo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("apo", "Apalik"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Apasco-Apoala Mixtec", "mip"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mip", "Apasco-Apoala Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Apatani", "apt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("apt", "Apatani"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Apiaká", "api"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("api", "Apiaká"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Apinayé", "apn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("apn", "Apinayé"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Apma", "app"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("app", "Apma"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Aproumu Aizi", "ahp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ahp", "Aproumu Aizi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Apurinã", "apu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("apu", "Apurinã"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Aputai", "apx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("apx", "Aputai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Aquitanian", "xaq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xaq", "Aquitanian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Arabana", "ard"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ard", "Arabana"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Arabela", "arl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("arl", "Arabela"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Arabic", "ara"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ara", "Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Aragonese", "arg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("arg", "Aragonese"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Araki", "akr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("akr", "Araki"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Aramanik", "aam"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aam", "Aramanik"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Arammba", "stk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("stk", "Arammba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Aranadan", "aaf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aaf", "Aranadan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Aranama-Tamique", "xrt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xrt", "Aranama-Tamique"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Arandai", "jbj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jbj", "Arandai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Araona", "aro"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aro", "Araona"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Arapaho", "arp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("arp", "Arapaho"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Arapaso", "arj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("arj", "Arapaso"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Arawak", "arw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("arw", "Arawak"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Araweté", "awt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("awt", "Araweté"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Arawum", "awm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("awm", "Arawum"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Arawá", "aru"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aru", "Arawá"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Arbore", "arv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("arv", "Arbore"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Arbëreshë Albanian", "aae"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aae", "Arbëreshë Albanian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Archi", "aqc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aqc", "Archi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ardhamāgadhī Prākrit", "pka"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pka", "Ardhamāgadhī Prākrit"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Are", "mwc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mwc", "Are"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Areba", "aea"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aea", "Areba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Arem", "aem"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aem", "Arem"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Arequipa-La Unión Quechua", "qxu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("qxu", "Arequipa-La Unión Quechua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Argentine Sign Language", "aed"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aed", "Argentine Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Argobba", "agj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("agj", "Argobba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Arguni", "agf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("agf", "Arguni"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Arhuaco", "arh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("arh", "Arhuaco"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Arhâ", "aqr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aqr", "Arhâ"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Arhö", "aok"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aok", "Arhö"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ari", "aac"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aac", "Ari"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Aribwatsa", "laz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("laz", "Aribwatsa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Aribwaung", "ylu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ylu", "Aribwaung"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Arigidi", "aqg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aqg", "Arigidi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Arikapú", "ark"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ark", "Arikapú"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Arikara", "ari"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ari", "Arikara"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Arikem", "ait"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ait", "Arikem"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Arin", "xrn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xrn", "Arin"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Aringa", "luc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("luc", "Aringa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Arma", "aoh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aoh", "Arma"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Armazic", "xrm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xrm", "Armazic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Armenian", "hye"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hye", "Armenian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Armenian Sign Language", "aen"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aen", "Armenian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Aromanian", "rup"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rup", "Aromanian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Arop-Lokep", "apr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("apr", "Arop-Lokep"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Arop-Sissano", "aps"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aps", "Arop-Sissano"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Arosi", "aia"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aia", "Arosi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Arpitan", "frp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("frp", "Arpitan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Arta", "atz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("atz", "Arta"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Aruamu", "msy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("msy", "Aruamu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Aruek", "aur"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aur", "Aruek"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Arumanian", "rup"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rup", "Arumanian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Aruop", "lsr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lsr", "Aruop"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Arutani", "atx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("atx", "Arutani"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Aruá (Amazonas State)", "aru"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aru", "Aruá (Amazonas State)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Aruá (Rodonia State)", "arx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("arx", "Aruá (Rodonia State)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Arvanitika Albanian", "aat"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aat", "Arvanitika Albanian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("As", "asz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("asz", "As"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Asaro'o", "mtv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mtv", "Asaro'o"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Asas", "asd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("asd", "Asas"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ashe", "ahs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ahs", "Ashe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ashkun", "ask"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ask", "Ashkun"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Asho Chin", "csh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("csh", "Asho Chin"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ashtiani", "atn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("atn", "Ashtiani"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Asháninka", "cni"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cni", "Asháninka"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ashéninka Pajonal", "cjo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cjo", "Ashéninka Pajonal"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ashéninka Perené", "prq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("prq", "Ashéninka Perené"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Asilulu", "asl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("asl", "Asilulu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Askopan", "eiv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("eiv", "Askopan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Asoa", "asv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("asv", "Asoa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Assamese", "asm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("asm", "Assamese"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Assan", "xss"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xss", "Assan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Assangori", "sjg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sjg", "Assangori"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Assiniboine", "asb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("asb", "Assiniboine"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Assyrian Neo-Aramaic", "aii"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aii", "Assyrian Neo-Aramaic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Asturian", "ast"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ast", "Asturian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Asturleonese", "ast"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ast", "Asturleonese"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Asu (Nigeria)", "aum"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aum", "Asu (Nigeria)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Asu (Tanzania)", "asa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("asa", "Asu (Tanzania)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Asue Awyu", "psa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("psa", "Asue Awyu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Asumboa", "aua"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aua", "Asumboa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Asunción Mixtepec Zapotec", "zoo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zoo", "Asunción Mixtepec Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Asuri", "asr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("asr", "Asuri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ata", "atm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("atm", "Ata"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ata Manobo", "atd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("atd", "Ata Manobo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Atakapa", "aqp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aqp", "Atakapa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Atampaya", "amz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("amz", "Atampaya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Atatláhuca Mixtec", "mib"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mib", "Atatláhuca Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Atayal", "tay"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tay", "Atayal"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Atemble", "ate"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ate", "Atemble"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Athpariya", "aph"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aph", "Athpariya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ati", "atk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("atk", "Ati"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Atikamekw", "atj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("atj", "Atikamekw"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Atohwaim", "aqm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aqm", "Atohwaim"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Atong", "ato"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ato", "Atong"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Atorada", "aox"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aox", "Atorada"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Atsahuaca", "atc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("atc", "Atsahuaca"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Atsam", "cch"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cch", "Atsam"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Atsugewi", "atw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("atw", "Atsugewi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Attapady Kurumba", "pkr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pkr", "Attapady Kurumba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Attié", "ati"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ati", "Attié"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Atzingo Matlatzinca", "ocu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ocu", "Atzingo Matlatzinca"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Au", "avt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("avt", "Au"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Aulua", "aul"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aul", "Aulua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Aurá", "aux"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aux", "Aurá"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Aushi", "auh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("auh", "Aushi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Aushiri", "avs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("avs", "Aushiri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Austral", "aut"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aut", "Austral"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Australian Aborigines Sign Language", "asw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("asw", "Australian Aborigines Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Australian Sign Language", "asf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("asf", "Australian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Austrian Sign Language", "asq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("asq", "Austrian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Auwe", "smf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("smf", "Auwe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Auye", "auu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("auu", "Auye"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Auyokawa", "auo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("auo", "Auyokawa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ava Guaraní", "nhd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nhd", "Ava Guaraní"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Avaric", "ava"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ava", "Avaric"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Avatime", "avn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("avn", "Avatime"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Avau", "avb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("avb", "Avau"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Avava", "tmb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tmb", "Avava"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Avestan", "ave"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ave", "Avestan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Avikam", "avi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("avi", "Avikam"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Avokaya", "avu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("avu", "Avokaya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Avá-Canoeiro", "avv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("avv", "Avá-Canoeiro"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Awa (China)", "vwa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vwa", "Awa (China)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Awa (Papua New Guinea)", "awb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("awb", "Awa (Papua New Guinea)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Awa-Cuaiquer", "kwi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kwi", "Awa-Cuaiquer"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Awabakal", "awk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("awk", "Awabakal"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Awad Bing", "bcu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bcu", "Awad Bing"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Awadhi", "awa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("awa", "Awadhi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Awak", "awo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("awo", "Awak"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Awar", "aya"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aya", "Awar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Awara", "awx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("awx", "Awara"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Awbono", "awh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("awh", "Awbono"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Aweer", "bob"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bob", "Aweer"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Awera", "awr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("awr", "Awera"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Awetí", "awe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("awe", "Awetí"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Awing", "azo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("azo", "Awing"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Awiyaana", "auy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("auy", "Awiyaana"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Awjilah", "auj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("auj", "Awjilah"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Awngi", "awn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("awn", "Awngi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Awtuw", "kmn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kmn", "Awtuw"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Awu", "yiu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yiu", "Awu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Awun", "aww"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aww", "Awun"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Awutu", "afu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("afu", "Awutu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Awyi", "auw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("auw", "Awyi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Axamb", "ahb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ahb", "Axamb"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Axi Yi", "yix"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yix", "Axi Yi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ayabadhu", "ayd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ayd", "Ayabadhu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ayacucho Quechua", "quy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("quy", "Ayacucho Quechua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ayangan Ifugao", "ifb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ifb", "Ayangan Ifugao"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ayautla Mazatec", "vmy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vmy", "Ayautla Mazatec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ayere", "aye"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aye", "Ayere"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ayi (China)", "ayx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ayx", "Ayi (China)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ayi (Papua New Guinea)", "ayq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ayq", "Ayi (Papua New Guinea)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ayiwo", "nfl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nfl", "Ayiwo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ayizi", "yyz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yyz", "Ayizi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ayizo Gbe", "ayb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ayb", "Ayizo Gbe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Aymara", "aym"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aym", "Aymara"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ayoquesco Zapotec", "zaf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zaf", "Ayoquesco Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ayoreo", "ayo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ayo", "Ayoreo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ayu", "ayu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ayu", "Ayu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ayutla Mixtec", "miy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("miy", "Ayutla Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Azerbaijani", "aze"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aze", "Azerbaijani"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Azha", "aza"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aza", "Azha"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Azhe", "yiz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yiz", "Azhe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Azoyú Me'phaa", "tpc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tpc", "Azoyú Me'phaa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Azoyú Tlapanec", "tpc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tpc", "Azoyú Tlapanec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Baan", "bvj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bvj", "Baan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Baangi", "bqx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bqx", "Baangi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Baatonum", "bba"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bba", "Baatonum"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Baba", "bbw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bbw", "Baba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Baba Malay", "mbf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mbf", "Baba Malay"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Babalia Creole Arabic", "bbz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bbz", "Babalia Creole Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Babango", "bbm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bbm", "Babango"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Babanki", "bbk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bbk", "Babanki"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Babatana", "baa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("baa", "Babatana"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Babine", "bcr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bcr", "Babine"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bable", "ast"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ast", "Bable"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Babuza", "bzg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bzg", "Babuza"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bacama", "bcy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bcy", "Bacama"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bacanese Malay", "btj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("btj", "Bacanese Malay"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bactrian", "xbc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xbc", "Bactrian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bada (Indonesia)", "bhz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bhz", "Bada (Indonesia)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bada (Nigeria)", "bau"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bau", "Bada (Nigeria)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Badaga", "bfq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bfq", "Badaga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bade", "bde"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bde", "Bade"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Badeshi", "bdz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bdz", "Badeshi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Badimaya", "bia"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bia", "Badimaya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Badui", "bac"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bac", "Badui"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Badyara", "pbp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pbp", "Badyara"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Baeggu", "bvd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bvd", "Baeggu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Baelelea", "bvc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bvc", "Baelelea"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Baetora", "btr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("btr", "Baetora"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bafanji", "bfj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bfj", "Bafanji"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bafaw-Balong", "bwt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bwt", "Bafaw-Balong"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bafia", "ksf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ksf", "Bafia"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bafut", "bfd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bfd", "Bafut"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Baga Binari", "bcg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bcg", "Baga Binari"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Baga Kaloum", "bqf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bqf", "Baga Kaloum"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Baga Koga", "bgo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bgo", "Baga Koga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Baga Manduri", "bmd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bmd", "Baga Manduri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Baga Mboteni", "bgm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bgm", "Baga Mboteni"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Baga Sitemu", "bsp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bsp", "Baga Sitemu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Baga Sobané", "bsv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bsv", "Baga Sobané"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bagheli", "bfy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bfy", "Bagheli"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bagirmi", "bmi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bmi", "Bagirmi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bagirmi Fulfulde", "fui"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("fui", "Bagirmi Fulfulde"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bago-Kusuntu", "bqg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bqg", "Bago-Kusuntu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bagri", "bgq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bgq", "Bagri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bagupi", "bpi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bpi", "Bagupi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bagusa", "bqb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bqb", "Bagusa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bagvalal", "kva"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kva", "Bagvalal"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Baha Buyang", "yha"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yha", "Baha Buyang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Baham", "bdw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bdw", "Baham"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bahamas Creole English", "bah"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bah", "Bahamas Creole English"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Baharna Arabic", "abv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("abv", "Baharna Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bahau", "bhv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bhv", "Bahau"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bahinemo", "bjh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bjh", "Bahinemo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bahing", "bhj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bhj", "Bahing"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bahnar", "bdq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bdq", "Bahnar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bahonsuai", "bsu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bsu", "Bahonsuai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bai", "bdj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bdj", "Bai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Baibai", "bbf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bbf", "Baibai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Baikeno", "bkx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bkx", "Baikeno"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Baima", "bqh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bqh", "Baima"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Baimak", "bmx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bmx", "Baimak"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bainouk-Gunyuño", "bab"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bab", "Bainouk-Gunyuño"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bainouk-Samik", "bcb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bcb", "Bainouk-Samik"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Baiso", "bsw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bsw", "Baiso"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Baissa Fali", "fah"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("fah", "Baissa Fali"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bajan", "bjs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bjs", "Bajan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bajelani", "bjm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bjm", "Bajelani"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Baka (Cameroon)", "bkc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bkc", "Baka (Cameroon)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Baka (Sudan)", "bdh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bdh", "Baka (Sudan)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bakairí", "bkq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bkq", "Bakairí"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bakaka", "bqz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bqz", "Bakaka"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bakhtiari", "bqi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bqi", "Bakhtiari"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Baki", "bki"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bki", "Baki"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bakoko", "bkh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bkh", "Bakoko"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bakole", "kme"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kme", "Bakole"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bakpinka", "bbs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bbs", "Bakpinka"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bakumpai", "bkr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bkr", "Bakumpai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bakwé", "bjw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bjw", "Bakwé"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Balaesang", "bls"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bls", "Balaesang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Balangao", "blw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("blw", "Balangao"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Balangingi", "sse"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sse", "Balangingi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Balanta-Ganja", "bjt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bjt", "Balanta-Ganja"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Balanta-Kentohe", "ble"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ble", "Balanta-Kentohe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Balantak", "blz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("blz", "Balantak"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Balau", "blg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("blg", "Balau"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Baldemu", "bdn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bdn", "Baldemu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bali (Democratic Republic of Congo)", "bcp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bcp", "Bali (Democratic Republic of Congo)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bali (Nigeria)", "bcn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bcn", "Bali (Nigeria)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Balinese", "ban"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ban", "Balinese"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Balinese Malay", "mhp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mhp", "Balinese Malay"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Balkan Gagauz Turkish", "bgx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bgx", "Balkan Gagauz Turkish"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Balkan Romani", "rmn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rmn", "Balkan Romani"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Balo", "bqo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bqo", "Balo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Baloi", "biz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("biz", "Baloi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Balti", "bft"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bft", "Balti"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Baltic Romani", "rml"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rml", "Baltic Romani"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Baluan-Pam", "blq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("blq", "Baluan-Pam"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Baluchi", "bal"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bal", "Baluchi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bamako Sign Language", "bog"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bog", "Bamako Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bamali", "bbq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bbq", "Bamali"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bambalang", "bmo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bmo", "Bambalang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bambam", "ptu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ptu", "Bambam"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bambara", "bam"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bam", "Bambara"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bambassi", "myf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("myf", "Bambassi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bambili-Bambui", "baw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("baw", "Bambili-Bambui"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bamenyam", "bce"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bce", "Bamenyam"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bamu", "bcf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bcf", "Bamu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bamukumbit", "bqt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bqt", "Bamukumbit"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bamun", "bax"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bax", "Bamun"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bamunka", "bvm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bvm", "Bamunka"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bamwe", "bmg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bmg", "Bamwe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ban Khor Sign Language", "bfk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bfk", "Ban Khor Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bana", "bcw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bcw", "Bana"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Banam Bay", "vrt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vrt", "Banam Bay"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Banao Itneg", "bjx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bjx", "Banao Itneg"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Banaro", "byz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("byz", "Banaro"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Banda (Indonesia)", "bnd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bnd", "Banda (Indonesia)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Banda Malay", "bpq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bpq", "Banda Malay"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Banda-Bambari", "liy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("liy", "Banda-Bambari"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Banda-Banda", "bpd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bpd", "Banda-Banda"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Banda-Mbrès", "bqk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bqk", "Banda-Mbrès"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Banda-Ndélé", "bfl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bfl", "Banda-Ndélé"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Banda-Yangere", "yaj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yaj", "Banda-Yangere"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bandi", "bza"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bza", "Bandi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bandial", "bqj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bqj", "Bandial"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bandjalang", "bdy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bdy", "Bandjalang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bandjigali", "bjd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bjd", "Bandjigali"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bangala", "bxg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bxg", "Bangala"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bangandu", "bgf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bgf", "Bangandu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bangba", "bbe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bbe", "Bangba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Banggai", "bgz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bgz", "Banggai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Banggarla", "bjb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bjb", "Banggarla"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bangi", "bni"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bni", "Bangi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bangi Me", "dba"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dba", "Bangi Me"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bangingih Sama", "sse"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sse", "Bangingih Sama"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bangka", "mfb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mfb", "Bangka"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bangolan", "bgj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bgj", "Bangolan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bangubangu", "bnx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bnx", "Bangubangu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bangwinji", "bsj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bsj", "Bangwinji"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Baniva", "bvv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bvv", "Baniva"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Baniwa", "bwi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bwi", "Baniwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Banjar", "bjn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bjn", "Banjar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bankagooma", "bxw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bxw", "Bankagooma"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bankon", "abb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("abb", "Bankon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bannoni", "bcm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bcm", "Bannoni"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bantawa", "bap"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bap", "Bantawa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bantayanon", "bfx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bfx", "Bantayanon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bantik", "bnq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bnq", "Bantik"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bantoanon", "bno"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bno", "Bantoanon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Baoulé", "bci"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bci", "Baoulé"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bara Malagasy", "bhr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bhr", "Bara Malagasy"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Baraamu", "brd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("brd", "Baraamu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Barai", "bbb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bbb", "Barai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Barakai", "baj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("baj", "Barakai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Baram Kayan", "kys"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kys", "Baram Kayan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Barama", "bbg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bbg", "Barama"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Barambu", "brm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("brm", "Barambu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Baramu", "bmz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bmz", "Baramu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Barapasi", "brp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("brp", "Barapasi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Baras", "brs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("brs", "Baras"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Barasana-Eduria", "bsn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bsn", "Barasana-Eduria"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Barbacoas", "bpb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bpb", "Barbacoas"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Barbareño", "boi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("boi", "Barbareño"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Barclayville Grebo", "gry"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gry", "Barclayville Grebo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bardi", "bcj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bcj", "Bardi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Barein", "bva"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bva", "Barein"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bargam", "mlp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mlp", "Bargam"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bari", "bfa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bfa", "Bari"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bariai", "bch"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bch", "Bariai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bariji", "bjc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bjc", "Bariji"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Barikanchi", "bxo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bxo", "Barikanchi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Barok", "bjk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bjk", "Barok"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Barombi", "bbi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bbi", "Barombi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Barro Negro Tunebo", "tbn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tbn", "Barro Negro Tunebo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Barrow Point", "bpt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bpt", "Barrow Point"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Baruga", "bjz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bjz", "Baruga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Baruya", "byr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("byr", "Baruya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Barwe", "bwg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bwg", "Barwe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Barzani Jewish Neo-Aramaic", "bjf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bjf", "Barzani Jewish Neo-Aramaic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Baré", "bae"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bae", "Baré"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Barí", "mot"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mot", "Barí"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Basa (Cameroon)", "bas"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bas", "Basa (Cameroon)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Basa (Nigeria)", "bzw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bzw", "Basa (Nigeria)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Basa-Gumna", "bsl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bsl", "Basa-Gumna"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Basa-Gurmana", "buj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("buj", "Basa-Gurmana"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Basap", "bdb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bdb", "Basap"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Basay", "byq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("byq", "Basay"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bashkardi", "bsg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bsg", "Bashkardi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bashkir", "bak"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bak", "Bashkir"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Basketo", "bst"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bst", "Basketo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Basque", "eus"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("eus", "Basque"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bassa", "bsq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bsq", "Bassa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bassa-Kontagora", "bsr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bsr", "Bassa-Kontagora"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bassari", "bsc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bsc", "Bassari"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bassossi", "bsi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bsi", "Bassossi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bata", "bta"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bta", "Bata"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Batad Ifugao", "ifb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ifb", "Batad Ifugao"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Batak", "bya"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bya", "Batak"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Batak Alas-Kluet", "btz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("btz", "Batak Alas-Kluet"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Batak Angkola", "akb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("akb", "Batak Angkola"); //$NON-NLS-1$ //$NON-NLS-2$

	}

	private void createLanguagesMap2() {

		langMap.put("Batak Dairi", "btd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("btd", "Batak Dairi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Batak Karo", "btx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("btx", "Batak Karo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Batak Mandailing", "btm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("btm", "Batak Mandailing"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Batak Simalungun", "bts"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bts", "Batak Simalungun"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Batak Toba", "bbc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bbc", "Batak Toba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Batanga", "bnm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bnm", "Batanga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Batek", "btq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("btq", "Batek"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bateri", "btv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("btv", "Bateri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bathari", "bhm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bhm", "Bathari"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bati (Cameroon)", "btc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("btc", "Bati (Cameroon)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bati (Indonesia)", "bvt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bvt", "Bati (Indonesia)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bats", "bbl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bbl", "Bats"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Batu", "btu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("btu", "Batu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Batui", "zbt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zbt", "Batui"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Batuley", "bay"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bay", "Batuley"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bau", "bbd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bbd", "Bau"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bau Bidayuh", "sne"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sne", "Bau Bidayuh"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bauchi", "bsf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bsf", "Bauchi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Baure", "brg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("brg", "Baure"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bauria", "bge"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bge", "Bauria"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bauro", "bxa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bxa", "Bauro"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bauwaki", "bwk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bwk", "Bauwaki"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bauzi", "bvz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bvz", "Bauzi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bavarian", "bar"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bar", "Bavarian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bawm Chin", "bgr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bgr", "Bawm Chin"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bay Miwok", "mkq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mkq", "Bay Miwok"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bayali", "bjy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bjy", "Bayali"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Baybayanon", "bvy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bvy", "Baybayanon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Baygo", "byg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("byg", "Baygo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bayono", "byl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("byl", "Bayono"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bayot", "bda"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bda", "Bayot"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bayungu", "bxj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bxj", "Bayungu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bazigar", "bfr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bfr", "Bazigar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Beami", "beo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("beo", "Beami"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Beaver", "bea"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bea", "Beaver"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Beba", "bfp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bfp", "Beba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bebe", "bzv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bzv", "Bebe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bebele", "beb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("beb", "Bebele"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bebeli", "bek"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bek", "Bebeli"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bebil", "bxp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bxp", "Bebil"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bedawiyet", "bej"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bej", "Bedawiyet"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bedik", "tnr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tnr", "Bedik"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bedjond", "bjv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bjv", "Bedjond"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bedoanas", "bed"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bed", "Bedoanas"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Beeke", "bkf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bkf", "Beeke"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Beele", "bxq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bxq", "Beele"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Beembe", "beq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("beq", "Beembe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Beezen", "bnz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bnz", "Beezen"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Befang", "bby"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bby", "Befang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Begbere-Ejar", "bqv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bqv", "Begbere-Ejar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Beja", "bej"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bej", "Beja"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bekati'", "bei"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bei", "Bekati'"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bekwarra", "bkv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bkv", "Bekwarra"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bekwil", "bkw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bkw", "Bekwil"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Belait", "beg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("beg", "Belait"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Belanda Bor", "bxb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bxb", "Belanda Bor"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Belanda Viri", "bvi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bvi", "Belanda Viri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Belarusian", "bel"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bel", "Belarusian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Belhariya", "byw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("byw", "Belhariya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Beli (Papua New Guinea)", "bey"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bey", "Beli (Papua New Guinea)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Beli (Sudan)", "blm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("blm", "Beli (Sudan)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Belize Kriol English", "bzj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bzj", "Belize Kriol English"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bella Coola", "blc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("blc", "Bella Coola"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bellari", "brw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("brw", "Bellari"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bemba (Democratic Republic of Congo)", "bmy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bmy", "Bemba (Democratic Republic of Congo)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bemba (Zambia)", "bem"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bem", "Bemba (Zambia)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bembe", "bmb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bmb", "Bembe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bena (Nigeria)", "yun"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yun", "Bena (Nigeria)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bena (Tanzania)", "bez"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bez", "Bena (Tanzania)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Benabena", "bef"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bef", "Benabena"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bench", "bcq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bcq", "Bench"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bende", "bdp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bdp", "Bende"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bendi", "bct"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bct", "Bendi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Beneraf", "bnv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bnv", "Beneraf"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Beng", "nhb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nhb", "Beng"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Benga", "bng"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bng", "Benga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bengali", "ben"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ben", "Bengali"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Benggoi", "bgy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bgy", "Benggoi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bengkala Sign Language", "bqy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bqy", "Bengkala Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bentong", "bnu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bnu", "Bentong"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Benyadu'", "byd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("byd", "Benyadu'"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Beothuk", "bue"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bue", "Beothuk"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bepour", "bie"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bie", "Bepour"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bera", "brf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("brf", "Bera"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Berakou", "bxv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bxv", "Berakou"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Berau Malay", "bve"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bve", "Berau Malay"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Berbice Creole Dutch", "brc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("brc", "Berbice Creole Dutch"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Berik", "bkl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bkl", "Berik"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Berinomo", "bit"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bit", "Berinomo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Berom", "bom"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bom", "Berom"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Berta", "wti"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wti", "Berta"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Berti", "byt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("byt", "Berti"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Besisi", "mhe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mhe", "Besisi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Besme", "bes"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bes", "Besme"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Besoa", "bep"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bep", "Besoa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Betaf", "bfe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bfe", "Betaf"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Betawi", "bew"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bew", "Betawi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bete", "byf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("byf", "Bete"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bete-Bendi", "btt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("btt", "Bete-Bendi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Beti (Côte d'Ivoire)", "eot"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("eot", "Beti (Côte d'Ivoire)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Betta Kurumba", "xub"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xub", "Betta Kurumba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bezhta", "kap"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kap", "Bezhta"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bhadrawahi", "bhd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bhd", "Bhadrawahi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bhalay", "bhx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bhx", "Bhalay"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bharia", "bha"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bha", "Bharia"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bhatola", "btl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("btl", "Bhatola"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bhatri", "bgw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bgw", "Bhatri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bhattiyali", "bht"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bht", "Bhattiyali"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bhaya", "bhe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bhe", "Bhaya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bhele", "bhy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bhy", "Bhele"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bhilali", "bhi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bhi", "Bhilali"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bhili", "bhb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bhb", "Bhili"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bhojpuri", "bho"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bho", "Bhojpuri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bhoti Kinnauri", "nes"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nes", "Bhoti Kinnauri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bhujel", "byh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("byh", "Bhujel"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bhunjia", "bhu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bhu", "Bhunjia"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Biafada", "bif"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bif", "Biafada"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Biage", "bdf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bdf", "Biage"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Biak", "bhw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bhw", "Biak"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Biali", "beh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("beh", "Biali"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bian Marind", "bpv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bpv", "Bian Marind"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Biangai", "big"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("big", "Biangai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Biao", "byk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("byk", "Biao"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Biao Mon", "bmt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bmt", "Biao Mon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Biao-Jiao Mien", "bje"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bje", "Biao-Jiao Mien"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Biatah Bidayuh", "bth"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bth", "Biatah Bidayuh"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bidiyo", "bid"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bid", "Bidiyo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bidyara", "bym"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bym", "Bidyara"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bidyogo", "bjg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bjg", "Bidyogo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Biem", "bmc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bmc", "Biem"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bierebo", "bnk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bnk", "Bierebo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bieria", "brj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("brj", "Bieria"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Biete", "biu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("biu", "Biete"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Big Nambas", "nmb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nmb", "Big Nambas"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Biga", "bhc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bhc", "Biga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bijori", "bix"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bix", "Bijori"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bikaru", "bic"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bic", "Bikaru"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bikol", "bik"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bik", "Bikol"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bikya", "byb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("byb", "Bikya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bila", "bip"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bip", "Bila"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bilakura", "bql"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bql", "Bilakura"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bilaspuri", "kfs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kfs", "Bilaspuri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bilba", "bpz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bpz", "Bilba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bilbil", "brz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("brz", "Bilbil"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bile", "bil"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bil", "Bile"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bilin", "byn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("byn", "Bilin"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bilma Kanuri", "bms"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bms", "Bilma Kanuri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Biloxi", "bll"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bll", "Biloxi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bilua", "blb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("blb", "Bilua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bilur", "bxf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bxf", "Bilur"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bima", "bhp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bhp", "Bima"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bimin", "bhl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bhl", "Bimin"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bimoba", "bim"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bim", "Bimoba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bina (Nigeria)", "byj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("byj", "Bina (Nigeria)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bina (Papua New Guinea)", "bmn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bmn", "Bina (Papua New Guinea)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Binahari", "bxz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bxz", "Binahari"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Binandere", "bhg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bhg", "Binandere"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bine", "bon"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bon", "Bine"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bini", "bin"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bin", "Bini"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Binji", "bpj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bpj", "Binji"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Binongan Itneg", "itb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("itb", "Binongan Itneg"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bintauna", "bne"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bne", "Bintauna"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bintulu", "bny"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bny", "Bintulu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Binukid", "bkd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bkd", "Binukid"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Binumarien", "bjr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bjr", "Binumarien"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bipi", "biq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("biq", "Bipi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Birale", "bxe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bxe", "Birale"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Birao", "brr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("brr", "Birao"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Birgit", "btf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("btf", "Birgit"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Birhor", "biy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("biy", "Birhor"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Biri", "bzr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bzr", "Biri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Biritai", "bqq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bqq", "Biritai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Birked", "brk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("brk", "Birked"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Birri", "bvq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bvq", "Birri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Birwa", "brl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("brl", "Birwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Biseni", "ije"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ije", "Biseni"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bishnupriya", "bpy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bpy", "Bishnupriya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bishuo", "bwh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bwh", "Bishuo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bisis", "bnw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bnw", "Bisis"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bislama", "bis"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bis", "Bislama"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bisorio", "bir"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bir", "Bisorio"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bissa", "bib"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bib", "Bissa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bisu", "bzi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bzi", "Bisu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bit", "bgk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bgk", "Bit"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bitare", "brt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("brt", "Bitare"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bitur", "mcc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mcc", "Bitur"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Biwat", "bwm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bwm", "Biwat"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Biyo", "byo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("byo", "Biyo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Biyom", "bpm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bpm", "Biyom"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Blablanga", "blp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("blp", "Blablanga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Blafe", "bfh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bfh", "Blafe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Blagar", "beu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("beu", "Blagar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Blang", "blr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("blr", "Blang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Blin", "byn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("byn", "Blin"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bliss", "zbl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zbl", "Bliss"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Blissymbolics", "zbl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zbl", "Blissymbolics"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Blissymbols", "zbl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zbl", "Blissymbols"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bo (Laos)", "bgl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bgl", "Bo (Laos)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bo (Papua New Guinea)", "bpw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bpw", "Bo (Papua New Guinea)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bo-Rukul", "mae"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mae", "Bo-Rukul"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bo-Ung", "mux"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mux", "Bo-Ung"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Boano (Maluku)", "bzn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bzn", "Boano (Maluku)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Boano (Sulawesi)", "bzl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bzl", "Boano (Sulawesi)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bobongko", "bgb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bgb", "Bobongko"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bobot", "bty"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bty", "Bobot"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bodo (Central African Republic)", "boy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("boy", "Bodo (Central African Republic)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bodo (India)", "brx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("brx", "Bodo (India)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bodo Gadaba", "gbj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gbj", "Bodo Gadaba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bodo Parja", "bdv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bdv", "Bodo Parja"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bofi", "bff"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bff", "Bofi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Boga", "bvw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bvw", "Boga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bogaya", "boq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("boq", "Bogaya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Boghom", "bux"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bux", "Boghom"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Boguru", "bqu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bqu", "Boguru"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bohtan Neo-Aramaic", "bhn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bhn", "Bohtan Neo-Aramaic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Boikin", "bzf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bzf", "Boikin"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bokha", "ybk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ybk", "Bokha"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Boko (Benin)", "bqc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bqc", "Boko (Benin)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Boko (Democratic Republic of Congo)", "bkp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bkp", "Boko (Democratic Republic of Congo)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bokobaru", "bus"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bus", "Bokobaru"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bokoto", "bdt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bdt", "Bokoto"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bokyi", "bky"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bky", "Bokyi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bola", "bnp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bnp", "Bola"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bolango", "bld"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bld", "Bolango"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bole", "bol"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bol", "Bole"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bolgarian", "xbo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xbo", "Bolgarian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bolgo", "bvo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bvo", "Bolgo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bolia", "bli"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bli", "Bolia"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bolinao", "smk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("smk", "Bolinao"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bolivian Sign Language", "bvl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bvl", "Bolivian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bolo", "blv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("blv", "Bolo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Boloki", "bkt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bkt", "Boloki"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bolon", "bof"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bof", "Bolon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bolondo", "bzm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bzm", "Bolondo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bolongan", "blj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("blj", "Bolongan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bolyu", "ply"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ply", "Bolyu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bom", "bmf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bmf", "Bom"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Boma", "boh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("boh", "Boma"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bomboli", "bml"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bml", "Bomboli"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bomboma", "bws"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bws", "Bomboma"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bomitaba", "zmx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zmx", "Bomitaba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bomu", "bmq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bmq", "Bomu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bomwali", "bmw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bmw", "Bomwali"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bon Gula", "glc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("glc", "Bon Gula"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bonan", "peh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("peh", "Bonan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bondei", "bou"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bou", "Bondei"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bondo", "bfw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bfw", "Bondo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bondoukou Kulango", "kzc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kzc", "Bondoukou Kulango"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bondum Dom Dogon", "dbu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dbu", "Bondum Dom Dogon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bonerate", "bna"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bna", "Bonerate"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bonerif", "bnv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bnv", "Bonerif"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bonggi", "bdg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bdg", "Bonggi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bonggo", "bpg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bpg", "Bonggo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bongili", "bui"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bui", "Bongili"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bongo", "bot"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bot", "Bongo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bongu", "bpu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bpu", "Bongu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bonjo", "bok"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bok", "Bonjo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bonkeng", "bvg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bvg", "Bonkeng"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bonkiman", "bop"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bop", "Bonkiman"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bontok", "bnc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bnc", "Bontok"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bookan", "bnb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bnb", "Bookan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Boon", "bnl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bnl", "Boon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Boor", "bvf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bvf", "Boor"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bora", "boa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("boa", "Bora"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Borana-Arsi-Guji Oromo", "gax"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gax", "Borana-Arsi-Guji Oromo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Border Kuna", "kvn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kvn", "Border Kuna"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Borei", "gai"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gai", "Borei"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Borgu Fulfulde", "fue"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("fue", "Borgu Fulfulde"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Borna (Democratic Republic of Congo)", "bxx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bxx", "Borna (Democratic Republic of Congo)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Borna (Ethiopia)", "bwo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bwo", "Borna (Ethiopia)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Boro (Ethiopia)", "bwo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bwo", "Boro (Ethiopia)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Boro (Ghana)", "xxb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xxb", "Boro (Ghana)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Borong", "ksr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ksr", "Borong"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Boruca", "brn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("brn", "Boruca"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Borôro", "bor"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bor", "Borôro"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Boselewa", "bwf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bwf", "Boselewa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bosngun", "bqs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bqs", "Bosngun"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bosnian", "bos"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bos", "Bosnian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bote-Majhi", "bmj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bmj", "Bote-Majhi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Botlikh", "bph"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bph", "Botlikh"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Botolan Sambal", "sbl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sbl", "Botolan Sambal"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bouna Kulango", "nku"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nku", "Bouna Kulango"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bouyei", "pcc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pcc", "Bouyei"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bozaba", "bzo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bzo", "Bozaba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bragat", "aof"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aof", "Bragat"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Brahui", "brh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("brh", "Brahui"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Braj", "bra"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bra", "Braj"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Brazilian Sign Language", "bzs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bzs", "Brazilian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Brek Karen", "kvl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kvl", "Brek Karen"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Brem", "buq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("buq", "Brem"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Breri", "brq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("brq", "Breri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Breton", "bre"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bre", "Breton"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bribri", "bzd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bzd", "Bribri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Brithenig", "bzt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bzt", "Brithenig"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("British Sign Language", "bfi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bfi", "British Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Brokkat", "bro"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bro", "Brokkat"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Brokpake", "sgt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sgt", "Brokpake"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Brokskat", "bkk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bkk", "Brokskat"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Brooke's Point Palawano", "plw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("plw", "Brooke's Point Palawano"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Broome Pearling Lugger Pidgin", "bpl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bpl", "Broome Pearling Lugger Pidgin"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Brunei", "kxd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kxd", "Brunei"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Brunei Bisaya", "bsb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bsb", "Brunei Bisaya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bu", "jid"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jid", "Bu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bu-Nao Bunu", "bwx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bwx", "Bu-Nao Bunu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bua", "bub"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bub", "Bua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bualkhaw Chin", "cbl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cbl", "Bualkhaw Chin"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Buamu", "box"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("box", "Buamu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bube", "bvb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bvb", "Bube"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bubi", "buw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("buw", "Bubi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bubia", "bbx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bbx", "Bubia"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Budeh Stieng", "stt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("stt", "Budeh Stieng"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Budibud", "btp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("btp", "Budibud"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Budong-Budong", "bdx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bdx", "Budong-Budong"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Budu", "buu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("buu", "Budu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Budukh", "bdk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bdk", "Budukh"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Buduma", "bdm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bdm", "Buduma"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Budza", "bja"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bja", "Budza"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bugan", "bbh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bbh", "Bugan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bugawac", "buk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("buk", "Bugawac"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bughotu", "bgt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bgt", "Bughotu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Buginese", "bug"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bug", "Buginese"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Buglere", "sab"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sab", "Buglere"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bugun", "bgg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bgg", "Bugun"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Buhi'non Bikol", "ubl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ubl", "Buhi'non Bikol"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Buhid", "bku"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bku", "Buhid"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Buhutu", "bxh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bxh", "Buhutu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bukar-Sadung Bidayuh", "sdo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sdo", "Bukar-Sadung Bidayuh"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bukat", "bvk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bvk", "Bukat"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bukharic", "bhh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bhh", "Bukharic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bukit Malay", "bvu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bvu", "Bukit Malay"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bukitan", "bkn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bkn", "Bukitan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bukiyip", "ape"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ape", "Bukiyip"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Buksa", "tkb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tkb", "Buksa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bukusu", "bxk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bxk", "Bukusu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bukwen", "buz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("buz", "Bukwen"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bulgarian", "bul"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bul", "Bulgarian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bulgarian Sign Language", "bqn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bqn", "Bulgarian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bulgebi", "bmp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bmp", "Bulgebi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Buli (Ghana)", "bwu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bwu", "Buli (Ghana)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Buli (Indonesia)", "bzq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bzq", "Buli (Indonesia)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bullom So", "buy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("buy", "Bullom So"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bulo Stieng", "sti"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sti", "Bulo Stieng"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bulu (Cameroon)", "bum"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bum", "Bulu (Cameroon)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bulu (Papua New Guinea)", "bjl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bjl", "Bulu (Papua New Guinea)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bum", "bmv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bmv", "Bum"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bumaji", "byp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("byp", "Bumaji"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bumbita Arapesh", "aon"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aon", "Bumbita Arapesh"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bumthangkha", "kjz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kjz", "Bumthangkha"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bun", "buv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("buv", "Bun"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Buna", "bvn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bvn", "Buna"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bunaba", "bck"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bck", "Bunaba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bunak", "bfn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bfn", "Bunak"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bunama", "bdd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bdd", "Bunama"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bundeli", "bns"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bns", "Bundeli"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bung", "bqd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bqd", "Bung"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bungain", "but"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("but", "Bungain"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bungku", "bkz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bkz", "Bungku"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bungu", "wun"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wun", "Bungu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bunoge Dogon", "dgb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dgb", "Bunoge Dogon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bunun", "bnn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bnn", "Bunun"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Buol", "blf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("blf", "Buol"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bura-Pabir", "bwr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bwr", "Bura-Pabir"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Burak", "bys"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bys", "Burak"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Buraka", "bkg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bkg", "Buraka"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Burarra", "bvr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bvr", "Burarra"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Burate", "bti"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bti", "Burate"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Burduna", "bxn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bxn", "Burduna"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bure", "bvh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bvh", "Bure"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Buriat", "bua"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bua", "Buriat"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Burji", "bji"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bji", "Burji"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Burmbar", "vrt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vrt", "Burmbar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Burmese", "mya"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mya", "Burmese"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Burmeso", "bzu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bzu", "Burmeso"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Buru (Indonesia)", "mhs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mhs", "Buru (Indonesia)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Buru (Nigeria)", "bqw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bqw", "Buru (Nigeria)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Burui", "bry"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bry", "Burui"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Burum-Mindik", "bmu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bmu", "Burum-Mindik"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Burumakok", "aip"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aip", "Burumakok"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Burun", "bdi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bdi", "Burun"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Burunge", "bds"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bds", "Burunge"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Burushaski", "bsk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bsk", "Burushaski"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Burusu", "bqr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bqr", "Burusu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Buruwai", "asi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("asi", "Buruwai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Busa", "bqp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bqp", "Busa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Busam", "bxs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bxs", "Busam"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Busami", "bsm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bsm", "Busami"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Busang Kayan", "bfg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bfg", "Busang Kayan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bushi", "buc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("buc", "Bushi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bushoong", "buf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("buf", "Bushoong"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Businenge Tongo", "djk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("djk", "Businenge Tongo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Buso", "bso"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bso", "Buso"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Busoa", "bup"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bup", "Busoa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bussa", "dox"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dox", "Bussa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Busuu", "bju"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bju", "Busuu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Butbut Kalinga", "kyb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kyb", "Butbut Kalinga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Butmas-Tur", "bnr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bnr", "Butmas-Tur"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Butuanon", "btw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("btw", "Butuanon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Buwal", "bhs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bhs", "Buwal"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Buxinhua", "bgk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bgk", "Buxinhua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Buya", "byy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("byy", "Buya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Buyu", "byi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("byi", "Buyu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Buyuan Jinuo", "jiy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jiy", "Buyuan Jinuo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bwa", "bww"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bww", "Bwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bwaidoka", "bwd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bwd", "Bwaidoka"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bwanabwana", "tte"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tte", "Bwanabwana"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bwatoo", "bwa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bwa", "Bwatoo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bwe Karen", "bwe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bwe", "Bwe Karen"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bwela", "bwl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bwl", "Bwela"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bwile", "bwc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bwc", "Bwile"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bwisi", "bwz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bwz", "Bwisi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Byangsi", "bee"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bee", "Byangsi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Byep", "mkk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mkk", "Byep"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Bädi Kanum", "khd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("khd", "Bädi Kanum"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("C'lela", "dri"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dri", "C'lela"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Caac", "msq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("msq", "Caac"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cabiyarí", "cbb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cbb", "Cabiyarí"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cabécar", "cjp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cjp", "Cabécar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cacaloxtepec Mixtec", "miu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("miu", "Cacaloxtepec Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cacaopera", "ccr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ccr", "Cacaopera"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cacgia Roglai", "roc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("roc", "Cacgia Roglai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cacua", "cbv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cbv", "Cacua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Caddo", "cad"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cad", "Caddo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cafundo Creole", "ccd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ccd", "Cafundo Creole"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cagua", "cbh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cbh", "Cagua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cahuarano", "cah"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cah", "Cahuarano"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cahuilla", "chl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("chl", "Cahuilla"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cajamarca Quechua", "qvc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("qvc", "Cajamarca Quechua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cajatambo North Lima Quechua", "qvl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("qvl", "Cajatambo North Lima Quechua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cajonos Zapotec", "zad"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zad", "Cajonos Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cajun French", "frc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("frc", "Cajun French"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Caka", "ckx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ckx", "Caka"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cakchiquel", "cak"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cak", "Cakchiquel"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cakchiquel-Quiché Mixed Language", "ckz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ckz", "Cakchiquel-Quiché Mixed Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cakfem-Mushere", "cky"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cky", "Cakfem-Mushere"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Calamian Tagbanwa", "tbk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tbk", "Calamian Tagbanwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Calderón Highland Quichua", "qud"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("qud", "Calderón Highland Quichua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Callawalla", "caw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("caw", "Callawalla"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Caluyanun", "clu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("clu", "Caluyanun"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Caló", "rmq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rmq", "Caló"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cameroon Mambila", "mcu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mcu", "Cameroon Mambila"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cameroon Pidgin", "wes"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wes", "Cameroon Pidgin"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Camling", "rab"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rab", "Camling"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Campalagian", "cml"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cml", "Campalagian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Campidanese Sardinian", "sro"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sro", "Campidanese Sardinian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Camsá", "kbh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kbh", "Camsá"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Camtho", "cmt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cmt", "Camtho"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Camunic", "xcc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xcc", "Camunic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Candoshi-Shapra", "cbu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cbu", "Candoshi-Shapra"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Canela", "ram"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ram", "Canela"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Canichana", "caz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("caz", "Canichana"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cao Lan", "mlc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mlc", "Cao Lan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cao Miao", "cov"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cov", "Cao Miao"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Capanahua", "kaq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kaq", "Capanahua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Capiznon", "cps"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cps", "Capiznon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cappadocian Greek", "cpg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cpg", "Cappadocian Greek"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Caquinte", "cot"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cot", "Caquinte"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Car Nicobarese", "caq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("caq", "Car Nicobarese"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cara", "cfd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cfd", "Cara"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Carabayo", "cby"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cby", "Carabayo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Caramanta", "crf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("crf", "Caramanta"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Carapana", "cbc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cbc", "Carapana"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Carian", "xcr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xcr", "Carian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Caribbean Hindustani", "hns"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hns", "Caribbean Hindustani"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Caribbean Javanese", "jvn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jvn", "Caribbean Javanese"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Carijona", "cbd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cbd", "Carijona"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Carolina Algonquian", "crr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("crr", "Carolina Algonquian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Carolinian", "cal"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cal", "Carolinian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Carpathian Romani", "rmc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rmc", "Carpathian Romani"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Carrier", "crx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("crx", "Carrier"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cashinahua", "cbs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cbs", "Cashinahua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Casiguran Dumagat Agta", "dgc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dgc", "Casiguran Dumagat Agta"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Castilian", "spa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("spa", "Castilian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Casuarina Coast Asmat", "asc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("asc", "Casuarina Coast Asmat"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Catalan", "cat"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cat", "Catalan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Catalan Sign Language", "csc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("csc", "Catalan Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Catawba", "chc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("chc", "Catawba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cauca", "cca"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cca", "Cauca"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cavineña", "cav"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cav", "Cavineña"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cayubaba", "cyb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cyb", "Cayubaba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cayuga", "cay"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cay", "Cayuga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cayuse", "xcy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xcy", "Cayuse"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cañar Highland Quichua", "qxr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("qxr", "Cañar Highland Quichua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cebaara Senoufo", "sef"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sef", "Cebaara Senoufo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cebuano", "ceb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ceb", "Cebuano"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Celtiberian", "xce"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xce", "Celtiberian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cemuhî", "cam"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cam", "Cemuhî"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cen", "cen"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cen", "Cen"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Central Asmat", "cns"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cns", "Central Asmat"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Central Atlas Tamazight", "tzm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tzm", "Central Atlas Tamazight"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Central Awyu", "awu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("awu", "Central Awyu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Central Aymara", "ayr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ayr", "Central Aymara"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Central Bai", "bca"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bca", "Central Bai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Central Berawan", "zbc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zbc", "Central Berawan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Central Bicolano", "bcl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bcl", "Central Bicolano"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Central Bontok", "lbk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lbk", "Central Bontok"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Central Cagayan Agta", "agt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("agt", "Central Cagayan Agta"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Central Dusun", "dtp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dtp", "Central Dusun"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Central Grebo", "grv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("grv", "Central Grebo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Central Hongshuihe Zhuang", "zch"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zch", "Central Hongshuihe Zhuang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Central Huasteca Nahuatl", "nch"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nch", "Central Huasteca Nahuatl"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Central Huishui Hmong", "hmc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hmc", "Central Huishui Hmong"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Central Huishui Miao", "hmc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hmc", "Central Huishui Miao"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Central Kanuri", "knc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("knc", "Central Kanuri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Central Khmer", "khm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("khm", "Central Khmer"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Central Kurdish", "ckb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ckb", "Central Kurdish"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Central Maewo", "mwo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mwo", "Central Maewo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Central Malay", "pse"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pse", "Central Malay"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Central Masela", "mxz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mxz", "Central Masela"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Central Mashan Hmong", "hmm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hmm", "Central Mashan Hmong"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Central Mashan Miao", "hmm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hmm", "Central Mashan Miao"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Central Mazahua", "maz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("maz", "Central Mazahua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Central Melanau", "mel"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mel", "Central Melanau"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Central Mnong", "cmo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cmo", "Central Mnong"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Central Nahuatl", "nhn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nhn", "Central Nahuatl"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Central Nicobarese", "ncb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ncb", "Central Nicobarese"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Central Ojibwa", "ojc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ojc", "Central Ojibwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Central Okinawan", "ryu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ryu", "Central Okinawan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Central Palawano", "plc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("plc", "Central Palawano"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Central Pame", "pbs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pbs", "Central Pame"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Central Pashto", "pst"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pst", "Central Pashto"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Central Pomo", "poo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("poo", "Central Pomo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Central Puebla Nahuatl", "ncx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ncx", "Central Puebla Nahuatl"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Central Sama", "sml"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sml", "Central Sama"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Central Siberian Yupik", "ess"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ess", "Central Siberian Yupik"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Central Sierra Miwok", "csm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("csm", "Central Sierra Miwok"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Central Subanen", "syb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("syb", "Central Subanen"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Central Tagbanwa", "tgt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tgt", "Central Tagbanwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Central Tarahumara", "tar"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tar", "Central Tarahumara"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Central Tunebo", "tuf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tuf", "Central Tunebo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Central Yupik", "esu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("esu", "Central Yupik"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Central-Eastern Niger Fulfulde", "fuq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("fuq", "Central-Eastern Niger Fulfulde"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Centúúm", "cet"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cet", "Centúúm"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cerma", "cme"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cme", "Cerma"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chachapoyas Quechua", "quk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("quk", "Chachapoyas Quechua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chachi", "cbi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cbi", "Chachi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chadian Arabic", "shu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("shu", "Chadian Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chadian Sign Language", "cds"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cds", "Chadian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chadong", "cdy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cdy", "Chadong"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chagatai", "chg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("chg", "Chagatai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chaima", "ciy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ciy", "Chaima"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chak", "ckh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ckh", "Chak"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chakali", "cli"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cli", "Chakali"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chakma", "ccp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ccp", "Chakma"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chala", "cll"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cll", "Chala"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chaldean Neo-Aramaic", "cld"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cld", "Chaldean Neo-Aramaic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chalikha", "tgf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tgf", "Chalikha"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chamacoco", "ceg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ceg", "Chamacoco"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chamalal", "cji"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cji", "Chamalal"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chamari", "cdg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cdg", "Chamari"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chambeali", "cdh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cdh", "Chambeali"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chambri", "can"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("can", "Chambri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chamicuro", "ccc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ccc", "Chamicuro"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chamorro", "cha"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cha", "Chamorro"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chang Naga", "nbc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nbc", "Chang Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Changriwa", "cga"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cga", "Changriwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Changthang", "cna"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cna", "Changthang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chantyal", "chx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("chx", "Chantyal"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chané", "caj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("caj", "Chané"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chara", "cra"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cra", "Chara"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chaudangsi", "cdn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cdn", "Chaudangsi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chaungtha", "ccq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ccq", "Chaungtha"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chaura", "crv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("crv", "Chaura"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chavacano", "cbk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cbk", "Chavacano"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chayahuita", "cbt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cbt", "Chayahuita"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chayuco Mixtec", "mih"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mih", "Chayuco Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chazumba Mixtec", "xtb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xtb", "Chazumba Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Che", "ruk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ruk", "Che"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chechen", "che"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("che", "Chechen"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cheke Holo", "mrn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mrn", "Cheke Holo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chemakum", "xch"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xch", "Chemakum"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chenapian", "cjn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cjn", "Chenapian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chenchu", "cde"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cde", "Chenchu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chenoua", "cnu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cnu", "Chenoua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chepang", "cdm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cdm", "Chepang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chepya", "ycp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ycp", "Chepya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cheq Wong", "cwg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cwg", "Cheq Wong"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cherepon", "cpn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cpn", "Cherepon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cherokee", "chr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("chr", "Cherokee"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chesu", "ych"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ych", "Chesu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chetco", "ctc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ctc", "Chetco"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chewa", "nya"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nya", "Chewa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chewong", "cwg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cwg", "Chewong"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cheyenne", "chy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("chy", "Cheyenne"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chhattisgarhi", "hne"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hne", "Chhattisgarhi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chhintange", "ctn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ctn", "Chhintange"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chhulung", "cur"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cur", "Chhulung"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chiangmai Sign Language", "csd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("csd", "Chiangmai Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chiapanec", "cip"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cip", "Chiapanec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chibcha", "chb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("chb", "Chibcha"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chicahuaxtla Triqui", "trs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("trs", "Chicahuaxtla Triqui"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chichewa", "nya"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nya", "Chichewa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chichicapan Zapotec", "zpv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zpv", "Chichicapan Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chickasaw", "cic"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cic", "Chickasaw"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chicomuceltec", "cob"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cob", "Chicomuceltec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chidigo", "dig"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dig", "Chidigo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chiduruma", "dug"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dug", "Chiduruma"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chiga", "cgg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cgg", "Chiga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chigmecatitlán Mixtec", "mii"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mii", "Chigmecatitlán Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chilcotin", "clc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("clc", "Chilcotin"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chilean Quechua", "cqu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cqu", "Chilean Quechua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chilean Sign Language", "csg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("csg", "Chilean Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chilisso", "clh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("clh", "Chilisso"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chiltepec Chinantec", "csa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("csa", "Chiltepec Chinantec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chimakum", "xch"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xch", "Chimakum"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chimalapa Zoque", "zoh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zoh", "Chimalapa Zoque"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chimariko", "cid"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cid", "Chimariko"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chimborazo Highland Quichua", "qug"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("qug", "Chimborazo Highland Quichua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chimila", "cbg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cbg", "Chimila"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("China Buriat", "bxu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bxu", "China Buriat"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chinali", "cih"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cih", "Chinali"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chinbon Chin", "cnb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cnb", "Chinbon Chin"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chincha Quechua", "qxc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("qxc", "Chincha Quechua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chinese", "zho"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zho", "Chinese"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chinese Pidgin English", "cpi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cpi", "Chinese Pidgin English"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chinese Sign Language", "csl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("csl", "Chinese Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chinook", "chh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("chh", "Chinook"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chinook jargon", "chn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("chn", "Chinook jargon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chipaya", "cap"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cap", "Chipaya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chipewyan", "chp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("chp", "Chipewyan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chipiajes", "cbe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cbe", "Chipiajes"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chippewa", "ciw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ciw", "Chippewa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chiquihuitlán Mazatec", "maq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("maq", "Chiquihuitlán Mazatec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chiquitano", "cax"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cax", "Chiquitano"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chiquián Ancash Quechua", "qxa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("qxa", "Chiquián Ancash Quechua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chiripá", "nhd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nhd", "Chiripá"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chiru", "cdf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cdf", "Chiru"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chitimacha", "ctm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ctm", "Chitimacha"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chitkuli Kinnauri", "cik"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cik", "Chitkuli Kinnauri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chittagonian", "ctg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ctg", "Chittagonian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chitwania Tharu", "the"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("the", "Chitwania Tharu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Choapan Zapotec", "zpc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zpc", "Choapan Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chocangacakha", "cgk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cgk", "Chocangacakha"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chochotec", "coz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("coz", "Chochotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Choctaw", "cho"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cho", "Choctaw"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chodri", "cdi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cdi", "Chodri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chokri Naga", "nri"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nri", "Chokri Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chokwe", "cjk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cjk", "Chokwe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chol", "ctu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ctu", "Chol"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cholón", "cht"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cht", "Cholón"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chong", "cog"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cog", "Chong"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Choni", "cda"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cda", "Choni"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chopi", "cce"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cce", "Chopi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chorasmian", "xco"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xco", "Chorasmian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chortí", "caa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("caa", "Chortí"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chothe Naga", "nct"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nct", "Chothe Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chrau", "crw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("crw", "Chrau"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chru", "cje"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cje", "Chru"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chuang", "zha"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zha", "Chuang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chuanqiandian Cluster Miao", "cqd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cqd", "Chuanqiandian Cluster Miao"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chuave", "cjv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cjv", "Chuave"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chug", "cvg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cvg", "Chug"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chuj", "cac"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cac", "Chuj"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chuka", "cuh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cuh", "Chuka"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chukot", "ckt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ckt", "Chukot"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chukwa", "cuw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cuw", "Chukwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chulym", "clw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("clw", "Chulym"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chumburung", "ncu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ncu", "Chumburung"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Churahi", "cdj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cdj", "Churahi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Church Slavic", "chu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("chu", "Church Slavic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Church Slavonic", "chu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("chu", "Church Slavonic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chut", "scb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("scb", "Chut"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chuukese", "chk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("chk", "Chuukese"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chuvantsy", "xcv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xcv", "Chuvantsy"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chuvash", "chv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("chv", "Chuvash"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chuwabu", "chw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("chw", "Chuwabu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Chácobo", "cao"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cao", "Chácobo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ci Gbe", "cib"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cib", "Ci Gbe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cia-Cia", "cia"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cia", "Cia-Cia"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cibak", "ckl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ckl", "Cibak"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cicipu", "awc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("awc", "Cicipu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cimbrian", "cim"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cim", "Cimbrian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cineni", "cie"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cie", "Cineni"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cinta Larga", "cin"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cin", "Cinta Larga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cisalpine Gaulish", "xcg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xcg", "Cisalpine Gaulish"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cishingini", "asg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("asg", "Cishingini"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Citak", "txt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("txt", "Citak"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ciwogai", "tgd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tgd", "Ciwogai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Clallam", "clm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("clm", "Clallam"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Classical Armenian", "xcl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xcl", "Classical Armenian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Classical Mandaic", "myz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("myz", "Classical Mandaic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Classical Mongolian", "cmg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cmg", "Classical Mongolian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Classical Nahuatl", "nci"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nci", "Classical Nahuatl"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Classical Nepal Bhasa", "nwc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nwc", "Classical Nepal Bhasa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Classical Newari", "nwc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nwc", "Classical Newari"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Classical Quechua", "qwc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("qwc", "Classical Quechua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Classical Syriac", "syc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("syc", "Classical Syriac"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Classical Tibetan", "xct"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xct", "Classical Tibetan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Coahuilteco", "xcw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xcw", "Coahuilteco"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Coast Miwok", "csi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("csi", "Coast Miwok"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Coastal Kadazan", "kzj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kzj", "Coastal Kadazan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Coastal Konjo", "kjc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kjc", "Coastal Konjo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Coatecas Altas Zapotec", "zca"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zca", "Coatecas Altas Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Coatepec Nahuatl", "naz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("naz", "Coatepec Nahuatl"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Coatlán Mixe", "mco"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mco", "Coatlán Mixe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Coatlán Zapotec", "zps"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zps", "Coatlán Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Coatzospan Mixtec", "miz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("miz", "Coatzospan Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cochimi", "coj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("coj", "Cochimi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cocopa", "coc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("coc", "Cocopa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cocos Islands Malay", "coa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("coa", "Cocos Islands Malay"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Coeur d'Alene", "crd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("crd", "Coeur d'Alene"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cofán", "con"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("con", "Cofán"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cogui", "kog"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kog", "Cogui"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Col", "liw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("liw", "Col"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Colombian Sign Language", "csn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("csn", "Colombian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Colonia Tovar German", "gct"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gct", "Colonia Tovar German"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Colorado", "cof"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cof", "Colorado"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Comaltepec Chinantec", "cco"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cco", "Comaltepec Chinantec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Comanche", "com"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("com", "Comanche"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Comecrudo", "xcm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xcm", "Comecrudo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Como Karim", "cfg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cfg", "Como Karim"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Comox", "coo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("coo", "Comox"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Con", "cno"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cno", "Con"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Congo Swahili", "swc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("swc", "Congo Swahili"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cook Islands Maori", "rar"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rar", "Cook Islands Maori"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Coos", "csz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("csz", "Coos"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Copainalá Zoque", "zoc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zoc", "Copainalá Zoque"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Copala Triqui", "trc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("trc", "Copala Triqui"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Coptic", "cop"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cop", "Coptic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Coquille", "coq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("coq", "Coquille"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cori", "cry"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cry", "Cori"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cornish", "cor"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cor", "Cornish"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Corongo Ancash Quechua", "qwa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("qwa", "Corongo Ancash Quechua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Corsican", "cos"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cos", "Corsican"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Costa Rican Sign Language", "csr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("csr", "Costa Rican Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cotabato Manobo", "mta"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mta", "Cotabato Manobo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cotoname", "xcn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xcn", "Cotoname"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cowlitz", "cow"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cow", "Cowlitz"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Coxima", "kox"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kox", "Coxima"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Coyaima", "coy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("coy", "Coyaima"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Coyotepec Popoloca", "pbf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pbf", "Coyotepec Popoloca"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Coyutla Totonac", "toc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("toc", "Coyutla Totonac"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cree", "cre"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cre", "Cree"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Creek", "mus"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mus", "Creek"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Crimean Tatar", "crh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("crh", "Crimean Tatar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Crimean Turkish", "crh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("crh", "Crimean Turkish"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Croatia Sign Language", "csq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("csq", "Croatia Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Croatian", "hrv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hrv", "Croatian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cross River Mbembe", "mfn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mfn", "Cross River Mbembe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Crow", "cro"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cro", "Crow"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cruzeño", "crz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("crz", "Cruzeño"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cua", "cua"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cua", "Cua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cuba Sign Language", "csf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("csf", "Cuba Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cubeo", "cub"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cub", "Cubeo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cuiba", "cui"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cui", "Cuiba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Culina", "cul"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cul", "Culina"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cumanagoto", "cuo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cuo", "Cumanagoto"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cumbric", "xcb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xcb", "Cumbric"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cumeral", "cum"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cum", "Cumeral"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cun", "cuq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cuq", "Cun"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cuneiform Luwian", "xlu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xlu", "Cuneiform Luwian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cung", "cug"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cug", "Cung"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cupeño", "cup"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cup", "Cupeño"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Curonian", "xcu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xcu", "Curonian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Curripaco", "kpc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kpc", "Curripaco"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cusco Quechua", "quz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("quz", "Cusco Quechua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cutchi-Swahili", "ccl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ccl", "Cutchi-Swahili"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cuvok", "cuv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cuv", "Cuvok"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cuyamecalco Mixtec", "xtu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xtu", "Cuyamecalco Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cuyonon", "cyo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cyo", "Cuyonon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cwi Bwamu", "bwy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bwy", "Cwi Bwamu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Cypriot Arabic", "acy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("acy", "Cypriot Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Czech", "ces"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ces", "Czech"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Czech Sign Language", "cse"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cse", "Czech Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Côông", "cnc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cnc", "Côông"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Da'a Kaili", "kzf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kzf", "Da'a Kaili"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Da-Hua Miao", "hmd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hmd", "Da-Hua Miao"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Daai Chin", "dao"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dao", "Daai Chin"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Daantanai'", "lni"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lni", "Daantanai'"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Daasanach", "dsh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dsh", "Daasanach"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Daba", "dbq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dbq", "Daba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dabarre", "dbr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dbr", "Dabarre"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dabe", "dbe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dbe", "Dabe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dacian", "xdc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xdc", "Dacian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dadibi", "mps"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mps", "Dadibi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dadiya", "dbd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dbd", "Dadiya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Daga", "dgz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dgz", "Daga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dagaari Dioula", "dgd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dgd", "Dagaari Dioula"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dagba", "dgk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dgk", "Dagba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dagbani", "dag"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dag", "Dagbani"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dagik", "dec"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dec", "Dagik"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dagoman", "dgn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dgn", "Dagoman"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dahalo", "dal"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dal", "Dahalo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Daho-Doo", "das"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("das", "Daho-Doo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dai", "dij"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dij", "Dai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dai Zhuang", "zhd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zhd", "Dai Zhuang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dair", "drb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("drb", "Dair"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dakaka", "bpa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bpa", "Dakaka"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dakka", "dkk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dkk", "Dakka"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dakota", "dak"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dak", "Dakota"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dakpakha", "dka"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dka", "Dakpakha"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dalmatian", "dlm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dlm", "Dalmatian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Daloa Bété", "bev"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bev", "Daloa Bété"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dama", "dmm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dmm", "Dama"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Damakawa", "dam"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dam", "Damakawa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Damal", "uhn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("uhn", "Damal"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dambi", "dac"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dac", "Dambi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dameli", "dml"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dml", "Dameli"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dampelas", "dms"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dms", "Dampelas"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dan", "daf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("daf", "Dan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Danaru", "dnr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dnr", "Danaru"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Danau", "dnu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dnu", "Danau"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dandami Maria", "daq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("daq", "Dandami Maria"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dangaléat", "daa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("daa", "Dangaléat"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dangaura Tharu", "thl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("thl", "Dangaura Tharu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Danish", "dan"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dan", "Danish"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Danish Sign Language", "dsl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dsl", "Danish Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dano", "aso"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aso", "Dano"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dao", "daz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("daz", "Dao"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Daonda", "dnd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dnd", "Daonda"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dar Daju Daju", "djc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("djc", "Dar Daju Daju"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dar Fur Daju", "daj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("daj", "Dar Fur Daju"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dar Sila Daju", "dau"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dau", "Dar Sila Daju"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Darai", "dry"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dry", "Darai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Darang Deng", "mhu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mhu", "Darang Deng"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dargwa", "dar"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dar", "Dargwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dari", "prs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("prs", "Dari"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Darling", "drl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("drl", "Darling"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Darlong", "dln"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dln", "Darlong"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Darmiya", "drd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("drd", "Darmiya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Daro-Matu Melanau", "dro"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dro", "Daro-Matu Melanau"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dass", "dot"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dot", "Dass"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Datooga", "tcc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tcc", "Datooga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Daur", "dta"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dta", "Daur"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Davawenyo", "daw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("daw", "Davawenyo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dawawa", "dww"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dww", "Dawawa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dawera-Daweloor", "ddw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ddw", "Dawera-Daweloor"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dawida", "dav"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dav", "Dawida"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dawro", "dwr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dwr", "Dawro"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Day", "dai"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dai", "Day"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dayi", "dax"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dax", "Dayi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Daza", "dzd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dzd", "Daza"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dazaga", "dzg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dzg", "Dazaga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Deccan", "dcc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dcc", "Deccan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dedua", "ded"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ded", "Dedua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Defaka", "afn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("afn", "Defaka"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Defi Gbe", "gbh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gbh", "Defi Gbe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Deg", "mzw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mzw", "Deg"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Degaru", "dgu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dgu", "Degaru"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Degema", "deg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("deg", "Degema"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Degenan", "dge"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dge", "Degenan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Degexit'an", "ing"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ing", "Degexit'an"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dehu", "dhv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dhv", "Dehu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dehwari", "deh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("deh", "Dehwari"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dek", "dek"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dek", "Dek"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dela-Oenale", "row"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("row", "Dela-Oenale"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Delaware", "del"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("del", "Delaware"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Delo", "ntr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ntr", "Delo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dem", "dem"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dem", "Dem"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dema", "dmx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dmx", "Dema"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Demisa", "dei"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dei", "Demisa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Demta", "dmy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dmy", "Demta"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dendi (Benin)", "ddn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ddn", "Dendi (Benin)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dendi (Central African Republic)", "deq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("deq", "Dendi (Central African Republic)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dene Suline", "chp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("chp", "Dene Suline"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dengese", "dez"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dez", "Dengese"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dengka", "dnk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dnk", "Dengka"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Deno", "dbb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dbb", "Deno"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Denya", "anv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("anv", "Denya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dení", "dny"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dny", "Dení"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Deori", "der"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("der", "Deori"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dera (Indonesia)", "kbv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kbv", "Dera (Indonesia)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dera (Nigeria)", "kna"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kna", "Dera (Nigeria)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Desano", "des"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("des", "Desano"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Desiya", "dso"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dso", "Desiya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dewoin", "dee"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dee", "Dewoin"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dezfuli", "def"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("def", "Dezfuli"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dghwede", "dgh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dgh", "Dghwede"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dhaiso", "dhs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dhs", "Dhaiso"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dhalandji", "dhl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dhl", "Dhalandji"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dhangu", "dhg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dhg", "Dhangu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dhanki", "dhn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dhn", "Dhanki"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dhanwar (India)", "dha"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dha", "Dhanwar (India)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dhanwar (Nepal)", "dhw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dhw", "Dhanwar (Nepal)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dhao", "nfa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nfa", "Dhao"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dhargari", "dhr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dhr", "Dhargari"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dhatki", "mki"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mki", "Dhatki"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dhimal", "dhi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dhi", "Dhimal"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dhivehi", "div"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("div", "Dhivehi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dhodia", "dho"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dho", "Dhodia"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dhofari Arabic", "adf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("adf", "Dhofari Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dholuo", "luo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("luo", "Dholuo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dhundari", "dhd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dhd", "Dhundari"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dhurga", "dhu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dhu", "Dhurga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dhuwal", "duj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("duj", "Dhuwal"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dia", "dia"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dia", "Dia"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dibabawon Manobo", "mbd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mbd", "Dibabawon Manobo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dibiyaso", "dby"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dby", "Dibiyaso"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dibo", "dio"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dio", "Dibo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dibole", "bvx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bvx", "Dibole"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dicamay Agta", "duy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("duy", "Dicamay Agta"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Didinga", "did"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("did", "Didinga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dido", "ddo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ddo", "Dido"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Diebroud", "tbp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tbp", "Diebroud"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dieri", "dif"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dif", "Dieri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Digaro-Mishmi", "mhu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mhu", "Digaro-Mishmi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Digo", "dig"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dig", "Digo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dii", "dur"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dur", "Dii"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dijim-Bwilim", "cfa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cfa", "Dijim-Bwilim"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dilling", "dil"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dil", "Dilling"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dima", "jma"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jma", "Dima"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dimasa", "dis"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dis", "Dimasa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dimbong", "dii"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dii", "Dimbong"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dime", "dim"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dim", "Dime"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dimili", "zza"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zza", "Dimili"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dimir", "dmc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dmc", "Dimir"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dimli (individual language)", "diq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("diq", "Dimli (individual language)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dimli (macrolanguage)", "zza"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zza", "Dimli (macrolanguage)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dineor", "mrx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mrx", "Dineor"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ding", "diz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("diz", "Ding"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dinka", "din"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("din", "Dinka"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dirari", "dit"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dit", "Dirari"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dirasha", "gdl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gdl", "Dirasha"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Diri", "dwa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dwa", "Diri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Diriku", "diu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("diu", "Diriku"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dirim", "dir"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dir", "Dirim"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Disa", "dsi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dsi", "Disa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ditammari", "tbz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tbz", "Ditammari"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Diuwe", "diy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("diy", "Diuwe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Diuxi-Tilantongo Mixtec", "xtd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xtd", "Diuxi-Tilantongo Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Divehi", "div"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("div", "Divehi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dixon Reef", "dix"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dix", "Dixon Reef"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dizin", "mdx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mdx", "Dizin"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Djambarrpuyngu", "djr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("djr", "Djambarrpuyngu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Djamindjung", "djd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("djd", "Djamindjung"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Djangun", "djf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("djf", "Djangun"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Djauan", "djn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("djn", "Djauan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Djawi", "djw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("djw", "Djawi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Djeebbana", "djj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("djj", "Djeebbana"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Djimini Senoufo", "dyi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dyi", "Djimini Senoufo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Djinang", "dji"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dji", "Djinang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Djinba", "djb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("djb", "Djinba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Djingili", "jig"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jig", "Djingili"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Djiwarli", "djl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("djl", "Djiwarli"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dobel", "kvo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kvo", "Dobel"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dobu", "dob"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dob", "Dobu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Doe", "doe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("doe", "Doe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Doga", "dgg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dgg", "Doga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Doghoro", "dgx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dgx", "Doghoro"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dogoso", "dgs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dgs", "Dogoso"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dogosé", "dos"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dos", "Dogosé"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dogri (individual language)", "dgo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dgo", "Dogri (individual language)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dogri (macrolanguage)", "doi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("doi", "Dogri (macrolanguage)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dogrib", "dgr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dgr", "Dogrib"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dogul Dom Dogon", "dbg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dbg", "Dogul Dom Dogon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Doka", "dbi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dbi", "Doka"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Doko-Uyanga", "uya"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("uya", "Doko-Uyanga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dolgan", "dlg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dlg", "Dolgan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dolpo", "dre"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dre", "Dolpo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dom", "doa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("doa", "Dom"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Domaaki", "dmk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dmk", "Domaaki"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Domari", "rmt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rmt", "Domari"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dombe", "dov"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dov", "Dombe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dominican Sign Language", "doq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("doq", "Dominican Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dompo", "doy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("doy", "Dompo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Domu", "dof"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dof", "Domu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Domung", "dev"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dev", "Domung"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dondo", "dok"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dok", "Dondo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dong", "doh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("doh", "Dong"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dongo", "doo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("doo", "Dongo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dongotono", "ddd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ddd", "Dongotono"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dongshanba Lalo", "yik"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yik", "Dongshanba Lalo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dongxiang", "sce"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sce", "Dongxiang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Donno So Dogon", "dds"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dds", "Donno So Dogon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Doondo", "dde"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dde", "Doondo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dori'o", "dor"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dor", "Dori'o"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dorig", "wwo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wwo", "Dorig"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Doromu-Koki", "kqc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kqc", "Doromu-Koki"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dororo", "drr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("drr", "Dororo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dorze", "doz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("doz", "Dorze"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Doso", "dol"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dol", "Doso"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Doutai", "tds"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tds", "Doutai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Doyayo", "dow"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dow", "Doyayo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Drents", "drt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("drt", "Drents"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Drung", "duu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("duu", "Drung"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Duala", "dua"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dua", "Duala"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Duano", "dup"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dup", "Duano"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Duau", "dva"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dva", "Duau"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dubli", "dub"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dub", "Dubli"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dubu", "dmu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dmu", "Dubu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Duduela", "duk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("duk", "Duduela"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dugun", "ndu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ndu", "Dugun"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Duguri", "dbm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dbm", "Duguri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dugwor", "dme"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dme", "Dugwor"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Duhwa", "kbz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kbz", "Duhwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Duke", "nke"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nke", "Duke"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dulbu", "dbo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dbo", "Dulbu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Duli", "duz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("duz", "Duli"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Duma", "dma"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dma", "Duma"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dumbea", "duf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("duf", "Dumbea"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dumi", "dus"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dus", "Dumi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dumpas", "dmv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dmv", "Dumpas"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dumpu", "wtf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wtf", "Dumpu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dumun", "dui"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dui", "Dumun"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Duna", "duc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("duc", "Duna"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dungan", "dng"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dng", "Dungan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dungmali", "raa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("raa", "Dungmali"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dungra Bhil", "duh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("duh", "Dungra Bhil"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dungu", "dbv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dbv", "Dungu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dupaninan Agta", "duo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("duo", "Dupaninan Agta"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dura", "drq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("drq", "Dura"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Durango Nahuatl", "nln"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nln", "Durango Nahuatl"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Duri", "mvp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mvp", "Duri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Duriankere", "dbn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dbn", "Duriankere"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Duruma", "dug"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dug", "Duruma"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Duruwa", "pci"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pci", "Duruwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dusner", "dsn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dsn", "Dusner"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dusun Deyah", "dun"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dun", "Dusun Deyah"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dusun Malang", "duq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("duq", "Dusun Malang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dusun Witu", "duw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("duw", "Dusun Witu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dutch", "nld"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nld", "Dutch"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dutch Sign Language", "dse"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dse", "Dutch Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dutton World Speedwords", "dws"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dws", "Dutton World Speedwords"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Duungooma", "dux"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dux", "Duungooma"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Duupa", "dae"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dae", "Duupa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Duvle", "duv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("duv", "Duvle"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Duwai", "dbp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dbp", "Duwai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Duwet", "gve"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gve", "Duwet"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dwang", "nnu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nnu", "Dwang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dyaabugay", "dyy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dyy", "Dyaabugay"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dyaberdyaber", "dyb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dyb", "Dyaberdyaber"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dyan", "dya"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dya", "Dyan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dyangadi", "dyn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dyn", "Dyangadi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dyirbal", "dbl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dbl", "Dyirbal"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dyugun", "dyd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dyd", "Dyugun"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dyula", "dyu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dyu", "Dyula"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dza", "jen"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jen", "Dza"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dzalakha", "dzl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dzl", "Dzalakha"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dzando", "dzn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dzn", "Dzando"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dzao Min", "bpn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bpn", "Dzao Min"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dzodinka", "add"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("add", "Dzodinka"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dzongkha", "dzo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dzo", "Dzongkha"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dzùùngoo", "dnn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dnn", "Dzùùngoo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Dâw", "kwa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kwa", "Dâw"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("E", "eee"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("eee", "E"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("E'ma Buyang", "yzg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yzg", "E'ma Buyang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("E'ñapa Woromaipu", "pbh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pbh", "E'ñapa Woromaipu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Early Tripuri", "xtr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xtr", "Early Tripuri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("East Ambae", "omb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("omb", "East Ambae"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("East Berawan", "zbe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zbe", "East Berawan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("East Damar", "dmr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dmr", "East Damar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("East Futuna", "fud"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("fud", "East Futuna"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("East Kewa", "kjs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kjs", "East Kewa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("East Limba", "lma"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lma", "East Limba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("East Makian", "mky"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mky", "East Makian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("East Masela", "vme"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vme", "East Masela"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("East Nyala", "nle"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nle", "East Nyala"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("East Tarangan", "tre"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tre", "East Tarangan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("East Yugur", "yuy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yuy", "East Yugur"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Eastern Abnaki", "aaq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aaq", "Eastern Abnaki"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Eastern Acipa", "acp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("acp", "Eastern Acipa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Eastern Apurímac Quechua", "qve"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("qve", "Eastern Apurímac Quechua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Eastern Arrernte", "aer"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aer", "Eastern Arrernte"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Eastern Balochi", "bgp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bgp", "Eastern Balochi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Eastern Bolivian Guaraní", "gui"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gui", "Eastern Bolivian Guaraní"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Eastern Bontok", "ebk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ebk", "Eastern Bontok"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Eastern Bru", "bru"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bru", "Eastern Bru"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Eastern Canadian Inuktitut", "ike"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ike", "Eastern Canadian Inuktitut"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Eastern Cham", "cjm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cjm", "Eastern Cham"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Eastern Egyptian Bedawi Arabic", "avl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("avl", "Eastern Egyptian Bedawi Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Eastern Frisian", "frs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("frs", "Eastern Frisian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Eastern Gorkha Tamang", "tge"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tge", "Eastern Gorkha Tamang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Eastern Gurung", "ggn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ggn", "Eastern Gurung"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Eastern Highland Chatino", "cly"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cly", "Eastern Highland Chatino"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Eastern Highland Otomi", "otm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("otm", "Eastern Highland Otomi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Eastern Hongshuihe Zhuang", "zeh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zeh", "Eastern Hongshuihe Zhuang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Eastern Huasteca Nahuatl", "nhe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nhe", "Eastern Huasteca Nahuatl"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Eastern Huishui Hmong", "hme"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hme", "Eastern Huishui Hmong"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Eastern Huishui Miao", "hme"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hme", "Eastern Huishui Miao"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Eastern Karaboro", "xrb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xrb", "Eastern Karaboro"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Eastern Katu", "ktv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ktv", "Eastern Katu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Eastern Kayah", "eky"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("eky", "Eastern Kayah"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Eastern Keres", "kee"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kee", "Eastern Keres"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Eastern Krahn", "kqo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kqo", "Eastern Krahn"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Eastern Lalu", "yit"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yit", "Eastern Lalu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Eastern Lawa", "lwl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lwl", "Eastern Lawa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Eastern Magar", "mgp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mgp", "Eastern Magar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Eastern Maninkakan", "emk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("emk", "Eastern Maninkakan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Eastern Mari", "mhr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mhr", "Eastern Mari"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Eastern Maroon Creole", "djk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("djk", "Eastern Maroon Creole"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Eastern Meohang", "emg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("emg", "Eastern Meohang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Eastern Mnong", "mng"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mng", "Eastern Mnong"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Eastern Muria", "emu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("emu", "Eastern Muria"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Eastern Ngad'a", "nea"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nea", "Eastern Ngad'a"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Eastern Nisu", "nos"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nos", "Eastern Nisu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Eastern Ojibwa", "ojg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ojg", "Eastern Ojibwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Eastern Oromo", "hae"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hae", "Eastern Oromo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Eastern Parbate Kham", "kif"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kif", "Eastern Parbate Kham"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Eastern Penan", "pez"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pez", "Eastern Penan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Eastern Pomo", "peb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("peb", "Eastern Pomo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Eastern Qiandong Miao", "hmq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hmq", "Eastern Qiandong Miao"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Eastern Tamang", "taj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("taj", "Eastern Tamang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Eastern Tawbuid", "bnj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bnj", "Eastern Tawbuid"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Eastern Xiangxi Miao", "muq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("muq", "Eastern Xiangxi Miao"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Eastern Xwla Gbe", "gbx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gbx", "Eastern Xwla Gbe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Eastern Yiddish", "ydd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ydd", "Eastern Yiddish"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ebira", "igb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("igb", "Ebira"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Eblan", "xeb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xeb", "Eblan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ebrié", "ebr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ebr", "Ebrié"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ebughu", "ebg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ebg", "Ebughu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ecuadorian Sign Language", "ecs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ecs", "Ecuadorian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ede Cabe", "cbj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cbj", "Ede Cabe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ede Ica", "ica"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ica", "Ede Ica"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ede Idaca", "idd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("idd", "Ede Idaca"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ede Ije", "ijj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ijj", "Ede Ije"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Edera Awyu", "awy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("awy", "Edera Awyu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Edo", "bin"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bin", "Edo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Edolo", "etr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("etr", "Edolo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Edomite", "xdm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xdm", "Edomite"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Edopi", "dbf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dbf", "Edopi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Edwas", "bnv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bnv", "Edwas"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Efai", "efa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("efa", "Efai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Efe", "efe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("efe", "Efe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Efik", "efi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("efi", "Efik"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Efutop", "ofu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ofu", "Efutop"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ega", "ega"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ega", "Ega"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Eggon", "ego"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ego", "Eggon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Egypt Sign Language", "esl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("esl", "Egypt Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Egyptian (Ancient)", "egy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("egy", "Egyptian (Ancient)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Egyptian Arabic", "arz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("arz", "Egyptian Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ehueun", "ehu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ehu", "Ehueun"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Eipomek", "eip"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("eip", "Eipomek"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Eitiep", "eit"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("eit", "Eitiep"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ejagham", "etu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("etu", "Ejagham"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ejamat", "eja"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("eja", "Ejamat"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ekajuk", "eka"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("eka", "Ekajuk"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ekari", "ekg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ekg", "Ekari"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ekegusii", "guz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("guz", "Ekegusii"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Eki", "eki"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("eki", "Eki"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ekit", "eke"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("eke", "Ekit"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ekpeye", "ekp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ekp", "Ekpeye"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("El Alto Zapotec", "zpp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zpp", "El Alto Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("El Hugeirat", "elh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("elh", "El Hugeirat"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("El Molo", "elo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("elo", "El Molo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("El Nayar Cora", "crn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("crn", "El Nayar Cora"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Elamite", "elx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("elx", "Elamite"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Eleme", "elm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("elm", "Eleme"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Elepi", "ele"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ele", "Elepi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Elip", "ekm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ekm", "Elip"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Elkei", "elk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("elk", "Elkei"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Elotepec Zapotec", "zte"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zte", "Elotepec Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Eloyi", "afo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("afo", "Eloyi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Elpaputih", "elp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("elp", "Elpaputih"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Elseng", "mrf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mrf", "Elseng"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Elu", "elu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("elu", "Elu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Elymian", "xly"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xly", "Elymian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Emae", "mmw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mmw", "Emae"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Emai-Iuleha-Ora", "ema"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ema", "Emai-Iuleha-Ora"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Eman", "emn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("emn", "Eman"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Embaloh", "emb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("emb", "Embaloh"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Emberá-Baudó", "bdc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bdc", "Emberá-Baudó"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Emberá-Catío", "cto"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cto", "Emberá-Catío"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Emberá-Chamí", "cmi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cmi", "Emberá-Chamí"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Emberá-Tadó", "tdc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tdc", "Emberá-Tadó"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Embu", "ebu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ebu", "Embu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Emem", "enr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("enr", "Emem"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Emerillon", "eme"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("eme", "Emerillon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Emilian", "egl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("egl", "Emilian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Emok", "emo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("emo", "Emok"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Emplawas", "emw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("emw", "Emplawas"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Emumu", "enr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("enr", "Emumu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("En", "enc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("enc", "En"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Enawené-Nawé", "unk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("unk", "Enawené-Nawé"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ende", "end"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("end", "Ende"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Enga", "enq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("enq", "Enga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Engenni", "enn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("enn", "Engenni"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Enggano", "eno"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("eno", "Enggano"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("English", "eng"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("eng", "English"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Enrekang", "ptt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ptt", "Enrekang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Enu", "enu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("enu", "Enu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Enwan (Akwa Ibom State)", "enw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("enw", "Enwan (Akwa Ibom State)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Enwan (Edu State)", "env"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("env", "Enwan (Edu State)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Enya", "gey"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gey", "Enya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Epena", "sja"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sja", "Epena"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Epi-Olmec", "xep"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xep", "Epi-Olmec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Epie", "epi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("epi", "Epie"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Epigraphic Mayan", "emy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("emy", "Epigraphic Mayan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Eravallan", "era"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("era", "Eravallan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Erave", "kjy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kjy", "Erave"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ere", "twp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("twp", "Ere"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Eritai", "ert"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ert", "Eritai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Erokwanas", "erw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("erw", "Erokwanas"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Erre", "err"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("err", "Erre"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Erromintxela", "emx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("emx", "Erromintxela"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ersu", "ers"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ers", "Ersu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Eruwa", "erh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("erh", "Eruwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Erzya", "myv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("myv", "Erzya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Esan", "ish"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ish", "Esan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ese", "mcq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mcq", "Ese"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ese Ejja", "ese"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ese", "Ese Ejja"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Eshtehardi", "esh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("esh", "Eshtehardi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Esimbi", "ags"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ags", "Esimbi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Esperanto", "epo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("epo", "Esperanto"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Esselen", "esq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("esq", "Esselen"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Estado de México Otomi", "ots"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ots", "Estado de México Otomi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Estonian", "est"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("est", "Estonian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Estonian Sign Language", "eso"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("eso", "Estonian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Esuma", "esm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("esm", "Esuma"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Etchemin", "etc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("etc", "Etchemin"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Etebi", "etb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("etb", "Etebi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Eten", "etx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("etx", "Eten"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Eteocretan", "ecr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ecr", "Eteocretan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Eteocypriot", "ecy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ecy", "Eteocypriot"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ethiopian Sign Language", "eth"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("eth", "Ethiopian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Etkywan", "ich"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ich", "Etkywan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Eton (Cameroon)", "eto"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("eto", "Eton (Cameroon)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Eton (Vanuatu)", "etn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("etn", "Eton (Vanuatu)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Etruscan", "ett"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ett", "Etruscan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Etulo", "utr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("utr", "Etulo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Evant", "bzz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bzz", "Evant"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Even", "eve"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("eve", "Even"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Evenki", "evn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("evn", "Evenki"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ewage-Notu", "nou"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nou", "Ewage-Notu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ewe", "ewe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ewe", "Ewe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ewondo", "ewo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ewo", "Ewondo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Extremaduran", "ext"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ext", "Extremaduran"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Eyak", "eya"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("eya", "Eyak"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Fa D'ambu", "fab"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("fab", "Fa D'ambu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Fagani", "faf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("faf", "Fagani"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Faire Atta", "azt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("azt", "Faire Atta"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Faita", "faj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("faj", "Faita"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Faiwol", "fai"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("fai", "Faiwol"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Fala", "fax"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("fax", "Fala"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Falam Chin", "cfm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cfm", "Falam Chin"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Fali", "fli"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("fli", "Fali"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Faliscan", "xfa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xfa", "Faliscan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Fam", "fam"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("fam", "Fam"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Fanagalo", "fng"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("fng", "Fanagalo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Fang (Cameroon)", "fak"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("fak", "Fang (Cameroon)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Fang (Equatorial Guinea)", "fan"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("fan", "Fang (Equatorial Guinea)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Fania", "fni"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("fni", "Fania"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Fanti", "fat"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("fat", "Fanti"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Far Western Muria", "fmu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("fmu", "Far Western Muria"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Farefare", "gur"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gur", "Farefare"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Faroese", "fao"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("fao", "Faroese"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Fas", "fqs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("fqs", "Fas"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Fasu", "faa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("faa", "Fasu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Fataleka", "far"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("far", "Fataleka"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Fataluku", "ddg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ddg", "Fataluku"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Fayu", "fau"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("fau", "Fayu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Fe'fe'", "fmp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("fmp", "Fe'fe'"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Fedan", "pdn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pdn", "Fedan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Fembe", "agl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("agl", "Fembe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Fernando Po Creole English", "fpe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("fpe", "Fernando Po Creole English"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Feroge", "fer"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("fer", "Feroge"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Fiji Hindi", "hif"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hif", "Fiji Hindi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Fijian", "fij"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("fij", "Fijian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Filipino", "fil"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("fil", "Filipino"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Filomena Mata-Coahuitlán Totonac", "tlp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tlp", "Filomena Mata-Coahuitlán Totonac"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Finland-Swedish Sign Language", "fss"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("fss", "Finland-Swedish Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Finnish", "fin"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("fin", "Finnish"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Finnish Sign Language", "fse"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("fse", "Finnish Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Finongan", "fag"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("fag", "Finongan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Fipa", "fip"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("fip", "Fipa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Firan", "fir"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("fir", "Firan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Fiwaga", "fiw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("fiw", "Fiwaga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Flemish", "nld"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nld", "Flemish"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Flemish Sign Language", "vgt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vgt", "Flemish Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Flinders Island", "fln"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("fln", "Flinders Island"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Foau", "flh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("flh", "Foau"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Foi", "foi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("foi", "Foi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Foia Foia", "ffi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ffi", "Foia Foia"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Folopa", "ppo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ppo", "Folopa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Foma", "fom"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("fom", "Foma"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Fon", "fon"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("fon", "Fon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Fongoro", "fgr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("fgr", "Fongoro"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Foodo", "fod"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("fod", "Foodo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Forak", "frq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("frq", "Forak"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Fordata", "frd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("frd", "Fordata"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Fore", "for"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("for", "Fore"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Forest Enets", "enf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("enf", "Forest Enets"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Forest Maninka", "myq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("myq", "Forest Maninka"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Fortsenal", "frt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("frt", "Fortsenal"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Francisco León Zoque", "zos"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zos", "Francisco León Zoque"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Francoprovençal", "frp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("frp", "Francoprovençal"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Frankish", "frk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("frk", "Frankish"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("French", "fra"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("fra", "French"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("French Belgian Sign Language", "sfb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sfb", "French Belgian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("French Sign Language", "fsl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("fsl", "French Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Friulian", "fur"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("fur", "Friulian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Fulah", "ful"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ful", "Fulah"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Fuliiru", "flr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("flr", "Fuliiru"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Fulniô", "fun"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("fun", "Fulniô"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Fum", "fum"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("fum", "Fum"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Fungwa", "ula"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ula", "Fungwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Fur", "fvr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("fvr", "Fur"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Furu", "fuu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("fuu", "Furu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Futuna-Aniwa", "fut"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("fut", "Futuna-Aniwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Fuyug", "fuy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("fuy", "Fuyug"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Fwe", "fwe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("fwe", "Fwe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Fwâi", "fwa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("fwa", "Fwâi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Fyam", "pym"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pym", "Fyam"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Fyer", "fie"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("fie", "Fyer"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ga", "gaa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gaa", "Ga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ga'anda", "gqa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gqa", "Ga'anda"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ga'dang", "gdg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gdg", "Ga'dang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gaa", "ttb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ttb", "Gaa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gaam", "tbi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tbi", "Gaam"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gabri", "gab"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gab", "Gabri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gadang", "gdk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gdk", "Gadang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gaddang", "gad"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gad", "Gaddang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gaddi", "gbk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gbk", "Gaddi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gade", "ged"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ged", "Gade"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gade Lohar", "gda"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gda", "Gade Lohar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gadjerawang", "gdh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gdh", "Gadjerawang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gadsup", "gaj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gaj", "Gadsup"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gaelic", "gla"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gla", "Gaelic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gafat", "gft"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gft", "Gafat"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gagadu", "gbu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gbu", "Gagadu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gagauz", "gag"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gag", "Gagauz"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gagnoa Bété", "btg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("btg", "Gagnoa Bété"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gagu", "ggu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ggu", "Gagu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gahri", "bfu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bfu", "Gahri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gaikundi", "gbf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gbf", "Gaikundi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gail", "gic"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gic", "Gail"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gaina", "gcn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gcn", "Gaina"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gal", "gap"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gap", "Gal"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Galambu", "glo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("glo", "Galambu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Galatian", "xga"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xga", "Galatian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Galela", "gbi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gbi", "Galela"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Galeya", "gar"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gar", "Galeya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Galibi Carib", "car"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("car", "Galibi Carib"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Galice", "gce"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gce", "Galice"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Galician", "glg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("glg", "Galician"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Galindan", "xgl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xgl", "Galindan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gallurese Sardinian", "sdn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sdn", "Gallurese Sardinian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Galo", "adl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("adl", "Galo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Galoli", "gal"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gal", "Galoli"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gamale Kham", "kgj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kgj", "Gamale Kham"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gambera", "gma"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gma", "Gambera"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gambian Wolof", "wof"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wof", "Gambian Wolof"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gamilaraay", "kld"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kld", "Gamilaraay"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gamit", "gbl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gbl", "Gamit"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gamkonora", "gak"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gak", "Gamkonora"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gamo", "gmv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gmv", "Gamo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gamo-Ningi", "bte"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bte", "Gamo-Ningi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gan Chinese", "gan"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gan", "Gan Chinese"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gana", "gnq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gnq", "Gana"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ganang", "gne"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gne", "Ganang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ganda", "lug"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lug", "Ganda"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gane", "gzn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gzn", "Gane"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ganggalida", "gcd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gcd", "Ganggalida"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ganglau", "ggl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ggl", "Ganglau"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gangte", "gnb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gnb", "Gangte"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gangulu", "gnl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gnl", "Gangulu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gants", "gao"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gao", "Gants"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ganza", "gza"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gza", "Ganza"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ganzi", "gnz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gnz", "Ganzi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gao", "gga"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gga", "Gao"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gapapaiwa", "pwg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pwg", "Gapapaiwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Garawa", "gbc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gbc", "Garawa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Garhwali", "gbm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gbm", "Garhwali"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Garifuna", "cab"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cab", "Garifuna"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Garig-Ilgar", "ilg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ilg", "Garig-Ilgar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Garo", "grt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("grt", "Garo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Garre", "gex"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gex", "Garre"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Garus", "gyb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gyb", "Garus"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Garza", "xgr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xgr", "Garza"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gata'", "gaq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gaq", "Gata'"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gavar", "gou"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gou", "Gavar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gavião Do Jiparaná", "gvo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gvo", "Gavião Do Jiparaná"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gawar-Bati", "gwt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gwt", "Gawar-Bati"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gawwada", "gwd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gwd", "Gawwada"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gayil", "gyl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gyl", "Gayil"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gayo", "gay"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gay", "Gayo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gazi", "gzi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gzi", "Gazi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gbagyi", "gbr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gbr", "Gbagyi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gbanu", "gbv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gbv", "Gbanu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gbanziri", "gbg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gbg", "Gbanziri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gbari", "gby"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gby", "Gbari"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gbati-ri", "gti"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gti", "Gbati-ri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gbaya (Central African Republic)", "gba"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gba", "Gbaya (Central African Republic)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gbaya (Sudan)", "krs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("krs", "Gbaya (Sudan)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gbaya-Bossangoa", "gbp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gbp", "Gbaya-Bossangoa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gbaya-Bozoum", "gbq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gbq", "Gbaya-Bozoum"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gbaya-Mbodomo", "gmm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gmm", "Gbaya-Mbodomo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gbayi", "gyg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gyg", "Gbayi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gbesi Gbe", "gbs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gbs", "Gbesi Gbe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gbii", "ggb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ggb", "Gbii"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gbiri-Niragu", "grh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("grh", "Gbiri-Niragu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gboloo Grebo", "gec"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gec", "Gboloo Grebo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ge", "hmj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hmj", "Ge"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Geba Karen", "kvq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kvq", "Geba Karen"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gebe", "gei"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gei", "Gebe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gedaged", "gdd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gdd", "Gedaged"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gedeo", "drs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("drs", "Gedeo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Geez", "gez"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gez", "Geez"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Geji", "gji"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gji", "Geji"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gejia", "hmj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hmj", "Gejia"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Geko Karen", "ghk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ghk", "Geko Karen"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gela", "nlg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nlg", "Gela"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gelao", "gio"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gio", "Gelao"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Geman Deng", "mxj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mxj", "Geman Deng"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Geme", "geq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("geq", "Geme"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gen", "gej"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gej", "Gen"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gende", "gaf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gaf", "Gende"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gengle", "geg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("geg", "Gengle"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Georgian", "kat"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kat", "Georgian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gepo", "ygp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ygp", "Gepo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gera", "gew"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gew", "Gera"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("German", "deu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("deu", "German"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("German Sign Language", "gsg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gsg", "German Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Geruma", "gea"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gea", "Geruma"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Geser-Gorom", "ges"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ges", "Geser-Gorom"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gey", "guv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("guv", "Gey"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ghadamès", "gha"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gha", "Ghadamès"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ghanaian Sign Language", "gse"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gse", "Ghanaian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ghanongga", "ghn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ghn", "Ghanongga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ghari", "gri"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gri", "Ghari"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ghayavi", "bmk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bmk", "Ghayavi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gheg Albanian", "aln"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aln", "Gheg Albanian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ghera", "ghr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ghr", "Ghera"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ghodoberi", "gdo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gdo", "Ghodoberi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ghomara", "gho"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gho", "Ghomara"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ghomálá'", "bbj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bbj", "Ghomálá'"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ghotuo", "aaa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aaa", "Ghotuo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ghulfan", "ghl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ghl", "Ghulfan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Giangan", "bgi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bgi", "Giangan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gibanawa", "gib"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gib", "Gibanawa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gichuka", "cuh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cuh", "Gichuka"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gidar", "gid"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gid", "Gidar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Giiwo", "kks"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kks", "Giiwo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gikuyu", "kik"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kik", "Gikuyu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gikyode", "acd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("acd", "Gikyode"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gilaki", "glk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("glk", "Gilaki"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gilbertese", "gil"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gil", "Gilbertese"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gilima", "gix"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gix", "Gilima"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gilyak", "niv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("niv", "Gilyak"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gimi (Eastern Highlands)", "gim"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gim", "Gimi (Eastern Highlands)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gimi (West New Britain)", "gip"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gip", "Gimi (West New Britain)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gimme", "kmp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kmp", "Gimme"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gimnime", "gmn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gmn", "Gimnime"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ginuman", "gnm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gnm", "Ginuman"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ginyanga", "ayg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ayg", "Ginyanga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Girawa", "bbr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bbr", "Girawa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Giryama", "nyf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nyf", "Giryama"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gitonga", "toh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("toh", "Gitonga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gitua", "ggt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ggt", "Gitua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gitxsan", "git"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("git", "Gitxsan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Giyug", "giy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("giy", "Giyug"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gizrra", "tof"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tof", "Gizrra"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Glaro-Twabo", "glr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("glr", "Glaro-Twabo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Glavda", "glw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("glw", "Glavda"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Glio-Oubi", "oub"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("oub", "Glio-Oubi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gnau", "gnu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gnu", "Gnau"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Goan Konkani", "gom"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gom", "Goan Konkani"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Goaria", "gig"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gig", "Goaria"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gobasi", "goi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("goi", "Gobasi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gobu", "gox"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gox", "Gobu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Godié", "god"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("god", "Godié"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Godwari", "gdx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gdx", "Godwari"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Goemai", "ank"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ank", "Goemai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gofa", "gof"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gof", "Gofa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gogo", "gog"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gog", "Gogo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gogodala", "ggw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ggw", "Gogodala"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gokana", "gkn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gkn", "Gokana"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gola", "gol"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gol", "Gola"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Golin", "gvf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gvf", "Golin"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gondi", "gon"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gon", "Gondi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gone Dau", "goo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("goo", "Gone Dau"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gongduk", "goe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("goe", "Gongduk"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gonja", "gjn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gjn", "Gonja"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gooniyandi", "gni"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gni", "Gooniyandi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gor", "gqr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gqr", "Gor"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gorakor", "goc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("goc", "Gorakor"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gorap", "goq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("goq", "Gorap"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gorontalo", "gor"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gor", "Gorontalo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gorovu", "grq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("grq", "Gorovu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gorowa", "gow"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gow", "Gorowa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gothic", "got"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("got", "Gothic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Goundo", "goy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("goy", "Goundo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gourmanchéma", "gux"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gux", "Gourmanchéma"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gowlan", "goj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("goj", "Gowlan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gowli", "gok"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gok", "Gowli"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gowro", "gwf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gwf", "Gowro"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gozarkhani", "goz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("goz", "Gozarkhani"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Grangali", "nli"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nli", "Grangali"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Grass Koiari", "kbk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kbk", "Grass Koiari"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Grebo", "grb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("grb", "Grebo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Greek Sign Language", "gss"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gss", "Greek Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Green Gelao", "giq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("giq", "Green Gelao"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Greenlandic", "kal"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kal", "Greenlandic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Grenadian Creole English", "gcl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gcl", "Grenadian Creole English"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gresi", "grs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("grs", "Gresi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Groma", "gro"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gro", "Groma"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gronings", "gos"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gos", "Gronings"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gros Ventre", "ats"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ats", "Gros Ventre"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gua", "gwx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gwx", "Gua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Guadeloupean Creole French", "gcf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gcf", "Guadeloupean Creole French"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Guahibo", "guh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("guh", "Guahibo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Guajajára", "gub"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gub", "Guajajára"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Guajá", "gvj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gvj", "Guajá"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Guambiano", "gum"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gum", "Guambiano"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Guana (Brazil)", "gqn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gqn", "Guana (Brazil)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Guana (Paraguay)", "gva"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gva", "Guana (Paraguay)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Guanano", "gvc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gvc", "Guanano"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Guanche", "gnc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gnc", "Guanche"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Guanyinqiao", "jiq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jiq", "Guanyinqiao"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Guarani", "grn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("grn", "Guarani"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Guarayu", "gyr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gyr", "Guarayu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Guarequena", "gae"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gae", "Guarequena"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Guatemalan Sign Language", "gsm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gsm", "Guatemalan Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Guató", "gta"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gta", "Guató"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Guayabero", "guo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("guo", "Guayabero"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gudanji", "nji"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nji", "Gudanji"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gude", "gde"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gde", "Gude"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gudu", "gdu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gdu", "Gudu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Guduf-Gava", "gdf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gdf", "Guduf-Gava"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Guerrero Amuzgo", "amu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("amu", "Guerrero Amuzgo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Guerrero Nahuatl", "ngu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ngu", "Guerrero Nahuatl"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Guevea De Humboldt Zapotec", "zpg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zpg", "Guevea De Humboldt Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gugadj", "ggd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ggd", "Gugadj"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gugu Badhun", "gdc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gdc", "Gugu Badhun"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gugu Warra", "wrw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wrw", "Gugu Warra"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gugubera", "kkp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kkp", "Gugubera"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Guguyimidjir", "kky"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kky", "Guguyimidjir"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Guhu-Samane", "ghs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ghs", "Guhu-Samane"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Guianese Creole French", "gcr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gcr", "Guianese Creole French"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Guibei Zhuang", "zgb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zgb", "Guibei Zhuang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Guiberoua Béte", "bet"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bet", "Guiberoua Béte"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Guibian Zhuang", "zgn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zgn", "Guibian Zhuang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Guinea Kpelle", "gkp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gkp", "Guinea Kpelle"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Guinean Sign Language", "gus"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gus", "Guinean Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Guiqiong", "gqi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gqi", "Guiqiong"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gujarati", "guj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("guj", "Gujarati"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gujari", "gju"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gju", "Gujari"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gula (Central African Republic)", "kcm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kcm", "Gula (Central African Republic)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gula (Chad)", "glu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("glu", "Gula (Chad)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gula Iro", "glj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("glj", "Gula Iro"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gula'alaa", "gmb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gmb", "Gula'alaa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gulay", "gvl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gvl", "Gulay"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gule", "gly"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gly", "Gule"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gulf Arabic", "afb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("afb", "Gulf Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Guliguli", "gli"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gli", "Guliguli"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gumalu", "gmu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gmu", "Gumalu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gumatj", "gnn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gnn", "Gumatj"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gumawana", "gvs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gvs", "Gumawana"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gumuz", "guk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("guk", "Gumuz"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gun", "guw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("guw", "Gun"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gundi", "gdi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gdi", "Gundi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gungabula", "gyf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gyf", "Gungabula"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gungu", "rub"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rub", "Gungu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Guntai", "gnt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gnt", "Guntai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gunwinggu", "gup"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gup", "Gunwinggu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gunya", "gyy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gyy", "Gunya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gupa-Abawa", "gpa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gpa", "Gupa-Abawa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gupapuyngu", "guf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("guf", "Gupapuyngu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Guragone", "gge"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gge", "Guragone"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Guramalum", "grz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("grz", "Guramalum"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gurani", "hac"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hac", "Gurani"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gurdjar", "gdj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gdj", "Gurdjar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gureng Gureng", "gnr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gnr", "Gureng Gureng"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gurgula", "ggg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ggg", "Gurgula"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Guriaso", "grx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("grx", "Guriaso"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gurinji", "gue"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gue", "Gurinji"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gurmana", "gvm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gvm", "Gurmana"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Guro", "goa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("goa", "Guro"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Guruntum-Mbaaru", "grd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("grd", "Guruntum-Mbaaru"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gusan", "gsn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gsn", "Gusan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gusii", "guz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("guz", "Gusii"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gusilay", "gsl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gsl", "Gusilay"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Guwamu", "gwu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gwu", "Guwamu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Guya", "gka"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gka", "Guya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Guyanese Creole English", "gyn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gyn", "Guyanese Creole English"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Guyani", "gvy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gvy", "Guyani"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gvoko", "ngs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ngs", "Gvoko"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gwa", "gwb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gwb", "Gwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gwahatike", "dah"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dah", "Gwahatike"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gwamhi-Wuri", "bga"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bga", "Gwamhi-Wuri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gwandara", "gwn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gwn", "Gwandara"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gweda", "grw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("grw", "Gweda"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gweno", "gwe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gwe", "Gweno"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gwere", "gwr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gwr", "Gwere"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gwichʼin", "gwi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gwi", "Gwichʼin"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gyele", "gyi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gyi", "Gyele"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Gyem", "gye"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gye", "Gyem"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Güilá Zapotec", "ztu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ztu", "Güilá Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ha", "haq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("haq", "Ha"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Habu", "hbu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hbu", "Habu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hadiyya", "hdy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hdy", "Hadiyya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hadothi", "hoj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hoj", "Hadothi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hadrami", "xhd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xhd", "Hadrami"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hadrami Arabic", "ayh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ayh", "Hadrami Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hadza", "hts"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hts", "Hadza"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Haeke", "aek"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aek", "Haeke"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hahon", "hah"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hah", "Hahon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hai//om", "hgm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hgm", "Hai//om"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Haida", "hai"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hai", "Haida"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Haigwai", "hgw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hgw", "Haigwai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hainyaxo Bozo", "bzx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bzx", "Hainyaxo Bozo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Haiphong Sign Language", "haf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("haf", "Haiphong Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Haisla", "has"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("has", "Haisla"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Haitian", "hat"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hat", "Haitian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Haitian Creole", "hat"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hat", "Haitian Creole"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Haitian Vodoun Culture Language", "hvc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hvc", "Haitian Vodoun Culture Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Haji", "hji"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hji", "Haji"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hajong", "haj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("haj", "Hajong"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Haka Chin", "cnh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cnh", "Haka Chin"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hakka Chinese", "hak"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hak", "Hakka Chinese"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hakö", "hao"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hao", "Hakö"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Halang", "hal"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hal", "Halang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Halang Doan", "hld"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hld", "Halang Doan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Halbi", "hlb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hlb", "Halbi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Halh Mongolian", "khk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("khk", "Halh Mongolian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Halia", "hla"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hla", "Halia"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Halkomelem", "hur"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hur", "Halkomelem"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hamap", "hmu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hmu", "Hamap"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hamba", "hba"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hba", "Hamba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hamer-Banna", "amf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("amf", "Hamer-Banna"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hamtai", "hmt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hmt", "Hamtai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Han", "haa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("haa", "Han"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hanga", "hag"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hag", "Hanga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hanga Hundi", "wos"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wos", "Hanga Hundi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hangaza", "han"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("han", "Hangaza"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hani", "hni"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hni", "Hani"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hano", "lml"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lml", "Hano"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hanoi Sign Language", "hab"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hab", "Hanoi Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hanunoo", "hnn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hnn", "Hanunoo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Harami", "xha"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xha", "Harami"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Harari", "har"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("har", "Harari"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Harijan Kinnauri", "kjo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kjo", "Harijan Kinnauri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Haroi", "hro"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hro", "Haroi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Harsusi", "hss"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hss", "Harsusi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Haruai", "tmd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tmd", "Haruai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Haruku", "hrk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hrk", "Haruku"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Haryanvi", "bgc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bgc", "Haryanvi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Harzani", "hrz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hrz", "Harzani"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hasha", "ybj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ybj", "Hasha"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hassaniyya", "mey"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mey", "Hassaniyya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hatam", "had"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("had", "Hatam"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hattic", "xht"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xht", "Hattic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hausa", "hau"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hau", "Hausa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hausa Sign Language", "hsl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hsl", "Hausa Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Haveke", "hvk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hvk", "Haveke"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Havu", "hav"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hav", "Havu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hawai'i Creole English", "hwc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hwc", "Hawai'i Creole English"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hawai'i Pidgin Sign Language", "hps"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hps", "Hawai'i Pidgin Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hawaiian", "haw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("haw", "Hawaiian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Haya", "hay"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hay", "Haya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hazaragi", "haz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("haz", "Hazaragi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hdi", "xed"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xed", "Hdi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hebrew", "heb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("heb", "Hebrew"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hehe", "heh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("heh", "Hehe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Heiban", "hbn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hbn", "Heiban"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Heiltsuk", "hei"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hei", "Heiltsuk"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Helambu Sherpa", "scp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("scp", "Helambu Sherpa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Helong", "heg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("heg", "Helong"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hema", "nix"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nix", "Hema"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hemba", "hem"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hem", "Hemba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Herdé", "hed"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hed", "Herdé"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Herero", "her"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("her", "Herero"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hermit", "llf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("llf", "Hermit"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hernican", "xhr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xhr", "Hernican"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Heung Kong Sau Yue", "hks"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hks", "Heung Kong Sau Yue"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hewa", "ham"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ham", "Hewa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Heyo", "auk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("auk", "Heyo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hiberno-Scottish Gaelic", "ghc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ghc", "Hiberno-Scottish Gaelic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hibito", "hib"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hib", "Hibito"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hidatsa", "hid"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hid", "Hidatsa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hieroglyphic Luwian", "hlu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hlu", "Hieroglyphic Luwian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Higaonon", "mba"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mba", "Higaonon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Highland Konjo", "kjk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kjk", "Highland Konjo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Highland Oaxaca Chontal", "chd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("chd", "Highland Oaxaca Chontal"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Highland Popoluca", "poi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("poi", "Highland Popoluca"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Highland Puebla Nahuatl", "azz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("azz", "Highland Puebla Nahuatl"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Highland Totonac", "tos"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tos", "Highland Totonac"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hijazi Arabic", "acw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("acw", "Hijazi Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hijuk", "hij"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hij", "Hijuk"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hiligaynon", "hil"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hil", "Hiligaynon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Himarimã", "hir"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hir", "Himarimã"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hindi", "hin"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hin", "Hindi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hinduri", "hii"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hii", "Hinduri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hinukh", "gin"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gin", "Hinukh"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hiri Motu", "hmo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hmo", "Hiri Motu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hittite", "hit"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hit", "Hittite"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hitu", "htu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("htu", "Hitu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hiw", "hiw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hiw", "Hiw"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hixkaryána", "hix"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hix", "Hixkaryána"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hlai", "lic"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lic", "Hlai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hlepho Phowa", "yhl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yhl", "Hlepho Phowa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hlersu", "hle"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hle", "Hlersu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hmar", "hmr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hmr", "Hmar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hmong", "hmn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hmn", "Hmong"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hmong Daw", "mww"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mww", "Hmong Daw"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hmong Don", "hmf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hmf", "Hmong Don"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hmong Dô", "hmv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hmv", "Hmong Dô"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hmong Njua", "hnj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hnj", "Hmong Njua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hmong Shua", "hmz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hmz", "Hmong Shua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hmwaveke", "mrk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mrk", "Hmwaveke"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ho", "hoc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hoc", "Ho"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ho Chi Minh City Sign Language", "hos"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hos", "Ho Chi Minh City Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ho-Chunk", "win"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("win", "Ho-Chunk"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hoava", "hoa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hoa", "Hoava"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hobyót", "hoh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hoh", "Hobyót"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hoia Hoia", "hhi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hhi", "Hoia Hoia"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Holikachuk", "hoi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hoi", "Holikachuk"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Holiya", "hoy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hoy", "Holiya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Holma", "hod"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hod", "Holma"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Holoholo", "hoo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hoo", "Holoholo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Holu", "hol"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hol", "Holu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Homa", "hom"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hom", "Homa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Honduras Sign Language", "hds"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hds", "Honduras Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hong Kong Sign Language", "hks"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hks", "Hong Kong Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Honi", "how"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("how", "Honi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hopi", "hop"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hop", "Hopi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Horned Miao", "hrm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hrm", "Horned Miao"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Horo", "hor"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hor", "Horo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Horom", "hoe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hoe", "Horom"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Horpa", "ero"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ero", "Horpa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Horuru", "hrr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hrr", "Horuru"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hote", "hot"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hot", "Hote"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hoti", "hti"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hti", "Hoti"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hovongan", "hov"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hov", "Hovongan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hoyahoya", "hhy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hhy", "Hoyahoya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hozo", "hoz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hoz", "Hozo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hpon", "hpo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hpo", "Hpon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hrangkhol", "hra"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hra", "Hrangkhol"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hre", "hre"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hre", "Hre"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hruso", "hru"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hru", "Hruso"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hu", "huo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("huo", "Hu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Huachipaeri", "hug"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hug", "Huachipaeri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Huallaga Huánuco Quechua", "qub"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("qub", "Huallaga Huánuco Quechua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Huamalíes-Dos de Mayo Huánuco Quechua", "qvh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("qvh", "Huamalíes-Dos de Mayo Huánuco Quechua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Huambisa", "hub"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hub", "Huambisa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Huarijio", "var"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("var", "Huarijio"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Huastec", "hus"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hus", "Huastec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Huaulu", "hud"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hud", "Huaulu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Huautla Mazatec", "mau"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mau", "Huautla Mazatec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Huaxcaleca Nahuatl", "nhq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nhq", "Huaxcaleca Nahuatl"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Huaylas Ancash Quechua", "qwh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("qwh", "Huaylas Ancash Quechua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Huaylla Wanca Quechua", "qvw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("qvw", "Huaylla Wanca Quechua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Huba", "hbb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hbb", "Huba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Huehuetla Tepehua", "tee"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tee", "Huehuetla Tepehua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Huichol", "hch"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hch", "Huichol"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Huilliche", "huh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("huh", "Huilliche"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Huitepec Mixtec", "mxs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mxs", "Huitepec Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Huizhou Chinese", "czh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("czh", "Huizhou Chinese"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hukumina", "huw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("huw", "Hukumina"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hula", "hul"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hul", "Hula"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hulaulá", "huy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("huy", "Hulaulá"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Huli", "hui"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hui", "Huli"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hulung", "huk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("huk", "Hulung"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Humburi Senni Songhay", "hmb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hmb", "Humburi Senni Songhay"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Humene", "huf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("huf", "Humene"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Humla", "hut"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hut", "Humla"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hun-Saare", "dud"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dud", "Hun-Saare"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hunde", "hke"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hke", "Hunde"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hung", "hnu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hnu", "Hung"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hungana", "hum"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hum", "Hungana"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hungarian", "hun"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hun", "Hungarian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hungarian Sign Language", "hsh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hsh", "Hungarian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hungworo", "nat"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nat", "Hungworo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hunjara-Kaina Ke", "hkk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hkk", "Hunjara-Kaina Ke"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hunnic", "xhc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xhc", "Hunnic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hunsrik", "hrx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hrx", "Hunsrik"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hunzib", "huz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("huz", "Hunzib"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hupa", "hup"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hup", "Hupa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hupdë", "jup"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jup", "Hupdë"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hupla", "hap"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hap", "Hupla"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hurrian", "xhu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xhu", "Hurrian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hutterisch", "geh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("geh", "Hutterisch"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hutterite German", "geh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("geh", "Hutterite German"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hwana", "hwo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hwo", "Hwana"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hya", "hya"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hya", "Hya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hyam", "jab"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jab", "Hyam"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hértevin", "hrt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hrt", "Hértevin"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Hõne", "juh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("juh", "Hõne"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("I'saka", "ksi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ksi", "I'saka"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("I-Wak", "iwk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("iwk", "I-Wak"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Iaai", "iai"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("iai", "Iaai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Iamalele", "yml"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yml", "Iamalele"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Iapama", "iap"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("iap", "Iapama"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Iatmul", "ian"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ian", "Iatmul"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Iau", "tmu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tmu", "Iau"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ibali Teke", "tek"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tek", "Ibali Teke"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ibaloi", "ibl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ibl", "Ibaloi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Iban", "iba"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("iba", "Iban"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ibanag", "ibg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ibg", "Ibanag"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ibani", "iby"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("iby", "Ibani"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ibatan", "ivb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ivb", "Ibatan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Iberian", "xib"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xib", "Iberian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ibibio", "ibb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ibb", "Ibibio"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ibilo", "ibi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ibi", "Ibilo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ibino", "ibn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ibn", "Ibino"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ibu", "ibu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ibu", "Ibu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ibuoro", "ibr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ibr", "Ibuoro"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Icelandic", "isl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("isl", "Icelandic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Icelandic Sign Language", "icl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("icl", "Icelandic Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Iceve-Maci", "bec"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bec", "Iceve-Maci"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ida'an", "dbj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dbj", "Ida'an"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Idaté", "idt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("idt", "Idaté"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Idere", "ide"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ide", "Idere"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Idesa", "ids"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ids", "Idesa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Idi", "idi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("idi", "Idi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ido", "ido"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ido", "Ido"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Idoma", "idu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("idu", "Idoma"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Idon", "idc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("idc", "Idon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Idu-Mishmi", "clk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("clk", "Idu-Mishmi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Idun", "ldb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ldb", "Idun"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Iduna", "viv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("viv", "Iduna"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ifo", "iff"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("iff", "Ifo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ifè", "ife"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ife", "Ifè"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Igala", "igl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("igl", "Igala"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Igana", "igg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("igg", "Igana"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Igbo", "ibo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ibo", "Igbo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Igede", "ige"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ige", "Igede"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ignaciano", "ign"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ign", "Ignaciano"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Igo", "ahl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ahl", "Igo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Iguta", "nar"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nar", "Iguta"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Igwe", "igw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("igw", "Igwe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Iha", "ihp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ihp", "Iha"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Iha Based Pidgin", "ihb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ihb", "Iha Based Pidgin"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ihievbe", "ihi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ihi", "Ihievbe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ija-Zuba", "vki"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vki", "Ija-Zuba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ik", "ikx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ikx", "Ik"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ika", "ikk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ikk", "Ika"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ikizu", "ikz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ikz", "Ikizu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Iko", "iki"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("iki", "Iko"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ikobi-Mena", "meb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("meb", "Ikobi-Mena"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ikpeng", "txi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("txi", "Ikpeng"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ikpeshi", "ikp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ikp", "Ikpeshi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ikposo", "kpo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kpo", "Ikposo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Iku-Gora-Ankwa", "ikv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ikv", "Iku-Gora-Ankwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ikulu", "ikl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ikl", "Ikulu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ikwere", "ikw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ikw", "Ikwere"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ila", "ilb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ilb", "Ila"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ile Ape", "ila"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ila", "Ile Ape"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ili Turki", "ili"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ili", "Ili Turki"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ili'uun", "ilu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ilu", "Ili'uun"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ilianen Manobo", "mbi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mbi", "Ilianen Manobo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Illyrian", "xil"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xil", "Illyrian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Iloko", "ilo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ilo", "Iloko"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ilongot", "ilk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ilk", "Ilongot"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ilue", "ilv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ilv", "Ilue"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ilwana", "mlk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mlk", "Ilwana"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Imbabura Highland Quichua", "qvi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("qvi", "Imbabura Highland Quichua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Imbongu", "imo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("imo", "Imbongu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Imeraguen", "ime"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ime", "Imeraguen"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Imonda", "imn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("imn", "Imonda"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Imperial Aramaic (700-300 BCE)", "arc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("arc", "Imperial Aramaic (700-300 BCE)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Imroing", "imr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("imr", "Imroing"); //$NON-NLS-1$ //$NON-NLS-2$
	}

	private void createLanguagesMap3() {
		langMap.put("Inabaknon", "abx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("abx", "Inabaknon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Inapang", "mzu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mzu", "Inapang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Inari Sami", "smn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("smn", "Inari Sami"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Indian Sign Language", "ins"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ins", "Indian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Indo-Portuguese", "idb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("idb", "Indo-Portuguese"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Indonesian", "ind"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ind", "Indonesian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Indonesian Bajau", "bdl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bdl", "Indonesian Bajau"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Indonesian Sign Language", "inl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("inl", "Indonesian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Indri", "idr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("idr", "Indri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Indus Kohistani", "mvy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mvy", "Indus Kohistani"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Indus Valley Language", "xiv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xiv", "Indus Valley Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Inebu One", "oin"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("oin", "Inebu One"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ineseño", "inz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("inz", "Ineseño"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Inga", "inb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("inb", "Inga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ingrian", "izh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("izh", "Ingrian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ingush", "inh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("inh", "Ingush"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Inlaod Itneg", "iti"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("iti", "Inlaod Itneg"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Inoke-Yate", "ino"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ino", "Inoke-Yate"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Inonhan", "loc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("loc", "Inonhan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Inor", "ior"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ior", "Inor"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Inpui Naga", "nkf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nkf", "Inpui Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Interglossa", "igs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("igs", "Interglossa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put(
				"Interlingua (International Auxiliary Language Association)", //$NON-NLS-1$
				"ina"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ina", //$NON-NLS-1$
				"Interlingua (International Auxiliary Language Association)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Interlingue", "ile"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ile", "Interlingue"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("International Sign", "ils"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ils", "International Sign"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Intha", "int"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("int", "Intha"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Inuktitut", "iku"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("iku", "Inuktitut"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Inupiaq", "ipk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ipk", "Inupiaq"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Iowa-Oto", "iow"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("iow", "Iowa-Oto"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ipalapa Amuzgo", "azm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("azm", "Ipalapa Amuzgo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ipiko", "ipo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ipo", "Ipiko"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ipili", "ipi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ipi", "Ipili"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ipulo", "ass"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ass", "Ipulo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Iquito", "iqu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("iqu", "Iquito"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ir", "irr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("irr", "Ir"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Iranian Persian", "pes"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pes", "Iranian Persian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Iranun", "ill"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ill", "Iranun"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Iraqw", "irk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("irk", "Iraqw"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Irarutu", "irh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("irh", "Irarutu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Iraya", "iry"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("iry", "Iraya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Iresim", "ire"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ire", "Iresim"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Irigwe", "iri"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("iri", "Irigwe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Irish", "gle"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gle", "Irish"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Irish Sign Language", "isg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("isg", "Irish Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Irula", "iru"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("iru", "Irula"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Irántxe", "irn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("irn", "Irántxe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Isabi", "isa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("isa", "Isabi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Isanzu", "isn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("isn", "Isanzu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Isarog Agta", "agk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("agk", "Isarog Agta"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Isconahua", "isc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("isc", "Isconahua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Isebe", "igo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("igo", "Isebe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Isekiri", "its"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("its", "Isekiri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ishkashimi", "isk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("isk", "Ishkashimi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Isinai", "inn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("inn", "Isinai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Isirawa", "srl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("srl", "Isirawa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Island Carib", "crb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("crb", "Island Carib"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Islander Creole English", "icr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("icr", "Islander Creole English"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Isnag", "isd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("isd", "Isnag"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Isoko", "iso"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("iso", "Isoko"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Israeli Sign Language", "isr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("isr", "Israeli Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Isthmus Mixe", "mir"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mir", "Isthmus Mixe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Isthmus Zapotec", "zai"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zai", "Isthmus Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Isthmus-Cosoleacaque Nahuatl", "nhk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nhk", "Isthmus-Cosoleacaque Nahuatl"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Isthmus-Mecayapan Nahuatl", "nhx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nhx", "Isthmus-Mecayapan Nahuatl"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Isthmus-Pajapan Nahuatl", "nhp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nhp", "Isthmus-Pajapan Nahuatl"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Istriot", "ist"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ist", "Istriot"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Istro Romanian", "ruo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ruo", "Istro Romanian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Isu (Fako Division)", "szv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("szv", "Isu (Fako Division)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Isu (Menchum Division)", "isu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("isu", "Isu (Menchum Division)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Italian", "ita"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ita", "Italian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Italian Sign Language", "ise"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ise", "Italian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Itawit", "itv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("itv", "Itawit"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Itelmen", "itl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("itl", "Itelmen"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Itene", "ite"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ite", "Itene"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Iteri", "itr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("itr", "Iteri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Itik", "itx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("itx", "Itik"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ito", "itw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("itw", "Ito"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Itonama", "ito"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ito", "Itonama"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Itu Mbon Uzo", "itm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("itm", "Itu Mbon Uzo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Itundujia Mixtec", "mce"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mce", "Itundujia Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Itzá", "itz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("itz", "Itzá"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Iu Mien", "ium"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ium", "Iu Mien"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ivatan", "ivv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ivv", "Ivatan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ivbie North-Okpela-Arhe", "atg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("atg", "Ivbie North-Okpela-Arhe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Iwaidja", "ibd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ibd", "Iwaidja"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Iwal", "kbm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kbm", "Iwal"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Iwam", "iwm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("iwm", "Iwam"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Iwur", "iwo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("iwo", "Iwur"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ixcatec", "ixc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ixc", "Ixcatec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ixcatlán Mazatec", "mzi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mzi", "Ixcatlán Mazatec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ixil", "ixl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ixl", "Ixil"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ixtayutla Mixtec", "vmj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vmj", "Ixtayutla Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ixtenco Otomi", "otz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("otz", "Ixtenco Otomi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Iyayu", "iya"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("iya", "Iyayu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Iyive", "uiv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("uiv", "Iyive"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Iyo", "nca"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nca", "Iyo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Iyo'wujwa Chorote", "crq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("crq", "Iyo'wujwa Chorote"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Iyojwa'ja Chorote", "crt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("crt", "Iyojwa'ja Chorote"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Izere", "izr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("izr", "Izere"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Izon", "ijc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ijc", "Izon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Izora", "cbo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cbo", "Izora"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Iñapari", "inp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("inp", "Iñapari"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jabutí", "jbt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jbt", "Jabutí"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jad", "jda"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jda", "Jad"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jadgali", "jdg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jdg", "Jadgali"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jah Hut", "jah"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jah", "Jah Hut"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jahanka", "jad"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jad", "Jahanka"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jair Awyu", "awv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("awv", "Jair Awyu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jakalteko", "jac"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jac", "Jakalteko"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jakati", "jat"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jat", "Jakati"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jakun", "jak"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jak", "Jakun"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jalapa De Díaz Mazatec", "maj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("maj", "Jalapa De Díaz Mazatec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jalkunan", "bxl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bxl", "Jalkunan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jamaican Country Sign Language", "jcs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jcs", "Jamaican Country Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jamaican Creole English", "jam"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jam", "Jamaican Creole English"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jamaican Sign Language", "jls"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jls", "Jamaican Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jamamadí", "jaa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jaa", "Jamamadí"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jambi Malay", "jax"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jax", "Jambi Malay"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jamiltepec Mixtec", "mxt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mxt", "Jamiltepec Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jamsay Dogon", "djm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("djm", "Jamsay Dogon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jandavra", "jnd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jnd", "Jandavra"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jangkang", "djo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("djo", "Jangkang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jangshung", "jna"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jna", "Jangshung"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Janji", "jni"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jni", "Janji"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Japanese", "jpn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jpn", "Japanese"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Japanese Sign Language", "jsl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jsl", "Japanese Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Japrería", "jru"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jru", "Japrería"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jaqaru", "jqr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jqr", "Jaqaru"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jara", "jaf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jaf", "Jara"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jarai", "jra"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jra", "Jarai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jarawa (India)", "anq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("anq", "Jarawa (India)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jarawa (Nigeria)", "jar"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jar", "Jarawa (Nigeria)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jaru", "ddj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ddj", "Jaru"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jauja Wanca Quechua", "qxw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("qxw", "Jauja Wanca Quechua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jaunsari", "jns"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jns", "Jaunsari"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Javanese", "jav"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jav", "Javanese"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Javindo", "jvd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jvd", "Javindo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jawe", "jaz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jaz", "Jawe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jaya", "jyy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jyy", "Jaya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jebero", "jeb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jeb", "Jebero"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jeh", "jeh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jeh", "Jeh"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jehai", "jhi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jhi", "Jehai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jemez", "tow"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tow", "Jemez"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jenaama Bozo", "bze"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bze", "Jenaama Bozo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jeng", "jeg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jeg", "Jeng"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jennu Kurumba", "xuj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xuj", "Jennu Kurumba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jere", "jer"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jer", "Jere"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jeri Kuo", "jek"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jek", "Jeri Kuo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jerung", "jee"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jee", "Jerung"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jewish Babylonian Aramaic (ca. 200-1200 CE)", "tmr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tmr", "Jewish Babylonian Aramaic (ca. 200-1200 CE)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jewish Palestinian Aramaic", "jpa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jpa", "Jewish Palestinian Aramaic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jhankot Sign Language", "jhs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jhs", "Jhankot Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jiamao", "jio"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jio", "Jiamao"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jiarong", "jya"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jya", "Jiarong"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jiba", "juo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("juo", "Jiba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jibu", "jib"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jib", "Jibu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jicarilla Apache", "apj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("apj", "Jicarilla Apache"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jiiddu", "jii"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jii", "Jiiddu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jilbe", "jie"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jie", "Jilbe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jilim", "jil"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jil", "Jilim"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jimi (Cameroon)", "jim"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jim", "Jimi (Cameroon)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jimi (Nigeria)", "jmi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jmi", "Jimi (Nigeria)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jina", "jia"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jia", "Jina"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jingpho", "kac"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kac", "Jingpho"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jinyu Chinese", "cjy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cjy", "Jinyu Chinese"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jiongnai Bunu", "pnu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pnu", "Jiongnai Bunu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jirel", "jul"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jul", "Jirel"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jiru", "jrr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jrr", "Jiru"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jita", "jit"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jit", "Jita"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jju", "kaj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kaj", "Jju"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Joba", "job"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("job", "Joba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jofotek-Bromnya", "jbr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jbr", "Jofotek-Bromnya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jola-Fonyi", "dyo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dyo", "Jola-Fonyi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jola-Kasa", "csk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("csk", "Jola-Kasa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jonkor Bourmataguil", "jeu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jeu", "Jonkor Bourmataguil"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jordanian Sign Language", "jos"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jos", "Jordanian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jorto", "jrt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jrt", "Jorto"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jorá", "jor"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jor", "Jorá"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jowulu", "jow"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jow", "Jowulu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ju", "juu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("juu", "Ju"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ju/'hoan", "ktz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ktz", "Ju/'hoan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Juang", "jun"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jun", "Juang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Judeo-Arabic", "jrb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jrb", "Judeo-Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Judeo-Berber", "jbe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jbe", "Judeo-Berber"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Judeo-Georgian", "jge"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jge", "Judeo-Georgian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Judeo-Iraqi Arabic", "yhd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yhd", "Judeo-Iraqi Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Judeo-Italian", "itk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("itk", "Judeo-Italian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Judeo-Moroccan Arabic", "aju"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aju", "Judeo-Moroccan Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Judeo-Persian", "jpr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jpr", "Judeo-Persian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Judeo-Tat", "jdt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jdt", "Judeo-Tat"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Judeo-Tripolitanian Arabic", "yud"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yud", "Judeo-Tripolitanian Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Judeo-Tunisian Arabic", "ajt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ajt", "Judeo-Tunisian Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Judeo-Yemeni Arabic", "jye"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jye", "Judeo-Yemeni Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jukun Takum", "jbu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jbu", "Jukun Takum"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jumjum", "jum"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jum", "Jumjum"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jumla Sign Language", "jus"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jus", "Jumla Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jumli", "jml"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jml", "Jumli"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jungle Inga", "inj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("inj", "Jungle Inga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Juquila Mixe", "mxq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mxq", "Juquila Mixe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jur Modo", "bex"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bex", "Jur Modo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Juray", "juy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("juy", "Juray"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jurchen", "juc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("juc", "Jurchen"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jurúna", "jur"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jur", "Jurúna"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jutish", "jut"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jut", "Jutish"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Juwal", "mwb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mwb", "Juwal"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Juxtlahuaca Mixtec", "vmc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vmc", "Juxtlahuaca Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Jwira-Pepesa", "jwi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jwi", "Jwira-Pepesa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Júma", "jua"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jua", "Júma"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("K'iche'", "quc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("quc", "K'iche'"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kaamba", "xku"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xku", "Kaamba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kaan", "ldl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ldl", "Kaan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kaansa", "gna"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gna", "Kaansa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kaapor", "urb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("urb", "Kaapor"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kaapor Sign Language", "uks"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("uks", "Kaapor Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kaba", "ksp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ksp", "Kaba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kabalai", "kvf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kvf", "Kabalai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kabardian", "kbd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kbd", "Kabardian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kabatei", "xkp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xkp", "Kabatei"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kabixí", "xbx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xbx", "Kabixí"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kabiyè", "kbp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kbp", "Kabiyè"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kabola", "klz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("klz", "Kabola"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kabore One", "onk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("onk", "Kabore One"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kabras", "lkb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lkb", "Kabras"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kaburi", "uka"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("uka", "Kaburi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kabutra", "kbu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kbu", "Kabutra"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kabuverdianu", "kea"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kea", "Kabuverdianu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kabwa", "cwa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cwa", "Kabwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kabwari", "kcw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kcw", "Kabwari"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kabyle", "kab"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kab", "Kabyle"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kachama-Ganjule", "kcx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kcx", "Kachama-Ganjule"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kachari", "xac"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xac", "Kachari"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kachchi", "kfr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kfr", "Kachchi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kachi Koli", "gjk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gjk", "Kachi Koli"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kachin", "kac"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kac", "Kachin"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kacipo-Balesi", "koe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("koe", "Kacipo-Balesi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kaco'", "xkk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xkk", "Kaco'"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kadai", "kzd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kzd", "Kadai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kadar", "kej"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kej", "Kadar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kadara", "kad"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kad", "Kadara"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kadaru", "kdu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kdu", "Kadaru"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kadiwéu", "kbc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kbc", "Kadiwéu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kado", "kdv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kdv", "Kado"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kaduo", "ktp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ktp", "Kaduo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kafa", "kbr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kbr", "Kafa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kafoa", "kpu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kpu", "Kafoa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kagan Kalagan", "kll"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kll", "Kagan Kalagan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kagate", "syw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("syw", "Kagate"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kagayanen", "cgc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cgc", "Kagayanen"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kagoma", "kdm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kdm", "Kagoma"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kagoro", "xkg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xkg", "Kagoro"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kagulu", "kki"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kki", "Kagulu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kahe", "hka"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hka", "Kahe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kahua", "agw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("agw", "Kahua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kaian", "kct"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kct", "Kaian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kaibobo", "kzb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kzb", "Kaibobo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kaidipang", "kzp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kzp", "Kaidipang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kaiep", "kbw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kbw", "Kaiep"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kaikadi", "kep"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kep", "Kaikadi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kaike", "kzq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kzq", "Kaike"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kaiku", "kkq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kkq", "Kaiku"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kaimbulawa", "zka"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zka", "Kaimbulawa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kaimbé", "xai"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xai", "Kaimbé"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kaingang", "kgp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kgp", "Kaingang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kairak", "ckr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ckr", "Kairak"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kairiru", "kxa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kxa", "Kairiru"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kairui-Midiki", "krd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("krd", "Kairui-Midiki"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kais", "kzm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kzm", "Kais"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kaivi", "kce"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kce", "Kaivi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kaiwá", "kgk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kgk", "Kaiwá"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kaiy", "tcq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tcq", "Kaiy"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kajakse", "ckq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ckq", "Kajakse"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kajali", "xkj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xkj", "Kajali"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kajaman", "kag"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kag", "Kajaman"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kakabai", "kqf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kqf", "Kakabai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kakabe", "kke"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kke", "Kakabe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kakanda", "kka"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kka", "Kakanda"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kakauhua", "kbf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kbf", "Kakauhua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kaki Ae", "tbd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tbd", "Kaki Ae"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kakihum", "kxe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kxe", "Kakihum"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kako", "kkj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kkj", "Kako"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kakwa", "keo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("keo", "Kakwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kala Lagaw Ya", "mwp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mwp", "Kala Lagaw Ya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kalaallisut", "kal"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kal", "Kalaallisut"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kalabakan", "kve"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kve", "Kalabakan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kalabari", "ijn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ijn", "Kalabari"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kalabra", "kzz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kzz", "Kalabra"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kalagan", "kqe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kqe", "Kalagan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kalaktang Monpa", "kkf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kkf", "Kalaktang Monpa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kalam", "kmh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kmh", "Kalam"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kalami", "gwc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gwc", "Kalami"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kalamsé", "knz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("knz", "Kalamsé"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kalanadi", "wkl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wkl", "Kalanadi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kalanga", "kck"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kck", "Kalanga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kalao", "kly"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kly", "Kalao"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kalapuya", "kyl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kyl", "Kalapuya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kalarko", "kba"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kba", "Kalarko"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kalasha", "kls"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kls", "Kalasha"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kalenjin", "kln"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kln", "Kalenjin"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kalispel-Pend d'Oreille", "fla"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("fla", "Kalispel-Pend d'Oreille"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kalkoti", "xka"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xka", "Kalkoti"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kalkutung", "ktg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ktg", "Kalkutung"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kalmyk", "xal"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xal", "Kalmyk"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kalo Finnish Romani", "rmf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rmf", "Kalo Finnish Romani"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kalou", "ywa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ywa", "Kalou"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kaluli", "bco"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bco", "Kaluli"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kalumpang", "kli"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kli", "Kalumpang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kam", "kdx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kdx", "Kam"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kamakan", "vkm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vkm", "Kamakan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kamang", "woi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("woi", "Kamang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kamano", "kbq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kbq", "Kamano"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kamantan", "kci"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kci", "Kamantan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kamar", "keq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("keq", "Kamar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kamara", "jmr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jmr", "Kamara"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kamarian", "kzx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kzx", "Kamarian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kamaru", "kgx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kgx", "Kamaru"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kamas", "xas"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xas", "Kamas"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kamasa", "klp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("klp", "Kamasa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kamasau", "kms"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kms", "Kamasau"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kamayo", "kyk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kyk", "Kamayo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kamayurá", "kay"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kay", "Kamayurá"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kamba (Brazil)", "xba"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xba", "Kamba (Brazil)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kamba (Kenya)", "kam"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kam", "Kamba (Kenya)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kambaata", "ktb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ktb", "Kambaata"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kambaira", "kyy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kyy", "Kambaira"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kambera", "xbr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xbr", "Kambera"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kamberau", "irx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("irx", "Kamberau"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kambiwá", "xbw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xbw", "Kambiwá"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kami (Nigeria)", "kmi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kmi", "Kami (Nigeria)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kami (Tanzania)", "kcu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kcu", "Kami (Tanzania)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kamo", "kcq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kcq", "Kamo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kamoro", "kgq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kgq", "Kamoro"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kamta", "rkt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rkt", "Kamta"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kamu", "xmu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xmu", "Kamu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kamula", "xla"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xla", "Kamula"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kamviri", "xvi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xvi", "Kamviri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kamwe", "hig"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hig", "Kamwe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kanakanabu", "xnb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xnb", "Kanakanabu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kanamarí", "knm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("knm", "Kanamarí"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kanashi", "xns"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xns", "Kanashi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kanasi", "soq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("soq", "Kanasi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kanauji", "bjj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bjj", "Kanauji"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kandas", "kqw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kqw", "Kandas"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kandawo", "gam"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gam", "Kandawo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kande", "kbs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kbs", "Kande"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kanembu", "kbl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kbl", "Kanembu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kang", "kyp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kyp", "Kang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kanga", "kcp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kcp", "Kanga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kangean", "kkv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kkv", "Kangean"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kanggape", "igm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("igm", "Kanggape"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kangjia", "kxs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kxs", "Kangjia"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kango (Bas-Uélé District)", "kty"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kty", "Kango (Bas-Uélé District)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kango (Tshopo District)", "kzy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kzy", "Kango (Tshopo District)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kangri", "xnr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xnr", "Kangri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kaniet", "ktk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ktk", "Kaniet"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kanikkaran", "kev"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kev", "Kanikkaran"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kaningi", "kzo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kzo", "Kaningi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kaningra", "knr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("knr", "Kaningra"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kaninuwa", "wat"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wat", "Kaninuwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kanite", "kmu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kmu", "Kanite"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kanjari", "kft"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kft", "Kanjari"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kanjobal", "kjb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kjb", "Kanjobal"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kanju", "kbe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kbe", "Kanju"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kankanaey", "kne"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kne", "Kankanaey"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kannada", "kan"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kan", "Kannada"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kannada Kurumba", "kfi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kfi", "Kannada Kurumba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kanowit-Tanjong Melanau", "kxn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kxn", "Kanowit-Tanjong Melanau"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kanoé", "kxo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kxo", "Kanoé"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kansa", "ksk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ksk", "Kansa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kantosi", "xkt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xkt", "Kantosi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kanu", "khx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("khx", "Kanu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kanufi", "kni"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kni", "Kanufi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kanuri", "kau"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kau", "Kanuri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kanyok", "kny"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kny", "Kanyok"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kao", "kax"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kax", "Kao"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kaonde", "kqn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kqn", "Kaonde"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kap", "ykm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ykm", "Kap"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kapampangan", "pam"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pam", "Kapampangan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kapauri", "khp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("khp", "Kapauri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kapin", "tbx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tbx", "Kapin"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kapinawá", "xpn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xpn", "Kapinawá"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kapingamarangi", "kpg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kpg", "Kapingamarangi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kapori", "khp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("khp", "Kapori"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kapriman", "dju"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dju", "Kapriman"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kaptiau", "kbi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kbi", "Kaptiau"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kapya", "klo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("klo", "Kapya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kaqchikel", "cak"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cak", "Kaqchikel"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kara (Central African Republic)", "kah"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kah", "Kara (Central African Republic)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kara (Korea)", "zra"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zra", "Kara (Korea)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kara (Papua New Guinea)", "leu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("leu", "Kara (Papua New Guinea)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kara (Tanzania)", "reg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("reg", "Kara (Tanzania)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kara-Kalpak", "kaa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kaa", "Kara-Kalpak"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Karachay-Balkar", "krc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("krc", "Karachay-Balkar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Karadjeri", "gbd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gbd", "Karadjeri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Karagas", "kim"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kim", "Karagas"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Karahawyana", "xkh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xkh", "Karahawyana"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Karaim", "kdr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kdr", "Karaim"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Karajá", "kpj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kpj", "Karajá"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Karakhanid", "xqa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xqa", "Karakhanid"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Karami", "xar"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xar", "Karami"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Karamojong", "kdj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kdj", "Karamojong"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Karang", "kzr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kzr", "Karang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Karanga", "kth"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kth", "Karanga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Karankawa", "zkk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zkk", "Karankawa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Karao", "kyj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kyj", "Karao"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Karas", "kgv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kgv", "Karas"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Karata", "kpt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kpt", "Karata"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Karawa", "xrw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xrw", "Karawa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Karbi", "mjw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mjw", "Karbi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kare (Central African Republic)", "kbn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kbn", "Kare (Central African Republic)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kare (Papua New Guinea)", "kmf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kmf", "Kare (Papua New Guinea)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Karekare", "kai"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kai", "Karekare"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Karelian", "krl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("krl", "Karelian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Karey", "kyd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kyd", "Karey"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kari", "kbj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kbj", "Kari"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Karingani", "kgn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kgn", "Karingani"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Karipuna", "kuq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kuq", "Karipuna"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Karipúna", "kgm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kgm", "Karipúna"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Karipúna Creole French", "kmv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kmv", "Karipúna Creole French"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Karirí-Xocó", "kzw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kzw", "Karirí-Xocó"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Karitiâna", "ktn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ktn", "Karitiâna"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kariya", "kil"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kil", "Kariya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kariyarra", "vka"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vka", "Kariyarra"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Karkar-Yuri", "yuj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yuj", "Karkar-Yuri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Karkin", "krb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("krb", "Karkin"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Karko", "kko"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kko", "Karko"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Karnai", "bbv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bbv", "Karnai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Karo (Brazil)", "arr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("arr", "Karo (Brazil)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Karo (Ethiopia)", "kxh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kxh", "Karo (Ethiopia)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Karok", "kyh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kyh", "Karok"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Karon", "krx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("krx", "Karon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Karon Dori", "kgw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kgw", "Karon Dori"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Karore", "xkx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xkx", "Karore"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kasanga", "ccj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ccj", "Kasanga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kasem", "xsm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xsm", "Kasem"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kashaya", "kju"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kju", "Kashaya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kashmiri", "kas"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kas", "Kashmiri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kashubian", "csb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("csb", "Kashubian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kasiguranin", "ksn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ksn", "Kasiguranin"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kaska", "kkz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kkz", "Kaska"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kaskean", "zsk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zsk", "Kaskean"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kasseng", "kgc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kgc", "Kasseng"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kasua", "khs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("khs", "Kasua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kataang", "kgd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kgd", "Kataang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Katabaga", "ktq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ktq", "Katabaga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Katawixi", "xat"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xat", "Katawixi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Katbol", "tmb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tmb", "Katbol"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kathoriya Tharu", "tkt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tkt", "Kathoriya Tharu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kathu", "ykt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ykt", "Kathu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kati", "bsh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bsh", "Kati"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Katkari", "kfu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kfu", "Katkari"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Katla", "kcr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kcr", "Katla"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kato", "ktw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ktw", "Kato"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Katso", "kaf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kaf", "Katso"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Katua", "kta"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kta", "Katua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Katukína", "kav"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kav", "Katukína"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kaulong", "pss"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pss", "Kaulong"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kaur", "vkk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vkk", "Kaur"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kaure", "bpp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bpp", "Kaure"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kaurna", "zku"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zku", "Kaurna"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kauwera", "xau"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xau", "Kauwera"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kavalan", "ckv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ckv", "Kavalan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kavet", "krv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("krv", "Kavet"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kawacha", "kcb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kcb", "Kawacha"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kawaiisu", "xaw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xaw", "Kawaiisu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kawe", "kgb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kgb", "Kawe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kawi", "kaw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kaw", "Kawi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kaxararí", "ktx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ktx", "Kaxararí"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kaxuiâna", "kbb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kbb", "Kaxuiâna"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kayabí", "kyz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kyz", "Kayabí"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kayagar", "kyt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kyt", "Kayagar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kayan", "pdu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pdu", "Kayan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kayan Mahakam", "xay"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xay", "Kayan Mahakam"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kayan River Kayan", "xkn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xkn", "Kayan River Kayan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kayapa Kallahan", "kak"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kak", "Kayapa Kallahan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kayapó", "txu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("txu", "Kayapó"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kayardild", "gyd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gyd", "Kayardild"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kayeli", "kzl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kzl", "Kayeli"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kayong", "kxy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kxy", "Kayong"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kayort", "kyv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kyv", "Kayort"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kaytetye", "gbb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gbb", "Kaytetye"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kayupulau", "kzu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kzu", "Kayupulau"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kazakh", "kaz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kaz", "Kazakh"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kazukuru", "kzk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kzk", "Kazukuru"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ke'o", "xxk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xxk", "Ke'o"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Keak", "keh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("keh", "Keak"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Keapara", "khz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("khz", "Keapara"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kedah Malay", "meo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("meo", "Kedah Malay"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kedang", "ksx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ksx", "Kedang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Keder", "kdy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kdy", "Keder"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kehu", "khh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("khh", "Kehu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kei", "kei"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kei", "Kei"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Keiga", "kec"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kec", "Keiga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Keijar", "kdy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kdy", "Keijar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kein", "bmh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bmh", "Kein"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Keiyo", "eyo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("eyo", "Keiyo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kekchí", "kek"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kek", "Kekchí"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kela (Democratic Republic of Congo)", "kel"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kel", "Kela (Democratic Republic of Congo)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kela (Papua New Guinea)", "kcl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kcl", "Kela (Papua New Guinea)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kelabit", "kzi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kzi", "Kelabit"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kele (Democratic Republic of Congo)", "khy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("khy", "Kele (Democratic Republic of Congo)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kele (Papua New Guinea)", "sbc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sbc", "Kele (Papua New Guinea)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Keley-I Kallahan", "ify"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ify", "Keley-I Kallahan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Keliko", "kbo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kbo", "Keliko"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kelo", "xel"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xel", "Kelo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kelon", "kyo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kyo", "Kelon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kemak", "kem"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kem", "Kemak"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kembayan", "xem"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xem", "Kembayan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kemberano", "bzp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bzp", "Kemberano"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kembra", "xkw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xkw", "Kembra"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kemezung", "dmo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dmo", "Kemezung"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kemi Sami", "sjk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sjk", "Kemi Sami"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kemiehua", "kfj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kfj", "Kemiehua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kemtuik", "kmt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kmt", "Kemtuik"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kenaboi", "xbn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xbn", "Kenaboi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kenati", "gat"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gat", "Kenati"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kendayan", "knx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("knx", "Kendayan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kendeje", "klf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("klf", "Kendeje"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kendem", "kvm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kvm", "Kendem"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kenga", "kyq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kyq", "Kenga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Keningau Murut", "kxi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kxi", "Keningau Murut"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Keninjal", "knl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("knl", "Keninjal"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kensiu", "kns"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kns", "Kensiu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kenswei Nsei", "ndb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ndb", "Kenswei Nsei"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kenuzi-Dongola", "kzh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kzh", "Kenuzi-Dongola"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kenyan Sign Language", "xki"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xki", "Kenyan Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kenyang", "ken"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ken", "Kenyang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kenyi", "lke"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lke", "Kenyi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Keoru-Ahia", "xeu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xeu", "Keoru-Ahia"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kepkiriwát", "kpn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kpn", "Kepkiriwát"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kepo'", "kuk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kuk", "Kepo'"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kera", "ker"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ker", "Kera"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kerak", "hhr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hhr", "Kerak"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kereho", "xke"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xke", "Kereho"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kerek", "krk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("krk", "Kerek"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kerewe", "ked"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ked", "Kerewe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kerewo", "kxz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kxz", "Kerewo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kerinci", "kvr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kvr", "Kerinci"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kesawai", "xes"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xes", "Kesawai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ket", "ket"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ket", "Ket"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ketangalan", "kae"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kae", "Ketangalan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kete", "kcv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kcv", "Kete"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ketengban", "xte"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xte", "Ketengban"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ketum", "ktt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ktt", "Ketum"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Keyagana", "kyg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kyg", "Keyagana"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kgalagadi", "xkv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xkv", "Kgalagadi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Khakas", "kjh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kjh", "Khakas"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Khalaj", "kjf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kjf", "Khalaj"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Khaling", "klr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("klr", "Khaling"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Khamba", "kbg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kbg", "Khamba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Khams Tibetan", "khg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("khg", "Khams Tibetan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Khamti", "kht"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kht", "Khamti"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Khamyang", "ksu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ksu", "Khamyang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Khana", "ogo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ogo", "Khana"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Khandesi", "khn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("khn", "Khandesi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Khanty", "kca"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kca", "Khanty"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Khao", "xao"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xao", "Khao"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kharam Naga", "kfw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kfw", "Kharam Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kharia", "khr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("khr", "Kharia"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kharia Thar", "ksy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ksy", "Kharia Thar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Khasi", "kha"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kha", "Khasi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Khayo", "lko"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lko", "Khayo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Khazar", "zkz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zkz", "Khazar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Khe", "kqg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kqg", "Khe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Khehek", "tlx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tlx", "Khehek"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Khengkha", "xkf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xkf", "Khengkha"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Khetrani", "xhe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xhe", "Khetrani"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Khezha Naga", "nkh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nkh", "Khezha Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Khiamniungan Naga", "kix"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kix", "Khiamniungan Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Khinalugh", "kjj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kjj", "Khinalugh"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Khirwar", "kwx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kwx", "Khirwar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Khisa", "kqm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kqm", "Khisa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Khlor", "llo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("llo", "Khlor"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Khlula", "ykl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ykl", "Khlula"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Khmu", "kjg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kjg", "Khmu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kho'ini", "xkc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xkc", "Kho'ini"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Khoibu Naga", "nkb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nkb", "Khoibu Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kholok", "ktc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ktc", "Kholok"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Khonso", "kxc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kxc", "Khonso"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Khorasani Turkish", "kmz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kmz", "Khorasani Turkish"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Khorezmian", "zkh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zkh", "Khorezmian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Khotanese", "kho"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kho", "Khotanese"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Khowar", "khw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("khw", "Khowar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Khua", "xhv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xhv", "Khua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Khuen", "khf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("khf", "Khuen"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Khumi Awa Chin", "cka"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cka", "Khumi Awa Chin"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Khumi Chin", "cnk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cnk", "Khumi Chin"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Khunsari", "kfm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kfm", "Khunsari"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Khvarshi", "khv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("khv", "Khvarshi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kháng", "kjm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kjm", "Kháng"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Khün", "kkh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kkh", "Khün"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kibet", "kie"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kie", "Kibet"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kibiri", "prm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("prm", "Kibiri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kickapoo", "kic"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kic", "Kickapoo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kiembu", "ebu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ebu", "Kiembu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kigiryama", "nyf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nyf", "Kigiryama"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kikai", "kzg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kzg", "Kikai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kikuyu", "kik"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kik", "Kikuyu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kildin Sami", "sjd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sjd", "Kildin Sami"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kilivila", "kij"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kij", "Kilivila"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kiliwa", "klb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("klb", "Kiliwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kilmeri", "kih"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kih", "Kilmeri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kim", "kia"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kia", "Kim"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kim Mun", "mji"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mji", "Kim Mun"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kimaama", "kig"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kig", "Kimaama"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kimaghima", "kig"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kig", "Kimaghima"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kimaragang", "kqr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kqr", "Kimaragang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kimbu", "kiv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kiv", "Kimbu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kimbundu", "kmb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kmb", "Kimbundu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kimki", "sbt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sbt", "Kimki"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kimré", "kqp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kqp", "Kimré"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kinabalian", "cbw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cbw", "Kinabalian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kinalakna", "kco"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kco", "Kinalakna"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kinamiging Manobo", "mkx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mkx", "Kinamiging Manobo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kinaray-A", "krj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("krj", "Kinaray-A"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kinga", "zga"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zga", "Kinga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kinnauri", "kfk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kfk", "Kinnauri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kintaq", "knq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("knq", "Kintaq"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kinuku", "kkd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kkd", "Kinuku"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kinyarwanda", "kin"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kin", "Kinyarwanda"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kioko", "ues"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ues", "Kioko"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kiong", "kkm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kkm", "Kiong"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kiorr", "xko"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xko", "Kiorr"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kiowa", "kio"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kio", "Kiowa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kiowa Apache", "apk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("apk", "Kiowa Apache"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kipfokomo", "pkb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pkb", "Kipfokomo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kipsigis", "sgc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sgc", "Kipsigis"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kiput", "kyi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kyi", "Kiput"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kir-Balar", "kkr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kkr", "Kir-Balar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kirdki", "zza"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zza", "Kirdki"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kire", "geb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("geb", "Kire"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kirghiz", "kir"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kir", "Kirghiz"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kirike", "okr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("okr", "Kirike"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kirikiri", "kiy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kiy", "Kirikiri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kirmanjki (individual language)", "kiu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kiu", "Kirmanjki (individual language)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kirmanjki (macrolanguage)", "zza"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zza", "Kirmanjki (macrolanguage)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kis", "kis"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kis", "Kis"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kisa", "lks"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lks", "Kisa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kisankasa", "kqh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kqh", "Kisankasa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kisar", "kje"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kje", "Kisar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kisi", "kiz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kiz", "Kisi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kistane", "gru"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gru", "Kistane"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kiswahili", "swh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("swh", "Kiswahili"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kita Maninkakan", "mwk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mwk", "Kita Maninkakan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kitan", "zkt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zkt", "Kitan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kitharaka", "thk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("thk", "Kitharaka"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kitja", "gia"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gia", "Kitja"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kitsai", "kii"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kii", "Kitsai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kituba (Congo)", "mkw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mkw", "Kituba (Congo)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kituba (Democratic Republic of Congo)", "ktu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ktu", "Kituba (Democratic Republic of Congo)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kiwilwana", "mlk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mlk", "Kiwilwana"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Klamath-Modoc", "kla"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kla", "Klamath-Modoc"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Klao", "klu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("klu", "Klao"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Klias River Kadazan", "kqt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kqt", "Klias River Kadazan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Klingon", "tlh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tlh", "Klingon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Knaanic", "czk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("czk", "Knaanic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ko", "fuj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("fuj", "Ko"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Koalib", "kib"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kib", "Koalib"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Koasati", "cku"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cku", "Koasati"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Koba", "kpd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kpd", "Koba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kobiana", "kcj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kcj", "Kobiana"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kobol", "kgu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kgu", "Kobol"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kobon", "kpw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kpw", "Kobon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Koch", "kdq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kdq", "Koch"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kochila Tharu", "thq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("thq", "Kochila Tharu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Koda", "cdz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cdz", "Koda"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kodaku", "ksz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ksz", "Kodaku"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kodava", "kfa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kfa", "Kodava"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kodeoha", "vko"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vko", "Kodeoha"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kodi", "kod"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kod", "Kodi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kodia", "kwp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kwp", "Kodia"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Koenoem", "kcs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kcs", "Koenoem"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kofa", "kso"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kso", "Kofa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kofei", "kpi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kpi", "Kofei"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kofyar", "kwl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kwl", "Kofyar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kogi", "kog"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kog", "Kogi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Koguryo", "zkg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zkg", "Koguryo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kohin", "kkx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kkx", "Kohin"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kohistani Shina", "plk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("plk", "Kohistani Shina"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Koho", "kpm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kpm", "Koho"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kohumono", "bcs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bcs", "Kohumono"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Koi", "kkt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kkt", "Koi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Koibal", "zkb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zkb", "Koibal"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Koireng", "nkd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nkd", "Koireng"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Koitabu", "kqi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kqi", "Koitabu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Koiwat", "kxt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kxt", "Koiwat"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kok Borok", "trp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("trp", "Kok Borok"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kokata", "ktd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ktd", "Kokata"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Koke", "kou"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kou", "Koke"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kokoda", "xod"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xod", "Kokoda"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kokola", "kzn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kzn", "Kokola"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kokota", "kkk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kkk", "Kokota"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kol", "ekl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ekl", "Kol"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kol (Cameroon)", "biw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("biw", "Kol (Cameroon)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kol (Papua New Guinea)", "kol"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kol", "Kol (Papua New Guinea)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kola", "kvv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kvv", "Kola"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kolbila", "klc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("klc", "Kolbila"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kolibugan Subanon", "skn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("skn", "Kolibugan Subanon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kolum So Dogon", "dkl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dkl", "Kolum So Dogon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Koluwawa", "klx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("klx", "Koluwawa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kom (Cameroon)", "bkm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bkm", "Kom (Cameroon)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kom (India)", "kmm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kmm", "Kom (India)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Koma", "kmy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kmy", "Koma"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Komba", "kpf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kpf", "Komba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kombai", "tyn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tyn", "Kombai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kombio", "xbi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xbi", "Kombio"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Komering", "kge"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kge", "Komering"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Komi", "kom"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kom", "Komi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Komi-Permyak", "koi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("koi", "Komi-Permyak"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Komi-Zyrian", "kpv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kpv", "Komi-Zyrian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kominimung", "xoi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xoi", "Kominimung"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Komo (Democratic Republic of Congo)", "kmw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kmw", "Komo (Democratic Republic of Congo)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Komo (Sudan)", "xom"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xom", "Komo (Sudan)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Komodo", "kvh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kvh", "Komodo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kompane", "kvp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kvp", "Kompane"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Komyandaret", "kzv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kzv", "Komyandaret"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kon Keu", "kkn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kkn", "Kon Keu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Konabéré", "bbo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bbo", "Konabéré"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Konai", "kxw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kxw", "Konai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Konda", "knd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("knd", "Konda"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Konda-Dora", "kfc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kfc", "Konda-Dora"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Koneraw", "kdw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kdw", "Koneraw"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kongo", "kon"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kon", "Kongo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Konkani (individual language)", "knn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("knn", "Konkani (individual language)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Konkani (macrolanguage)", "kok"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kok", "Konkani (macrolanguage)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Konkomba", "xon"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xon", "Konkomba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Konni", "kma"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kma", "Konni"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kono (Guinea)", "knu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("knu", "Kono (Guinea)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kono (Nigeria)", "klk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("klk", "Kono (Nigeria)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kono (Sierra Leone)", "kno"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kno", "Kono (Sierra Leone)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Konomala", "koa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("koa", "Konomala"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Konongo", "kcz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kcz", "Konongo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Konso", "kxc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kxc", "Konso"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Konyak Naga", "nbe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nbe", "Konyak Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Konyanka Maninka", "mku"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mku", "Konyanka Maninka"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Konzo", "koo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("koo", "Konzo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Koongo", "kng"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kng", "Koongo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Koonzime", "ozm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ozm", "Koonzime"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Koorete", "kqy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kqy", "Koorete"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kopar", "xop"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xop", "Kopar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kopkaka", "opk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("opk", "Kopkaka"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Korafe-Yegha", "kpr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kpr", "Korafe-Yegha"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Korak", "koz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("koz", "Korak"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Korana", "kqz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kqz", "Korana"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Korandje", "kcy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kcy", "Korandje"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Korean", "kor"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kor", "Korean"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Korean Sign Language", "kvk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kvk", "Korean Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Koreguaje", "coe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("coe", "Koreguaje"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Koresh-e Rostam", "okh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("okh", "Koresh-e Rostam"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Korku", "kfq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kfq", "Korku"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Korlai Creole Portuguese", "vkp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vkp", "Korlai Creole Portuguese"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Koro (Côte d'Ivoire)", "kfo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kfo", "Koro (Côte d'Ivoire)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Koro (Papua New Guinea)", "kxr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kxr", "Koro (Papua New Guinea)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Koro (Vanuatu)", "krf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("krf", "Koro (Vanuatu)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Koromfé", "kfz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kfz", "Koromfé"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Koromira", "kqj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kqj", "Koromira"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Koronadal Blaan", "bpr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bpr", "Koronadal Blaan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Koroni", "xkq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xkq", "Koroni"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Korop", "krp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("krp", "Korop"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Koropó", "xxr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xxr", "Koropó"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Koroshi", "ktl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ktl", "Koroshi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Korowai", "khe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("khe", "Korowai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Korra Koraga", "kfd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kfd", "Korra Koraga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Korubo", "xor"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xor", "Korubo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Korupun-Sela", "kpq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kpq", "Korupun-Sela"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Korur", "auq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("auq", "Korur"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Korwa", "kfp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kfp", "Korwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Koryak", "kpy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kpy", "Koryak"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kosadle", "kiq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kiq", "Kosadle"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kosare", "kiq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kiq", "Kosare"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kosarek Yale", "kkl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kkl", "Kosarek Yale"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kosena", "kze"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kze", "Kosena"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Koshin", "kid"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kid", "Koshin"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kosraean", "kos"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kos", "Kosraean"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kota (Gabon)", "koq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("koq", "Kota (Gabon)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kota (India)", "kfe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kfe", "Kota (India)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kota Bangun Kutai Malay", "mqg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mqg", "Kota Bangun Kutai Malay"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kota Marudu Talantang", "grm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("grm", "Kota Marudu Talantang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kota Marudu Tinagas", "ktr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ktr", "Kota Marudu Tinagas"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kotafon Gbe", "kqk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kqk", "Kotafon Gbe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kotava", "avk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("avk", "Kotava"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Koti", "eko"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("eko", "Koti"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kott", "zko"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zko", "Kott"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kouya", "kyf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kyf", "Kouya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kovai", "kqb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kqb", "Kovai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kove", "kvc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kvc", "Kove"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kowaki", "xow"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xow", "Kowaki"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kowiai", "kwh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kwh", "Kowiai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Koy Sanjaq Surat", "kqd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kqd", "Koy Sanjaq Surat"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Koya", "kff"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kff", "Koya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Koyaga", "kga"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kga", "Koyaga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Koyo", "koh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("koh", "Koyo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Koyra Chiini Songhay", "khq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("khq", "Koyra Chiini Songhay"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Koyraboro Senni Songhai", "ses"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ses", "Koyraboro Senni Songhai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Koyukon", "koy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("koy", "Koyukon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kpagua", "kuw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kuw", "Kpagua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kpala", "kpl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kpl", "Kpala"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kpan", "kpk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kpk", "Kpan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kpasam", "pbn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pbn", "Kpasam"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kpati", "koc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("koc", "Kpati"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kpatili", "kym"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kym", "Kpatili"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kpelle", "kpe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kpe", "Kpelle"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kpessi", "kef"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kef", "Kpessi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kplang", "kph"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kph", "Kplang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Krache", "kye"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kye", "Krache"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Krahô", "xra"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xra", "Krahô"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kraol", "rka"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rka", "Kraol"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Krenak", "kqq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kqq", "Krenak"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Krevinian", "zkv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zkv", "Krevinian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kreye", "xre"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xre", "Kreye"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Krikati-Timbira", "xri"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xri", "Krikati-Timbira"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Krim", "krm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("krm", "Krim"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Krio", "kri"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kri", "Krio"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kriol", "rop"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rop", "Kriol"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Krisa", "ksi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ksi", "Krisa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Krobu", "kxb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kxb", "Krobu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Krongo", "kgo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kgo", "Krongo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kru'ng 2", "krr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("krr", "Kru'ng 2"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Krymchak", "jct"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jct", "Krymchak"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kryts", "kry"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kry", "Kryts"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kua", "tyu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tyu", "Kua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kuan", "uan"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("uan", "Kuan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kuanhua", "xnh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xnh", "Kuanhua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kuanua", "ksd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ksd", "Kuanua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kuanyama", "kua"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kua", "Kuanyama"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kube", "kgf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kgf", "Kube"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kubi", "kof"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kof", "Kubi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kubo", "jko"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jko", "Kubo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kubu", "kvb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kvb", "Kubu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kucong", "lkc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lkc", "Kucong"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kudiya", "kfg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kfg", "Kudiya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kudmali", "kyw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kyw", "Kudmali"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kudu-Camo", "kov"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kov", "Kudu-Camo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kugama", "kow"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kow", "Kugama"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kugbo", "kes"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kes", "Kugbo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kui (India)", "kxu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kxu", "Kui (India)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kui (Indonesia)", "kvd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kvd", "Kui (Indonesia)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kuijau", "dkr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dkr", "Kuijau"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kujarge", "vkj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vkj", "Kujarge"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kuk", "kfn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kfn", "Kuk"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kukatja", "kux"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kux", "Kukatja"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kukele", "kez"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kez", "Kukele"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kukna", "kex"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kex", "Kukna"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kuku-Mangk", "xmq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xmq", "Kuku-Mangk"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kuku-Mu'inh", "xmp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xmp", "Kuku-Mu'inh"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kuku-Muminh", "xmh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xmh", "Kuku-Muminh"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kuku-Ugbanh", "ugb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ugb", "Kuku-Ugbanh"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kuku-Uwanh", "uwa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("uwa", "Kuku-Uwanh"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kuku-Yalanji", "gvn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gvn", "Kuku-Yalanji"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kula", "tpg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tpg", "Kula"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kulere", "kul"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kul", "Kulere"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kulfa", "kxj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kxj", "Kulfa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kulina", "cul"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cul", "Kulina"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kulina Pano", "xpk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xpk", "Kulina Pano"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kulisusu", "vkl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vkl", "Kulisusu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kullu Pahari", "kfx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kfx", "Kullu Pahari"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kulon-Pazeh", "uun"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("uun", "Kulon-Pazeh"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kulung (Nepal)", "kle"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kle", "Kulung (Nepal)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kulung (Nigeria)", "bbu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bbu", "Kulung (Nigeria)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kumak", "nee"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nee", "Kumak"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kumalu", "ksl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ksl", "Kumalu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kumam", "kdi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kdi", "Kumam"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kuman", "kue"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kue", "Kuman"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kuman (Russia)", "qwm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("qwm", "Kuman (Russia)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kumaoni", "kfy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kfy", "Kumaoni"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kumarbhag Paharia", "kmj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kmj", "Kumarbhag Paharia"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kumba", "ksm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ksm", "Kumba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kumbainggar", "kgs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kgs", "Kumbainggar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kumbaran", "wkb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wkb", "Kumbaran"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kumbewaha", "xks"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xks", "Kumbewaha"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kumhali", "kra"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kra", "Kumhali"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kumiai", "dih"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dih", "Kumiai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kumukio", "kuo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kuo", "Kumukio"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kumyk", "kum"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kum", "Kumyk"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kumzari", "zum"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zum", "Kumzari"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kunama", "kun"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kun", "Kunama"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kunbarlang", "wlg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wlg", "Kunbarlang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kunda", "kdn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kdn", "Kunda"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kundal Shahi", "shd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("shd", "Kundal Shahi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kunduvadi", "wku"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wku", "Kunduvadi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kung", "kfl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kfl", "Kung"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kung-Ekoka", "knw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("knw", "Kung-Ekoka"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kungarakany", "ggk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ggk", "Kungarakany"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kunggara", "kvs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kvs", "Kunggara"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kunggari", "kgl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kgl", "Kunggari"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kuni", "kse"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kse", "Kuni"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kuni-Boazi", "kvg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kvg", "Kuni-Boazi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kunigami", "xug"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xug", "Kunigami"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kunimaipa", "kup"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kup", "Kunimaipa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kunja", "pep"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pep", "Kunja"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kunjen", "kjn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kjn", "Kunjen"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kunyi", "njx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("njx", "Kunyi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kunza", "kuz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kuz", "Kunza"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kuo", "xuo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xuo", "Kuo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kuot", "kto"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kto", "Kuot"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kupa", "kug"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kug", "Kupa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kupang Malay", "mkn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mkn", "Kupang Malay"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kupia", "key"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("key", "Kupia"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kupsabiny", "kpz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kpz", "Kupsabiny"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kur", "kuv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kuv", "Kur"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kura Ede Nago", "nqk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nqk", "Kura Ede Nago"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kurama", "krh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("krh", "Kurama"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kuranko", "knk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("knk", "Kuranko"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kurdish", "kur"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kur", "Kurdish"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kuri", "nbn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nbn", "Kuri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kuria", "kuj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kuj", "Kuria"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kurichiya", "kfh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kfh", "Kurichiya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kurmukar", "kfv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kfv", "Kurmukar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kurrama", "vku"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vku", "Kurrama"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kurti", "ktm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ktm", "Kurti"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kurtokha", "xkz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xkz", "Kurtokha"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kurudu", "kjr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kjr", "Kurudu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kurukh", "kru"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kru", "Kurukh"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kuruáya", "kyr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kyr", "Kuruáya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kusaal", "kus"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kus", "Kusaal"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kusaghe", "ksg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ksg", "Kusaghe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kushi", "kuh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kuh", "Kushi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kusu", "ksv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ksv", "Kusu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kusunda", "kgg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kgg", "Kusunda"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kutang Ghale", "ght"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ght", "Kutang Ghale"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kutenai", "kut"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kut", "Kutenai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kutep", "kub"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kub", "Kutep"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kuthant", "xut"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xut", "Kuthant"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kutto", "kpa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kpa", "Kutto"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kutu", "kdc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kdc", "Kutu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kuturmi", "khj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("khj", "Kuturmi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kuuku-Ya'u", "kuy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kuy", "Kuuku-Ya'u"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kuvi", "kxv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kxv", "Kuvi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kuwaa", "blh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("blh", "Kuwaa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kuwaataay", "cwt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cwt", "Kuwaataay"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kuy", "kdt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kdt", "Kuy"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kven Finnish", "fkv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("fkv", "Kven Finnish"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kw'adza", "wka"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wka", "Kw'adza"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kwa", "kwb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kwb", "Kwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kwa'", "bko"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bko", "Kwa'"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kwaami", "ksq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ksq", "Kwaami"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kwadi", "kwz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kwz", "Kwadi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kwaio", "kwd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kwd", "Kwaio"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kwaja", "kdz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kdz", "Kwaja"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kwak", "kwq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kwq", "Kwak"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kwakiutl", "kwk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kwk", "Kwakiutl"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kwakum", "kwu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kwu", "Kwakum"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kwama", "kmq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kmq", "Kwama"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kwambi", "kwm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kwm", "Kwambi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kwamera", "tnk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tnk", "Kwamera"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kwami", "ktf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ktf", "Kwami"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kwamtim One", "okk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("okk", "Kwamtim One"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kwang", "kvi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kvi", "Kwang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kwanga", "kwj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kwj", "Kwanga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kwangali", "kwn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kwn", "Kwangali"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kwanja", "knp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("knp", "Kwanja"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kwanyama", "kua"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kua", "Kwanyama"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kwara'ae", "kwf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kwf", "Kwara'ae"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kwasio", "nmg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nmg", "Kwasio"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kwato", "kop"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kop", "Kwato"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kwaya", "kya"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kya", "Kwaya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kwaza", "xwa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xwa", "Kwaza"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kwegu", "xwg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xwg", "Kwegu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kwer", "kwr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kwr", "Kwer"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kwerba", "kwe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kwe", "Kwerba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kwerba Mamberamo", "xwr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xwr", "Kwerba Mamberamo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kwere", "cwe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cwe", "Kwere"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kwerisa", "kkb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kkb", "Kwerisa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kwese", "kws"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kws", "Kwese"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kwesten", "kwt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kwt", "Kwesten"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kwini", "gww"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gww", "Kwini"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kwinsu", "kuc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kuc", "Kwinsu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kwinti", "kww"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kww", "Kwinti"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kwoma", "kmo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kmo", "Kwoma"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kwomtari", "kwo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kwo", "Kwomtari"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kxoe", "xuu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xuu", "Kxoe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kyak", "bka"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bka", "Kyak"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kyaka", "kyc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kyc", "Kyaka"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kyenele", "kql"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kql", "Kyenele"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kyenga", "tye"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tye", "Kyenga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kyerung", "kgy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kgy", "Kyerung"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kyrgyz", "kir"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kir", "Kyrgyz"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kâte", "kmg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kmg", "Kâte"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kélé", "keb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("keb", "Kélé"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Kölsch", "ksh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ksh", "Kölsch"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("La'bi", "lbi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lbi", "La'bi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Laal", "gdm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gdm", "Laal"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Laalaa", "cae"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cae", "Laalaa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Laari", "ldi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ldi", "Laari"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Laba", "lau"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lau", "Laba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Label", "lbb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lbb", "Label"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Labir", "jku"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jku", "Labir"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Labo", "mwi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mwi", "Labo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Labo Phowa", "ypb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ypb", "Labo Phowa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Labu", "lbu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lbu", "Labu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Labuk-Kinabatangan Kadazan", "dtb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dtb", "Labuk-Kinabatangan Kadazan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lacandon", "lac"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lac", "Lacandon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lachi", "lbt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lbt", "Lachi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lachiguiri Zapotec", "zpa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zpa", "Lachiguiri Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lachixío Zapotec", "zpl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zpl", "Lachixío Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ladakhi", "lbj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lbj", "Ladakhi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ladin", "lld"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lld", "Ladin"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ladino", "lad"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lad", "Ladino"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Laeko-Libuat", "lkl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lkl", "Laeko-Libuat"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lafofa", "laf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("laf", "Lafofa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Laghu", "lgb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lgb", "Laghu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Laghuu", "lgh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lgh", "Laghuu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lagwan", "kot"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kot", "Lagwan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Laha (Indonesia)", "lhh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lhh", "Laha (Indonesia)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Laha (Viet Nam)", "lha"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lha", "Laha (Viet Nam)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lahanan", "lhn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lhn", "Lahanan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lahnda", "lah"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lah", "Lahnda"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lahta Karen", "kvt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kvt", "Lahta Karen"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lahu", "lhu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lhu", "Lahu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lahu Shi", "lhi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lhi", "Lahu Shi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lahul Lohar", "lhl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lhl", "Lahul Lohar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Laimbue", "lmx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lmx", "Laimbue"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Laiyolo", "lji"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lji", "Laiyolo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lak", "lbe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lbe", "Lak"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Laka (Chad)", "lap"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lap", "Laka (Chad)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Laka (Nigeria)", "lak"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lak", "Laka (Nigeria)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lakalei", "lka"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lka", "Lakalei"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lake Miwok", "lmw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lmw", "Lake Miwok"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lakha", "lkh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lkh", "Lakha"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Laki", "lki"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lki", "Laki"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lakkia", "lbc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lbc", "Lakkia"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lakon", "lkn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lkn", "Lakon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lakondê", "lkd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lkd", "Lakondê"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lakota", "lkt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lkt", "Lakota"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lakota Dida", "dic"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dic", "Lakota Dida"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lala", "nrz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nrz", "Lala"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lala-Bisa", "leb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("leb", "Lala-Bisa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lala-Roba", "lla"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lla", "Lala-Roba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lalana Chinantec", "cnl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cnl", "Lalana Chinantec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lalia", "lal"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lal", "Lalia"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lama (Myanmar)", "lay"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lay", "Lama (Myanmar)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lama (Togo)", "las"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("las", "Lama (Togo)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lamaholot", "slp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("slp", "Lamaholot"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lamalera", "lmr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lmr", "Lamalera"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lamam", "lmm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lmm", "Lamam"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lamang", "hia"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hia", "Lamang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lamatuka", "lmq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lmq", "Lamatuka"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lamba", "lam"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lam", "Lamba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lambadi", "lmn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lmn", "Lambadi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lambayeque Quechua", "quf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("quf", "Lambayeque Quechua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lambichhong", "lmh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lmh", "Lambichhong"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lamboya", "lmy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lmy", "Lamboya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lambya", "lai"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lai", "Lambya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lame", "bma"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bma", "Lame"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lamenu", "lmu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lmu", "Lamenu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lamet", "lbn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lbn", "Lamet"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lamkang", "lmk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lmk", "Lamkang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lamma", "lev"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lev", "Lamma"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lamnso'", "lns"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lns", "Lamnso'"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lamogai", "lmg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lmg", "Lamogai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lampung Api", "ljp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ljp", "Lampung Api"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lampung Nyo", "abl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("abl", "Lampung Nyo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lamu", "llh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("llh", "Lamu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lamu-Lamu", "lby"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lby", "Lamu-Lamu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lanas Lobu", "ruu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ruu", "Lanas Lobu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Landoma", "ldm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ldm", "Landoma"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lang'e", "yne"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yne", "Lang'e"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Langam", "lnm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lnm", "Langam"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Langbashe", "lna"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lna", "Langbashe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Langi", "lag"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lag", "Langi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Langnian Buyang", "yln"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yln", "Langnian Buyang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lango (Sudan)", "lno"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lno", "Lango (Sudan)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lango (Uganda)", "laj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("laj", "Lango (Uganda)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Langobardic", "lng"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lng", "Langobardic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Langue des signes de Belgique Francophone", "sfb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sfb", "Langue des signes de Belgique Francophone"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lanoh", "lnh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lnh", "Lanoh"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lao", "lao"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lao", "Lao"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Laomian", "lwm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lwm", "Laomian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Laopang", "lbg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lbg", "Laopang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Laos Sign Language", "lso"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lso", "Laos Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lapaguía-Guivini Zapotec", "ztl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ztl", "Lapaguía-Guivini Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Laragia", "lrg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lrg", "Laragia"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Larantuka Malay", "lrt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lrt", "Larantuka Malay"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lardil", "lbz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lbz", "Lardil"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Larevat", "lrv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lrv", "Larevat"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Large Flowery Miao", "hmd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hmd", "Large Flowery Miao"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lari", "lrl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lrl", "Lari"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Larike-Wakasihu", "alo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("alo", "Larike-Wakasihu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Laro", "lro"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lro", "Laro"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Larteh", "lar"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lar", "Larteh"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Laru", "lan"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lan", "Laru"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lasalimu", "llm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("llm", "Lasalimu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lasgerdi", "lsa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lsa", "Lasgerdi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lashi", "lsi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lsi", "Lashi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lasi", "lss"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lss", "Lasi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Late Middle Chinese", "ltc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ltc", "Late Middle Chinese"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Latgalian", "ltg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ltg", "Latgalian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Latin", "lat"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lat", "Latin"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Latu", "ltu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ltu", "Latu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Latundê", "ltn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ltn", "Latundê"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Latvian", "lav"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lav", "Latvian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Latvian Sign Language", "lsl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lsl", "Latvian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lau", "llu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("llu", "Lau"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Laua", "luf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("luf", "Laua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lauan", "llx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("llx", "Lauan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lauje", "law"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("law", "Lauje"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Laura", "lur"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lur", "Laura"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Laurentian", "lre"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lre", "Laurentian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lave", "brb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("brb", "Lave"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Laven", "lbo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lbo", "Laven"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lavukaleve", "lvk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lvk", "Lavukaleve"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lawangan", "lbx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lbx", "Lawangan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lawunuia", "tgi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tgi", "Lawunuia"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Layakha", "lya"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lya", "Layakha"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Laz", "lzz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lzz", "Laz"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lealao Chinantec", "cle"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cle", "Lealao Chinantec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Leco", "lec"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lec", "Leco"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ledo Kaili", "lew"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lew", "Ledo Kaili"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Leelau", "ldk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ldk", "Leelau"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lefa", "lfa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lfa", "Lefa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lega-Mwenga", "lgm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lgm", "Lega-Mwenga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lega-Shabunda", "lea"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lea", "Lega-Shabunda"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Legbo", "agb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("agb", "Legbo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Legenyem", "lcc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lcc", "Legenyem"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lehali", "tql"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tql", "Lehali"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lehalurup", "urr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("urr", "Lehalurup"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lehar", "cae"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cae", "Lehar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Leinong Naga", "lzn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lzn", "Leinong Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Leipon", "lek"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lek", "Leipon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lelak", "llk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("llk", "Lelak"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lele (Chad)", "lln"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lln", "Lele (Chad)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lele (Democratic Republic of Congo)", "lel"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lel", "Lele (Democratic Republic of Congo)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lele (Guinea)", "llc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("llc", "Lele (Guinea)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lele (Papua New Guinea)", "lle"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lle", "Lele (Papua New Guinea)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lelemi", "lef"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lef", "Lelemi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lelepa", "lpa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lpa", "Lelepa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lembena", "leq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("leq", "Lembena"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lemerig", "lrz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lrz", "Lemerig"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lemio", "lei"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lei", "Lemio"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lemnian", "xle"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xle", "Lemnian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lemolang", "ley"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ley", "Lemolang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lemoro", "ldj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ldj", "Lemoro"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lenakel", "tnl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tnl", "Lenakel"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lenca", "len"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("len", "Lenca"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lendu", "led"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("led", "Lendu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lengilu", "lgi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lgi", "Lengilu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lengo", "lgr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lgr", "Lengo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lengola", "lej"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lej", "Lengola"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lengua", "leg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("leg", "Lengua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lengua de Señas Panameñas", "lsp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lsp", "Lengua de Señas Panameñas"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lengua de Señas del Paraguay", "pys"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pys", "Lengua de Señas del Paraguay"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lengua de señas catalana", "csc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("csc", "Lengua de señas catalana"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Leningitij", "lnj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lnj", "Leningitij"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lenje", "leh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("leh", "Lenje"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lenkau", "ler"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ler", "Lenkau"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lenyima", "ldg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ldg", "Lenyima"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Leonese", "ast"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ast", "Leonese"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lepcha", "lep"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lep", "Lepcha"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lepki", "lpe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lpe", "Lepki"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lepontic", "xlp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xlp", "Lepontic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lere", "gnh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gnh", "Lere"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lese", "les"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("les", "Lese"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lesing-Gelimi", "let"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("let", "Lesing-Gelimi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Letemboi", "nms"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nms", "Letemboi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Leti (Cameroon)", "leo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("leo", "Leti (Cameroon)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Leti (Indonesia)", "lti"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lti", "Leti (Indonesia)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Letzeburgesch", "ltz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ltz", "Letzeburgesch"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Levuka", "lvu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lvu", "Levuka"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lewo", "lww"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lww", "Lewo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lewo Eleng", "lwe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lwe", "Lewo Eleng"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lewotobi", "lwt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lwt", "Lewotobi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Leyigha", "ayi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ayi", "Leyigha"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lezghian", "lez"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lez", "Lezghian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lhaovo", "mhx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mhx", "Lhaovo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lhokpu", "lhp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lhp", "Lhokpu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lhomi", "lhm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lhm", "Lhomi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Li'o", "ljl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ljl", "Li'o"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Liabuku", "lix"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lix", "Liabuku"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Liana-Seti", "ste"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ste", "Liana-Seti"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Liangmai Naga", "njn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("njn", "Liangmai Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lianshan Zhuang", "zln"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zln", "Lianshan Zhuang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Liberia Kpelle", "xpe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xpe", "Liberia Kpelle"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Liberian English", "lir"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lir", "Liberian English"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Libido", "liq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("liq", "Libido"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Libinza", "liz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("liz", "Libinza"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Libon Bikol", "lbl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lbl", "Libon Bikol"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Liburnian", "xli"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xli", "Liburnian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Libyan Arabic", "ayl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ayl", "Libyan Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Libyan Sign Language", "lbs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lbs", "Libyan Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ligbi", "lig"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lig", "Ligbi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ligenza", "lgz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lgz", "Ligenza"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ligurian", "lij"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lij", "Ligurian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ligurian (Ancient)", "xlg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xlg", "Ligurian (Ancient)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lihir", "lih"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lih", "Lihir"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lijili", "mgi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mgi", "Lijili"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lika", "lik"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lik", "Lika"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Liki", "lio"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lio", "Liki"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Likila", "lie"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lie", "Likila"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Likuba", "kxx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kxx", "Likuba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Likum", "lib"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lib", "Likum"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Likwala", "kwc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kwc", "Likwala"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lilau", "lll"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lll", "Lilau"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lillooet", "lil"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lil", "Lillooet"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Limassa", "bme"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bme", "Limassa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Limbu", "lif"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lif", "Limbu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Limbum", "lmp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lmp", "Limbum"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Limburgan", "lim"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lim", "Limburgan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Limburger", "lim"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lim", "Limburger"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Limburgish", "lim"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lim", "Limburgish"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Limi", "ylm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ylm", "Limi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Limilngan", "lmc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lmc", "Limilngan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Limos Kalinga", "kmk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kmk", "Limos Kalinga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lindu", "klw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("klw", "Lindu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Linear A", "lab"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lab", "Linear A"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lingala", "lin"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lin", "Lingala"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lingao", "onb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("onb", "Lingao"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lingarak", "lgk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lgk", "Lingarak"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lingkhim", "lii"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lii", "Lingkhim"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lingua Franca", "pml"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pml", "Lingua Franca"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lingua Franca Nova", "lfn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lfn", "Lingua Franca Nova"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lipan Apache", "apl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("apl", "Lipan Apache"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lipo", "lpo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lpo", "Lipo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lisela", "lcl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lcl", "Lisela"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lish", "lsh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lsh", "Lish"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lishana Deni", "lsd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lsd", "Lishana Deni"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lishanid Noshan", "aij"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aij", "Lishanid Noshan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lishán Didán", "trg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("trg", "Lishán Didán"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lisu", "lis"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lis", "Lisu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Literary Chinese", "lzh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lzh", "Literary Chinese"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lithuanian", "lit"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lit", "Lithuanian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lithuanian Sign Language", "lls"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lls", "Lithuanian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Litzlitz", "lzl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lzl", "Litzlitz"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Liujiang Zhuang", "zlj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zlj", "Liujiang Zhuang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Liuqian Zhuang", "zlq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zlq", "Liuqian Zhuang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Liv", "liv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("liv", "Liv"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Livvi", "olo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("olo", "Livvi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Llengua de Signes Catalana", "csc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("csc", "Llengua de Signes Catalana"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Llengua de signes valenciana", "vsv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vsv", "Llengua de signes valenciana"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lo-Toga", "lht"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lht", "Lo-Toga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Loarki", "lrk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lrk", "Loarki"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lobala", "loq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("loq", "Lobala"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lobi", "lob"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lob", "Lobi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lodhi", "lbm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lbm", "Lodhi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Logba", "lgq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lgq", "Logba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Logo", "log"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("log", "Logo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Logol", "lof"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lof", "Logol"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Logooli", "rag"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rag", "Logooli"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Logorik", "liu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("liu", "Logorik"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Logudorese Sardinian", "src"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("src", "Logudorese Sardinian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Loja Highland Quichua", "qvj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("qvj", "Loja Highland Quichua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lojban", "jbo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jbo", "Lojban"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lokaa", "yaz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yaz", "Lokaa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Loko", "lok"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lok", "Loko"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lokoya", "lky"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lky", "Lokoya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lola", "lcd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lcd", "Lola"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lolak", "llq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("llq", "Lolak"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lole", "llg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("llg", "Lole"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lolo", "llb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("llb", "Lolo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Loloda", "loa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("loa", "Loloda"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lolopo", "ycl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ycl", "Lolopo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Loma (Côte d'Ivoire)", "loi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("loi", "Loma (Côte d'Ivoire)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Loma (Liberia)", "lom"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lom", "Loma (Liberia)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lomaiviti", "lmv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lmv", "Lomaiviti"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lomavren", "rmi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rmi", "Lomavren"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lombard", "lmo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lmo", "Lombard"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lombi", "lmi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lmi", "Lombi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lombo", "loo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("loo", "Lombo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lomwe", "ngl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ngl", "Lomwe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Loncong", "lce"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lce", "Loncong"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Long Phuri Naga", "lpn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lpn", "Long Phuri Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Long Wat", "ttw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ttw", "Long Wat"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Longgu", "lgu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lgu", "Longgu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Longto", "wok"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wok", "Longto"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Longuda", "lnu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lnu", "Longuda"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Loniu", "los"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("los", "Loniu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lonwolwol", "crc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("crc", "Lonwolwol"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lonzo", "lnz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lnz", "Lonzo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Loo", "ldo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ldo", "Loo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lopa", "lop"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lop", "Lopa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lopi", "lov"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lov", "Lopi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lopit", "lpx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lpx", "Lopit"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lorang", "lrn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lrn", "Lorang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lorediakarkar", "lnn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lnn", "Lorediakarkar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Loreto-Ucayali Spanish", "spq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("spq", "Loreto-Ucayali Spanish"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lote", "uvl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("uvl", "Lote"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lotha Naga", "njh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("njh", "Lotha Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lotud", "dtr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dtr", "Lotud"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lou", "loj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("loj", "Lou"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Louisiana Creole French", "lou"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lou", "Louisiana Creole French"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Loun", "lox"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lox", "Loun"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Loup A", "xlo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xlo", "Loup A"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Loup B", "xlb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xlb", "Loup B"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lovono", "vnk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vnk", "Lovono"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Low German", "nds"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nds", "Low German"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Low Saxon", "nds"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nds", "Low Saxon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lowa", "loy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("loy", "Lowa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lower Burdekin", "xbb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xbb", "Lower Burdekin"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lower Chehalis", "cea"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cea", "Lower Chehalis"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lower Grand Valley Dani", "dni"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dni", "Lower Grand Valley Dani"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lower Silesian", "sli"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sli", "Lower Silesian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lower Sorbian", "dsb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dsb", "Lower Sorbian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lower Ta'oih", "tto"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tto", "Lower Ta'oih"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lower Tanana", "taa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("taa", "Lower Tanana"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lower Tanudan Kalinga", "kml"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kml", "Lower Tanudan Kalinga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lowland Oaxaca Chontal", "clo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("clo", "Lowland Oaxaca Chontal"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lowland Tarahumara", "tac"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tac", "Lowland Tarahumara"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Loxicha Zapotec", "ztp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ztp", "Loxicha Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lozi", "loz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("loz", "Lozi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lua'", "prb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("prb", "Lua'"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Luang", "lex"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lex", "Luang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Luba-Katanga", "lub"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lub", "Luba-Katanga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Luba-Lulua", "lua"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lua", "Luba-Lulua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lubila", "kcc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kcc", "Lubila"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lubu", "lcf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lcf", "Lubu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lubuagan Kalinga", "knb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("knb", "Lubuagan Kalinga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lubukusu", "bxk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bxk", "Lubukusu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Luchazi", "lch"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lch", "Luchazi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lucumi", "luq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("luq", "Lucumi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ludian", "lud"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lud", "Ludian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lufu", "ldq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ldq", "Lufu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lugbara", "lgg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lgg", "Lugbara"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Luguru", "ruf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ruf", "Luguru"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Luhu", "lcq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lcq", "Luhu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lui", "lba"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lba", "Lui"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Luimbi", "lum"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lum", "Luimbi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Luiseno", "lui"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lui", "Luiseno"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lukabaras", "lkb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lkb", "Lukabaras"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lukpa", "dop"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dop", "Lukpa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lule Sami", "smj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("smj", "Lule Sami"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lulogooli", "rag"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rag", "Lulogooli"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lumba-Yakkha", "luu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("luu", "Lumba-Yakkha"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lumbee", "lmz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lmz", "Lumbee"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lumbu", "lup"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lup", "Lumbu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lumun", "lmd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lmd", "Lumun"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lun Bawang", "lnd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lnd", "Lun Bawang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Luna", "luj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("luj", "Luna"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lunanakha", "luk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("luk", "Lunanakha"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lunda", "lun"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lun", "Lunda"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lundayeh", "lnd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lnd", "Lundayeh"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lungga", "lga"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lga", "Lungga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Luo (Cameroon)", "luw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("luw", "Luo (Cameroon)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Luo (Kenya and Tanzania)", "luo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("luo", "Luo (Kenya and Tanzania)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Luopohe Hmong", "hml"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hml", "Luopohe Hmong"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Luopohe Miao", "hml"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hml", "Luopohe Miao"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Luri", "ldd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ldd", "Luri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lusengo", "lse"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lse", "Lusengo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lushai", "lus"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lus", "Lushai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lushootseed", "lut"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lut", "Lushootseed"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lusi", "khl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("khl", "Lusi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lusitanian", "xls"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xls", "Lusitanian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lutachoni", "lts"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lts", "Lutachoni"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lutos", "ndy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ndy", "Lutos"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Luvale", "lue"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lue", "Luvale"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Luwati", "luv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("luv", "Luwati"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Luwo", "lwo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lwo", "Luwo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Luxembourgish", "ltz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ltz", "Luxembourgish"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Luyana", "lyn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lyn", "Luyana"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Luyia", "luy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("luy", "Luyia"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lwalu", "lwa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lwa", "Lwalu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lycian", "xlc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xlc", "Lycian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lydian", "xld"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xld", "Lydian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lyngngam", "lyg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lyg", "Lyngngam"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lyons Sign Language", "lsg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lsg", "Lyons Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lyélé", "lee"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lee", "Lyélé"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Láadan", "ldn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ldn", "Láadan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Láá Láá Bwamu", "bwj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bwj", "Láá Láá Bwamu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Löyöp", "urr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("urr", "Löyöp"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Lü", "khb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("khb", "Lü"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ma (Democratic Republic of Congo)", "msj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("msj", "Ma (Democratic Republic of Congo)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ma (Papua New Guinea)", "mjn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mjn", "Ma (Papua New Guinea)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ma'anyan", "mhy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mhy", "Ma'anyan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ma'di", "mhi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mhi", "Ma'di"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ma'ya", "slz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("slz", "Ma'ya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Maa", "cma"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cma", "Maa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Maaka", "mew"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mew", "Maaka"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Maasina Fulfulde", "ffm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ffm", "Maasina Fulfulde"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Maay", "ymm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ymm", "Maay"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Maba (Chad)", "mde"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mde", "Maba (Chad)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Maba (Indonesia)", "mqa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mqa", "Maba (Indonesia)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mabaale", "mmz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mmz", "Mabaale"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mabaan", "mfz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mfz", "Mabaan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mabaka Valley Kalinga", "kkg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kkg", "Mabaka Valley Kalinga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mabire", "muj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("muj", "Mabire"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Maca", "mca"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mca", "Maca"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Macaguaje", "mcl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mcl", "Macaguaje"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Macaguán", "mbn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mbn", "Macaguán"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Macanese", "mzs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mzs", "Macanese"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Macedo-Romanian", "rup"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rup", "Macedo-Romanian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Macedonian", "mkd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mkd", "Macedonian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Machame", "jmc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jmc", "Machame"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Machiguenga", "mcb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mcb", "Machiguenga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Machinere", "mpd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mpd", "Machinere"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Machinga", "mvw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mvw", "Machinga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Maco", "wpc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wpc", "Maco"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Macuna", "myy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("myy", "Macuna"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Macushi", "mbc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mbc", "Macushi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mada (Cameroon)", "mxu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mxu", "Mada (Cameroon)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mada (Nigeria)", "mda"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mda", "Mada (Nigeria)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Madagascar Sign Language", "mzc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mzc", "Madagascar Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Madak", "mmx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mmx", "Madak"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Maden", "xmx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xmx", "Maden"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Madi", "grg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("grg", "Madi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Madngele", "zml"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zml", "Madngele"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Madurese", "mad"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mad", "Madurese"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mae", "mme"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mme", "Mae"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Maek", "hmk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hmk", "Maek"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Maeng Itneg", "itt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("itt", "Maeng Itneg"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mafa", "maf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("maf", "Mafa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mafea", "mkv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mkv", "Mafea"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mag-Indi Ayta", "blx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("blx", "Mag-Indi Ayta"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mag-antsi Ayta", "sgb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sgb", "Mag-antsi Ayta"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Magahi", "mag"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mag", "Magahi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Magbukun Ayta", "ayt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ayt", "Magbukun Ayta"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Magdalena Peñasco Mixtec", "xtm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xtm", "Magdalena Peñasco Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Magoma", "gmx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gmx", "Magoma"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Magori", "zgr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zgr", "Magori"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Maguindanaon", "mdh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mdh", "Maguindanaon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mah Meri", "mhe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mhe", "Mah Meri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mahali", "mjx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mjx", "Mahali"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mahasu Pahari", "bfz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bfz", "Mahasu Pahari"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mahei", "mja"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mja", "Mahei"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mahican", "mjy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mjy", "Mahican"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mahongwe", "mhb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mhb", "Mahongwe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mahou", "mxx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mxx", "Mahou"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mai Brat", "ayz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ayz", "Mai Brat"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Maia", "sks"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sks", "Maia"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Maiadomu", "mzz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mzz", "Maiadomu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Maiani", "tnh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tnh", "Maiani"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Maii", "mmm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mmm", "Maii"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mailu", "mgu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mgu", "Mailu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Maindo", "cwb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cwb", "Maindo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mainfränkisch", "vmf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vmf", "Mainfränkisch"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mainstream Kenyah", "xkl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xkl", "Mainstream Kenyah"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mairasi", "zrs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zrs", "Mairasi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Maisin", "mbq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mbq", "Maisin"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Maithili", "mai"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mai", "Maithili"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Maiwa (Indonesia)", "wmm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wmm", "Maiwa (Indonesia)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Maiwa (Papua New Guinea)", "mti"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mti", "Maiwa (Papua New Guinea)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Maiwala", "mum"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mum", "Maiwala"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Majang", "mpe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mpe", "Majang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Majera", "xmj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xmj", "Majera"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Majhi", "mjz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mjz", "Majhi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Majhwar", "mmj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mmj", "Majhwar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Majukayang Kalinga", "kmd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kmd", "Majukayang Kalinga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mak (China)", "mkg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mkg", "Mak (China)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mak (Nigeria)", "pbl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pbl", "Mak (Nigeria)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Makaa", "mcp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mcp", "Makaa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Makah", "myh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("myh", "Makah"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Makasae", "mkz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mkz", "Makasae"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Makasar", "mak"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mak", "Makasar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Makassar Malay", "mfp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mfp", "Makassar Malay"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Makayam", "aup"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aup", "Makayam"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Makhuwa", "vmw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vmw", "Makhuwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Makhuwa-Meetto", "mgh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mgh", "Makhuwa-Meetto"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Makhuwa-Moniga", "mhm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mhm", "Makhuwa-Moniga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Makhuwa-Saka", "xsq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xsq", "Makhuwa-Saka"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Makhuwa-Shirima", "vmk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vmk", "Makhuwa-Shirima"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Maklew", "mgf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mgf", "Maklew"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Makolkol", "zmh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zmh", "Makolkol"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Makonde", "kde"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kde", "Makonde"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Maku'a", "lva"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lva", "Maku'a"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Makuri Naga", "jmn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jmn", "Makuri Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Makuráp", "mpu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mpu", "Makuráp"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Makwe", "ymk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ymk", "Makwe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Makyan Naga", "umn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("umn", "Makyan Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mal", "mlf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mlf", "Mal"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mal Paharia", "mkb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mkb", "Mal Paharia"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mala (Nigeria)", "ruy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ruy", "Mala (Nigeria)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mala (Papua New Guinea)", "ped"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ped", "Mala (Papua New Guinea)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mala Malasar", "ima"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ima", "Mala Malasar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Malaccan Creole Malay", "ccm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ccm", "Malaccan Creole Malay"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Malaccan Creole Portuguese", "mcm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mcm", "Malaccan Creole Portuguese"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Malagasy", "mlg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mlg", "Malagasy"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Malakhel", "mld"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mld", "Malakhel"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Malalamai", "mmt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mmt", "Malalamai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Malango", "mln"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mln", "Malango"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Malankuravan", "mjo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mjo", "Malankuravan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Malapandaram", "mjp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mjp", "Malapandaram"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Malaryan", "mjq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mjq", "Malaryan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Malas", "mkr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mkr", "Malas"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Malasanga", "mqz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mqz", "Malasanga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Malasar", "ymr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ymr", "Malasar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Malavedan", "mjr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mjr", "Malavedan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Malawi Lomwe", "lon"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lon", "Malawi Lomwe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Malawi Sena", "swk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("swk", "Malawi Sena"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Malay (individual language)", "zlm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zlm", "Malay (individual language)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Malay (macrolanguage)", "msa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("msa", "Malay (macrolanguage)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Malayalam", "mal"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mal", "Malayalam"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Malayic Dayak", "xdy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xdy", "Malayic Dayak"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Malaynon", "mlz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mlz", "Malaynon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Malayo", "mbp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mbp", "Malayo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Malaysian Sign Language", "xml"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xml", "Malaysian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Malba Birifor", "bfo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bfo", "Malba Birifor"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Maldivian", "div"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("div", "Maldivian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Male (Ethiopia)", "mdy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mdy", "Male (Ethiopia)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Male (Papua New Guinea)", "mdc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mdc", "Male (Papua New Guinea)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Maleng", "pkt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pkt", "Maleng"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Maleu-Kilenge", "mgl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mgl", "Maleu-Kilenge"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Malfaxal", "mlx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mlx", "Malfaxal"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Malgana", "vml"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vml", "Malgana"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Malgbe", "mxf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mxf", "Malgbe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mali", "gcc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gcc", "Mali"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Maligo", "mwj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mwj", "Maligo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Malila", "mgq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mgq", "Malila"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Malimba", "mzd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mzd", "Malimba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Malimpung", "mli"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mli", "Malimpung"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Malinaltepec Me'phaa", "tcf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tcf", "Malinaltepec Me'phaa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Malinaltepec Tlapanec", "tcf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tcf", "Malinaltepec Tlapanec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Malo", "mla"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mla", "Malo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Malol", "mbk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mbk", "Malol"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Maltese", "mlt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mlt", "Maltese"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Maltese Sign Language", "mdl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mdl", "Maltese Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Malua Bay", "mll"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mll", "Malua Bay"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Malvi", "mup"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mup", "Malvi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Maléku Jaíka", "gut"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gut", "Maléku Jaíka"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Malê", "hot"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hot", "Malê"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mam", "mam"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mam", "Mam"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mama", "mma"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mma", "Mama"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mamaa", "mhf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mhf", "Mamaa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mamaindé", "wmd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wmd", "Mamaindé"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mamanwa", "mmn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mmn", "Mamanwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mamara Senoufo", "myk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("myk", "Mamara Senoufo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mamasa", "mqj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mqj", "Mamasa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mambae", "mgm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mgm", "Mambae"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mambai", "mcs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mcs", "Mambai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mamboru", "mvd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mvd", "Mamboru"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mambwe-Lungu", "mgr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mgr", "Mambwe-Lungu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mampruli", "maw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("maw", "Mampruli"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mamuju", "mqx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mqx", "Mamuju"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mamulique", "emm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("emm", "Mamulique"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mamusi", "kdf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kdf", "Mamusi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mamvu", "mdi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mdi", "Mamvu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Man Met", "mml"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mml", "Man Met"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Manado Malay", "xmm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xmm", "Manado Malay"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Manam", "mva"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mva", "Manam"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Manambu", "mle"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mle", "Manambu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Manangba", "nmm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nmm", "Manangba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Manangkari", "znk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("znk", "Manangkari"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Manchu", "mnc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mnc", "Manchu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Manda (Australia)", "zma"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zma", "Manda (Australia)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Manda (India)", "mha"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mha", "Manda (India)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Manda (Tanzania)", "mgs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mgs", "Manda (Tanzania)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mandahuaca", "mht"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mht", "Mandahuaca"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mandaic", "mid"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mid", "Mandaic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mandan", "mhq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mhq", "Mandan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mandandanyi", "zmk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zmk", "Mandandanyi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mandar", "mdr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mdr", "Mandar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mandara", "tbf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tbf", "Mandara"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mandari", "mqu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mqu", "Mandari"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mandarin Chinese", "cmn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cmn", "Mandarin Chinese"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mandaya", "mry"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mry", "Mandaya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mandeali", "mjl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mjl", "Mandeali"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mander", "mqr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mqr", "Mander"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mandingo", "man"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("man", "Mandingo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mandinka", "mnk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mnk", "Mandinka"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mandjak", "mfv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mfv", "Mandjak"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mandobo Atas", "aax"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aax", "Mandobo Atas"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mandobo Bawah", "bwp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bwp", "Mandobo Bawah"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Manem", "jet"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jet", "Manem"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mang", "zng"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zng", "Mang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Manga Kanuri", "kby"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kby", "Manga Kanuri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mangala", "mem"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mem", "Mangala"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mangarayi", "mpc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mpc", "Mangarayi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mangareva", "mrv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mrv", "Mangareva"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mangas", "zns"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zns", "Mangas"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mangayat", "myj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("myj", "Mangayat"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mangbetu", "mdj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mdj", "Mangbetu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mangbutu", "mdk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mdk", "Mangbutu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mangerr", "zme"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zme", "Mangerr"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mangga Buang", "mmo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mmo", "Mangga Buang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Manggarai", "mqy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mqy", "Manggarai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mango", "mge"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mge", "Mango"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mangole", "mqc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mqc", "Mangole"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mangseng", "mbh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mbh", "Mangseng"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mangue", "mom"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mom", "Mangue"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Manichaean Middle Persian", "xmn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xmn", "Manichaean Middle Persian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Manide", "abd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("abd", "Manide"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Manikion", "mnx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mnx", "Manikion"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Manipa", "mqp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mqp", "Manipa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Manipuri", "mni"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mni", "Manipuri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mankanya", "knf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("knf", "Mankanya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mann", "mev"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mev", "Mann"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Manna-Dora", "mju"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mju", "Manna-Dora"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mannan", "mjv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mjv", "Mannan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Manombai", "woo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("woo", "Manombai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mansaka", "msk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("msk", "Mansaka"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mansi", "mns"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mns", "Mansi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mansoanka", "msw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("msw", "Mansoanka"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Manta", "myg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("myg", "Manta"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mantsi", "nty"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nty", "Mantsi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Manumanaw Karen", "kxf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kxf", "Manumanaw Karen"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Manusela", "wha"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wha", "Manusela"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Manx", "glv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("glv", "Manx"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Manya", "mzj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mzj", "Manya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Manyawa", "mny"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mny", "Manyawa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Manyika", "mxc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mxc", "Manyika"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Manza", "mzv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mzv", "Manza"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mao Naga", "nbi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nbi", "Mao Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Maonan", "mmd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mmd", "Maonan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Maore Comorian", "swb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("swb", "Maore Comorian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Maori", "mri"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mri", "Maori"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mape", "mlh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mlh", "Mape"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mapena", "mnm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mnm", "Mapena"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mapia", "mpy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mpy", "Mapia"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mapidian", "mpw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mpw", "Mapidian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mapos Buang", "bzh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bzh", "Mapos Buang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mapoyo", "mcg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mcg", "Mapoyo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mapuche", "arn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("arn", "Mapuche"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mapudungun", "arn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("arn", "Mapudungun"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mapun", "sjm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sjm", "Mapun"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Maquiritari", "mch"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mch", "Maquiritari"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mara", "mec"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mec", "Mara"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mara Chin", "mrh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mrh", "Mara Chin"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Marachi", "lri"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lri", "Marachi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Maraghei", "vmh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vmh", "Maraghei"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Maragus", "mrs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mrs", "Maragus"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Maram Naga", "nma"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nma", "Maram Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Marama", "lrm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lrm", "Marama"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Maramba", "myd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("myd", "Maramba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Maranao", "mrw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mrw", "Maranao"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Maranunggu", "zmr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zmr", "Maranunggu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mararit", "mgb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mgb", "Mararit"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Marathi", "mar"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mar", "Marathi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Marau", "mvr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mvr", "Marau"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Marba", "mpg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mpg", "Marba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Maremgi", "mrx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mrx", "Maremgi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Marenje", "vmr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vmr", "Marenje"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Marfa", "mvu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mvu", "Marfa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Margany", "zmc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zmc", "Margany"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Marghi Central", "mrt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mrt", "Marghi Central"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Marghi South", "mfm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mfm", "Marghi South"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Margos-Yarowilca-Lauricocha Quechua", "qvm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("qvm", "Margos-Yarowilca-Lauricocha Quechua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Margu", "mhg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mhg", "Margu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mari (East Sepik Province)", "mbx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mbx", "Mari (East Sepik Province)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mari (Madang Province)", "hob"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hob", "Mari (Madang Province)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mari (Russia)", "chm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("chm", "Mari (Russia)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Maria (India)", "mrr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mrr", "Maria (India)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Maria (Papua New Guinea)", "mds"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mds", "Maria (Papua New Guinea)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Maricopa", "mrc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mrc", "Maricopa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Maridan", "zmd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zmd", "Maridan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Maridjabin", "zmj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zmj", "Maridjabin"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Marik", "dad"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dad", "Marik"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Marimanindji", "zmm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zmm", "Marimanindji"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Marind", "mrz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mrz", "Marind"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Maring", "mbw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mbw", "Maring"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Maring Naga", "nng"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nng", "Maring Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Maringarr", "zmt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zmt", "Maringarr"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Marino", "mrb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mrb", "Marino"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mariri", "mqi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mqi", "Mariri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Marithiel", "mfr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mfr", "Marithiel"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Maritime Sign Language", "nsr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nsr", "Maritime Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Maritsauá", "msp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("msp", "Maritsauá"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mariyedi", "zmy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zmy", "Mariyedi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Marka", "rkm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rkm", "Marka"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Markweeta", "enb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("enb", "Markweeta"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Marma", "rmz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rmz", "Marma"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Marovo", "mvo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mvo", "Marovo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Marriammu", "xru"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xru", "Marriammu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Marrucinian", "umc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("umc", "Marrucinian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Marshallese", "mah"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mah", "Marshallese"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Marsian", "ims"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ims", "Marsian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Martha's Vineyard Sign Language", "mre"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mre", "Martha's Vineyard Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Marti Ke", "zmg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zmg", "Marti Ke"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Martu Wangka", "mpj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mpj", "Martu Wangka"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Martuyhunira", "vma"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vma", "Martuyhunira"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Maru", "mhx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mhx", "Maru"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Marwari", "mwr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mwr", "Marwari"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Marwari (India)", "rwr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rwr", "Marwari (India)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Marwari (Pakistan)", "mve"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mve", "Marwari (Pakistan)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Marúbo", "mzr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mzr", "Marúbo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Masaaba", "myx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("myx", "Masaaba"); //$NON-NLS-1$ //$NON-NLS-2$
	}

	private void createLanguagesMap4() {
		langMap.put("Masadiit Itneg", "tis"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tis", "Masadiit Itneg"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Masai", "mas"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mas", "Masai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Masalit", "mls"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mls", "Masalit"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Masana", "mcn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mcn", "Masana"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Masbatenyo", "msb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("msb", "Masbatenyo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mashco Piro", "cuj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cuj", "Mashco Piro"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mashi (Nigeria)", "jms"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jms", "Mashi (Nigeria)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mashi (Zambia)", "mho"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mho", "Mashi (Zambia)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Masikoro Malagasy", "msh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("msh", "Masikoro Malagasy"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Masimasi", "ism"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ism", "Masimasi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Masiwang", "bnf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bnf", "Masiwang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Maskelynes", "klv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("klv", "Maskelynes"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Maskoy Pidgin", "mhh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mhh", "Maskoy Pidgin"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Maslam", "msv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("msv", "Maslam"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Masmaje", "mes"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mes", "Masmaje"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Massalat", "mdg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mdg", "Massalat"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Massep", "mvs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mvs", "Massep"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Matagalpa", "mtn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mtn", "Matagalpa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Matal", "mfh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mfh", "Matal"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Matbat", "xmt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xmt", "Matbat"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Matengo", "mgv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mgv", "Matengo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Matepi", "mqe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mqe", "Matepi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Matigsalug Manobo", "mbt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mbt", "Matigsalug Manobo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Matipuhy", "mzo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mzo", "Matipuhy"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mato", "met"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("met", "Mato"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mato Grosso Arára", "axg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("axg", "Mato Grosso Arára"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mator", "mtm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mtm", "Mator"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Matsés", "mcf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mcf", "Matsés"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mattole", "mvb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mvb", "Mattole"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Matukar", "mjk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mjk", "Matukar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Matumbi", "mgw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mgw", "Matumbi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Matya Samo", "stj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("stj", "Matya Samo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Matís", "mpq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mpq", "Matís"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Maung", "mph"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mph", "Maung"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mauritian Sign Language", "lsy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lsy", "Mauritian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mauwake", "mhl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mhl", "Mauwake"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mawa (Chad)", "mcw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mcw", "Mawa (Chad)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mawa (Nigeria)", "wma"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wma", "Mawa (Nigeria)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mawak", "mjj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mjj", "Mawak"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mawan", "mcz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mcz", "Mawan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mawayana", "mzx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mzx", "Mawayana"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mawchi", "mke"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mke", "Mawchi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mawes", "mgk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mgk", "Mawes"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mawukakan", "mxx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mxx", "Mawukakan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Maxakalí", "mbl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mbl", "Maxakalí"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Maxi Gbe", "mxl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mxl", "Maxi Gbe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Maya Samo", "sym"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sym", "Maya Samo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mayaguduna", "xmy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xmy", "Mayaguduna"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mayangna", "yan"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yan", "Mayangna"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mayeka", "myc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("myc", "Mayeka"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Maykulan", "mnt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mnt", "Maykulan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mayo", "mfy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mfy", "Mayo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mayogo", "mdm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mdm", "Mayogo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mayoyao Ifugao", "ifu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ifu", "Mayoyao Ifugao"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mazagway", "dkx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dkx", "Mazagway"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mazaltepec Zapotec", "zpy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zpy", "Mazaltepec Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mazanderani", "mzn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mzn", "Mazanderani"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mazatlán Mazatec", "vmz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vmz", "Mazatlán Mazatec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mazatlán Mixe", "mzl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mzl", "Mazatlán Mixe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mba", "mfc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mfc", "Mba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mbabaram", "vmb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vmb", "Mbabaram"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mbala", "mdp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mdp", "Mbala"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mbalanhu", "lnb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lnb", "Mbalanhu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mbandja", "zmz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zmz", "Mbandja"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mbangala", "mxg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mxg", "Mbangala"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mbangi", "mgn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mgn", "Mbangi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mbangwe", "zmn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zmn", "Mbangwe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mbara (Australia)", "mvl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mvl", "Mbara (Australia)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mbara (Chad)", "mpk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mpk", "Mbara (Chad)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mbati", "mdn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mdn", "Mbati"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mbato", "gwa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gwa", "Mbato"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mbay", "myb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("myb", "Mbay"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mbe", "mfo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mfo", "Mbe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mbe'", "mtk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mtk", "Mbe'"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mbedam", "xmd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xmd", "Mbedam"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mbelime", "mql"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mql", "Mbelime"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mbere", "mdt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mdt", "Mbere"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mbesa", "zms"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zms", "Mbesa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mbo (Cameroon)", "mbo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mbo", "Mbo (Cameroon)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mbo (Democratic Republic of Congo)", "zmw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zmw", "Mbo (Democratic Republic of Congo)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mboi", "moi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("moi", "Mboi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mboko", "mdu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mdu", "Mboko"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mbole", "mdq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mdq", "Mbole"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mbonga", "xmb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xmb", "Mbonga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mbongno", "bgu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bgu", "Mbongno"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mbosi", "mdw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mdw", "Mbosi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mbowe", "mxo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mxo", "Mbowe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mbre", "mka"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mka", "Mbre"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mbu'", "muc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("muc", "Mbu'"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mbugu", "mhd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mhd", "Mbugu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mbugwe", "mgz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mgz", "Mbugwe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mbuko", "mqb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mqb", "Mbuko"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mbukushu", "mhw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mhw", "Mbukushu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mbula", "mna"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mna", "Mbula"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mbula-Bwazza", "mbu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mbu", "Mbula-Bwazza"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mbule", "mlb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mlb", "Mbule"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mbulungish", "mbv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mbv", "Mbulungish"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mbum", "mdd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mdd", "Mbum"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mbunda", "mck"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mck", "Mbunda"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mbunga", "mgy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mgy", "Mbunga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mburku", "bbt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bbt", "Mburku"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mbwela", "mfu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mfu", "Mbwela"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mbyá Guaraní", "gun"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gun", "Mbyá Guaraní"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Me'en", "mym"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mym", "Me'en"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mea", "meg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("meg", "Mea"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Medebur", "mjm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mjm", "Medebur"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Media Lengua", "mue"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mue", "Media Lengua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mediak", "mwx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mwx", "Mediak"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Median", "xme"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xme", "Median"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mednyj Aleut", "mud"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mud", "Mednyj Aleut"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Medumba", "byv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("byv", "Medumba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mefele", "mfj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mfj", "Mefele"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Megam", "mef"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mef", "Megam"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Megleno Romanian", "ruq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ruq", "Megleno Romanian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mehek", "nux"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nux", "Mehek"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mehináku", "mmh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mmh", "Mehináku"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mehri", "gdq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gdq", "Mehri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mekeo", "mek"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mek", "Mekeo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mekmek", "mvk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mvk", "Mekmek"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mekwei", "msf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("msf", "Mekwei"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mele-Fila", "mxe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mxe", "Mele-Fila"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Melo", "mfx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mfx", "Melo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Melpa", "med"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("med", "Melpa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Memoni", "mby"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mby", "Memoni"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mendalam Kayan", "xkd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xkd", "Mendalam Kayan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mendankwe-Nkwen", "mfd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mfd", "Mendankwe-Nkwen"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mende (Papua New Guinea)", "sim"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sim", "Mende (Papua New Guinea)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mende (Sierra Leone)", "men"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("men", "Mende (Sierra Leone)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mengaka", "xmg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xmg", "Mengaka"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mengen", "mee"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mee", "Mengen"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mengisa", "mct"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mct", "Mengisa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Menka", "mea"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mea", "Menka"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Menominee", "mez"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mez", "Menominee"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mentawai", "mwv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mwv", "Mentawai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Menya", "mcr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mcr", "Menya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Meoswar", "mvx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mvx", "Meoswar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mer", "mnu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mnu", "Mer"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Meramera", "mxm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mxm", "Meramera"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Merei", "lmb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lmb", "Merei"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Merey", "meq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("meq", "Merey"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Meriam", "ulk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ulk", "Meriam"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Merlav", "mrm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mrm", "Merlav"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Meroitic", "xmr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xmr", "Meroitic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Meru", "mer"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mer", "Meru"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Merwari", "wry"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wry", "Merwari"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mesaka", "iyo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("iyo", "Mesaka"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mescalero-Chiricahua Apache", "apm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("apm", "Mescalero-Chiricahua Apache"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mese", "mci"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mci", "Mese"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Meskwaki", "sac"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sac", "Meskwaki"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mesme", "zim"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zim", "Mesme"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mesmes", "mys"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mys", "Mesmes"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mesopotamian Arabic", "acm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("acm", "Mesopotamian Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mesqan", "mvz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mvz", "Mesqan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Messapic", "cms"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cms", "Messapic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Meta'", "mgo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mgo", "Meta'"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Metlatónoc Mixtec", "mxv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mxv", "Metlatónoc Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mewari", "mtr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mtr", "Mewari"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mewati", "wtm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wtm", "Mewati"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mexican Sign Language", "mfs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mfs", "Mexican Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Meyah", "mej"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mej", "Meyah"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mezontla Popoloca", "pbe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pbe", "Mezontla Popoloca"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mezquital Otomi", "ote"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ote", "Mezquital Otomi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mfinu", "zmf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zmf", "Mfinu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mfumte", "nfu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nfu", "Mfumte"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mi'kmaq", "mic"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mic", "Mi'kmaq"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Miahuatlán Zapotec", "zam"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zam", "Miahuatlán Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Miami", "mia"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mia", "Miami"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mian", "mpt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mpt", "Mian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Miani", "pla"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pla", "Miani"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Michif", "crg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("crg", "Michif"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Michigamea", "cmm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cmm", "Michigamea"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Michoacán Mazahua", "mmc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mmc", "Michoacán Mazahua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Michoacán Nahuatl", "ncl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ncl", "Michoacán Nahuatl"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Micmac", "mic"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mic", "Micmac"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mid Grand Valley Dani", "dnt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dnt", "Mid Grand Valley Dani"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mid-Southern Banda", "bjo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bjo", "Mid-Southern Banda"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Middle Armenian", "axm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("axm", "Middle Armenian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Middle Breton", "xbm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xbm", "Middle Breton"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Middle Cornish", "cnx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cnx", "Middle Cornish"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Middle Dutch (ca. 1050-1350)", "dum"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dum", "Middle Dutch (ca. 1050-1350)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Middle English (1100-1500)", "enm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("enm", "Middle English (1100-1500)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Middle French (ca. 1400-1600)", "frm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("frm", "Middle French (ca. 1400-1600)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Middle High German (ca. 1050-1500)", "gmh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gmh", "Middle High German (ca. 1050-1500)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Middle Hittite", "htx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("htx", "Middle Hittite"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Middle Irish (900-1200)", "mga"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mga", "Middle Irish (900-1200)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Middle Korean (10th-16th cent.)", "okm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("okm", "Middle Korean (10th-16th cent.)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Middle Low German", "gml"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gml", "Middle Low German"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Middle Mongolian", "xng"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xng", "Middle Mongolian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Middle Newar", "nwx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nwx", "Middle Newar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Middle Watut", "mpl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mpl", "Middle Watut"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Middle Welsh", "wlm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wlm", "Middle Welsh"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Midob", "mei"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mei", "Midob"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Migaama", "mmy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mmy", "Migaama"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Migabac", "mpp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mpp", "Migabac"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Migum", "klm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("klm", "Migum"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Miji", "sjl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sjl", "Miji"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Miju-Mishmi", "mxj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mxj", "Miju-Mishmi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mikasuki", "mik"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mik", "Mikasuki"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mili", "ymh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ymh", "Mili"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Miltu", "mlj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mlj", "Miltu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Miluk", "iml"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("iml", "Miluk"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Milyan", "imy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("imy", "Milyan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Min Bei Chinese", "mnp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mnp", "Min Bei Chinese"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Min Dong Chinese", "cdo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cdo", "Min Dong Chinese"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Min Nan Chinese", "nan"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nan", "Min Nan Chinese"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Min Zhong Chinese", "czo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("czo", "Min Zhong Chinese"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mina (Cameroon)", "hna"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hna", "Mina (Cameroon)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mina (India)", "myi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("myi", "Mina (India)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Minaean", "inm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("inm", "Minaean"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Minangkabau", "min"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("min", "Minangkabau"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Minanibai", "mcv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mcv", "Minanibai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Minaveha", "mvn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mvn", "Minaveha"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mindiri", "mpn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mpn", "Mindiri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mingang Doso", "mko"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mko", "Mingang Doso"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mingrelian", "xmf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xmf", "Mingrelian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Minica Huitoto", "hto"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hto", "Minica Huitoto"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Minidien", "wii"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wii", "Minidien"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Minigir", "vmg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vmg", "Minigir"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Minoan", "omn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("omn", "Minoan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Minokok", "mqq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mqq", "Minokok"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Minriq", "mnq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mnq", "Minriq"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mintil", "mzt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mzt", "Mintil"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Minz Zhuang", "zgm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zgm", "Minz Zhuang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Miqie", "yiq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yiq", "Miqie"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mirandese", "mwl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mwl", "Mirandese"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Miraya Bikol", "rbl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rbl", "Miraya Bikol"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mire", "mvh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mvh", "Mire"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mirgan", "zrg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zrg", "Mirgan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Miri", "mrg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mrg", "Miri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Miriti", "mmv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mmv", "Miriti"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Miriwung", "mep"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mep", "Miriwung"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mirpur Panjabi", "pmu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pmu", "Mirpur Panjabi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Miship", "mjs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mjs", "Miship"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Misima-Paneati", "mpx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mpx", "Misima-Paneati"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mitla Zapotec", "zaw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zaw", "Mitla Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mitlatongo Mixtec", "vmm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vmm", "Mitlatongo Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mittu", "mwu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mwu", "Mittu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mituku", "zmq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zmq", "Mituku"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Miu", "mpo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mpo", "Miu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Miwa", "vmi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vmi", "Miwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mixed Great Andamanese", "gac"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gac", "Mixed Great Andamanese"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mixifore", "mfg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mfg", "Mixifore"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mixtepec Mixtec", "mix"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mix", "Mixtepec Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mixtepec Zapotec", "zpm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zpm", "Mixtepec Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Miya", "mkf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mkf", "Miya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Miyako", "mvi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mvi", "Miyako"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Miyobe", "soy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("soy", "Miyobe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mlabri", "mra"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mra", "Mlabri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mlahsö", "lhs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lhs", "Mlahsö"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mlap", "kja"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kja", "Mlap"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mlomp", "mlo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mlo", "Mlomp"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mmaala", "mmu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mmu", "Mmaala"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mmen", "bfm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bfm", "Mmen"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mo", "wkd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wkd", "Mo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mo'da", "gbn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gbn", "Mo'da"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Moabite", "obm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("obm", "Moabite"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Moba", "mfq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mfq", "Moba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mobilian", "mod"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mod", "Mobilian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mobumrin Aizi", "ahm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ahm", "Mobumrin Aizi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mochi", "old"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("old", "Mochi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mochica", "omc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("omc", "Mochica"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mocho", "mhc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mhc", "Mocho"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mocoví", "moc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("moc", "Mocoví"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Modang", "mxd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mxd", "Modang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Modern Greek (1453-)", "ell"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ell", "Modern Greek (1453-)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Modole", "mqo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mqo", "Modole"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Moere", "mvq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mvq", "Moere"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mofu-Gudur", "mif"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mif", "Mofu-Gudur"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mogholi", "mhj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mhj", "Mogholi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mogum", "mou"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mou", "Mogum"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mohave", "mov"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mov", "Mohave"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mohawk", "moh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("moh", "Mohawk"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mohegan-Pequot", "xpq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xpq", "Mohegan-Pequot"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Moi (Congo)", "mow"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mow", "Moi (Congo)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Moi (Indonesia)", "mxn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mxn", "Moi (Indonesia)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Moikodi", "mkp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mkp", "Moikodi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Moingi", "mwz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mwz", "Moingi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Moji", "ymi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ymi", "Moji"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mok", "mqt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mqt", "Mok"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Moken", "mwt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mwt", "Moken"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mokerang", "mft"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mft", "Mokerang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mokilese", "mkj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mkj", "Mokilese"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Moklen", "mkm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mkm", "Moklen"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mokole", "mkl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mkl", "Mokole"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mokpwe", "bri"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bri", "Mokpwe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Moksela", "vms"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vms", "Moksela"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Moksha", "mdf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mdf", "Moksha"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Molale", "mbe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mbe", "Molale"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Molbog", "pwm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pwm", "Molbog"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Moldavian", "ron"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ron", "Moldavian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Moldova Sign Language", "vsi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vsi", "Moldova Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Moldovan", "ron"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ron", "Moldovan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Molengue", "bxc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bxc", "Molengue"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Molima", "mox"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mox", "Molima"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Molmo One", "aun"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aun", "Molmo One"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Molo", "zmo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zmo", "Molo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Molof", "msl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("msl", "Molof"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Moloko", "mlw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mlw", "Moloko"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mom Jango", "ver"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ver", "Mom Jango"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Moma", "myl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("myl", "Moma"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Momare", "msz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("msz", "Momare"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mombum", "mso"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mso", "Mombum"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Momina", "mmb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mmb", "Momina"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Momuna", "mqf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mqf", "Momuna"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mon", "mnw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mnw", "Mon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Monastic Sign Language", "mzg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mzg", "Monastic Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mondropolon", "npn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("npn", "Mondropolon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mondé", "mnd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mnd", "Mondé"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mong", "hmn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hmn", "Mong"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mong Leng", "hnj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hnj", "Mong Leng"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mong Njua", "hnj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hnj", "Mong Njua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mongo", "lol"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lol", "Mongo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mongol", "mgt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mgt", "Mongol"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mongolia Buriat", "bxm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bxm", "Mongolia Buriat"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mongolian", "mon"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mon", "Mongolian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mongolian Sign Language", "msr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("msr", "Mongolian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mongondow", "mog"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mog", "Mongondow"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Moni", "mnz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mnz", "Moni"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mono (Cameroon)", "mru"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mru", "Mono (Cameroon)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mono (Democratic Republic of Congo)", "mnh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mnh", "Mono (Democratic Republic of Congo)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mono (Solomon Islands)", "mte"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mte", "Mono (Solomon Islands)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mono (USA)", "mnr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mnr", "Mono (USA)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Monom", "moo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("moo", "Monom"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Monsang Naga", "nmh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nmh", "Monsang Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Montagnais", "moe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("moe", "Montagnais"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Montol", "mtl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mtl", "Montol"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Monumbo", "mxk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mxk", "Monumbo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Monzombo", "moj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("moj", "Monzombo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Moo", "gwg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gwg", "Moo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Moose Cree", "crm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("crm", "Moose Cree"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mopán Maya", "mop"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mop", "Mopán Maya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mor (Bomberai Peninsula)", "moq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("moq", "Mor (Bomberai Peninsula)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mor (Mor Islands)", "mhz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mhz", "Mor (Mor Islands)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Moraid", "msg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("msg", "Moraid"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Morawa", "mze"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mze", "Morawa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Morelos Nahuatl", "nhm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nhm", "Morelos Nahuatl"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Morerebi", "xmo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xmo", "Morerebi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Moresada", "msx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("msx", "Moresada"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mori Atas", "mzq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mzq", "Mori Atas"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mori Bawah", "xmz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xmz", "Mori Bawah"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Morigi", "mdb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mdb", "Morigi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Morisyen", "mfe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mfe", "Morisyen"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Moro", "mor"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mor", "Moro"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Moroccan Arabic", "ary"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ary", "Moroccan Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Moroccan Sign Language", "xms"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xms", "Moroccan Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Morokodo", "mgc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mgc", "Morokodo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Morom", "bdo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bdo", "Morom"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Moronene", "mqn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mqn", "Moronene"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Morori", "mok"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mok", "Morori"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Morouas", "mrp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mrp", "Morouas"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mortlockese", "mrl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mrl", "Mortlockese"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Moru", "mgd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mgd", "Moru"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mosimo", "mqv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mqv", "Mosimo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mosiro", "mwy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mwy", "Mosiro"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Moskona", "mtj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mtj", "Moskona"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mossi", "mos"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mos", "Mossi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mota", "mtt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mtt", "Mota"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Motlav", "mlv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mlv", "Motlav"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Motu", "meu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("meu", "Motu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mouk-Aria", "mwh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mwh", "Mouk-Aria"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mountain Koiali", "kpx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kpx", "Mountain Koiali"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Movima", "mzp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mzp", "Movima"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Moyadan Itneg", "ity"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ity", "Moyadan Itneg"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Moyon Naga", "nmo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nmo", "Moyon Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mozambican Sign Language", "mzy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mzy", "Mozambican Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mozarabic", "mxi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mxi", "Mozarabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mpade", "mpi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mpi", "Mpade"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mpi", "mpz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mpz", "Mpi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mpiemo", "mcx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mcx", "Mpiemo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mpongmpong", "mgg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mgg", "Mpongmpong"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mpoto", "mpa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mpa", "Mpoto"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mpotovoro", "mvt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mvt", "Mpotovoro"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mpuono", "zmp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zmp", "Mpuono"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mpur", "akc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("akc", "Mpur"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mro Chin", "cmr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cmr", "Mro Chin"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mru", "mro"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mro", "Mru"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mser", "kqx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kqx", "Mser"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mt. Iraya Agta", "atl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("atl", "Mt. Iraya Agta"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mt. Iriga Agta", "agz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("agz", "Mt. Iriga Agta"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mualang", "mtd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mtd", "Mualang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mubami", "tsx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tsx", "Mubami"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mubi", "mub"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mub", "Mubi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Muda", "ymd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ymd", "Muda"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mudbura", "mwd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mwd", "Mudbura"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mudhili Gadaba", "gau"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gau", "Mudhili Gadaba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mudu Koraga", "vmd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vmd", "Mudu Koraga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Muduapa", "wiv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wiv", "Muduapa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Muduga", "udg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("udg", "Muduga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mufian", "aoj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aoj", "Mufian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mugom", "muk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("muk", "Mugom"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Muinane", "bmr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bmr", "Muinane"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mukha-Dora", "mmk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mmk", "Mukha-Dora"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mukulu", "moz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("moz", "Mukulu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mulaha", "mfw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mfw", "Mulaha"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mulam", "mlm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mlm", "Mulam"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mullu Kurumba", "kpb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kpb", "Mullu Kurumba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mullukmulluk", "mpb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mpb", "Mullukmulluk"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Multiple languages", "mul"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mul", "Multiple languages"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Muluridyi", "vmu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vmu", "Muluridyi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mum", "kqa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kqa", "Mum"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mumuye", "mzm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mzm", "Mumuye"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Muna", "mnb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mnb", "Muna"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Munda", "unx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("unx", "Munda"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mundabli", "boe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("boe", "Mundabli"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mundang", "mua"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mua", "Mundang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mundani", "mnf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mnf", "Mundani"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mundari", "unr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("unr", "Mundari"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mundat", "mmf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mmf", "Mundat"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mundurukú", "myu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("myu", "Mundurukú"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mungaka", "mhk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mhk", "Mungaka"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Munggui", "mth"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mth", "Munggui"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mungkip", "mpv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mpv", "Mungkip"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Muniche", "myr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("myr", "Muniche"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Munit", "mtc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mtc", "Munit"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Munji", "mnj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mnj", "Munji"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Munsee", "umu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("umu", "Munsee"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Muong", "mtq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mtq", "Muong"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Muratayak", "asx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("asx", "Muratayak"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Murik (Malaysia)", "mxr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mxr", "Murik (Malaysia)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Murik (Papua New Guinea)", "mtf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mtf", "Murik (Papua New Guinea)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Murkim", "rmh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rmh", "Murkim"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Murle", "mur"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mur", "Murle"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Murrinh-Patha", "mwf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mwf", "Murrinh-Patha"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mursi", "muz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("muz", "Mursi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Murui Huitoto", "huu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("huu", "Murui Huitoto"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Murupi", "mqw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mqw", "Murupi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Muruwari", "zmu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zmu", "Muruwari"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Musak", "mmq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mmq", "Musak"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Musar", "mmi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mmi", "Musar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Musasa", "smm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("smm", "Musasa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Musey", "mse"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mse", "Musey"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Musgu", "mug"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mug", "Musgu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mushungulu", "xma"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xma", "Mushungulu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Musi", "mui"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mui", "Musi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Muskum", "mje"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mje", "Muskum"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Muslim Tat", "ttt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ttt", "Muslim Tat"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Musom", "msu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("msu", "Musom"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mussau-Emira", "emi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("emi", "Mussau-Emira"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Muthuvan", "muv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("muv", "Muthuvan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mutu", "tuc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tuc", "Mutu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Muya", "mvm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mvm", "Muya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Muyang", "muy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("muy", "Muyang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Muyuw", "myw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("myw", "Muyuw"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Muzi", "ymz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ymz", "Muzi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mvanip", "mcj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mcj", "Mvanip"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mvuba", "mxh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mxh", "Mvuba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mwaghavul", "sur"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sur", "Mwaghavul"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mwali Comorian", "wlc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wlc", "Mwali Comorian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mwan", "moa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("moa", "Mwan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mwani", "wmw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wmw", "Mwani"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mwatebu", "mwa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mwa", "Mwatebu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mwera (Chimwera)", "mwe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mwe", "Mwera (Chimwera)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mwera (Nyasa)", "mjh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mjh", "Mwera (Nyasa)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mwerlap", "mrm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mrm", "Mwerlap"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mwimbi-Muthambi", "mws"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mws", "Mwimbi-Muthambi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mwotlap", "mlv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mlv", "Mwotlap"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mycenaean Greek", "gmy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gmy", "Mycenaean Greek"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Myene", "mye"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mye", "Myene"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mysian", "yms"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yms", "Mysian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mzieme Naga", "nme"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nme", "Mzieme Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mághdì", "gmd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gmd", "Mághdì"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mískito", "miq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("miq", "Mískito"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mócheno", "mhn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mhn", "Mócheno"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mün Chin", "mwq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mwq", "Mün Chin"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Mündü", "muh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("muh", "Mündü"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Māhārāṣṭri Prākrit", "pmh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pmh", "Māhārāṣṭri Prākrit"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("N'Ko", "nqo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nqo", "N'Ko"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("N/u", "ngh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ngh", "N/u"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Na", "nbt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nbt", "Na"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Naaba", "nao"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nao", "Naaba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Naasioi", "nas"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nas", "Naasioi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Naba", "mne"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mne", "Naba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nabak", "naf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("naf", "Nabak"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nabi", "mty"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mty", "Nabi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nachering", "ncd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ncd", "Nachering"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nadruvian", "ndf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ndf", "Nadruvian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nadëb", "mbj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mbj", "Nadëb"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nafaanra", "nfr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nfr", "Nafaanra"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nafi", "srf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("srf", "Nafi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nafri", "nxx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nxx", "Nafri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nafusi", "jbn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jbn", "Nafusi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Naga Pidgin", "nag"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nag", "Naga Pidgin"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nagarchal", "nbg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nbg", "Nagarchal"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nage", "nxe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nxe", "Nage"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nagu", "ngr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ngr", "Nagu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nagumi", "ngv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ngv", "Nagumi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Naha'ai", "mlx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mlx", "Naha'ai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nahali", "nlx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nlx", "Nahali"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nahari", "nhh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nhh", "Nahari"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nahavaq", "sns"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sns", "Nahavaq"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nai", "bio"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bio", "Nai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Najdi Arabic", "ars"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ars", "Najdi Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Naka'ela", "nae"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nae", "Naka'ela"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nakai", "nkj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nkj", "Nakai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nakama", "nib"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nib", "Nakama"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nakanai", "nak"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nak", "Nakanai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nakara", "nck"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nck", "Nakara"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nake", "nbk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nbk", "Nake"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Naki", "mff"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mff", "Naki"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nakwi", "nax"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nax", "Nakwi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nalca", "nlc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nlc", "Nalca"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nali", "nss"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nss", "Nali"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nalik", "nal"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nal", "Nalik"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nalu", "naj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("naj", "Nalu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Naluo Yi", "ylo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ylo", "Naluo Yi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nalögo", "nlz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nlz", "Nalögo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nama (Namibia)", "naq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("naq", "Nama (Namibia)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nama (Papua New Guinea)", "nmx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nmx", "Nama (Papua New Guinea)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Namakura", "nmk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nmk", "Namakura"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Namat", "nkm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nkm", "Namat"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nambo", "ncm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ncm", "Nambo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nambya", "nmq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nmq", "Nambya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Namia", "nnm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nnm", "Namia"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Namiae", "nvm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nvm", "Namiae"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Namibian Sign Language", "nbs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nbs", "Namibian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Namla", "naa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("naa", "Namla"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Namo", "mxw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mxw", "Namo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Namonuito", "nmt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nmt", "Namonuito"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Namuyi", "nmy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nmy", "Namuyi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nanai", "gld"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gld", "Nanai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nancere", "nnc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nnc", "Nancere"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nande", "nnb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nnb", "Nande"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nandi", "niq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("niq", "Nandi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nanerigé Sénoufo", "sen"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sen", "Nanerigé Sénoufo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nanga Dama Dogon", "nzz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nzz", "Nanga Dama Dogon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nanggu", "ngr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ngr", "Nanggu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nankina", "nnk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nnk", "Nankina"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nanti", "cox"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cox", "Nanti"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nanticoke", "nnt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nnt", "Nanticoke"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nanubae", "afk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("afk", "Nanubae"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Napo Lowland Quechua", "qvo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("qvo", "Napo Lowland Quechua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Napu", "npy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("npy", "Napu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nar Phu", "npa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("npa", "Nar Phu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nara", "nrb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nrb", "Nara"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Narak", "nac"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nac", "Narak"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Narango", "nrg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nrg", "Narango"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Narau", "nxu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nxu", "Narau"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Narim", "loh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("loh", "Narim"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Naro", "nhr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nhr", "Naro"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Narom", "nrm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nrm", "Narom"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Narragansett", "xnt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xnt", "Narragansett"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Narrinyeri", "nay"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nay", "Narrinyeri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Narungga", "nnr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nnr", "Narungga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nasal", "nsy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nsy", "Nasal"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nasarian", "nvh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nvh", "Nasarian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Naskapi", "nsk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nsk", "Naskapi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Natagaimas", "nts"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nts", "Natagaimas"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Natanzi", "ntz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ntz", "Natanzi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nataoran Amis", "ais"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ais", "Nataoran Amis"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Natchez", "ncz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ncz", "Natchez"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nateni", "ntm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ntm", "Nateni"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nathembo", "nte"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nte", "Nathembo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Natioro", "nti"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nti", "Natioro"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Natügu", "ntu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ntu", "Natügu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nauete", "nxa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nxa", "Nauete"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Naukan Yupik", "ynk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ynk", "Naukan Yupik"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nauna", "ncn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ncn", "Nauna"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nauru", "nau"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nau", "Nauru"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Navaho", "nav"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nav", "Navaho"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Navajo", "nav"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nav", "Navajo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Navut", "nsw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nsw", "Navut"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nawaru", "nwr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nwr", "Nawaru"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nawathinehena", "nwa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nwa", "Nawathinehena"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nawdm", "nmz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nmz", "Nawdm"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nawuri", "naw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("naw", "Nawuri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Naxi", "nbf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nbf", "Naxi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nayi", "noz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("noz", "Nayi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nayini", "nyq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nyq", "Nayini"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ncane", "ncr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ncr", "Ncane"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nchumbulu", "nlu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nlu", "Nchumbulu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nda'nda'", "nnz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nnz", "Nda'nda'"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ndai", "gke"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gke", "Ndai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ndaka", "ndk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ndk", "Ndaka"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ndaktup", "ncp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ncp", "Ndaktup"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ndali", "ndh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ndh", "Ndali"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ndam", "ndm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ndm", "Ndam"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ndamba", "ndj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ndj", "Ndamba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ndasa", "nda"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nda", "Ndasa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ndau", "ndc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ndc", "Ndau"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nde-Gbite", "ned"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ned", "Nde-Gbite"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nde-Nsele-Nta", "ndd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ndd", "Nde-Nsele-Nta"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ndemli", "nml"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nml", "Ndemli"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ndendeule", "dne"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dne", "Ndendeule"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ndengereko", "ndg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ndg", "Ndengereko"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nding", "eli"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("eli", "Nding"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ndo", "ndp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ndp", "Ndo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ndobo", "ndw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ndw", "Ndobo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ndoe", "nbb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nbb", "Ndoe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ndogo", "ndz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ndz", "Ndogo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ndolo", "ndl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ndl", "Ndolo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ndom", "nqm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nqm", "Ndom"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ndombe", "ndq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ndq", "Ndombe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ndonde Hamba", "njd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("njd", "Ndonde Hamba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ndonga", "ndo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ndo", "Ndonga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ndoola", "ndr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ndr", "Ndoola"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nduga", "ndx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ndx", "Nduga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ndumu", "nmd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nmd", "Ndumu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ndunda", "nuh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nuh", "Ndunda"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ndunga", "ndt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ndt", "Ndunga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ndut", "ndv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ndv", "Ndut"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ndyuka-Trio Pidgin", "njt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("njt", "Ndyuka-Trio Pidgin"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ndzwani Comorian", "wni"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wni", "Ndzwani Comorian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Neapolitan", "nap"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nap", "Neapolitan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nedebang", "nec"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nec", "Nedebang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nefamese", "nef"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nef", "Nefamese"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Negerhollands", "dcr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dcr", "Negerhollands"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Negeri Sembilan Malay", "zmi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zmi", "Negeri Sembilan Malay"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Negidal", "neg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("neg", "Negidal"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nehan", "nsn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nsn", "Nehan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nek", "nif"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nif", "Nek"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nekgini", "nkg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nkg", "Nekgini"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Neko", "nej"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nej", "Neko"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Neku", "nek"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nek", "Neku"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Neme", "nex"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nex", "Neme"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nemi", "nem"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nem", "Nemi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nen", "nqn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nqn", "Nen"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nend", "anh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("anh", "Nend"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nenets", "yrk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yrk", "Nenets"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nenge", "djk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("djk", "Nenge"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nengone", "nen"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nen", "Nengone"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Neo-Hittite", "nei"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nei", "Neo-Hittite"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nepal Bhasa", "new"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("new", "Nepal Bhasa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nepalese Sign Language", "nsp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nsp", "Nepalese Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nepali", "nep"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nep", "Nepali"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nepali Kurux", "kxl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kxl", "Nepali Kurux"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nete", "net"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("net", "Nete"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Neve'ei", "vnm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vnm", "Neve'ei"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Neverver", "lgk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lgk", "Neverver"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("New Caledonian Javanese", "jas"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jas", "New Caledonian Javanese"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("New Zealand Sign Language", "nzs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nzs", "New Zealand Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Newari", "new"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("new", "Newari"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Neyo", "ney"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ney", "Neyo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nez Perce", "nez"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nez", "Nez Perce"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nga La", "hlt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hlt", "Nga La"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ngaanyatjarra", "ntj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ntj", "Ngaanyatjarra"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ngad'a", "nxg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nxg", "Ngad'a"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ngadjunmaya", "nju"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nju", "Ngadjunmaya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ngaing", "nnf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nnf", "Ngaing"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ngaju", "nij"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nij", "Ngaju"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ngala", "nud"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nud", "Ngala"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ngalakan", "nig"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nig", "Ngalakan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ngalkbun", "ngk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ngk", "Ngalkbun"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ngalum", "szb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("szb", "Ngalum"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ngam", "nmc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nmc", "Ngam"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ngamambo", "nbv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nbv", "Ngamambo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ngambay", "sba"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sba", "Ngambay"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ngamini", "nmv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nmv", "Ngamini"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ngamo", "nbh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nbh", "Ngamo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nganasan", "nio"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nio", "Nganasan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ngandi", "nid"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nid", "Ngandi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ngando (Central African Republic)", "ngd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ngd", "Ngando (Central African Republic)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ngando (Democratic Republic of Congo)", "nxd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nxd", "Ngando (Democratic Republic of Congo)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ngandyera", "nne"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nne", "Ngandyera"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ngangam", "gng"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gng", "Ngangam"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nganyaywana", "nyx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nyx", "Nganyaywana"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ngarinman", "nbj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nbj", "Ngarinman"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ngarinyin", "ung"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ung", "Ngarinyin"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ngarla", "nlr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nlr", "Ngarla"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ngarluma", "nrl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nrl", "Ngarluma"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ngas", "anc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("anc", "Ngas"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ngasa", "nsg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nsg", "Ngasa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ngatik Men's Creole", "ngm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ngm", "Ngatik Men's Creole"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ngawn Chin", "cnw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cnw", "Ngawn Chin"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ngawun", "nxn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nxn", "Ngawun"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ngazidja Comorian", "zdj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zdj", "Ngazidja Comorian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ngbaka", "nga"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nga", "Ngbaka"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ngbaka Ma'bo", "nbm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nbm", "Ngbaka Ma'bo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ngbaka Manza", "ngg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ngg", "Ngbaka Manza"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ngbee", "jgb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jgb", "Ngbee"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ngbinda", "nbd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nbd", "Ngbinda"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ngbundu", "nuu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nuu", "Ngbundu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ngelima", "agh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("agh", "Ngelima"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ngemba", "nge"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nge", "Ngemba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ngeq", "ngt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ngt", "Ngeq"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ngete", "nnn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nnn", "Ngete"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nggem", "nbq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nbq", "Nggem"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nggwahyi", "ngx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ngx", "Nggwahyi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ngie", "ngj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ngj", "Ngie"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ngiemboon", "nnh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nnh", "Ngiemboon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ngile", "jle"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jle", "Ngile"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ngindo", "nnq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nnq", "Ngindo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ngiti", "niy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("niy", "Ngiti"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ngizim", "ngi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ngi", "Ngizim"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ngkâlmpw Kanum", "kcd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kcd", "Ngkâlmpw Kanum"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ngom", "nra"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nra", "Ngom"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ngomba", "jgo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jgo", "Ngomba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ngombale", "nla"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nla", "Ngombale"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ngombe (Central African Republic)", "nmj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nmj", "Ngombe (Central African Republic)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ngombe (Democratic Republic of Congo)", "ngc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ngc", "Ngombe (Democratic Republic of Congo)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ngong", "nnx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nnx", "Ngong"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ngongo", "noq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("noq", "Ngongo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ngoni", "ngo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ngo", "Ngoni"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ngoreme", "ngq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ngq", "Ngoreme"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ngoshie", "nsh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nsh", "Ngoshie"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ngul", "nlo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nlo", "Ngul"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ngulu", "ngp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ngp", "Ngulu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nguluwan", "nuw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nuw", "Nguluwan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ngumbi", "nui"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nui", "Ngumbi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ngundi", "ndn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ndn", "Ngundi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ngundu", "nue"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nue", "Ngundu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ngungwel", "ngz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ngz", "Ngungwel"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ngura", "nbx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nbx", "Ngura"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ngurimi", "ngq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ngq", "Ngurimi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ngurmbur", "nrx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nrx", "Ngurmbur"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nguôn", "nuo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nuo", "Nguôn"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ngwaba", "ngw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ngw", "Ngwaba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ngwe", "nwe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nwe", "Ngwe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ngwo", "ngn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ngn", "Ngwo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ngäbere", "gym"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gym", "Ngäbere"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nhanda", "nha"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nha", "Nhanda"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nhengatu", "yrl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yrl", "Nhengatu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nhuwala", "nhf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nhf", "Nhuwala"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nias", "nia"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nia", "Nias"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nicaragua Creole English", "bzk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bzk", "Nicaragua Creole English"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nicaraguan Sign Language", "ncs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ncs", "Nicaraguan Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nicarao", "ppl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ppl", "Nicarao"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Niellim", "nie"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nie", "Niellim"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nigeria Mambila", "mzk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mzk", "Nigeria Mambila"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nigerian Fulfulde", "fuv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("fuv", "Nigerian Fulfulde"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nigerian Pidgin", "pcm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pcm", "Nigerian Pidgin"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nigerian Sign Language", "nsi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nsi", "Nigerian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nihali", "nll"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nll", "Nihali"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nii", "nii"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nii", "Nii"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nijadali", "nad"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nad", "Nijadali"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Niksek", "gbe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gbe", "Niksek"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nila", "nil"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nil", "Nila"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nilamba", "nim"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nim", "Nilamba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nimadi", "noe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("noe", "Nimadi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nimanbur", "nmp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nmp", "Nimanbur"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nimbari", "nmr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nmr", "Nimbari"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nimboran", "nir"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nir", "Nimboran"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nimi", "nis"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nis", "Nimi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nimo", "niw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("niw", "Nimo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nimoa", "nmw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nmw", "Nimoa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ninam", "shb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("shb", "Ninam"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ninde", "mwi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mwi", "Ninde"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nindi", "nxi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nxi", "Nindi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ningera", "nby"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nby", "Ningera"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ninggerum", "nxr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nxr", "Ninggerum"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ningil", "niz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("niz", "Ningil"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ningye", "nns"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nns", "Ningye"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ninia Yali", "nlk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nlk", "Ninia Yali"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ninzo", "nin"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nin", "Ninzo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nipsan", "nps"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nps", "Nipsan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nisa", "njs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("njs", "Nisa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nisenan", "nsz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nsz", "Nisenan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nisga'a", "ncg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ncg", "Nisga'a"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nisi (China)", "yso"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yso", "Nisi (China)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nisi (India)", "dap"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dap", "Nisi (India)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Niuafo'ou", "num"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("num", "Niuafo'ou"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Niuatoputapu", "nkp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nkp", "Niuatoputapu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Niuean", "niu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("niu", "Niuean"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nivaclé", "cag"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cag", "Nivaclé"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Njalgulgule", "njl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("njl", "Njalgulgule"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Njebi", "nzb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nzb", "Njebi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Njen", "njj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("njj", "Njen"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Njerep", "njr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("njr", "Njerep"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Njyem", "njy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("njy", "Njyem"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nkami", "nkq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nkq", "Nkami"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nkangala", "nkn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nkn", "Nkangala"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nkari", "nkz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nkz", "Nkari"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nkem-Nkum", "isi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("isi", "Nkem-Nkum"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nkhumbi", "khu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("khu", "Nkhumbi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nkongho", "nkc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nkc", "Nkongho"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nkonya", "nko"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nko", "Nkonya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nkoroo", "nkx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nkx", "Nkoroo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nkoya", "nka"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nka", "Nkoya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nkukoli", "nbo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nbo", "Nkukoli"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nkutu", "nkw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nkw", "Nkutu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nnam", "nbp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nbp", "Nnam"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("No linguistic content", "zxx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zxx", "No linguistic content"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nobiin", "fia"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("fia", "Nobiin"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nobonob", "gaw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gaw", "Nobonob"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nocamán", "nom"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nom", "Nocamán"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nocte Naga", "njb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("njb", "Nocte Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nogai", "nog"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nog", "Nogai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Noiri", "noi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("noi", "Noiri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nokuku", "nkk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nkk", "Nokuku"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nomaande", "lem"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lem", "Nomaande"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nomane", "nof"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nof", "Nomane"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nomatsiguenga", "not"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("not", "Nomatsiguenga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nomu", "noh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("noh", "Nomu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nong Zhuang", "zhn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zhn", "Nong Zhuang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nonuya", "noj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("noj", "Nonuya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nooksack", "nok"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nok", "Nooksack"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Noon", "snf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("snf", "Noon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Noone", "nhu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nhu", "Noone"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nootka", "noo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("noo", "Nootka"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nopala Chatino", "cya"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cya", "Nopala Chatino"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Noric", "nrc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nrc", "Noric"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Norn", "nrn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nrn", "Norn"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Norra", "nrr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nrr", "Norra"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("North Alaskan Inupiatun", "esi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("esi", "North Alaskan Inupiatun"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("North Ambrym", "mmg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mmg", "North Ambrym"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("North Asmat", "nks"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nks", "North Asmat"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("North Awyu", "yir"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yir", "North Awyu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("North Azerbaijani", "azj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("azj", "North Azerbaijani"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("North Babar", "bcd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bcd", "North Babar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("North Bolivian Quechua", "qul"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("qul", "North Bolivian Quechua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("North Central Mixe", "neq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("neq", "North Central Mixe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("North Efate", "llp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("llp", "North Efate"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("North Fali", "fll"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("fll", "North Fali"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("North Giziga", "gis"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gis", "North Giziga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("North Junín Quechua", "qvn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("qvn", "North Junín Quechua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("North Levantine Arabic", "apc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("apc", "North Levantine Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("North Marquesan", "mrq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mrq", "North Marquesan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("North Mesopotamian Arabic", "ayp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ayp", "North Mesopotamian Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("North Mofu", "mfk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mfk", "North Mofu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("North Moluccan Malay", "max"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("max", "North Moluccan Malay"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("North Muyu", "kti"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kti", "North Muyu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("North Ndebele", "nde"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nde", "North Ndebele"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("North Nuaulu", "nni"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nni", "North Nuaulu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("North Picene", "nrp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nrp", "North Picene"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("North Slavey", "scs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("scs", "North Slavey"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("North Tairora", "tbg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tbg", "North Tairora"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("North Tanna", "tnn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tnn", "North Tanna"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("North Wahgi", "whg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("whg", "North Wahgi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("North Watut", "una"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("una", "North Watut"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("North Wemale", "weo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("weo", "North Wemale"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northeast Kiwai", "kiw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kiw", "Northeast Kiwai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northeast Maidu", "nmu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nmu", "Northeast Maidu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northeast Pashayi", "aee"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aee", "Northeast Pashayi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northeastern Dinka", "dip"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dip", "Northeastern Dinka"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northeastern Pomo", "pef"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pef", "Northeastern Pomo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northeastern Thai", "tts"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tts", "Northeastern Thai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northern Alta", "aqn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aqn", "Northern Alta"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northern Altai", "atv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("atv", "Northern Altai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northern Amami-Oshima", "ryn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ryn", "Northern Amami-Oshima"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northern Bai", "bfc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bfc", "Northern Bai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northern Betsimisaraka Malagasy", "bmm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bmm", "Northern Betsimisaraka Malagasy"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northern Binukidnon", "kyn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kyn", "Northern Binukidnon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northern Bobo Madaré", "bbo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bbo", "Northern Bobo Madaré"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northern Bontok", "rbk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rbk", "Northern Bontok"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northern Catanduanes Bicolano", "cts"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cts", "Northern Catanduanes Bicolano"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northern Conchucos Ancash Quechua", "qxn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("qxn", "Northern Conchucos Ancash Quechua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northern Dagara", "dgi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dgi", "Northern Dagara"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northern Dong", "doc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("doc", "Northern Dong"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northern East Cree", "crl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("crl", "Northern East Cree"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northern Emberá", "emp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("emp", "Northern Emberá"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northern Frisian", "frr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("frr", "Northern Frisian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northern Ghale", "ghh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ghh", "Northern Ghale"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northern Gondi", "gno"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gno", "Northern Gondi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northern Grebo", "gbo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gbo", "Northern Grebo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northern Guiyang Hmong", "huj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("huj", "Northern Guiyang Hmong"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northern Guiyang Miao", "huj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("huj", "Northern Guiyang Miao"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northern Haida", "hdn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hdn", "Northern Haida"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northern Hindko", "hno"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hno", "Northern Hindko"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northern Huishui Hmong", "hmi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hmi", "Northern Huishui Hmong"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northern Huishui Miao", "hmi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hmi", "Northern Huishui Miao"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northern Kalapuya", "nrt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nrt", "Northern Kalapuya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northern Kankanay", "xnn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xnn", "Northern Kankanay"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northern Khmer", "kxm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kxm", "Northern Khmer"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northern Kissi", "kqs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kqs", "Northern Kissi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northern Kurdish", "kmr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kmr", "Northern Kurdish"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northern Lorung", "lbr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lbr", "Northern Lorung"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northern Luri", "lrc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lrc", "Northern Luri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northern Mashan Hmong", "hmp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hmp", "Northern Mashan Hmong"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northern Mashan Miao", "hmp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hmp", "Northern Mashan Miao"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northern Muji", "ymx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ymx", "Northern Muji"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northern Nago", "xkb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xkb", "Northern Nago"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northern Ngbandi", "ngb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ngb", "Northern Ngbandi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northern Nisu", "yiv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yiv", "Northern Nisu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northern Nuni", "nuv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nuv", "Northern Nuni"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northern Oaxaca Nahuatl", "nhy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nhy", "Northern Oaxaca Nahuatl"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northern Ohlone", "cst"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cst", "Northern Ohlone"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northern One", "onr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("onr", "Northern One"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northern Paiute", "pao"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pao", "Northern Paiute"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northern Pame", "pmq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pmq", "Northern Pame"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northern Pashto", "pbu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pbu", "Northern Pashto"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northern Pastaza Quichua", "qvz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("qvz", "Northern Pastaza Quichua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northern Pomo", "pej"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pej", "Northern Pomo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northern Puebla Nahuatl", "ncj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ncj", "Northern Puebla Nahuatl"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northern Pumi", "pmi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pmi", "Northern Pumi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northern Qiandong Miao", "hea"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hea", "Northern Qiandong Miao"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northern Qiang", "cng"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cng", "Northern Qiang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northern Rengma Naga", "nnl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nnl", "Northern Rengma Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northern Roglai", "rog"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rog", "Northern Roglai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northern Sami", "sme"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sme", "Northern Sami"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northern Sierra Miwok", "nsq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nsq", "Northern Sierra Miwok"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northern Sorsoganon", "bks"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bks", "Northern Sorsoganon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northern Sotho", "nso"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nso", "Northern Sotho"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northern Subanen", "stb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("stb", "Northern Subanen"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northern Tarahumara", "thh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("thh", "Northern Tarahumara"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northern Tepehuan", "ntp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ntp", "Northern Tepehuan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northern Thai", "nod"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nod", "Northern Thai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northern Tiwa", "twf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("twf", "Northern Tiwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northern Tlaxiaco Mixtec", "xtn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xtn", "Northern Tlaxiaco Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northern Toussian", "tsp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tsp", "Northern Toussian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northern Tujia", "tji"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tji", "Northern Tujia"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northern Tutchone", "ttm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ttm", "Northern Tutchone"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northern Uzbek", "uzn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("uzn", "Northern Uzbek"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northern Yukaghir", "ykg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ykg", "Northern Yukaghir"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northwest Alaska Inupiatun", "esk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("esk", "Northwest Alaska Inupiatun"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northwest Gbaya", "gya"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gya", "Northwest Gbaya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northwest Maidu", "mjd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mjd", "Northwest Maidu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northwest Oaxaca Mixtec", "mxa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mxa", "Northwest Oaxaca Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northwest Pashayi", "glh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("glh", "Northwest Pashayi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northwestern Dinka", "diw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("diw", "Northwestern Dinka"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northwestern Fars", "faz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("faz", "Northwestern Fars"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northwestern Kolami", "kfb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kfb", "Northwestern Kolami"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northwestern Ojibwa", "ojb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ojb", "Northwestern Ojibwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Northwestern Tamang", "tmk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tmk", "Northwestern Tamang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Norwegian", "nor"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nor", "Norwegian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Norwegian Bokmål", "nob"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nob", "Norwegian Bokmål"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Norwegian Nynorsk", "nno"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nno", "Norwegian Nynorsk"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Norwegian Sign Language", "nsl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nsl", "Norwegian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Not applicable", "zxx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zxx", "Not applicable"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Notre", "bly"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bly", "Notre"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Notsi", "ncf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ncf", "Notsi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nottoway", "ntw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ntw", "Nottoway"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Novial", "nov"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nov", "Novial"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Noy", "noy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("noy", "Noy"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nsari", "asj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("asj", "Nsari"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nsenga", "nse"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nse", "Nsenga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nshi", "nsc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nsc", "Nshi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nsongo", "nsx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nsx", "Nsongo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ntcham", "bud"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bud", "Ntcham"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ntomba", "nto"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nto", "Ntomba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nubaca", "baf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("baf", "Nubaca"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nubi", "kcn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kcn", "Nubi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nubri", "kte"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kte", "Nubri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nuer", "nus"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nus", "Nuer"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nugunu (Australia)", "nnv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nnv", "Nugunu (Australia)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nugunu (Cameroon)", "yas"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yas", "Nugunu (Cameroon)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nuk", "noc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("noc", "Nuk"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nukak Makú", "mbr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mbr", "Nukak Makú"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nukna", "klt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("klt", "Nukna"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nukuini", "nuc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nuc", "Nukuini"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nukumanu", "nuq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nuq", "Nukumanu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nukuoro", "nkr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nkr", "Nukuoro"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nukuria", "nur"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nur", "Nukuria"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Numanggang", "nop"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nop", "Numanggang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Numbami", "sij"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sij", "Numbami"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nume", "tgs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tgs", "Nume"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Numee", "kdk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kdk", "Numee"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Numidian", "nxm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nxm", "Numidian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nung (Myanmar)", "nun"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nun", "Nung (Myanmar)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nung (Viet Nam)", "nut"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nut", "Nung (Viet Nam)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nungali", "nug"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nug", "Nungali"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nunggubuyu", "nuy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nuy", "Nunggubuyu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nungu", "rin"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rin", "Nungu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nuosu", "iii"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("iii", "Nuosu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nupbikha", "npb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("npb", "Nupbikha"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nupe-Nupe-Tako", "nup"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nup", "Nupe-Nupe-Tako"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nusa Laut", "nul"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nul", "Nusa Laut"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nusu", "nuf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nuf", "Nusu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nyabwa", "nwb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nwb", "Nyabwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nyaheun", "nev"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nev", "Nyaheun"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nyahkur", "cbn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cbn", "Nyahkur"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nyakyusa-Ngonde", "nyy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nyy", "Nyakyusa-Ngonde"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nyali", "nlj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nlj", "Nyali"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nyam", "nmi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nmi", "Nyam"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nyamal", "nly"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nly", "Nyamal"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nyambo", "now"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("now", "Nyambo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nyamusa-Molo", "nwm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nwm", "Nyamusa-Molo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nyamwanga", "mwn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mwn", "Nyamwanga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nyamwezi", "nym"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nym", "Nyamwezi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nyaneka", "nyk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nyk", "Nyaneka"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nyang'i", "nyp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nyp", "Nyang'i"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nyanga", "nyj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nyj", "Nyanga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nyanga-li", "nyc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nyc", "Nyanga-li"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nyangatom", "nnj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nnj", "Nyangatom"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nyangbo", "nyb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nyb", "Nyangbo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nyangga", "nny"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nny", "Nyangga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nyangumarta", "nna"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nna", "Nyangumarta"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nyanja", "nya"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nya", "Nyanja"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nyankole", "nyn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nyn", "Nyankole"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nyarafolo Senoufo", "sev"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sev", "Nyarafolo Senoufo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nyaturu", "rim"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rim", "Nyaturu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nyaw", "nyw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nyw", "Nyaw"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nyawaygi", "nyt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nyt", "Nyawaygi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nyemba", "nba"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nba", "Nyemba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nyengo", "nye"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nye", "Nyengo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nyenkha", "neh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("neh", "Nyenkha"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nyeu", "nyl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nyl", "Nyeu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nyigina", "nyh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nyh", "Nyigina"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nyiha (Malawi)", "nyr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nyr", "Nyiha (Malawi)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nyiha (Tanzania)", "nih"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nih", "Nyiha (Tanzania)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nyika (Malawi and Zambia)", "nkv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nkv", "Nyika (Malawi and Zambia)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nyika (Tanzania)", "nkt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nkt", "Nyika (Tanzania)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nyindrou", "lid"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lid", "Nyindrou"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nyindu", "nyg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nyg", "Nyindu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nyole", "nuj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nuj", "Nyole"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nyong", "muo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("muo", "Nyong"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nyore", "nyd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nyd", "Nyore"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nyoro", "nyo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nyo", "Nyoro"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nyulnyul", "nyv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nyv", "Nyulnyul"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nyunga", "nys"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nys", "Nyunga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nyungwe", "nyu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nyu", "Nyungwe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nyâlayu", "yly"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yly", "Nyâlayu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nzakambay", "nzy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nzy", "Nzakambay"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nzakara", "nzk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nzk", "Nzakara"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nzanyi", "nja"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nja", "Nzanyi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nzima", "nzi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nzi", "Nzima"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ná-Meo", "neo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("neo", "Ná-Meo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Nüpode Huitoto", "hux"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hux", "Nüpode Huitoto"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("O'chi'chi'", "xoc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xoc", "O'chi'chi'"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("O'du", "tyh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tyh", "O'du"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Obanliku", "bzy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bzy", "Obanliku"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Obispeño", "obi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("obi", "Obispeño"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Oblo", "obl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("obl", "Oblo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Obo Manobo", "obo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("obo", "Obo Manobo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Obokuitai", "afz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("afz", "Obokuitai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Obolo", "ann"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ann", "Obolo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Obulom", "obu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("obu", "Obulom"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ocaina", "oca"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("oca", "Ocaina"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Occidental", "ile"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ile", "Occidental"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Occitan (post 1500)", "oci"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("oci", "Occitan (post 1500)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ocotepec Mixtec", "mie"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mie", "Ocotepec Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ocotlán Zapotec", "zac"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zac", "Ocotlán Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Od", "odk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("odk", "Od"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Odiai", "bhf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bhf", "Odiai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Odoodee", "kkc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kkc", "Odoodee"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Odual", "odu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("odu", "Odual"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Odut", "oda"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("oda", "Odut"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ofayé", "opy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("opy", "Ofayé"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Official Aramaic (700-300 BCE)", "arc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("arc", "Official Aramaic (700-300 BCE)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ofo", "ofo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ofo", "Ofo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ogbah", "ogc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ogc", "Ogbah"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ogbia", "ogb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ogb", "Ogbia"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ogbogolo", "ogg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ogg", "Ogbogolo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ogbronuagum", "ogu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ogu", "Ogbronuagum"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ogea", "eri"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("eri", "Ogea"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Oirat", "xal"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xal", "Oirat"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Oirata", "oia"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("oia", "Oirata"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ojibwa", "oji"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("oji", "Ojibwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ojitlán Chinantec", "chj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("chj", "Ojitlán Chinantec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Okanagan", "oka"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("oka", "Okanagan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Oki-No-Erabu", "okn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("okn", "Oki-No-Erabu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Okiek", "oki"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("oki", "Okiek"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Oko-Eni-Osayen", "oks"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("oks", "Oko-Eni-Osayen"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Oko-Juwoi", "okj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("okj", "Oko-Juwoi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Okobo", "okb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("okb", "Okobo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Okodia", "okd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("okd", "Okodia"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Okolod", "kqv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kqv", "Okolod"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Okpamheri", "opa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("opa", "Okpamheri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Okpe (Northwestern Edo)", "okx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("okx", "Okpe (Northwestern Edo)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Okpe (Southwestern Edo)", "oke"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("oke", "Okpe (Southwestern Edo)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Oksapmin", "opm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("opm", "Oksapmin"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Oku", "oku"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("oku", "Oku"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Old Aramaic (up to 700 BCE)", "oar"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("oar", "Old Aramaic (up to 700 BCE)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Old Avar", "oav"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("oav", "Old Avar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Old Breton", "obt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("obt", "Old Breton"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Old Bulgarian", "chu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("chu", "Old Bulgarian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Old Burmese", "obr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("obr", "Old Burmese"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Old Chinese", "och"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("och", "Old Chinese"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Old Church Slavonic", "chu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("chu", "Old Church Slavonic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Old Cornish", "oco"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("oco", "Old Cornish"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Old Dutch", "odt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("odt", "Old Dutch"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Old English (ca. 450-1100)", "ang"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ang", "Old English (ca. 450-1100)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Old French (842-ca. 1400)", "fro"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("fro", "Old French (842-ca. 1400)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Old Frisian", "ofs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ofs", "Old Frisian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Old Georgian", "oge"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("oge", "Old Georgian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Old High German (ca. 750-1050)", "goh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("goh", "Old High German (ca. 750-1050)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Old Hittite", "oht"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("oht", "Old Hittite"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Old Hungarian", "ohu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ohu", "Old Hungarian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Old Irish (to 900)", "sga"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sga", "Old Irish (to 900)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Old Japanese", "ojp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ojp", "Old Japanese"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Old Kentish Sign Language", "okl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("okl", "Old Kentish Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Old Korean (3rd-9th cent.)", "oko"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("oko", "Old Korean (3rd-9th cent.)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Old Manipuri", "omp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("omp", "Old Manipuri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Old Marathi", "omr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("omr", "Old Marathi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Old Mon", "omx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("omx", "Old Mon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Old Newari", "nwc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nwc", "Old Newari"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Old Norse", "non"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("non", "Old Norse"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Old Nubian", "onw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("onw", "Old Nubian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Old Occitan (to 1500)", "pro"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pro", "Old Occitan (to 1500)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Old Ossetic", "oos"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("oos", "Old Ossetic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Old Persian (ca. 600-400 B.C.)", "peo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("peo", "Old Persian (ca. 600-400 B.C.)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Old Provençal (to 1500)", "pro"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pro", "Old Provençal (to 1500)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Old Russian", "orv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("orv", "Old Russian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Old Saxon", "osx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("osx", "Old Saxon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Old Slavonic", "chu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("chu", "Old Slavonic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Old Spanish", "osp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("osp", "Old Spanish"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Old Tamil", "oty"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("oty", "Old Tamil"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Old Tibetan", "otb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("otb", "Old Tibetan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Old Turkish", "otk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("otk", "Old Turkish"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Old Uighur", "oui"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("oui", "Old Uighur"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Old Welsh", "owl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("owl", "Old Welsh"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Olekha", "ole"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ole", "Olekha"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Olo", "ong"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ong", "Olo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Oloma", "olm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("olm", "Oloma"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Olrat", "olr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("olr", "Olrat"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Olu'bo", "lul"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lul", "Olu'bo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Olukhayo", "lko"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lko", "Olukhayo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Olulumo-Ikom", "iko"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("iko", "Olulumo-Ikom"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Oluluyia", "luy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("luy", "Oluluyia"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Olumarachi", "lri"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lri", "Olumarachi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Olumarama", "lrm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lrm", "Olumarama"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Olunyole", "nyd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nyd", "Olunyole"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Olusamia", "lsm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lsm", "Olusamia"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Olushisa", "lks"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lks", "Olushisa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Oluta Popoluca", "plo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("plo", "Oluta Popoluca"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Olutsotso", "lto"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lto", "Olutsotso"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Oluwanga", "lwg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lwg", "Oluwanga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Omagua", "omg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("omg", "Omagua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Omaha-Ponca", "oma"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("oma", "Omaha-Ponca"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Omani Arabic", "acx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("acx", "Omani Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Omati", "mgx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mgx", "Omati"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ombamba", "mbm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mbm", "Ombamba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ombo", "oml"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("oml", "Ombo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Omejes", "ome"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ome", "Omejes"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ometepec Nahuatl", "nht"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nht", "Ometepec Nahuatl"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Omi", "omi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("omi", "Omi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Omok", "omk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("omk", "Omok"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Omotik", "omt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("omt", "Omotik"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Omurano", "omu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("omu", "Omurano"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ona", "ona"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ona", "Ona"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Oneida", "one"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("one", "Oneida"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ong", "oog"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("oog", "Ong"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Onin", "oni"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("oni", "Onin"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Onin Based Pidgin", "onx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("onx", "Onin Based Pidgin"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Oniyan", "bsc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bsc", "Oniyan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Onjob", "onj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("onj", "Onjob"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ono", "ons"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ons", "Ono"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Onobasulu", "onn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("onn", "Onobasulu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Onondaga", "ono"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ono", "Onondaga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ontenu", "ont"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ont", "Ontenu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ontong Java", "ojv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ojv", "Ontong Java"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Oorlams", "oor"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("oor", "Oorlams"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Opao", "opo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("opo", "Opao"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Opata", "opt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("opt", "Opata"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Opuuo", "lgn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lgn", "Opuuo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Orang Kanaq", "orn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("orn", "Orang Kanaq"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Orang Seletar", "ors"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ors", "Orang Seletar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Oraon Sadri", "sdr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sdr", "Oraon Sadri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Orejón", "ore"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ore", "Orejón"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Oring", "org"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("org", "Oring"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Oriya", "ori"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ori", "Oriya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Orizaba Nahuatl", "nlv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nlv", "Orizaba Nahuatl"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Orma", "orc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("orc", "Orma"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ormu", "orz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("orz", "Ormu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ormuri", "oru"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("oru", "Ormuri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Oro", "orx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("orx", "Oro"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Oro Win", "orw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("orw", "Oro Win"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Oroch", "oac"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("oac", "Oroch"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Oroha", "ora"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ora", "Oroha"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Orok", "oaa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("oaa", "Orok"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Orokaiva", "okv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("okv", "Orokaiva"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Oroko", "bdu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bdu", "Oroko"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Orokolo", "oro"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("oro", "Orokolo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Oromo", "orm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("orm", "Oromo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Oroqen", "orh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("orh", "Oroqen"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Orowe", "bpk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bpk", "Orowe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Oruma", "orr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("orr", "Oruma"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Orya", "ury"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ury", "Orya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Osage", "osa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("osa", "Osage"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Osatu", "ost"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ost", "Osatu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Oscan", "osc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("osc", "Oscan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Osing", "osi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("osi", "Osing"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ososo", "oso"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("oso", "Ososo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ossetian", "oss"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("oss", "Ossetian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ossetic", "oss"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("oss", "Ossetic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ot Danum", "otd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("otd", "Ot Danum"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Otank", "uta"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("uta", "Otank"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Oti", "oti"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("oti", "Oti"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Otoro", "otr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("otr", "Otoro"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ottawa", "otw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("otw", "Ottawa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ottoman Turkish (1500-1928)", "ota"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ota", "Ottoman Turkish (1500-1928)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Otuho", "lot"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lot", "Otuho"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Otuke", "otu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("otu", "Otuke"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ouma", "oum"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("oum", "Ouma"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ounge", "oue"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("oue", "Ounge"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Owa", "stn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("stn", "Owa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Owenia", "wsr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wsr", "Owenia"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Owiniga", "owi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("owi", "Owiniga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Oy", "oyb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("oyb", "Oy"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Oya'oya", "oyy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("oyy", "Oya'oya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Oyda", "oyd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("oyd", "Oyda"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ozolotepec Zapotec", "zao"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zao", "Ozolotepec Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ozumacín Chinantec", "chz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("chz", "Ozumacín Chinantec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pa Di", "pdi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pdi", "Pa Di"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pa'a", "pqa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pqa", "Pa'a"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pa'o Karen", "blk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("blk", "Pa'o Karen"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pa-Hng", "pha"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pha", "Pa-Hng"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Paama", "pma"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pma", "Paama"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Paasaal", "sig"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sig", "Paasaal"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pacahuara", "pcp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pcp", "Pacahuara"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pacaraos Quechua", "qvp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("qvp", "Pacaraos Quechua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pacific Gulf Yupik", "ems"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ems", "Pacific Gulf Yupik"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pacoh", "pac"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pac", "Pacoh"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Padoe", "pdo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pdo", "Padoe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Paekche", "pkc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pkc", "Paekche"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Paelignian", "pgn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pgn", "Paelignian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pagi", "pgi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pgi", "Pagi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pagibete", "pae"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pae", "Pagibete"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pagu", "pgu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pgu", "Pagu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pahanan Agta", "apf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("apf", "Pahanan Agta"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pahari-Potwari", "phr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("phr", "Pahari-Potwari"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pahi", "lgt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lgt", "Pahi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pahlavani", "phv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("phv", "Pahlavani"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pahlavi", "pal"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pal", "Pahlavi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pai Tavytera", "pta"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pta", "Pai Tavytera"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Paicî", "pri"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pri", "Paicî"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Paipai", "ppi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ppi", "Paipai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Paite Chin", "pck"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pck", "Paite Chin"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Paiwan", "pwn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pwn", "Paiwan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pak-Tong", "pkg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pkg", "Pak-Tong"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pakanha", "pkn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pkn", "Pakanha"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pakaásnovos", "pav"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pav", "Pakaásnovos"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pakistan Sign Language", "pks"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pks", "Pakistan Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Paku", "pku"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pku", "Paku"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Paku Karen", "kpp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kpp", "Paku Karen"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pal", "abw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("abw", "Pal"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Palaic", "plq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("plq", "Palaic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Palaka Senoufo", "plr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("plr", "Palaka Senoufo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Palantla Chinantec", "cpa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cpa", "Palantla Chinantec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Palauan", "pau"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pau", "Palauan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Palenquero", "pln"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pln", "Palenquero"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pali", "pli"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pli", "Pali"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Palikúr", "plu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("plu", "Palikúr"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Paliyan", "pcf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pcf", "Paliyan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Palor", "fap"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("fap", "Palor"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Palpa", "plp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("plp", "Palpa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Palu", "pbz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pbz", "Palu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Palu'e", "ple"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ple", "Palu'e"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Paluan", "plz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("plz", "Paluan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Palula", "phl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("phl", "Palula"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Palumata", "pmc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pmc", "Palumata"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Palya Bareli", "bpx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bpx", "Palya Bareli"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pam", "pmn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pmn", "Pam"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pambia", "pmb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pmb", "Pambia"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pamlico", "pmk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pmk", "Pamlico"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pamona", "pmf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pmf", "Pamona"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pamosu", "hih"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hih", "Pamosu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pampanga", "pam"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pam", "Pampanga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pamplona Atta", "att"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("att", "Pamplona Atta"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pana (Burkina Faso)", "pnq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pnq", "Pana (Burkina Faso)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pana (Central African Republic)", "pnz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pnz", "Pana (Central African Republic)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Panamanian Sign Language", "lsp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lsp", "Panamanian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Panamint", "par"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("par", "Panamint"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Panang", "pcr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pcr", "Panang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Panao Huánuco Quechua", "qxh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("qxh", "Panao Huánuco Quechua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Panará", "kre"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kre", "Panará"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Panasuan", "psn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("psn", "Panasuan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Panawa", "pwb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pwb", "Panawa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pancana", "pnp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pnp", "Pancana"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Panchpargania", "tdb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tdb", "Panchpargania"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pande", "bkj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bkj", "Pande"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pangasinan", "pag"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pag", "Pangasinan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pangseng", "pgs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pgs", "Pangseng"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pangutaran Sama", "slm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("slm", "Pangutaran Sama"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pangwa", "pbr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pbr", "Pangwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pangwali", "pgg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pgg", "Pangwali"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Panim", "pnr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pnr", "Panim"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Paniya", "pcg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pcg", "Paniya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Panjabi", "pan"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pan", "Panjabi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pankararé", "pax"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pax", "Pankararé"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pankararú", "paz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("paz", "Pankararú"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pankhu", "pkh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pkh", "Pankhu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pannei", "pnc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pnc", "Pannei"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Panoan Katukína", "knt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("knt", "Panoan Katukína"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Panobo", "pno"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pno", "Panobo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Panytyima", "pnw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pnw", "Panytyima"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pao", "ppa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ppa", "Pao"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Papantla Totonac", "top"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("top", "Papantla Totonac"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Papapana", "ppn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ppn", "Papapana"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Papar", "dpp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dpp", "Papar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Papasena", "pas"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pas", "Papasena"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Papel", "pbo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pbo", "Papel"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Papi", "ppe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ppe", "Papi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Papiamento", "pap"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pap", "Papiamento"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Papitalai", "pat"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pat", "Papitalai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Papora", "ppu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ppu", "Papora"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Papuan Malay", "pmy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pmy", "Papuan Malay"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Papuma", "ppm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ppm", "Papuma"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Para Naga", "pzn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pzn", "Para Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Parachi", "prc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("prc", "Parachi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Paraguayan Guaraní", "gug"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gug", "Paraguayan Guaraní"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Paraguayan Sign Language", "pys"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pys", "Paraguayan Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Parakanã", "pak"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pak", "Parakanã"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Paranan", "prf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("prf", "Paranan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Paranawát", "paf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("paf", "Paranawát"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Paraujano", "pbg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pbg", "Paraujano"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Parauk", "prk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("prk", "Parauk"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Parawen", "prw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("prw", "Parawen"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pardhan", "pch"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pch", "Pardhan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pardhi", "pcl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pcl", "Pardhi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pare", "ppt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ppt", "Pare"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Parecís", "pab"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pab", "Parecís"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Parenga", "pcj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pcj", "Parenga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Parkari Koli", "kvx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kvx", "Parkari Koli"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Parkwa", "pbi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pbi", "Parkwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Parsi", "prp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("prp", "Parsi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Parsi-Dari", "prd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("prd", "Parsi-Dari"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Parthian", "xpr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xpr", "Parthian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Parya", "paq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("paq", "Parya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pará Arára", "aap"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aap", "Pará Arára"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pará Gavião", "gvp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gvp", "Pará Gavião"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pashto", "pus"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pus", "Pashto"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pasi", "psq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("psq", "Pasi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pass Valley Yali", "yac"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yac", "Pass Valley Yali"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Patamona", "pbc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pbc", "Patamona"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Patani", "ptn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ptn", "Patani"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pataxó Hã-Ha-Hãe", "pth"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pth", "Pataxó Hã-Ha-Hãe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Patep", "ptp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ptp", "Patep"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pathiya", "pty"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pty", "Pathiya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Patpatar", "gfk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gfk", "Patpatar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pattani", "lae"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lae", "Pattani"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pattani Malay", "mfa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mfa", "Pattani Malay"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Paulohi", "plh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("plh", "Paulohi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Paumarí", "pad"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pad", "Paumarí"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pauri Bareli", "bfb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bfb", "Pauri Bareli"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pauserna", "psm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("psm", "Pauserna"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pawaia", "pwa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pwa", "Pawaia"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pawnee", "paw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("paw", "Pawnee"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Paynamar", "pmr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pmr", "Paynamar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pe", "pai"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pai", "Pe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pear", "pcb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pcb", "Pear"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pech", "pay"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pay", "Pech"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pecheneg", "xpc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xpc", "Pecheneg"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pedi", "nso"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nso", "Pedi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Peere", "pfe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pfe", "Peere"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pei", "ppq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ppq", "Pei"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pekal", "pel"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pel", "Pekal"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pela", "bxd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bxd", "Pela"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pele-Ata", "ata"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ata", "Pele-Ata"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pelende", "ppp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ppp", "Pelende"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pemon", "aoc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aoc", "Pemon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Penang Sign Language", "psg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("psg", "Penang Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Penchal", "pek"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pek", "Penchal"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pendau", "ums"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ums", "Pendau"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pengo", "peg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("peg", "Pengo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pennsylvania German", "pdc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pdc", "Pennsylvania German"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Penrhyn", "pnh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pnh", "Penrhyn"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pentlatch", "ptw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ptw", "Pentlatch"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Perai", "wet"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wet", "Perai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Peranakan Indonesian", "pea"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pea", "Peranakan Indonesian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Peripheral Mongolian", "mvf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mvf", "Peripheral Mongolian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pero", "pip"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pip", "Pero"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Persian", "fas"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("fas", "Persian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Persian Sign Language", "psc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("psc", "Persian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Peruvian Sign Language", "prl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("prl", "Peruvian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Petapa Zapotec", "zpe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zpe", "Petapa Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Petats", "pex"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pex", "Petats"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Petjo", "pey"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pey", "Petjo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Peñoles Mixtec", "mil"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mil", "Peñoles Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pfaelzisch", "pfl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pfl", "Pfaelzisch"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Phai", "prt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("prt", "Phai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Phake", "phk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("phk", "Phake"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Phala", "ypa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ypa", "Phala"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Phalura", "phl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("phl", "Phalura"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Phana'", "phq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("phq", "Phana'"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Phangduwali", "phw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("phw", "Phangduwali"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Phende", "pem"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pem", "Phende"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Philippine Sign Language", "psp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("psp", "Philippine Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Phimbi", "phm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("phm", "Phimbi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Phoenician", "phn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("phn", "Phoenician"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Phola", "ypg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ypg", "Phola"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pholo", "yip"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yip", "Pholo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Phom Naga", "nph"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nph", "Phom Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Phong-Kniang", "pnx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pnx", "Phong-Kniang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Phrae Pwo Karen", "kjt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kjt", "Phrae Pwo Karen"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Phrygian", "xpg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xpg", "Phrygian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Phu Thai", "pht"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pht", "Phu Thai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Phuan", "phu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("phu", "Phuan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Phudagi", "phd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("phd", "Phudagi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Phuie", "pug"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pug", "Phuie"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Phukha", "phh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("phh", "Phukha"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Phuma", "ypm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ypm", "Phuma"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Phunoi", "pho"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pho", "Phunoi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Phuong", "phg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("phg", "Phuong"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Phupa", "ypp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ypp", "Phupa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Phupha", "yph"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yph", "Phupha"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Phuza", "ypz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ypz", "Phuza"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Piamatsina", "ptr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ptr", "Piamatsina"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Piame", "pin"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pin", "Piame"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Piapoco", "pio"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pio", "Piapoco"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Piaroa", "pid"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pid", "Piaroa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Picard", "pcd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pcd", "Picard"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pichis Ashéninka", "cpu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cpu", "Pichis Ashéninka"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pictish", "xpi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xpi", "Pictish"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pidgin Delaware", "dep"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dep", "Pidgin Delaware"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Piemontese", "pms"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pms", "Piemontese"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pijao", "pij"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pij", "Pijao"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pije", "piz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("piz", "Pije"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pijin", "pis"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pis", "Pijin"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pilagá", "plg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("plg", "Pilagá"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pileni", "piv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("piv", "Pileni"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pilipino", "fil"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("fil", "Pilipino"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pima Bajo", "pia"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pia", "Pima Bajo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pimbwe", "piw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("piw", "Pimbwe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pinai-Hagahai", "pnn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pnn", "Pinai-Hagahai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pingelapese", "pif"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pif", "Pingelapese"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pini", "pii"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pii", "Pini"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pinigura", "pnv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pnv", "Pinigura"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pinji", "pic"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pic", "Pinji"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pinotepa Nacional Mixtec", "mio"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mio", "Pinotepa Nacional Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pintiini", "pti"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pti", "Pintiini"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pintupi-Luritja", "piu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("piu", "Pintupi-Luritja"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pinyin", "pny"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pny", "Pinyin"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pipil", "ppl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ppl", "Pipil"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pirahã", "myp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("myp", "Pirahã"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Piratapuyo", "pir"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pir", "Piratapuyo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pirlatapa", "bxi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bxi", "Pirlatapa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Piro", "pie"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pie", "Piro"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Piru", "ppr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ppr", "Piru"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pisabo", "pig"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pig", "Pisabo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pisaflores Tepehua", "tpp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tpp", "Pisaflores Tepehua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Piscataway", "psy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("psy", "Piscataway"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pisidian", "xps"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xps", "Pisidian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pite Sami", "sje"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sje", "Pite Sami"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Piti", "pcn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pcn", "Piti"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pitjantjatjara", "pjt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pjt", "Pitjantjatjara"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pitta Pitta", "pit"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pit", "Pitta Pitta"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Piu", "pix"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pix", "Piu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Piya-Kwonci", "piy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("piy", "Piya-Kwonci"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Plains Cree", "crk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("crk", "Plains Cree"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Plains Indian Sign Language", "psd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("psd", "Plains Indian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Plains Miwok", "pmw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pmw", "Plains Miwok"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Plapo Krumen", "ktj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ktj", "Plapo Krumen"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Plateau Malagasy", "plt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("plt", "Plateau Malagasy"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Plautdietsch", "pdt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pdt", "Plautdietsch"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Playero", "gob"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gob", "Playero"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pnar", "pbv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pbv", "Pnar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pochuri Naga", "npo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("npo", "Pochuri Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pochutec", "xpo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xpo", "Pochutec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Podena", "pdn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pdn", "Podena"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pogolo", "poy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("poy", "Pogolo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pohnpeian", "pon"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pon", "Pohnpeian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pokangá", "pok"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pok", "Pokangá"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Poke", "pof"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pof", "Poke"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pokomo", "pkb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pkb", "Pokomo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Polabian", "pox"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pox", "Polabian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Polari", "pld"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pld", "Polari"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Polci", "plj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("plj", "Polci"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Polish", "pol"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pol", "Polish"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Polish Sign Language", "pso"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pso", "Polish Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Polonombauk", "plb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("plb", "Polonombauk"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pom", "pmo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pmo", "Pom"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pomo", "pmm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pmm", "Pomo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ponam", "ncc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ncc", "Ponam"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ponares", "pod"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pod", "Ponares"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pongu", "png"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("png", "Pongu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pongyong", "pgy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pgy", "Pongyong"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ponosakan", "pns"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pns", "Ponosakan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pontic", "pnt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pnt", "Pontic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Popti'", "jac"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jac", "Popti'"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Poqomam", "poc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("poc", "Poqomam"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Poqomchi'", "poh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("poh", "Poqomchi'"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Porohanon", "prh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("prh", "Porohanon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Port Sandwich", "psw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("psw", "Port Sandwich"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Port Vato", "ptv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ptv", "Port Vato"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Portuguese", "por"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("por", "Portuguese"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Portuguese Sign Language", "psr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("psr", "Portuguese Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Potawatomi", "pot"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pot", "Potawatomi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Potiguára", "pog"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pog", "Potiguára"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pottangi Ollar Gadaba", "gdb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gdb", "Pottangi Ollar Gadaba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Poule", "msl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("msl", "Poule"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Poumei Naga", "pmx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pmx", "Poumei Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pouye", "bye"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bye", "Pouye"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Powari", "pwr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pwr", "Powari"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Powhatan", "pim"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pim", "Powhatan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Poyanáwa", "pyn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pyn", "Poyanáwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Prasuni", "prn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("prn", "Prasuni"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pray 3", "pry"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pry", "Pray 3"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Principense", "pre"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pre", "Principense"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Providencia Sign Language", "prz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("prz", "Providencia Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Prussian", "prg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("prg", "Prussian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Psikye", "kvj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kvj", "Psikye"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pu Ko", "puk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("puk", "Pu Ko"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pu-Xian Chinese", "cpx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cpx", "Pu-Xian Chinese"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Puare", "pux"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pux", "Puare"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pudtol Atta", "atp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("atp", "Pudtol Atta"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Puelche", "pue"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pue", "Puelche"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Puerto Rican Sign Language", "psl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("psl", "Puerto Rican Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Puimei Naga", "npu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("npu", "Puimei Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Puinave", "pui"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pui", "Puinave"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pukapuka", "pkp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pkp", "Pukapuka"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pulaar", "fuc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("fuc", "Pulaar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pulabu", "pup"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pup", "Pulabu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pular", "fuf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("fuf", "Pular"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Puluwatese", "puw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("puw", "Puluwatese"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Puma", "pum"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pum", "Puma"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pumpokol", "xpm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xpm", "Pumpokol"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pumé", "yae"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yae", "Pumé"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Punan Aput", "pud"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pud", "Punan Aput"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Punan Bah-Biau", "pna"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pna", "Punan Bah-Biau"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Punan Batu 1", "pnm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pnm", "Punan Batu 1"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Punan Merah", "puf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("puf", "Punan Merah"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Punan Merap", "puc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("puc", "Punan Merap"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Punan Tubu", "puj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("puj", "Punan Tubu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Punic", "xpu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xpu", "Punic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Punjabi", "pan"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pan", "Punjabi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Puno Quechua", "qxp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("qxp", "Puno Quechua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Punu", "puu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("puu", "Punu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Puoc", "puo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("puo", "Puoc"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Puquina", "puq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("puq", "Puquina"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Puragi", "pru"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pru", "Puragi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Purari", "iar"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("iar", "Purari"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Purepecha", "tsz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tsz", "Purepecha"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Puri", "prr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("prr", "Puri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Purik", "prx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("prx", "Purik"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Purisimeño", "puy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("puy", "Purisimeño"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Puruborá", "pur"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pur", "Puruborá"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Purum", "pub"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pub", "Purum"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Purum Naga", "puz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("puz", "Purum Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pushto", "pus"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pus", "Pushto"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Putai", "mfl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mfl", "Putai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Putoh", "put"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("put", "Putoh"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Putukwam", "afe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("afe", "Putukwam"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Puyo", "xpy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xpy", "Puyo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Puyo-Paekche", "xpp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xpp", "Puyo-Paekche"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Puyuma", "pyu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pyu", "Puyuma"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pwaamei", "pme"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pme", "Pwaamei"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pwapwa", "pop"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pop", "Pwapwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pwo Eastern Karen", "kjp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kjp", "Pwo Eastern Karen"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pwo Northern Karen", "pww"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pww", "Pwo Northern Karen"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pwo Western Karen", "pwo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pwo", "Pwo Western Karen"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pyapun", "pcw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pcw", "Pyapun"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pye Krumen", "pye"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pye", "Pye Krumen"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pyen", "pyy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pyy", "Pyen"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pyu", "pby"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pby", "Pyu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pyu (Myanmar)", "pyx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pyx", "Pyu (Myanmar)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Páez", "pbb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pbb", "Páez"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pááfang", "pfa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pfa", "Pááfang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Päri", "lkr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lkr", "Päri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pémono", "pev"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pev", "Pémono"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pévé", "lme"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lme", "Pévé"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Pökoot", "pko"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pko", "Pökoot"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Q'anjob'al", "kjb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kjb", "Q'anjob'al"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Qabiao", "laq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("laq", "Qabiao"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Qaqet", "byx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("byx", "Qaqet"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Qashqa'i", "qxq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("qxq", "Qashqa'i"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Qatabanian", "xqt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xqt", "Qatabanian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Qawasqar", "alc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("alc", "Qawasqar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Qila Muji", "ymq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ymq", "Qila Muji"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Qimant", "ahg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ahg", "Qimant"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Qiubei Zhuang", "zqe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zqe", "Qiubei Zhuang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Quapaw", "qua"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("qua", "Quapaw"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Quebec Sign Language", "fcs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("fcs", "Quebec Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Quechan", "yum"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yum", "Quechan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Quechua", "que"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("que", "Quechua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Quenya", "qya"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("qya", "Quenya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Querétaro Otomi", "otq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("otq", "Querétaro Otomi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Quetzaltepec Mixe", "pxm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pxm", "Quetzaltepec Mixe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Queyu", "qvy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("qvy", "Queyu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Quiavicuzas Zapotec", "zpj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zpj", "Quiavicuzas Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Quiché", "quc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("quc", "Quiché"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Quileute", "qui"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("qui", "Quileute"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Quinault", "qun"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("qun", "Quinault"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Quinqui", "quq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("quq", "Quinqui"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Quioquitani-Quierí Zapotec", "ztq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ztq", "Quioquitani-Quierí Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Quiotepec Chinantec", "chq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("chq", "Quiotepec Chinantec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Quiripi", "qyp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("qyp", "Quiripi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Rabha", "rah"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rah", "Rabha"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Rade", "rad"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rad", "Rade"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Raetic", "xrr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xrr", "Raetic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Rahambuu", "raz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("raz", "Rahambuu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Rajah Kabunsuwan Manobo", "mqk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mqk", "Rajah Kabunsuwan Manobo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Rajasthani", "raj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("raj", "Rajasthani"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Rajbanshi", "rjs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rjs", "Rajbanshi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Raji", "rji"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rji", "Raji"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Rajong", "rjg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rjg", "Rajong"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Rajput Garasia", "gra"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gra", "Rajput Garasia"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Rakhine", "rki"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rki", "Rakhine"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ralte", "ral"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ral", "Ralte"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Rama", "rma"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rma", "Rama"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ramoaaina", "rai"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rai", "Ramoaaina"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ramopa", "kjx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kjx", "Ramopa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Rampi", "lje"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lje", "Rampi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Rana Tharu", "thr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("thr", "Rana Tharu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Rang", "rax"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rax", "Rang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Rangkas", "rgk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rgk", "Rangkas"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ranglong", "rnl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rnl", "Ranglong"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Rangpuri", "rkt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rkt", "Rangpuri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Rao", "rao"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rao", "Rao"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Rapa", "ray"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ray", "Rapa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Rapanui", "rap"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rap", "Rapanui"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Rapoisi", "kyx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kyx", "Rapoisi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Rapting", "rpt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rpt", "Rapting"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Rara Bakati'", "lra"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lra", "Rara Bakati'"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Rarotongan", "rar"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rar", "Rarotongan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Rasawa", "rac"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rac", "Rasawa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ratagnon", "btn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("btn", "Ratagnon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ratahan", "rth"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rth", "Ratahan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Rathawi", "rtw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rtw", "Rathawi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Rathwi Bareli", "bgd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bgd", "Rathwi Bareli"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Raute", "rau"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rau", "Raute"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ravula", "yea"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yea", "Ravula"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Rawa", "rwo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rwo", "Rawa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Rawang", "raw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("raw", "Rawang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Rawat", "jnl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jnl", "Rawat"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Rawo", "rwa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rwa", "Rawo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Rayón Zoque", "zor"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zor", "Rayón Zoque"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Razajerdi", "rat"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rat", "Razajerdi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Red Gelao", "gir"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gir", "Red Gelao"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Reel", "atu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("atu", "Reel"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Rejang", "rej"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rej", "Rejang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Rejang Kayan", "ree"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ree", "Rejang Kayan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Reli", "rei"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rei", "Reli"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Rema", "bow"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bow", "Rema"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Rembarunga", "rmb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rmb", "Rembarunga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Rembong", "reb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("reb", "Rembong"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Remo", "rem"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rem", "Remo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Remontado Dumagat", "agv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("agv", "Remontado Dumagat"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Rempi", "rmp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rmp", "Rempi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Remun", "lkj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lkj", "Remun"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Rendille", "rel"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rel", "Rendille"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Rengao", "ren"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ren", "Rengao"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Rennell-Bellona", "mnv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mnv", "Rennell-Bellona"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Rennellese Sign Language", "rsi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rsi", "Rennellese Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Repanbitip", "rpn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rpn", "Repanbitip"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Rer Bare", "rer"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rer", "Rer Bare"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Rerau", "rea"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rea", "Rerau"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Rerep", "pgk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pgk", "Rerep"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Reshe", "res"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("res", "Reshe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Resígaro", "rgr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rgr", "Resígaro"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Retta", "ret"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ret", "Retta"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Reyesano", "rey"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rey", "Reyesano"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Riang (India)", "ria"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ria", "Riang (India)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Riang (Myanmar)", "ril"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ril", "Riang (Myanmar)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Riantana", "ran"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ran", "Riantana"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ribun", "rir"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rir", "Ribun"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Rien", "rie"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rie", "Rien"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Rikbaktsa", "rkb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rkb", "Rikbaktsa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Rinconada Bikol", "bto"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bto", "Rinconada Bikol"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Rincón Zapotec", "zar"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zar", "Rincón Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ringgou", "rgu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rgu", "Ringgou"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ririo", "rri"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rri", "Ririo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ritarungo", "rit"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rit", "Ritarungo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Riung", "riu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("riu", "Riung"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Riverain Sango", "snj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("snj", "Riverain Sango"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Rogo", "rod"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rod", "Rogo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Rohingya", "rhg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rhg", "Rohingya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Roma", "rmm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rmm", "Roma"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Romagnol", "rgn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rgn", "Romagnol"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Romam", "rmx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rmx", "Romam"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Romanian", "ron"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ron", "Romanian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Romanian Sign Language", "rms"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rms", "Romanian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Romano-Greek", "rge"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rge", "Romano-Greek"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Romano-Serbian", "rsb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rsb", "Romano-Serbian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Romanova", "rmv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rmv", "Romanova"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Romansh", "roh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("roh", "Romansh"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Romany", "rom"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rom", "Romany"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Romblomanon", "rol"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rol", "Romblomanon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Rombo", "rof"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rof", "Rombo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Romkun", "rmk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rmk", "Romkun"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ron", "cla"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cla", "Ron"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ronga", "rng"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rng", "Ronga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Rongga", "ror"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ror", "Rongga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Rongmei Naga", "nbu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nbu", "Rongmei Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Rongpo", "rnp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rnp", "Rongpo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ronji", "roe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("roe", "Ronji"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Roon", "rnn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rnn", "Roon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Roria", "rga"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rga", "Roria"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Rotokas", "roo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("roo", "Rotokas"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Rotuman", "rtm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rtm", "Rotuman"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Roviana", "rug"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rug", "Roviana"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ruching Palaung", "pce"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pce", "Ruching Palaung"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Rudbari", "rdb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rdb", "Rudbari"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Rufiji", "rui"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rui", "Rufiji"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ruga", "ruh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ruh", "Ruga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Rukai", "dru"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dru", "Rukai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ruma", "ruz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ruz", "Ruma"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Rumai Palaung", "rbb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rbb", "Rumai Palaung"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Rumu", "klq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("klq", "Rumu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Runa", "rna"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rna", "Runa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Rundi", "run"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("run", "Rundi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Runga", "rou"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rou", "Runga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Rungus", "drg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("drg", "Rungus"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Rungwa", "rnw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rnw", "Rungwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Russia Buriat", "bxr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bxr", "Russia Buriat"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Russian", "rus"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rus", "Russian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Russian Sign Language", "rsl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rsl", "Russian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Rusyn", "rue"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rue", "Rusyn"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Rutul", "rut"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rut", "Rutul"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ruuli", "ruc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ruc", "Ruuli"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ruund", "rnd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rnd", "Ruund"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Rwa", "rwk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rwk", "Rwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Réunion Creole French", "rcf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rcf", "Réunion Creole French"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("S'gaw Karen", "ksw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ksw", "S'gaw Karen"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sa", "sax"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sax", "Sa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sa'a", "apb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("apb", "Sa'a"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sa'ban", "snv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("snv", "Sa'ban"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sa'och", "scq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("scq", "Sa'och"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Saafi-Saafi", "sav"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sav", "Saafi-Saafi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Saam", "raq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("raq", "Saam"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Saamia", "lsm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lsm", "Saamia"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Saaroa", "sxr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sxr", "Saaroa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Saba", "saa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("saa", "Saba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sabaean", "xsa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xsa", "Sabaean"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sabah Bisaya", "bsy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bsy", "Sabah Bisaya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sabah Malay", "msi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("msi", "Sabah Malay"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sabanê", "sae"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sae", "Sabanê"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sabaot", "spy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("spy", "Sabaot"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sabine", "sbv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sbv", "Sabine"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sabu", "hvn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hvn", "Sabu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sabüm", "sbo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sbo", "Sabüm"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sacapulteco", "quv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("quv", "Sacapulteco"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sadri", "sck"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sck", "Sadri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Saek", "skb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("skb", "Saek"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Saep", "spd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("spd", "Saep"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Safaliba", "saf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("saf", "Safaliba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Safeyoka", "apz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("apz", "Safeyoka"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Safwa", "sbk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sbk", "Safwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sagala", "sbm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sbm", "Sagala"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sagalla", "tga"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tga", "Sagalla"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Saho", "ssy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ssy", "Saho"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sahu", "saj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("saj", "Sahu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Saidi Arabic", "aec"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aec", "Saidi Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Saint Lucian Creole French", "acf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("acf", "Saint Lucian Creole French"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Saisiyat", "xsy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xsy", "Saisiyat"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sajalong", "sjl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sjl", "Sajalong"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sajau Basap", "sjb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sjb", "Sajau Basap"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sakachep", "sch"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sch", "Sakachep"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sakalava Malagasy", "skg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("skg", "Sakalava Malagasy"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sakam", "skm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("skm", "Sakam"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sakan", "kho"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kho", "Sakan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sakao", "sku"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sku", "Sakao"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sakata", "skt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("skt", "Sakata"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sake", "sak"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sak", "Sake"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sakirabiá", "skf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("skf", "Sakirabiá"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sala", "shq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("shq", "Sala"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Salako", "knx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("knx", "Salako"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Salampasu", "slx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("slx", "Salampasu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Salar", "slr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("slr", "Salar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Salas", "sgu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sgu", "Salas"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Salasaca Highland Quichua", "qxl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("qxl", "Salasaca Highland Quichua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Salchuq", "slq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("slq", "Salchuq"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Saleman", "sau"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sau", "Saleman"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Saliba", "sbe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sbe", "Saliba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Salinan", "sln"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sln", "Salinan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sallands", "sdz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sdz", "Sallands"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Salt-Yui", "sll"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sll", "Salt-Yui"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Saluan", "loe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("loe", "Saluan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Salumá", "slj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("slj", "Salumá"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Salvadoran Sign Language", "esn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("esn", "Salvadoran Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sam", "snx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("snx", "Sam"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sama", "smd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("smd", "Sama"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Samaritan", "smp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("smp", "Samaritan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Samaritan Aramaic", "sam"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sam", "Samaritan Aramaic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Samarokena", "tmj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tmj", "Samarokena"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Samatao", "ysd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ysd", "Samatao"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Samba", "smx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("smx", "Samba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Samba Daka", "ccg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ccg", "Samba Daka"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Samba Leko", "ndi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ndi", "Samba Leko"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sambe", "xab"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xab", "Sambe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Samberigi", "ssx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ssx", "Samberigi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Samburu", "saq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("saq", "Samburu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Samei", "smh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("smh", "Samei"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Samo", "smq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("smq", "Samo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Samoan", "smo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("smo", "Samoan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Samogitian", "sgs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sgs", "Samogitian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Samosa", "swm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("swm", "Samosa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sampang", "rav"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rav", "Sampang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Samre", "sxm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sxm", "Samre"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Samtao", "stu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("stu", "Samtao"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Samvedi", "smv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("smv", "Samvedi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("San Agustín Mixtepec Zapotec", "ztm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ztm", "San Agustín Mixtepec Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("San Baltazar Loxicha Zapotec", "zpx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zpx", "San Baltazar Loxicha Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("San Blas Kuna", "cuk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cuk", "San Blas Kuna"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("San Dionisio Del Mar Huave", "hve"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hve", "San Dionisio Del Mar Huave"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("San Felipe Otlaltepec Popoloca", "pow"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pow", "San Felipe Otlaltepec Popoloca"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("San Francisco Del Mar Huave", "hue"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hue", "San Francisco Del Mar Huave"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("San Francisco Matlatzinca", "mat"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mat", "San Francisco Matlatzinca"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("San Jerónimo Tecóatl Mazatec", "maa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("maa", "San Jerónimo Tecóatl Mazatec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("San Juan Atzingo Popoloca", "poe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("poe", "San Juan Atzingo Popoloca"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("San Juan Colorado Mixtec", "mjc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mjc", "San Juan Colorado Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("San Juan Guelavía Zapotec", "zab"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zab", "San Juan Guelavía Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("San Juan Teita Mixtec", "xtj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xtj", "San Juan Teita Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("San Luís Temalacayuca Popoloca", "pps"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pps", "San Luís Temalacayuca Popoloca"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("San Marcos Tlalcoyalco Popoloca", "pls"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pls", "San Marcos Tlalcoyalco Popoloca"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("San Martín Itunyoso Triqui", "trq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("trq", "San Martín Itunyoso Triqui"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("San Martín Quechua", "qvs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("qvs", "San Martín Quechua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("San Mateo Del Mar Huave", "huv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("huv", "San Mateo Del Mar Huave"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("San Miguel Creole French", "scf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("scf", "San Miguel Creole French"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("San Miguel El Grande Mixtec", "mig"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mig", "San Miguel El Grande Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("San Miguel Piedras Mixtec", "xtp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xtp", "San Miguel Piedras Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("San Pedro Amuzgos Amuzgo", "azg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("azg", "San Pedro Amuzgos Amuzgo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("San Pedro Quiatoni Zapotec", "zpf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zpf", "San Pedro Quiatoni Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("San Salvador Kongo", "kwy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kwy", "San Salvador Kongo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("San Vicente Coatlán Zapotec", "zpt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zpt", "San Vicente Coatlán Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sanaani Arabic", "ayn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ayn", "Sanaani Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sanapaná", "sap"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sap", "Sanapaná"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sandawe", "sad"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sad", "Sandawe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sanga (Democratic Republic of Congo)", "sng"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sng", "Sanga (Democratic Republic of Congo)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sanga (Nigeria)", "xsn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xsn", "Sanga (Nigeria)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sanggau", "scg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("scg", "Sanggau"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sangil", "snl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("snl", "Sangil"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sangir", "sxn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sxn", "Sangir"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sangisari", "sgr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sgr", "Sangisari"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sangkong", "sgk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sgk", "Sangkong"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sanglechi", "sgy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sgy", "Sanglechi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sango", "sag"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sag", "Sango"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sangtam Naga", "nsa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nsa", "Sangtam Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sangu (Gabon)", "snq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("snq", "Sangu (Gabon)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sangu (Tanzania)", "sbp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sbp", "Sangu (Tanzania)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sani", "ysn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ysn", "Sani"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sanie", "ysy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ysy", "Sanie"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Saniyo-Hiyewe", "sny"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sny", "Saniyo-Hiyewe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sankaran Maninka", "msc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("msc", "Sankaran Maninka"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sansi", "ssi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ssi", "Sansi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sanskrit", "san"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("san", "Sanskrit"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sansu", "sca"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sca", "Sansu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Santa Ana de Tusi Pasco Quechua", "qxt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("qxt", "Santa Ana de Tusi Pasco Quechua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Santa Catarina Albarradas Zapotec", "ztn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ztn", "Santa Catarina Albarradas Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Santa Inés Ahuatempan Popoloca", "pca"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pca", "Santa Inés Ahuatempan Popoloca"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Santa Inés Yatzechi Zapotec", "zpn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zpn", "Santa Inés Yatzechi Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Santa Lucía Monteverde Mixtec", "mdv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mdv", "Santa Lucía Monteverde Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Santa María Del Mar Huave", "hvv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hvv", "Santa María Del Mar Huave"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Santa María La Alta Nahuatl", "nhz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nhz", "Santa María La Alta Nahuatl"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Santa María Quiegolani Zapotec", "zpi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zpi", "Santa María Quiegolani Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Santa María Zacatepec Mixtec", "mza"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mza", "Santa María Zacatepec Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Santa Teresa Cora", "cok"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cok", "Santa Teresa Cora"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Santali", "sat"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sat", "Santali"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Santiago Xanica Zapotec", "zpr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zpr", "Santiago Xanica Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Santiago del Estero Quichua", "qus"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("qus", "Santiago del Estero Quichua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Santo Domingo Albarradas Zapotec", "zas"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zas", "Santo Domingo Albarradas Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sanumá", "xsu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xsu", "Sanumá"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Saparua", "spr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("spr", "Saparua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sapo", "krn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("krn", "Sapo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Saponi", "spi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("spi", "Saponi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Saposa", "sps"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sps", "Saposa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sapuan", "spu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("spu", "Sapuan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sapé", "spc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("spc", "Sapé"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sar", "mwm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mwm", "Sar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sara", "sre"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sre", "Sara"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sara Dunjo", "koj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("koj", "Sara Dunjo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sara Kaba", "sbz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sbz", "Sara Kaba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sara Kaba Deme", "kwg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kwg", "Sara Kaba Deme"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sara Kaba Náà", "kwv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kwv", "Sara Kaba Náà"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Saramaccan", "srm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("srm", "Saramaccan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sarangani Blaan", "bps"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bps", "Sarangani Blaan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sarangani Manobo", "mbs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mbs", "Sarangani Manobo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sarasira", "zsa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zsa", "Sarasira"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Saraveca", "sar"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sar", "Saraveca"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sardinian", "srd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("srd", "Sardinian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sarikoli", "srh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("srh", "Sarikoli"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sarli", "sdf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sdf", "Sarli"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sarsi", "srs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("srs", "Sarsi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sartang", "onp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("onp", "Sartang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sarua", "swy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("swy", "Sarua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sarudu", "sdu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sdu", "Sarudu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Saruga", "sra"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sra", "Saruga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sasak", "sas"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sas", "Sasak"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sasaru", "sxs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sxs", "Sasaru"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sassarese Sardinian", "sdc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sdc", "Sassarese Sardinian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Satawalese", "stw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("stw", "Satawalese"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Saterfriesisch", "stq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("stq", "Saterfriesisch"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sateré-Mawé", "mav"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mav", "Sateré-Mawé"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Saudi Arabian Sign Language", "sdl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sdl", "Saudi Arabian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sauk", "skc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("skc", "Sauk"); //$NON-NLS-1$ //$NON-NLS-2$
	}

	private void createLanguagesMap5() {
		langMap.put("Sauraseni Prākrit", "psu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("psu", "Sauraseni Prākrit"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Saurashtra", "saz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("saz", "Saurashtra"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sauri", "srt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("srt", "Sauri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sauria Paharia", "mjt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mjt", "Sauria Paharia"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sause", "sao"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sao", "Sause"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sausi", "ssj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ssj", "Sausi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Savara", "svr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("svr", "Savara"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Savi", "sdg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sdg", "Savi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Savosavo", "svs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("svs", "Savosavo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sawai", "szw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("szw", "Sawai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Saweru", "swr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("swr", "Saweru"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sawi", "saw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("saw", "Sawi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sawila", "swt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("swt", "Sawila"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sawknah", "swn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("swn", "Sawknah"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Saxwe Gbe", "sxw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sxw", "Saxwe Gbe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Saya", "say"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("say", "Saya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sayula Popoluca", "pos"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pos", "Sayula Popoluca"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Scots", "sco"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sco", "Scots"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Scottish Gaelic", "gla"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gla", "Scottish Gaelic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Scythian", "xsc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xsc", "Scythian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sea Island Creole English", "gul"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gul", "Sea Island Creole English"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Seba", "kdg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kdg", "Seba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sebat Bet Gurage", "sgw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sgw", "Sebat Bet Gurage"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Seberuang", "sbx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sbx", "Seberuang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sebop", "sib"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sib", "Sebop"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sebuyau", "snb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("snb", "Sebuyau"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sechelt", "sec"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sec", "Sechelt"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Secoya", "sey"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sey", "Secoya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sedang", "sed"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sed", "Sedang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sedoa", "tvw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tvw", "Sedoa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Seeku", "sos"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sos", "Seeku"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Segai", "sge"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sge", "Segai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Segeju", "seg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("seg", "Segeju"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Seget", "sbg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sbg", "Seget"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sehwi", "sfw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sfw", "Sehwi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Seimat", "ssg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ssg", "Seimat"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Seit-Kaitetu", "hik"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hik", "Seit-Kaitetu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sekani", "sek"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sek", "Sekani"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sekapan", "skp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("skp", "Sekapan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sekar", "skz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("skz", "Sekar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Seke (Nepal)", "skj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("skj", "Seke (Nepal)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Seke (Vanuatu)", "ske"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ske", "Seke (Vanuatu)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Seki", "syi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("syi", "Seki"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Seko Padang", "skx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("skx", "Seko Padang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Seko Tengah", "sko"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sko", "Seko Tengah"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sekpele", "lip"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lip", "Sekpele"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Selangor Sign Language", "kgi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kgi", "Selangor Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Selaru", "slu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("slu", "Selaru"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Selayar", "sly"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sly", "Selayar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Selee", "snw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("snw", "Selee"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Selepet", "spl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("spl", "Selepet"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Selian", "sxl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sxl", "Selian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Selkup", "sel"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sel", "Selkup"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Selungai Murut", "slg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("slg", "Selungai Murut"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Seluwasan", "sws"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sws", "Seluwasan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Semai", "sea"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sea", "Semai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Semandang", "sdm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sdm", "Semandang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Semaq Beri", "szc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("szc", "Semaq Beri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sembakung Murut", "sbr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sbr", "Sembakung Murut"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Semelai", "sza"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sza", "Semelai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Semimi", "etz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("etz", "Semimi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Semnam", "ssm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ssm", "Semnam"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Semnani", "smy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("smy", "Semnani"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sempan", "xse"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xse", "Sempan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sena", "seh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("seh", "Sena"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Senara Sénoufo", "seq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("seq", "Senara Sénoufo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Senaya", "syn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("syn", "Senaya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sene", "sej"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sej", "Sene"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Seneca", "see"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("see", "Seneca"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sened", "sds"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sds", "Sened"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sengele", "szg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("szg", "Sengele"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Senggi", "snu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("snu", "Senggi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sengo", "spk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("spk", "Sengo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sengseng", "ssz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ssz", "Sengseng"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Senhaja De Srair", "sjs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sjs", "Senhaja De Srair"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sensi", "sni"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sni", "Sensi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sentani", "set"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("set", "Sentani"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Senthang Chin", "sez"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sez", "Senthang Chin"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sentinel", "std"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("std", "Sentinel"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sepa (Indonesia)", "spb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("spb", "Sepa (Indonesia)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sepa (Papua New Guinea)", "spe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("spe", "Sepa (Papua New Guinea)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sepedi", "nso"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nso", "Sepedi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sepen", "spm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("spm", "Sepen"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sepik Iwam", "iws"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("iws", "Sepik Iwam"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sera", "sry"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sry", "Sera"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Seraiki", "skr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("skr", "Seraiki"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Serbian", "srp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("srp", "Serbian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Serbo-Croatian", "hbs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hbs", "Serbo-Croatian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sere", "swf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("swf", "Sere"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Serer", "srr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("srr", "Serer"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Seri", "sei"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sei", "Seri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Serili", "sve"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sve", "Serili"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Seroa", "kqu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kqu", "Seroa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Serrano", "ser"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ser", "Serrano"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Seru", "szd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("szd", "Seru"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Serua", "srw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("srw", "Serua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Serudung Murut", "srk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("srk", "Serudung Murut"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Serui-Laut", "seu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("seu", "Serui-Laut"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Seselwa Creole French", "crs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("crs", "Seselwa Creole French"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Seta", "stf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("stf", "Seta"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Setaman", "stm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("stm", "Setaman"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Seti", "sbi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sbi", "Seti"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Settla", "sta"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sta", "Settla"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Severn Ojibwa", "ojs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ojs", "Severn Ojibwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sewa Bay", "sew"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sew", "Sewa Bay"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Seze", "sze"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sze", "Seze"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sha", "scw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("scw", "Sha"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Shabak", "sdb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sdb", "Shabak"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Shabo", "sbf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sbf", "Shabo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Shahmirzadi", "srz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("srz", "Shahmirzadi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Shahrudi", "shm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("shm", "Shahrudi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Shall-Zwall", "sha"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sha", "Shall-Zwall"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Shama-Sambuga", "sqa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sqa", "Shama-Sambuga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Shamang", "xsh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xsh", "Shamang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Shambala", "ksb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ksb", "Shambala"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Shan", "shn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("shn", "Shan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Shanenawa", "swo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("swo", "Shanenawa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Shanga", "sho"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sho", "Shanga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Shangzhai", "jih"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jih", "Shangzhai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sharanahua", "mcd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mcd", "Sharanahua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Shark Bay", "ssv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ssv", "Shark Bay"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sharwa", "swq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("swq", "Sharwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Shasta", "sht"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sht", "Shasta"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Shatt", "shj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("shj", "Shatt"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Shau", "sqh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sqh", "Shau"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Shawnee", "sjw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sjw", "Shawnee"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("She", "shx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("shx", "She"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Shehri", "shv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("shv", "Shehri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Shekhawati", "swv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("swv", "Shekhawati"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Shekkacho", "moy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("moy", "Shekkacho"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sheko", "she"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("she", "Sheko"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Shelta", "sth"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sth", "Shelta"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Shempire Senoufo", "seb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("seb", "Shempire Senoufo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Shendu", "shl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("shl", "Shendu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sheni", "scv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("scv", "Sheni"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sherbro", "bun"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bun", "Sherbro"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sherdukpen", "sdp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sdp", "Sherdukpen"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sherpa", "xsr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xsr", "Sherpa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sheshi Kham", "kip"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kip", "Sheshi Kham"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Shi", "shr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("shr", "Shi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Shihhi Arabic", "ssh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ssh", "Shihhi Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Shiki", "gua"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gua", "Shiki"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Shilluk", "shk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("shk", "Shilluk"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Shina", "scl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("scl", "Shina"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Shinabo", "snh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("snh", "Shinabo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Shipibo-Conibo", "shp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("shp", "Shipibo-Conibo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Shixing", "sxg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sxg", "Shixing"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sholaga", "sle"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sle", "Sholaga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Shom Peng", "sii"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sii", "Shom Peng"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Shona", "sna"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sna", "Shona"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Shoo-Minda-Nye", "bcv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bcv", "Shoo-Minda-Nye"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Shor", "cjs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cjs", "Shor"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Shoshoni", "shh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("shh", "Shoshoni"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Shua", "shg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("shg", "Shua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Shuadit", "sdt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sdt", "Shuadit"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Shuar", "jiv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jiv", "Shuar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Shubi", "suj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("suj", "Shubi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Shughni", "sgh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sgh", "Shughni"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Shumashti", "sts"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sts", "Shumashti"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Shumcho", "scu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("scu", "Shumcho"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Shuswap", "shs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("shs", "Shuswap"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Shuwa-Zamani", "ksa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ksa", "Shuwa-Zamani"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Shwai", "shw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("shw", "Shwai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Shwe Palaung", "pll"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pll", "Shwe Palaung"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sialum", "slw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("slw", "Sialum"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Siamou", "sif"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sif", "Siamou"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sian", "spg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("spg", "Sian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Siane", "snp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("snp", "Siane"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Siang", "sya"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sya", "Siang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Siar-Lak", "sjr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sjr", "Siar-Lak"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Siawi", "mmp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mmp", "Siawi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sibe", "nco"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nco", "Sibe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sibu Melanau", "sdx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sdx", "Sibu Melanau"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sicanian", "sxc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sxc", "Sicanian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sicel", "scx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("scx", "Sicel"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sichuan Yi", "iii"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("iii", "Sichuan Yi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sicilian", "scn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("scn", "Sicilian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Siculo Arabic", "sqr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sqr", "Siculo Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sidamo", "sid"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sid", "Sidamo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sidetic", "xsd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xsd", "Sidetic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sie", "erg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("erg", "Sie"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sierra Leone Sign Language", "sgx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sgx", "Sierra Leone Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sierra Negra Nahuatl", "nsu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nsu", "Sierra Negra Nahuatl"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sierra de Juárez Zapotec", "zaa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zaa", "Sierra de Juárez Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sighu", "sxe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sxe", "Sighu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sihan", "snr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("snr", "Sihan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sihuas Ancash Quechua", "qws"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("qws", "Sihuas Ancash Quechua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sika", "ski"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ski", "Sika"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sikaiana", "sky"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sky", "Sikaiana"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sikaritai", "tty"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tty", "Sikaritai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sikiana", "sik"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sik", "Sikiana"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sikkimese", "sip"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sip", "Sikkimese"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Siksika", "bla"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bla", "Siksika"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sikule", "skh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("skh", "Sikule"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sila", "slt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("slt", "Sila"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Silacayoapan Mixtec", "mks"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mks", "Silacayoapan Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sileibi", "sbq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sbq", "Sileibi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Silesian", "szl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("szl", "Silesian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Silimo", "wul"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wul", "Silimo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Siliput", "mkc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mkc", "Siliput"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Silopi", "xsp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xsp", "Silopi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Silt'e", "stv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("stv", "Silt'e"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Simaa", "sie"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sie", "Simaa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Simba", "sbw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sbw", "Simba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Simbali", "smg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("smg", "Simbali"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Simbari", "smb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("smb", "Simbari"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Simbo", "sbb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sbb", "Simbo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Simeku", "smz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("smz", "Simeku"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Simeulue", "smr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("smr", "Simeulue"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Simte", "smt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("smt", "Simte"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sinagen", "siu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("siu", "Sinagen"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sinasina", "sst"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sst", "Sinasina"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sinaugoro", "snc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("snc", "Sinaugoro"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sindarin", "sjn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sjn", "Sindarin"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sindhi", "snd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("snd", "Sindhi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sindhi Bhil", "sbn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sbn", "Sindhi Bhil"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sindihui Mixtec", "xts"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xts", "Sindihui Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Singa", "sgm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sgm", "Singa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Singapore Sign Language", "sls"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sls", "Singapore Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Singpho", "sgp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sgp", "Singpho"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sinhala", "sin"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sin", "Sinhala"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sinhalese", "sin"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sin", "Sinhalese"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sinicahua Mixtec", "xti"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xti", "Sinicahua Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sinicized Miao", "hmz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hmz", "Sinicized Miao"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sininkere", "skq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("skq", "Sininkere"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sinsauru", "snz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("snz", "Sinsauru"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sinte Romani", "rmo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rmo", "Sinte Romani"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sinyar", "sys"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sys", "Sinyar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sio", "xsi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xsi", "Sio"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Siona", "snn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("snn", "Siona"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sipacapense", "qum"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("qum", "Sipacapense"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sira", "swj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("swj", "Sira"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Siraya", "fos"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("fos", "Siraya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sirenik Yupik", "ysr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ysr", "Sirenik Yupik"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Siri", "sir"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sir", "Siri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Siriano", "sri"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sri", "Siriano"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sirionó", "srq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("srq", "Sirionó"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sirmauri", "srx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("srx", "Sirmauri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Siroi", "ssd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ssd", "Siroi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sissala", "sld"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sld", "Sissala"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sissano", "sso"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sso", "Sissano"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Siuslaw", "sis"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sis", "Siuslaw"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sivandi", "siy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("siy", "Sivandi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Siwai", "siw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("siw", "Siwai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Siwi", "siz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("siz", "Siwi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Siwu", "akp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("akp", "Siwu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Siyin Chin", "csy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("csy", "Siyin Chin"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Skagit", "ska"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ska", "Skagit"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Skalvian", "svx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("svx", "Skalvian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Skepi Creole Dutch", "skw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("skw", "Skepi Creole Dutch"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Skolt Sami", "sms"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sms", "Skolt Sami"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Skou", "skv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("skv", "Skou"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Slave (Athapascan)", "den"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("den", "Slave (Athapascan)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Slovak", "slk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("slk", "Slovak"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Slovakian Sign Language", "svk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("svk", "Slovakian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Slovenian", "slv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("slv", "Slovenian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Small Flowery Miao", "sfm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sfm", "Small Flowery Miao"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Smärky Kanum", "kxq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kxq", "Smärky Kanum"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Snohomish", "sno"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sno", "Snohomish"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("So (Cameroon)", "sox"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sox", "So (Cameroon)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("So (Democratic Republic of Congo)", "soc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("soc", "So (Democratic Republic of Congo)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("So'a", "ssq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ssq", "So'a"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sobei", "sob"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sob", "Sobei"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sochiapam Chinantec", "cso"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cso", "Sochiapam Chinantec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sochiapan Chinantec", "cso"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cso", "Sochiapan Chinantec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Soga", "xog"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xog", "Soga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sogdian", "sog"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sog", "Sogdian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Soi", "soj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("soj", "Soi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sok", "skk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("skk", "Sok"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sokoro", "sok"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sok", "Sokoro"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Solano", "xso"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xso", "Solano"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Soli", "sby"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sby", "Soli"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Solong", "aaw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aaw", "Solong"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Solos", "sol"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sol", "Solos"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Som", "smc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("smc", "Som"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Somali", "som"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("som", "Somali"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Somrai", "sor"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sor", "Somrai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Somray", "smu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("smu", "Somray"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Somyev", "kgt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kgt", "Somyev"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sonde", "shc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("shc", "Sonde"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Songa", "sgo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sgo", "Songa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Songe", "sop"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sop", "Songe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Songo", "soo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("soo", "Songo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Songomeno", "soe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("soe", "Songomeno"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Songoora", "sod"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sod", "Songoora"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sonha", "soi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("soi", "Sonha"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sonia", "siq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("siq", "Sonia"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Soninke", "snk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("snk", "Soninke"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sonsorol", "sov"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sov", "Sonsorol"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Soo", "teu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("teu", "Soo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sop", "urw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("urw", "Sop"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Soqotri", "sqt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sqt", "Soqotri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sora", "srb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("srb", "Sora"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sori-Harengan", "sbh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sbh", "Sori-Harengan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sorkhei", "sqo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sqo", "Sorkhei"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sorothaptic", "sxo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sxo", "Sorothaptic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sorsogon Ayta", "ays"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ays", "Sorsogon Ayta"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sos Kundi", "sdk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sdk", "Sos Kundi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sota Kanum", "krz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("krz", "Sota Kanum"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sou", "sqq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sqq", "Sou"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("South African Sign Language", "sfs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sfs", "South African Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("South Awyu", "aws"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aws", "South Awyu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("South Azerbaijani", "azb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("azb", "South Azerbaijani"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("South Bolivian Quechua", "quh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("quh", "South Bolivian Quechua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("South Central Banda", "lnl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lnl", "South Central Banda"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("South Central Dinka", "dib"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dib", "South Central Dinka"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("South Efate", "erk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("erk", "South Efate"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("South Fali", "fal"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("fal", "South Fali"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("South Giziga", "giz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("giz", "South Giziga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("South Lembata", "lmf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lmf", "South Lembata"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("South Levantine Arabic", "ajp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ajp", "South Levantine Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("South Marquesan", "mqm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mqm", "South Marquesan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("South Muyu", "kts"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kts", "South Muyu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("South Ndebele", "nbl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nbl", "South Ndebele"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("South Nuaulu", "nxl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nxl", "South Nuaulu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("South Picene", "spx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("spx", "South Picene"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("South Slavey", "xsl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xsl", "South Slavey"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("South Tairora", "omw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("omw", "South Tairora"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("South Ucayali Ashéninka", "cpy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cpy", "South Ucayali Ashéninka"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("South Watut", "mcy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mcy", "South Watut"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("South Wemale", "tlw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tlw", "South Wemale"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("South West Bay", "sns"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sns", "South West Bay"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southeast Ambrym", "tvk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tvk", "Southeast Ambrym"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southeast Babar", "vbb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vbb", "Southeast Babar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southeast Ijo", "ijs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ijs", "Southeast Ijo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southeast Pashayi", "psi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("psi", "Southeast Pashayi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southeastern Dinka", "dks"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dks", "Southeastern Dinka"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southeastern Ixtlán Zapotec", "zpd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zpd", "Southeastern Ixtlán Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southeastern Kolami", "nit"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nit", "Southeastern Kolami"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southeastern Nochixtlán Mixtec", "mxy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mxy", "Southeastern Nochixtlán Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southeastern Pomo", "pom"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pom", "Southeastern Pomo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southeastern Puebla Nahuatl", "npl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("npl", "Southeastern Puebla Nahuatl"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southeastern Tarahumara", "tcu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tcu", "Southeastern Tarahumara"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southeastern Tepehuan", "stp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("stp", "Southeastern Tepehuan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southern Alta", "agy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("agy", "Southern Alta"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southern Altai", "alt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("alt", "Southern Altai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southern Amami-Oshima", "ams"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ams", "Southern Amami-Oshima"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southern Aymara", "ayc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ayc", "Southern Aymara"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southern Bai", "bfs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bfs", "Southern Bai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southern Balochi", "bcc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bcc", "Southern Balochi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southern Betsimisaraka Malagasy", "bjq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bjq", "Southern Betsimisaraka Malagasy"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southern Binukidnon", "mtw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mtw", "Southern Binukidnon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southern Birifor", "biv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("biv", "Southern Birifor"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southern Bobo Madaré", "bwq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bwq", "Southern Bobo Madaré"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southern Bontok", "obk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("obk", "Southern Bontok"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southern Carrier", "caf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("caf", "Southern Carrier"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southern Catanduanes Bicolano", "bln"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bln", "Southern Catanduanes Bicolano"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southern Conchucos Ancash Quechua", "qxo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("qxo", "Southern Conchucos Ancash Quechua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southern Dagaare", "dga"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dga", "Southern Dagaare"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southern Dong", "kmc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kmc", "Southern Dong"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southern East Cree", "crj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("crj", "Southern East Cree"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southern Ghale", "ghe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ghe", "Southern Ghale"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southern Gondi", "ggo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ggo", "Southern Gondi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southern Grebo", "grj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("grj", "Southern Grebo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southern Guiyang Hmong", "hmy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hmy", "Southern Guiyang Hmong"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southern Guiyang Miao", "hmy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hmy", "Southern Guiyang Miao"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southern Haida", "hax"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hax", "Southern Haida"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southern Hindko", "hnd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hnd", "Southern Hindko"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southern Kalapuya", "sxk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sxk", "Southern Kalapuya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southern Kalinga", "ksc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ksc", "Southern Kalinga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southern Kisi", "kss"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kss", "Southern Kisi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southern Kiwai", "kjd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kjd", "Southern Kiwai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southern Kurdish", "sdh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sdh", "Southern Kurdish"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southern Lolopo", "ysp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ysp", "Southern Lolopo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southern Lorung", "lrr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lrr", "Southern Lorung"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southern Luri", "luz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("luz", "Southern Luri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southern Ma'di", "snm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("snm", "Southern Ma'di"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southern Mashan Hmong", "hma"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hma", "Southern Mashan Hmong"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southern Mashan Miao", "hma"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hma", "Southern Mashan Miao"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southern Mnong", "mnn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mnn", "Southern Mnong"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southern Muji", "ymc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ymc", "Southern Muji"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southern Nago", "nqg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nqg", "Southern Nago"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southern Nambikuára", "nab"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nab", "Southern Nambikuára"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southern Ngbandi", "nbw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nbw", "Southern Ngbandi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southern Nicobarese", "nik"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nik", "Southern Nicobarese"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southern Nisu", "nsd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nsd", "Southern Nisu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southern Nuni", "nnw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nnw", "Southern Nuni"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southern Ohlone", "css"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("css", "Southern Ohlone"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southern One", "osu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("osu", "Southern One"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southern Pame", "pmz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pmz", "Southern Pame"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southern Pashto", "pbt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pbt", "Southern Pashto"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southern Pastaza Quechua", "qup"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("qup", "Southern Pastaza Quechua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southern Pomo", "peq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("peq", "Southern Pomo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southern Puebla Mixtec", "mit"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mit", "Southern Puebla Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southern Puget Sound Salish", "slh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("slh", "Southern Puget Sound Salish"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southern Pumi", "pmj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pmj", "Southern Pumi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southern Qiandong Miao", "hms"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hms", "Southern Qiandong Miao"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southern Qiang", "qxs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("qxs", "Southern Qiang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southern Rengma Naga", "nre"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nre", "Southern Rengma Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southern Rincon Zapotec", "zsr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zsr", "Southern Rincon Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southern Roglai", "rgs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rgs", "Southern Roglai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southern Sama", "ssb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ssb", "Southern Sama"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southern Sami", "sma"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sma", "Southern Sami"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southern Samo", "sbd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sbd", "Southern Samo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southern Sierra Miwok", "skd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("skd", "Southern Sierra Miwok"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southern Sorsoganon", "srv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("srv", "Southern Sorsoganon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southern Sotho", "sot"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sot", "Southern Sotho"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southern Subanen", "laa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("laa", "Southern Subanen"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southern Thai", "sou"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sou", "Southern Thai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southern Tiwa", "tix"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tix", "Southern Tiwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southern Toussian", "wib"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wib", "Southern Toussian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southern Tujia", "tjs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tjs", "Southern Tujia"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southern Tutchone", "tce"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tce", "Southern Tutchone"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southern Uzbek", "uzs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("uzs", "Southern Uzbek"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southern Yukaghir", "yux"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yux", "Southern Yukaghir"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southwest Gbaya", "gso"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gso", "Southwest Gbaya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southwest Palawano", "plv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("plv", "Southwest Palawano"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southwest Pashayi", "psh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("psh", "Southwest Pashayi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southwest Tanna", "nwi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nwi", "Southwest Tanna"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southwestern Bontok", "vbk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vbk", "Southwestern Bontok"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southwestern Dinka", "dik"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dik", "Southwestern Dinka"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southwestern Fars", "fay"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("fay", "Southwestern Fars"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southwestern Guiyang Hmong", "hmg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hmg", "Southwestern Guiyang Hmong"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southwestern Huishui Hmong", "hmh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hmh", "Southwestern Huishui Hmong"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southwestern Huishui Miao", "hmh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hmh", "Southwestern Huishui Miao"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southwestern Nisu", "nsv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nsv", "Southwestern Nisu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southwestern Tamang", "tsf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tsf", "Southwestern Tamang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southwestern Tarahumara", "twr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("twr", "Southwestern Tarahumara"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southwestern Tepehuan", "tla"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tla", "Southwestern Tepehuan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Southwestern Tlaxiaco Mixtec", "meh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("meh", "Southwestern Tlaxiaco Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sowa", "sww"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sww", "Sowa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sowanda", "sow"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sow", "Sowanda"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sowari", "dmy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dmy", "Sowari"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Soyaltepec Mazatec", "vmp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vmp", "Soyaltepec Mazatec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Soyaltepec Mixtec", "vmq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vmq", "Soyaltepec Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Spanish", "spa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("spa", "Spanish"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Spanish Sign Language", "ssp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ssp", "Spanish Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Spiti Bhoti", "spt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("spt", "Spiti Bhoti"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Spokane", "spo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("spo", "Spokane"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Squamish", "squ"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("squ", "Squamish"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sranan Tongo", "srn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("srn", "Sranan Tongo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sri Lankan Creole Malay", "sci"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sci", "Sri Lankan Creole Malay"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sri Lankan Sign Language", "sqs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sqs", "Sri Lankan Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Standard Arabic", "arb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("arb", "Standard Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Standard Estonian", "ekk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ekk", "Standard Estonian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Standard Latvian", "lvs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lvs", "Standard Latvian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Standard Malay", "zsm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zsm", "Standard Malay"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Stellingwerfs", "stl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("stl", "Stellingwerfs"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Stod Bhoti", "sbu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sbu", "Stod Bhoti"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Stoney", "sto"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sto", "Stoney"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Straits Salish", "str"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("str", "Straits Salish"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Suabo", "szp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("szp", "Suabo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Suarmin", "seo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("seo", "Suarmin"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Suau", "swp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("swp", "Suau"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Suba", "sxb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sxb", "Suba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Suba-Simbiti", "ssc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ssc", "Suba-Simbiti"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Subi", "xsj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xsj", "Subi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Subiya", "sbs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sbs", "Subiya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Subtiaba", "sut"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sut", "Subtiaba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sudanese Arabic", "apd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("apd", "Sudanese Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sudanese Creole Arabic", "pga"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pga", "Sudanese Creole Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sudest", "tgo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tgo", "Sudest"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sudovian", "xsv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xsv", "Sudovian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Suena", "sue"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sue", "Suena"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Suga", "sgi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sgi", "Suga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Suganga", "sug"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sug", "Suganga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sugut Dusun", "kzs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kzs", "Sugut Dusun"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sui", "swi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("swi", "Sui"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Suki", "sui"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sui", "Suki"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Suku", "sub"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sub", "Suku"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sukuma", "suk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("suk", "Sukuma"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sukur", "syk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("syk", "Sukur"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sukurum", "zsu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zsu", "Sukurum"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sula", "szn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("szn", "Sula"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sulka", "sua"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sua", "Sulka"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sulod", "srg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("srg", "Sulod"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sulung", "suv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("suv", "Sulung"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Suma", "sqm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sqm", "Suma"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sumariup", "siv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("siv", "Sumariup"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sumau", "six"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("six", "Sumau"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sumbawa", "smw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("smw", "Sumbawa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sumbwa", "suw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("suw", "Sumbwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sumerian", "sux"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sux", "Sumerian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sumi Naga", "nsm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nsm", "Sumi Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sunam", "ssk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ssk", "Sunam"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sundanese", "sun"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sun", "Sundanese"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sunum", "ymn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ymn", "Sunum"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sunwar", "suz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("suz", "Sunwar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Suoy", "syo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("syo", "Suoy"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Supyire Senoufo", "spp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("spp", "Supyire Senoufo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sur", "tdl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tdl", "Sur"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Surbakhal", "sbj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sbj", "Surbakhal"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Suri", "suq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("suq", "Suri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Surigaonon", "sgd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sgd", "Surigaonon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Surjapuri", "sjp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sjp", "Surjapuri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sursurunga", "sgz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sgz", "Sursurunga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Suruahá", "swx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("swx", "Suruahá"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Surubu", "sde"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sde", "Surubu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Suruí", "sru"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sru", "Suruí"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Suruí Do Pará", "mdz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mdz", "Suruí Do Pará"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Susquehannock", "sqn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sqn", "Susquehannock"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Susu", "sus"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sus", "Susu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Susuami", "ssu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ssu", "Susuami"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Suundi", "sdj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sdj", "Suundi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Suwawa", "swu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("swu", "Suwawa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Suyá", "suy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("suy", "Suyá"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Svan", "sva"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sva", "Svan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Swabian", "swg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("swg", "Swabian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Swahili (individual language)", "swh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("swh", "Swahili (individual language)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Swahili (macrolanguage)", "swa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("swa", "Swahili (macrolanguage)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Swampy Cree", "csw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("csw", "Swampy Cree"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Swati", "ssw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ssw", "Swati"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Swedish", "swe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("swe", "Swedish"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Swedish Sign Language", "swl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("swl", "Swedish Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Swiss German", "gsw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gsw", "Swiss German"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Swiss-French Sign Language", "ssr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ssr", "Swiss-French Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Swiss-German Sign Language", "sgg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sgg", "Swiss-German Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Swiss-Italian Sign Language", "slf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("slf", "Swiss-Italian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Syenara Senoufo", "shz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("shz", "Syenara Senoufo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sylheti", "syl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("syl", "Sylheti"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Syriac", "syr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("syr", "Syriac"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sáliba", "slc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("slc", "Sáliba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("São Paulo Kaingáng", "zkp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zkp", "São Paulo Kaingáng"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sãotomense", "cri"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cri", "Sãotomense"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sìcìté Sénoufo", "sep"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sep", "Sìcìté Sénoufo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Sô", "sss"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sss", "Sô"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("T'en", "tct"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tct", "T'en"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ta'izzi-Adeni Arabic", "acq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("acq", "Ta'izzi-Adeni Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Taabwa", "tap"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tap", "Taabwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tabaa Zapotec", "zat"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zat", "Tabaa Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tabaru", "tby"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tby", "Tabaru"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tabasco Chontal", "chf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("chf", "Tabasco Chontal"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tabasco Nahuatl", "nhc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nhc", "Tabasco Nahuatl"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tabasco Zoque", "zoq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zoq", "Tabasco Zoque"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tabassaran", "tab"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tab", "Tabassaran"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tabla", "tnm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tnm", "Tabla"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tabo", "knv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("knv", "Tabo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tabriak", "tzx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tzx", "Tabriak"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tacahua Mixtec", "xtt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xtt", "Tacahua Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tacana", "tna"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tna", "Tacana"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tachawit", "shy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("shy", "Tachawit"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tachelhit", "shi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("shi", "Tachelhit"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tachoni", "lts"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lts", "Tachoni"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tadaksahak", "dsq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dsq", "Tadaksahak"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tadyawan", "tdy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tdy", "Tadyawan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tae'", "rob"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rob", "Tae'"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tafi", "tcd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tcd", "Tafi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tagabawa", "bgs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bgs", "Tagabawa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tagakaulo", "klg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("klg", "Tagakaulo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tagal Murut", "mvv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mvv", "Tagal Murut"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tagalog", "tgl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tgl", "Tagalog"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tagargrent", "oua"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("oua", "Tagargrent"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tagbanwa", "tbw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tbw", "Tagbanwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tagbu", "tbm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tbm", "Tagbu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tagdal", "tda"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tda", "Tagdal"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tagish", "tgx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tgx", "Tagish"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tagoi", "tag"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tag", "Tagoi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tagwana Senoufo", "tgw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tgw", "Tagwana Senoufo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tahaggart Tamahaq", "thv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("thv", "Tahaggart Tamahaq"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tahitian", "tah"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tah", "Tahitian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tahltan", "tht"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tht", "Tahltan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tai", "taw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("taw", "Tai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tai Daeng", "tyr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tyr", "Tai Daeng"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tai Dam", "blt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("blt", "Tai Dam"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tai Do", "tyj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tyj", "Tai Do"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tai Dón", "twh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("twh", "Tai Dón"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tai Hang Tong", "thc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("thc", "Tai Hang Tong"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tai Hongjin", "tiz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tiz", "Tai Hongjin"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tai Loi", "tlq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tlq", "Tai Loi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tai Long", "thi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("thi", "Tai Long"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tai Mène", "tmp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tmp", "Tai Mène"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tai Nüa", "tdd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tdd", "Tai Nüa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tai Pao", "tpo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tpo", "Tai Pao"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tai Thanh", "tmm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tmm", "Tai Thanh"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tai Ya", "cuu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cuu", "Tai Ya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Taiap", "gpn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gpn", "Taiap"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Taikat", "aos"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aos", "Taikat"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tainae", "ago"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ago", "Tainae"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Taino", "tnq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tnq", "Taino"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tairuma", "uar"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("uar", "Tairuma"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Taita", "dav"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dav", "Taita"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Taiwan Sign Language", "tss"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tss", "Taiwan Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Taje", "pee"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pee", "Taje"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tajik", "tgk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tgk", "Tajik"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tajiki Arabic", "abh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("abh", "Tajiki Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tajio", "tdj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tdj", "Tajio"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tajuasohn", "tja"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tja", "Tajuasohn"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Takelma", "tkm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tkm", "Takelma"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Takestani", "tks"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tks", "Takestani"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Takia", "tbc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tbc", "Takia"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Takpa", "tkk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tkk", "Takpa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Takua", "tkz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tkz", "Takua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Takuu", "nho"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nho", "Takuu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Takwane", "tke"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tke", "Takwane"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tal", "tal"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tal", "Tal"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tala", "tak"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tak", "Tala"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Talaandig", "bkd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bkd", "Talaandig"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Talaud", "tld"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tld", "Talaud"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Taliabu", "tlv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tlv", "Taliabu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Talieng", "tdf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tdf", "Talieng"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Talinga-Bwisi", "tlj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tlj", "Talinga-Bwisi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Talise", "tlr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tlr", "Talise"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Talodi", "tlo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tlo", "Talodi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Taloki", "tlk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tlk", "Taloki"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Talondo'", "tln"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tln", "Talondo'"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Talu", "yta"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yta", "Talu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Talur", "ilw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ilw", "Talur"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Talysh", "tly"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tly", "Talysh"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tama (Chad)", "tma"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tma", "Tama (Chad)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tama (Colombia)", "ten"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ten", "Tama (Colombia)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tamagario", "tcg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tcg", "Tamagario"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Taman (Indonesia)", "tmn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tmn", "Taman (Indonesia)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Taman (Myanmar)", "tcl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tcl", "Taman (Myanmar)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tamanaku", "tmz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tmz", "Tamanaku"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tamashek", "tmh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tmh", "Tamashek"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tamasheq", "taq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("taq", "Tamasheq"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tamazola Mixtec", "vmx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vmx", "Tamazola Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tambas", "tdk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tdk", "Tambas"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tambora", "xxt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xxt", "Tambora"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tambotalo", "tls"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tls", "Tambotalo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tambunan Dusun", "kzt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kzt", "Tambunan Dusun"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tami", "tmy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tmy", "Tami"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tamil", "tam"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tam", "Tamil"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tamki", "tax"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tax", "Tamki"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tamnim Citak", "tml"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tml", "Tamnim Citak"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tampias Lobu", "low"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("low", "Tampias Lobu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tampuan", "tpu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tpu", "Tampuan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tampulma", "tpm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tpm", "Tampulma"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tanacross", "tcb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tcb", "Tanacross"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tanahmerah", "tcm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tcm", "Tanahmerah"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tanaina", "tfn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tfn", "Tanaina"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tanapag", "tpv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tpv", "Tanapag"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tandaganon", "tgn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tgn", "Tandaganon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tandia", "tni"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tni", "Tandia"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tandroy-Mahafaly Malagasy", "tdx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tdx", "Tandroy-Mahafaly Malagasy"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tanema", "tnx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tnx", "Tanema"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tangale", "tan"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tan", "Tangale"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tangchangya", "tnv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tnv", "Tangchangya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tangga", "tgg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tgg", "Tangga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tanggu", "tgu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tgu", "Tanggu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tangkhul Naga", "nmf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nmf", "Tangkhul Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tangko", "tkx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tkx", "Tangko"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tanglang", "ytl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ytl", "Tanglang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tangoa", "tgp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tgp", "Tangoa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tanguat", "tbs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tbs", "Tanguat"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tangut", "txg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("txg", "Tangut"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tanimbili", "tbe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tbe", "Tanimbili"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tanjijili", "uji"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("uji", "Tanjijili"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tanosy Malagasy", "txy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("txy", "Tanosy Malagasy"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tanzanian Sign Language", "tza"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tza", "Tanzanian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tapeba", "tbb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tbb", "Tapeba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tapei", "afp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("afp", "Tapei"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tapieté", "tpj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tpj", "Tapieté"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tapirapé", "taf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("taf", "Tapirapé"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tarao Naga", "tro"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tro", "Tarao Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tareng", "tgr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tgr", "Tareng"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tariana", "tae"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tae", "Tariana"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tarifit", "rif"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rif", "Tarifit"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tarok", "yer"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yer", "Tarok"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Taroko", "trv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("trv", "Taroko"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tarpia", "tpf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tpf", "Tarpia"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tartessian", "txr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("txr", "Tartessian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tasawaq", "twq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("twq", "Tasawaq"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tase Naga", "nst"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nst", "Tase Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tasmanian", "xtz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xtz", "Tasmanian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tasmate", "tmt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tmt", "Tasmate"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tataltepec Chatino", "cta"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cta", "Tataltepec Chatino"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tatana", "txx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("txx", "Tatana"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tatar", "tat"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tat", "Tatar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tatuyo", "tav"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tav", "Tatuyo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tauade", "ttd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ttd", "Tauade"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Taulil", "tuh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tuh", "Taulil"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Taungyo", "tco"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tco", "Taungyo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Taupota", "tpa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tpa", "Taupota"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tause", "tad"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tad", "Tause"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Taushiro", "trr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("trr", "Taushiro"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tausug", "tsg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tsg", "Tausug"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tauya", "tya"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tya", "Tauya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Taveta", "tvs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tvs", "Taveta"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tavoyan", "tvn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tvn", "Tavoyan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tavringer Romani", "rmu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rmu", "Tavringer Romani"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tawala", "tbo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tbo", "Tawala"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tawallammat Tamajaq", "ttq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ttq", "Tawallammat Tamajaq"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tawandê", "xtw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xtw", "Tawandê"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tawang Monpa", "twm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("twm", "Tawang Monpa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tawara", "twl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("twl", "Tawara"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Taworta", "tbp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tbp", "Taworta"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tawoyan", "twy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("twy", "Tawoyan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tawr Chin", "tcp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tcp", "Tawr Chin"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tay Boi", "tas"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tas", "Tay Boi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tay Khang", "tnu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tnu", "Tay Khang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tayabas Ayta", "ayy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ayy", "Tayabas Ayta"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tayart Tamajeq", "thz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("thz", "Tayart Tamajeq"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tayo", "cks"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cks", "Tayo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Taznatit", "grr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("grr", "Taznatit"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tboli", "tbl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tbl", "Tboli"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tchitchege", "tck"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tck", "Tchitchege"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tchumbuli", "bqa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bqa", "Tchumbuli"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Te'un", "tve"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tve", "Te'un"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Teanu", "tkw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tkw", "Teanu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tebi", "dmu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dmu", "Tebi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tebul Sign Language", "tsy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tsy", "Tebul Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tebul Ure Dogon", "dtu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dtu", "Tebul Ure Dogon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tecpatlán Totonac", "tcw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tcw", "Tecpatlán Totonac"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tedaga", "tuq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tuq", "Tedaga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tedim Chin", "ctd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ctd", "Tedim Chin"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tee", "tkq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tkq", "Tee"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tefaro", "tfo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tfo", "Tefaro"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tegali", "ras"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ras", "Tegali"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tehit", "kps"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kps", "Tehit"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tehuelche", "teh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("teh", "Tehuelche"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tejalapan Zapotec", "ztt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ztt", "Tejalapan Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Teke-Ebo", "ebo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ebo", "Teke-Ebo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Teke-Fuumu", "ifm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ifm", "Teke-Fuumu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Teke-Kukuya", "kkw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kkw", "Teke-Kukuya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Teke-Laali", "lli"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lli", "Teke-Laali"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Teke-Nzikou", "nzu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nzu", "Teke-Nzikou"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Teke-Tege", "teg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("teg", "Teke-Tege"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Teke-Tsaayi", "tyi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tyi", "Teke-Tsaayi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Teke-Tyee", "tyx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tyx", "Teke-Tyee"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tektiteko", "ttc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ttc", "Tektiteko"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tela-Masbuar", "tvm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tvm", "Tela-Masbuar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Telefol", "tlf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tlf", "Telefol"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Telugu", "tel"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tel", "Telugu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Teluti", "tlt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tlt", "Teluti"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tem", "kdh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kdh", "Tem"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Temacine Tamazight", "tjo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tjo", "Temacine Tamazight"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Temascaltepec Nahuatl", "nhv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nhv", "Temascaltepec Nahuatl"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tembo (Kitembo)", "tbt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tbt", "Tembo (Kitembo)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tembo (Motembo)", "tmv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tmv", "Tembo (Motembo)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tembé", "tqb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tqb", "Tembé"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Teme", "tdo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tdo", "Teme"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Temein", "teq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("teq", "Temein"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Temi", "soz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("soz", "Temi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Temiar", "tea"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tea", "Temiar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Temoaya Otomi", "ott"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ott", "Temoaya Otomi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Temoq", "tmo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tmo", "Temoq"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tempasuk Dusun", "tdu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tdu", "Tempasuk Dusun"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Temuan", "tmw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tmw", "Temuan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tena", "bfe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bfe", "Tena"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tena Lowland Quichua", "quw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("quw", "Tena Lowland Quichua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tenango Otomi", "otn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("otn", "Tenango Otomi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tene Kan Dogon", "dtk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dtk", "Tene Kan Dogon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tenggarong Kutai Malay", "vkt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vkt", "Tenggarong Kutai Malay"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tengger", "tes"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tes", "Tengger"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tenharim", "pah"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pah", "Tenharim"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tenino", "tqn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tqn", "Tenino"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tenis", "tns"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tns", "Tenis"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tennet", "tex"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tex", "Tennet"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Teop", "tio"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tio", "Teop"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Teor", "tev"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tev", "Teor"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tepecano", "tep"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tep", "Tepecano"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tepetotutla Chinantec", "cnt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cnt", "Tepetotutla Chinantec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tepeuxila Cuicatec", "cux"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cux", "Tepeuxila Cuicatec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tepinapa Chinantec", "cte"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cte", "Tepinapa Chinantec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tepo Krumen", "ted"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ted", "Tepo Krumen"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ter Sami", "sjt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sjt", "Ter Sami"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tera", "ttr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ttr", "Tera"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Terebu", "trb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("trb", "Terebu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Terei", "buo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("buo", "Terei"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tereno", "ter"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ter", "Tereno"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Teressa", "tef"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tef", "Teressa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tereweng", "twg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("twg", "Tereweng"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Teribe", "tfr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tfr", "Teribe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Terik", "tec"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tec", "Terik"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Termanu", "twu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("twu", "Termanu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ternate", "tft"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tft", "Ternate"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ternateño", "tmg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tmg", "Ternateño"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tese", "keg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("keg", "Tese"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Teshenawa", "twc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("twc", "Teshenawa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Teso", "teo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("teo", "Teso"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tetela", "tll"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tll", "Tetela"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tetelcingo Nahuatl", "nhg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nhg", "Tetelcingo Nahuatl"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tetete", "teb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("teb", "Tetete"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tetum", "tet"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tet", "Tetum"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tetun Dili", "tdt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tdt", "Tetun Dili"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Teutila Cuicatec", "cut"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cut", "Teutila Cuicatec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tewa (Indonesia)", "twe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("twe", "Tewa (Indonesia)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tewa (USA)", "tew"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tew", "Tewa (USA)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tewe", "twx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("twx", "Tewe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Texcatepec Otomi", "otx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("otx", "Texcatepec Otomi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Texistepec Popoluca", "poq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("poq", "Texistepec Popoluca"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Texmelucan Zapotec", "zpz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zpz", "Texmelucan Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tezoatlán Mixtec", "mxb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mxb", "Tezoatlán Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tha", "thy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("thy", "Tha"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Thachanadan", "thn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("thn", "Thachanadan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Thado Chin", "tcz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tcz", "Thado Chin"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Thai", "tha"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tha", "Thai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Thai Sign Language", "tsq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tsq", "Thai Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Thai Song", "soa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("soa", "Thai Song"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Thakali", "ths"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ths", "Thakali"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Thangal Naga", "nki"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nki", "Thangal Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Thangmi", "thf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("thf", "Thangmi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Thao", "ssf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ssf", "Thao"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tharaka", "thk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("thk", "Tharaka"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Thayore", "thd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("thd", "Thayore"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Thaypan", "typ"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("typ", "Thaypan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("The", "thx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("thx", "The"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tho", "tou"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tou", "Tho"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Thompson", "thp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("thp", "Thompson"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Thopho", "ytp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ytp", "Thopho"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Thracian", "txh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("txh", "Thracian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Thu Lao", "tyl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tyl", "Thu Lao"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Thudam", "thw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("thw", "Thudam"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Thulung", "tdh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tdh", "Thulung"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Thurawal", "tbh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tbh", "Thurawal"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Thuri", "thu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("thu", "Thuri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tiagbamrin Aizi", "ahi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ahi", "Tiagbamrin Aizi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tiale", "mnl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mnl", "Tiale"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tiang", "tbj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tbj", "Tiang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tibea", "ngy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ngy", "Tibea"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tibetan", "bod"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bod", "Tibetan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tichurong", "tcn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tcn", "Tichurong"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ticuna", "tca"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tca", "Ticuna"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tidaá Mixtec", "mtx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mtx", "Tidaá Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tidikelt Tamazight", "tia"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tia", "Tidikelt Tamazight"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tidong", "tid"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tid", "Tidong"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tidore", "tvo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tvo", "Tidore"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tiemacèwè Bozo", "boo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("boo", "Tiemacèwè Bozo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tiene", "tii"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tii", "Tiene"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tifal", "tif"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tif", "Tifal"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tigak", "tgc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tgc", "Tigak"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tigon Mbembe", "nza"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nza", "Tigon Mbembe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tigre", "tig"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tig", "Tigre"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tigrinya", "tir"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tir", "Tigrinya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tii", "txq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("txq", "Tii"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tijaltepec Mixtec", "xtl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xtl", "Tijaltepec Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tikar", "tik"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tik", "Tikar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tikopia", "tkp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tkp", "Tikopia"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tilapa Otomi", "otl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("otl", "Tilapa Otomi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tillamook", "til"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("til", "Tillamook"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tilquiapan Zapotec", "zts"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zts", "Tilquiapan Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tilung", "tij"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tij", "Tilung"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tima", "tms"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tms", "Tima"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Timbe", "tim"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tim", "Timbe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Timbisha", "par"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("par", "Timbisha"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Timne", "tem"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tem", "Timne"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Timor Pidgin", "tvy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tvy", "Timor Pidgin"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Timucua", "tjm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tjm", "Timucua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Timugon Murut", "tih"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tih", "Timugon Murut"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tinani", "lbf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lbf", "Tinani"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tindi", "tin"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tin", "Tindi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tingal", "tie"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tie", "Tingal"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tingui-Boto", "tgv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tgv", "Tingui-Boto"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tinigua", "tit"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tit", "Tinigua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tinoc Kallahan", "tne"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tne", "Tinoc Kallahan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tinputz", "tpz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tpz", "Tinputz"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tinà Sambal", "xsb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xsb", "Tinà Sambal"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tippera", "tpe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tpe", "Tippera"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tira", "tic"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tic", "Tira"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tirahi", "tra"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tra", "Tirahi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tiranige Diga Dogon", "tde"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tde", "Tiranige Diga Dogon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tiri", "cir"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cir", "Tiri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tiruray", "tiy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tiy", "Tiruray"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tita", "tdq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tdq", "Tita"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Titan", "ttv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ttv", "Titan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tiv", "tiv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tiv", "Tiv"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tiwa", "lax"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lax", "Tiwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tiwi", "tiw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tiw", "Tiwi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tiéfo", "tiq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tiq", "Tiéfo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tiéyaxo Bozo", "boz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("boz", "Tiéyaxo Bozo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tjurruru", "tju"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tju", "Tjurruru"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tlachichilco Tepehua", "tpt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tpt", "Tlachichilco Tepehua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tlacoapa Me'phaa", "tpl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tpl", "Tlacoapa Me'phaa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tlacoapa Tlapanec", "tpl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tpl", "Tlacoapa Tlapanec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tlacoatzintepec Chinantec", "ctl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ctl", "Tlacoatzintepec Chinantec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tlacolulita Zapotec", "zpk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zpk", "Tlacolulita Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tlahuitoltepec Mixe", "mxp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mxp", "Tlahuitoltepec Mixe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tlamacazapa Nahuatl", "nuz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nuz", "Tlamacazapa Nahuatl"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tlazoyaltepec Mixtec", "mqh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mqh", "Tlazoyaltepec Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tlingit", "tli"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tli", "Tlingit"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("To", "toz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("toz", "To"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("To'abaita", "mlu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mlu", "To'abaita"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Toaripi", "tqo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tqo", "Toaripi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Toba", "tob"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tob", "Toba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Toba-Maskoy", "tmf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tmf", "Toba-Maskoy"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tobagonian Creole English", "tgh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tgh", "Tobagonian Creole English"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tobanga", "tng"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tng", "Tobanga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tobati", "tti"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tti", "Tobati"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tobelo", "tlb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tlb", "Tobelo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tobian", "tox"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tox", "Tobian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tobilung", "tgb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tgb", "Tobilung"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tobo", "tbv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tbv", "Tobo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tocantins Asurini", "asu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("asu", "Tocantins Asurini"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tocho", "taz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("taz", "Tocho"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Toda", "tcx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tcx", "Toda"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Todrah", "tdr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tdr", "Todrah"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tofanma", "tlg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tlg", "Tofanma"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tofin Gbe", "tfi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tfi", "Tofin Gbe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Togbo-Vara Banda", "tor"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tor", "Togbo-Vara Banda"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Togoyo", "tgy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tgy", "Togoyo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tohono O'odham", "ood"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ood", "Tohono O'odham"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tojolabal", "toj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("toj", "Tojolabal"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tok Pisin", "tpi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tpi", "Tok Pisin"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tokano", "zuh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zuh", "Tokano"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tokelau", "tkl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tkl", "Tokelau"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tokharian A", "xto"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xto", "Tokharian A"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tokharian B", "txb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("txb", "Tokharian B"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Toku-No-Shima", "tkn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tkn", "Toku-No-Shima"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tol", "jic"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jic", "Tol"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tolaki", "lbw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lbw", "Tolaki"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tolomako", "tlm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tlm", "Tolomako"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tolowa", "tol"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tol", "Tolowa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Toma", "tod"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tod", "Toma"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tomadino", "tdi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tdi", "Tomadino"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tombelala", "ttp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ttp", "Tombelala"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tombonuo", "txa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("txa", "Tombonuo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tombulu", "tom"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tom", "Tombulu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tomedes", "toe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("toe", "Tomedes"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tomini", "txm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("txm", "Tomini"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tomo Kan Dogon", "dtm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dtm", "Tomo Kan Dogon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tomoip", "tqp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tqp", "Tomoip"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tondano", "tdn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tdn", "Tondano"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tonga (Nyasa)", "tog"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tog", "Tonga (Nyasa)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tonga (Thailand)", "tnz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tnz", "Tonga (Thailand)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tonga (Tonga Islands)", "ton"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ton", "Tonga (Tonga Islands)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tonga (Zambia)", "toi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("toi", "Tonga (Zambia)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tongwe", "tny"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tny", "Tongwe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tonjon", "tjn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tjn", "Tonjon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tonkawa", "tqw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tqw", "Tonkawa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tonsawang", "tnw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tnw", "Tonsawang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tonsea", "txs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("txs", "Tonsea"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tontemboan", "tnt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tnt", "Tontemboan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tooro", "ttj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ttj", "Tooro"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Topoiyo", "toy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("toy", "Topoiyo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Toposa", "toq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("toq", "Toposa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Toraja-Sa'dan", "sda"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sda", "Toraja-Sa'dan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Toram", "trj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("trj", "Toram"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Torau", "ttu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ttu", "Torau"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tornedalen Finnish", "fit"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("fit", "Tornedalen Finnish"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Toro", "tdv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tdv", "Toro"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Toro So Dogon", "dts"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dts", "Toro So Dogon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Toro Tegu Dogon", "dtt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dtt", "Toro Tegu Dogon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Toromono", "tno"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tno", "Toromono"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Torona", "tqr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tqr", "Torona"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Torres Strait Creole", "tcs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tcs", "Torres Strait Creole"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Torricelli", "tei"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tei", "Torricelli"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Torwali", "trw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("trw", "Torwali"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Torá", "trz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("trz", "Torá"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tosk Albanian", "als"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("als", "Tosk Albanian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Totela", "ttl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ttl", "Totela"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Toto", "txo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("txo", "Toto"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Totoli", "txe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("txe", "Totoli"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Totomachapan Zapotec", "zph"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zph", "Totomachapan Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Totontepec Mixe", "mto"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mto", "Totontepec Mixe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Totoro", "ttk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ttk", "Totoro"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Touo", "tqu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tqu", "Touo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Toura (Côte d'Ivoire)", "neb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("neb", "Toura (Côte d'Ivoire)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Toura (Papua New Guinea)", "don"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("don", "Toura (Papua New Guinea)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Towei", "ttn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ttn", "Towei"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Transalpine Gaulish", "xtg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xtg", "Transalpine Gaulish"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Traveller Danish", "rmd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rmd", "Traveller Danish"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Traveller Norwegian", "rmg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rmg", "Traveller Norwegian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Traveller Scottish", "trl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("trl", "Traveller Scottish"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tregami", "trm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("trm", "Tregami"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tremembé", "tme"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tme", "Tremembé"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Trieng", "stg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("stg", "Trieng"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Trimuris", "tip"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tip", "Trimuris"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tring", "tgq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tgq", "Tring"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tringgus-Sembaan Bidayuh", "trx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("trx", "Tringgus-Sembaan Bidayuh"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Trinidad and Tobago Sign Language", "lst"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lst", "Trinidad and Tobago Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Trinidadian Creole English", "trf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("trf", "Trinidadian Creole English"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Trinitario", "trn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("trn", "Trinitario"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Trió", "tri"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tri", "Trió"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Truká", "tka"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tka", "Truká"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Trumai", "tpy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tpy", "Trumai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ts'ün-Lao", "tsl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tsl", "Ts'ün-Lao"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tsaangi", "tsa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tsa", "Tsaangi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tsakhur", "tkr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tkr", "Tsakhur"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tsakonian", "tsd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tsd", "Tsakonian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tsakwambo", "kvz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kvz", "Tsakwambo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tsamai", "tsb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tsb", "Tsamai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tsat", "huq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("huq", "Tsat"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tsaukambo", "kvz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kvz", "Tsaukambo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tseku", "tsk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tsk", "Tseku"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tsetsaut", "txc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("txc", "Tsetsaut"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tshangla", "tsj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tsj", "Tshangla"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tsikimba", "kdl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kdl", "Tsikimba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tsimané", "cas"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cas", "Tsimané"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tsimihety Malagasy", "xmw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xmw", "Tsimihety Malagasy"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tsimshian", "tsi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tsi", "Tsimshian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tsishingini", "tsw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tsw", "Tsishingini"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tso", "ldp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ldp", "Tso"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tsoa", "hio"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hio", "Tsoa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tsogo", "tsv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tsv", "Tsogo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tsonga", "tso"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tso", "Tsonga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tsotsitaal", "fly"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("fly", "Tsotsitaal"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tsotso", "lto"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lto", "Tsotso"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tsou", "tsu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tsu", "Tsou"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tsum", "ttz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ttz", "Tsum"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tsuvadi", "tvd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tvd", "Tsuvadi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tsuvan", "tsh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tsh", "Tsuvan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tswa", "tsc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tsc", "Tswa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tswana", "tsn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tsn", "Tswana"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tswapong", "two"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("two", "Tswapong"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tu", "mjg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mjg", "Tu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tuamotuan", "pmt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pmt", "Tuamotuan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tubar", "tbu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tbu", "Tubar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tucano", "tuo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tuo", "Tucano"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tugen", "tuy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tuy", "Tugen"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tugun", "tzn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tzn", "Tugun"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tugutil", "tuj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tuj", "Tugutil"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tukang Besi North", "khc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("khc", "Tukang Besi North"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tukang Besi South", "bhq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bhq", "Tukang Besi South"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tuki", "bag"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bag", "Tuki"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tukpa", "tpq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tpq", "Tukpa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tukudede", "tkd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tkd", "Tukudede"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tukumanféd", "tkf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tkf", "Tukumanféd"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tula", "tul"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tul", "Tula"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tulehu", "tlu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tlu", "Tulehu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tulishi", "tey"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tey", "Tulishi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tulu", "tcy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tcy", "Tulu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tulu-Bohuai", "rak"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rak", "Tulu-Bohuai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tuma-Irumu", "iou"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("iou", "Tuma-Irumu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tumak", "tmc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tmc", "Tumak"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tumari Kanuri", "krt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("krt", "Tumari Kanuri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tumbuka", "tum"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tum", "Tumbuka"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tumi", "kku"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kku", "Tumi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tumleo", "tmq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tmq", "Tumleo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tumshuqese", "xtq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xtq", "Tumshuqese"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tumtum", "tbr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tbr", "Tumtum"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tumulung Sisaala", "sil"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sil", "Tumulung Sisaala"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tumzabt", "mzb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mzb", "Tumzabt"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tundra Enets", "enh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("enh", "Tundra Enets"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tunen", "baz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("baz", "Tunen"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tungag", "lcm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lcm", "Tungag"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tunggare", "trt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("trt", "Tunggare"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tunia", "tug"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tug", "Tunia"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tunica", "tun"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tun", "Tunica"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tunisian Arabic", "aeb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aeb", "Tunisian Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tunisian Sign Language", "tse"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tse", "Tunisian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tunjung", "tjg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tjg", "Tunjung"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tunni", "tqq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tqq", "Tunni"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tunzu", "dza"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dza", "Tunzu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tuotomb", "ttf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ttf", "Tuotomb"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tuparí", "tpr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tpr", "Tuparí"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tupinambá", "tpn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tpn", "Tupinambá"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tupinikin", "tpk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tpk", "Tupinikin"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tupuri", "tui"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tui", "Tupuri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tupí", "tpw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tpw", "Tupí"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Turaka", "trh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("trh", "Turaka"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Turi", "trd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("trd", "Turi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Turiwára", "twt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("twt", "Turiwára"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Turka", "tuz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tuz", "Turka"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Turkana", "tuv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tuv", "Turkana"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Turkic Khalaj", "klj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("klj", "Turkic Khalaj"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Turkish", "tur"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tur", "Turkish"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Turkish Sign Language", "tsm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tsm", "Turkish Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Turkmen", "tuk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tuk", "Turkmen"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Turks And Caicos Creole English", "tch"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tch", "Turks And Caicos Creole English"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Turoyo", "tru"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tru", "Turoyo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Turumsa", "tqm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tqm", "Turumsa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Turung", "try"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("try", "Turung"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tuscarora", "tus"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tus", "Tuscarora"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tutelo", "tta"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tta", "Tutelo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tutong", "ttg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ttg", "Tutong"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tutsa Naga", "tvt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tvt", "Tutsa Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tutuba", "tmi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tmi", "Tutuba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tututepec Mixtec", "mtu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mtu", "Tututepec Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tututni", "tuu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tuu", "Tututni"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tuvalu", "tvl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tvl", "Tuvalu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tuvinian", "tyv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tyv", "Tuvinian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tuwali Ifugao", "ifk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ifk", "Tuwali Ifugao"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tuwari", "tww"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tww", "Tuwari"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tuwuli", "bov"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bov", "Tuwuli"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tuxináwa", "tux"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tux", "Tuxináwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tuxá", "tud"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tud", "Tuxá"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tuyuca", "tue"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tue", "Tuyuca"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Twana", "twa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("twa", "Twana"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Twendi", "twn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("twn", "Twendi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Twents", "twd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("twd", "Twents"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Twi", "twi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("twi", "Twi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tyap", "kcg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kcg", "Tyap"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tyaraity", "woa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("woa", "Tyaraity"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tz'utujil", "tzj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tzj", "Tz'utujil"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tzeltal", "tzh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tzh", "Tzeltal"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tzotzil", "tzo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tzo", "Tzotzil"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tày", "tyz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tyz", "Tày"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tày Sa Pa", "tys"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tys", "Tày Sa Pa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tày Tac", "tyt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tyt", "Tày Tac"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Téén", "lor"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lor", "Téén"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Tübatulabal", "tub"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tub", "Tübatulabal"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Türk İşaret Dili", "tsm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tsm", "Türk İşaret Dili"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("U", "uuu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("uuu", "U"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Uab Meto", "aoz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aoz", "Uab Meto"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Uamué", "uam"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("uam", "Uamué"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Uare", "ksj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ksj", "Uare"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ubaghara", "byc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("byc", "Ubaghara"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ubang", "uba"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("uba", "Ubang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ubi", "ubi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ubi", "Ubi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ubir", "ubr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ubr", "Ubir"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ubykh", "uby"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("uby", "Ubykh"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ucayali-Yurúa Ashéninka", "cpb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cpb", "Ucayali-Yurúa Ashéninka"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Uda", "uda"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("uda", "Uda"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Udi", "udi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("udi", "Udi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Udihe", "ude"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ude", "Udihe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Udmurt", "udm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("udm", "Udmurt"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Uduk", "udu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("udu", "Uduk"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ufim", "ufi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ufi", "Ufim"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ugandan Sign Language", "ugn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ugn", "Ugandan Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ugaritic", "uga"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("uga", "Ugaritic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ughele", "uge"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("uge", "Ughele"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ugong", "ugo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ugo", "Ugong"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Uhami", "uha"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("uha", "Uhami"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Uighur", "uig"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("uig", "Uighur"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Uisai", "uis"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("uis", "Uisai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ujir", "udj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("udj", "Ujir"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ukaan", "kcf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kcf", "Ukaan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ukhwejo", "ukh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ukh", "Ukhwejo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ukit", "umi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("umi", "Ukit"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ukpe-Bayobiri", "ukp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ukp", "Ukpe-Bayobiri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ukpet-Ehom", "akd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("akd", "Ukpet-Ehom"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ukrainian", "ukr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ukr", "Ukrainian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ukrainian Sign Language", "ukl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ukl", "Ukrainian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ukue", "uku"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("uku", "Ukue"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ukuriguma", "ukg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ukg", "Ukuriguma"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ukwa", "ukq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ukq", "Ukwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ulau-Suain", "svb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("svb", "Ulau-Suain"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ulch", "ulc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ulc", "Ulch"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ulithian", "uli"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("uli", "Ulithian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ullatan", "ull"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ull", "Ullatan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ulukwumi", "ulb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ulb", "Ulukwumi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ulumanda'", "ulm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ulm", "Ulumanda'"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ulwa", "ulw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ulw", "Ulwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Uma", "ppk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ppk", "Uma"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Uma' Lasan", "xky"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xky", "Uma' Lasan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Uma' Lung", "ulu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ulu", "Uma' Lung"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Umanakaina", "gdn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gdn", "Umanakaina"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Umatilla", "uma"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("uma", "Umatilla"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Umbindhamu", "umd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("umd", "Umbindhamu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Umbrian", "xum"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xum", "Umbrian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Umbu-Ungu", "ubu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ubu", "Umbu-Ungu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Umbugarla", "umr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("umr", "Umbugarla"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Umbundu", "umb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("umb", "Umbundu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Umbuygamu", "umg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("umg", "Umbuygamu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ume Sami", "sju"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sju", "Ume Sami"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Umeda", "upi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("upi", "Umeda"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Umiray Dumaget Agta", "due"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("due", "Umiray Dumaget Agta"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Umon", "umm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("umm", "Umon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Umotína", "umo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("umo", "Umotína"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Umpila", "ump"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ump", "Umpila"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Una", "mtg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mtg", "Una"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Unami", "unm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("unm", "Unami"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Uncoded languages", "mis"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mis", "Uncoded languages"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Unde Kaili", "unz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("unz", "Unde Kaili"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Undetermined", "und"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("und", "Undetermined"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Uneapa", "bbn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bbn", "Uneapa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Uneme", "une"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("une", "Uneme"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Unserdeutsch", "uln"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("uln", "Unserdeutsch"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Unua", "onu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("onu", "Unua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Uokha", "uok"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("uok", "Uokha"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Upper Chehalis", "cjh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cjh", "Upper Chehalis"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Upper Grand Valley Dani", "dna"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dna", "Upper Grand Valley Dani"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Upper Guinea Crioulo", "pov"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pov", "Upper Guinea Crioulo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Upper Kinabatangan", "dmg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dmg", "Upper Kinabatangan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Upper Kuskokwim", "kuu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kuu", "Upper Kuskokwim"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Upper Necaxa Totonac", "tku"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tku", "Upper Necaxa Totonac"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Upper Saxon", "sxu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sxu", "Upper Saxon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Upper Sorbian", "hsb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hsb", "Upper Sorbian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Upper Ta'oih", "tth"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tth", "Upper Ta'oih"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Upper Tanana", "tau"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tau", "Upper Tanana"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Upper Tanudan Kalinga", "kgh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kgh", "Upper Tanudan Kalinga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Upper Taromi", "tov"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tov", "Upper Taromi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Upper Umpqua", "xup"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xup", "Upper Umpqua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ura (Papua New Guinea)", "uro"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("uro", "Ura (Papua New Guinea)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ura (Vanuatu)", "uur"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("uur", "Ura (Vanuatu)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Uradhi", "urf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("urf", "Uradhi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Urak Lawoi'", "urk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("urk", "Urak Lawoi'"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Urali", "url"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("url", "Urali"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Urapmin", "urm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("urm", "Urapmin"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Urarina", "ura"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ura", "Urarina"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Urartian", "xur"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xur", "Urartian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Urat", "urt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("urt", "Urat"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Urdu", "urd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("urd", "Urdu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Urhobo", "urh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("urh", "Urhobo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Uri", "uvh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("uvh", "Uri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Urigina", "urg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("urg", "Urigina"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Urim", "uri"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("uri", "Urim"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Urimo", "urx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("urx", "Urimo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Urningangg", "urc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("urc", "Urningangg"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Uru", "ure"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ure", "Uru"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Uru-Eu-Wau-Wau", "urz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("urz", "Uru-Eu-Wau-Wau"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Uru-Pa-In", "urp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("urp", "Uru-Pa-In"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Uruangnirin", "urn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("urn", "Uruangnirin"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Uruava", "urv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("urv", "Uruava"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Urubú-Kaapor", "urb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("urb", "Urubú-Kaapor"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Urubú-Kaapor Sign Language", "uks"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("uks", "Urubú-Kaapor Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Uruguayan Sign Language", "ugy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ugy", "Uruguayan Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Urum", "uum"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("uum", "Urum"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Urumi", "uru"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("uru", "Urumi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Usaghade", "usk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("usk", "Usaghade"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Usan", "wnu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wnu", "Usan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Usarufa", "usa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("usa", "Usarufa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ushojo", "ush"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ush", "Ushojo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Usila Chinantec", "cuc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cuc", "Usila Chinantec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Usku", "ulf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ulf", "Usku"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Uspanteco", "usp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("usp", "Uspanteco"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Usui", "usi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("usi", "Usui"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Utarmbung", "omo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("omo", "Utarmbung"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ute-Southern Paiute", "ute"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ute", "Ute-Southern Paiute"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Utu", "utu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("utu", "Utu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Uvbie", "evh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("evh", "Uvbie"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Uya", "usu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("usu", "Uya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Uyghur", "uig"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("uig", "Uyghur"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Uzbek", "uzb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("uzb", "Uzbek"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Uzbeki Arabic", "auz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("auz", "Uzbeki Arabic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Uzekwe", "eze"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("eze", "Uzekwe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("V'ënen Taut", "nmb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nmb", "V'ënen Taut"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Vaagri Booli", "vaa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vaa", "Vaagri Booli"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Vaeakau-Taumako", "piv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("piv", "Vaeakau-Taumako"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Vafsi", "vaf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vaf", "Vafsi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Vaghri", "vgr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vgr", "Vaghri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Vaghua", "tva"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tva", "Vaghua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Vagla", "vag"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vag", "Vagla"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Vai", "vai"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vai", "Vai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Vaiphei", "vap"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vap", "Vaiphei"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Vale", "vae"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vae", "Vale"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Valencian", "cat"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cat", "Valencian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Valencian Sign Language", "vsv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vsv", "Valencian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Valle Nacional Chinantec", "cvn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cvn", "Valle Nacional Chinantec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Valley Maidu", "vmv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vmv", "Valley Maidu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Valman", "van"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("van", "Valman"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Valpei", "vlp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vlp", "Valpei"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Vamale", "mkt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mkt", "Vamale"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Vame", "mlr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mlr", "Vame"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Vandalic", "xvn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xvn", "Vandalic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Vangunu", "mpr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mpr", "Vangunu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Vanimo", "vam"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vam", "Vanimo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Vano", "vnk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vnk", "Vano"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Vanuma", "vau"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vau", "Vanuma"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Vao", "vao"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vao", "Vao"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Varhadi-Nagpuri", "vah"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vah", "Varhadi-Nagpuri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Varisi", "vrs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vrs", "Varisi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Varli", "vav"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vav", "Varli"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Vasavi", "vas"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vas", "Vasavi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Vasekela Bushman", "vaj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vaj", "Vasekela Bushman"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Veddah", "ved"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ved", "Veddah"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Vehes", "val"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("val", "Vehes"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Veluws", "vel"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vel", "Veluws"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Vemgo-Mabas", "vem"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vem", "Vemgo-Mabas"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Venda", "ven"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ven", "Venda"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Venetian", "vec"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vec", "Venetian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Venetic", "xve"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xve", "Venetic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Venezuelan Sign Language", "vsl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vsl", "Venezuelan Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Vengo", "bav"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bav", "Vengo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ventureño", "veo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("veo", "Ventureño"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Veps", "vep"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vep", "Veps"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Vera'a", "vra"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vra", "Vera'a"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Vestinian", "xvs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xvs", "Vestinian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Vidunda", "vid"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vid", "Vidunda"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Viemo", "vig"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vig", "Viemo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Vietnamese", "vie"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vie", "Vietnamese"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Viid", "snu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("snu", "Viid"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Vilela", "vil"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vil", "Vilela"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Vili", "vif"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vif", "Vili"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Villa Viciosa Agta", "dyg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dyg", "Villa Viciosa Agta"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Vincentian Creole English", "svc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("svc", "Vincentian Creole English"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Vinmavis", "vnm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vnm", "Vinmavis"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Vinza", "vin"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vin", "Vinza"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Virgin Islands Creole English", "vic"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vic", "Virgin Islands Creole English"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Vishavan", "vis"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vis", "Vishavan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Viti", "vit"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vit", "Viti"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Vitou", "vto"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vto", "Vitou"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Vlaams", "vls"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vls", "Vlaams"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Vlaamse Gebarentaal", "vgt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vgt", "Vlaamse Gebarentaal"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Vlax Romani", "rmy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rmy", "Vlax Romani"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Volapük", "vol"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vol", "Volapük"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Volscian", "xvo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xvo", "Volscian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Vono", "kch"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kch", "Vono"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Voro", "vor"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vor", "Voro"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Votic", "vot"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vot", "Votic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Vumbu", "vum"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vum", "Vumbu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Vunapu", "vnp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vnp", "Vunapu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Vunjo", "vun"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vun", "Vunjo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Vure", "lkn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lkn", "Vure"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Vurës", "msn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("msn", "Vurës"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Vute", "vut"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vut", "Vute"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Vwanji", "wbi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wbi", "Vwanji"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Võro", "vro"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vro", "Võro"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wa", "wbm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wbm", "Wa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wa'ema", "wag"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wag", "Wa'ema"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Waama", "wwa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wwa", "Waama"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Waamwang", "wmn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wmn", "Waamwang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Waata", "ssn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ssn", "Waata"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wab", "wab"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wab", "Wab"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wabo", "wbb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wbb", "Wabo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Waboda", "kmx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kmx", "Waboda"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Waci Gbe", "wci"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wci", "Waci Gbe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wadaginam", "wdg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wdg", "Wadaginam"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Waddar", "wbq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wbq", "Waddar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wadiyara Koli", "kxp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kxp", "Wadiyara Koli"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wadjiginy", "wdj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wdj", "Wadjiginy"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wadjigu", "wdu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wdu", "Wadjigu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wae Rana", "wrx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wrx", "Wae Rana"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Waffa", "waj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("waj", "Waffa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wagawaga", "wgb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wgb", "Wagawaga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wagaya", "wga"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wga", "Wagaya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wagdi", "wbr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wbr", "Wagdi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wageman", "waq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("waq", "Wageman"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wagi", "fad"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("fad", "Wagi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wahau Kayan", "whu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("whu", "Wahau Kayan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wahau Kenyah", "whk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("whk", "Wahau Kenyah"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wahgi", "wgi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wgi", "Wahgi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Waigali", "wbk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wbk", "Waigali"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Waigeo", "wgo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wgo", "Waigeo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wailaki", "wlk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wlk", "Wailaki"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wailapa", "wlr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wlr", "Wailapa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Waima", "rro"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rro", "Waima"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Waima'a", "wmh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wmh", "Waima'a"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Waimaha", "bao"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bao", "Waimaha"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Waimiri-Atroari", "atr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("atr", "Waimiri-Atroari"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Waioli", "wli"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wli", "Waioli"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Waiwai", "waw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("waw", "Waiwai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Waja", "wja"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wja", "Waja"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wajarri", "wbv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wbv", "Wajarri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Waka", "wav"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wav", "Waka"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wakawaka", "wkw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wkw", "Wakawaka"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wakde", "wkd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wkd", "Wakde"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wakhi", "wbl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wbl", "Wakhi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wakoná", "waf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("waf", "Wakoná"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wala", "lgl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lgl", "Wala"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Walak", "wlw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wlw", "Walak"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wali (Ghana)", "wlx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wlx", "Wali (Ghana)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wali (Sudan)", "wll"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wll", "Wali (Sudan)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Waling", "wly"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wly", "Waling"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Walio", "wla"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wla", "Walio"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Walla Walla", "waa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("waa", "Walla Walla"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wallisian", "wls"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wls", "Wallisian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Walloon", "wln"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wln", "Walloon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Walmajarri", "wmt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wmt", "Walmajarri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Walo Kumbe Dogon", "dwl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dwl", "Walo Kumbe Dogon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Walser", "wae"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wae", "Walser"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Walungge", "ola"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ola", "Walungge"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wamas", "wmc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wmc", "Wamas"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wambaya", "wmb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wmb", "Wambaya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wambon", "wms"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wms", "Wambon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wambule", "wme"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wme", "Wambule"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wamey", "cou"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cou", "Wamey"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wamin", "wmi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wmi", "Wamin"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wampanoag", "wam"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wam", "Wampanoag"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wampar", "lbq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lbq", "Wampar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wampur", "waz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("waz", "Wampur"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wan", "wan"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wan", "Wan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wanambre", "wnb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wnb", "Wanambre"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wanap", "wnp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wnp", "Wanap"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wanbasana", "alw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("alw", "Wanbasana"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wancho Naga", "nnp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nnp", "Wancho Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wanda", "wbh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wbh", "Wanda"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wandala", "mfi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mfi", "Wandala"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wandamen", "wad"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wad", "Wandamen"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wandarang", "wnd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wnd", "Wandarang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wandji", "wdd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wdd", "Wandji"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Waneci", "wne"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wne", "Waneci"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wanga", "lwg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lwg", "Wanga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wanggamala", "wnm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wnm", "Wanggamala"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wangganguru", "wgg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wgg", "Wangganguru"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wanggom", "wng"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wng", "Wanggom"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wanman", "wbt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wbt", "Wanman"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wannu", "jub"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jub", "Wannu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wano", "wno"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wno", "Wano"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wantoat", "wnc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wnc", "Wantoat"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wanukaka", "wnk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wnk", "Wanukaka"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wané", "hwa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hwa", "Wané"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Waorani", "auc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("auc", "Waorani"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wapan", "juk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("juk", "Wapan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wapishana", "wap"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wap", "Wapishana"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wappo", "wao"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wao", "Wappo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("War-Jaintia", "aml"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aml", "War-Jaintia"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wara", "wbf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wbf", "Wara"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Warao", "wba"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wba", "Warao"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Warapu", "wra"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wra", "Warapu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Waray (Australia)", "wrz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wrz", "Waray (Australia)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Waray (Philippines)", "war"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("war", "Waray (Philippines)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wardaman", "wrr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wrr", "Wardaman"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Warduji", "wrd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wrd", "Warduji"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Warembori", "wsa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wsa", "Warembori"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wares", "wai"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wai", "Wares"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Waris", "wrs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wrs", "Waris"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Waritai", "wbe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wbe", "Waritai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wariyangga", "wri"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wri", "Wariyangga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Warji", "wji"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wji", "Warji"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Warkay-Bipim", "bgv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bgv", "Warkay-Bipim"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Warlmanpa", "wrl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wrl", "Warlmanpa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Warlpiri", "wbp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wbp", "Warlpiri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Warluwara", "wrb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wrb", "Warluwara"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Warnang", "wrn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wrn", "Warnang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Waropen", "wrp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wrp", "Waropen"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Warrgamay", "wgy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wgy", "Warrgamay"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Warrwa", "wwr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wwr", "Warrwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Waru", "wru"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wru", "Waru"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Warumungu", "wrm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wrm", "Warumungu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Waruna", "wrv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wrv", "Waruna"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Warungu", "wrg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wrg", "Warungu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wasa", "wss"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wss", "Wasa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wasco-Wishram", "wac"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wac", "Wasco-Wishram"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wasembo", "gsp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gsp", "Wasembo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Washo", "was"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("was", "Washo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Waskia", "wsk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wsk", "Waskia"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wasu", "wsu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wsu", "Wasu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Watakataui", "wtk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wtk", "Watakataui"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Watam", "wax"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wax", "Watam"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Watubela", "wah"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wah", "Watubela"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Waurá", "wau"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wau", "Waurá"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wauyai", "wuy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wuy", "Wauyai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wawa", "www"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("www", "Wawa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wawonii", "wow"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wow", "Wawonii"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Waxianghua", "wxa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wxa", "Waxianghua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wayampi", "oym"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("oym", "Wayampi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wayana", "way"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("way", "Wayana"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wayanad Chetti", "ctt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ctt", "Wayanad Chetti"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wayoró", "wyr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wyr", "Wayoró"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wayu", "vay"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("vay", "Wayu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wayuu", "guc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("guc", "Wayuu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wedau", "wed"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wed", "Wedau"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Weh", "weh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("weh", "Weh"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wejewa", "wew"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wew", "Wejewa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Welaung", "weu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("weu", "Welaung"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Weliki", "klh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("klh", "Weliki"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Welsh", "cym"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cym", "Welsh"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Welsh Romani", "rmw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rmw", "Welsh Romani"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Weme Gbe", "wem"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wem", "Weme Gbe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Were", "wei"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wei", "Were"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Weri", "wer"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wer", "Weri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wersing", "kvw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kvw", "Wersing"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("West Albay Bikol", "fbl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("fbl", "West Albay Bikol"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("West Ambae", "nnd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nnd", "West Ambae"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("West Berawan", "zbw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zbw", "West Berawan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("West Central Banda", "bbp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bbp", "West Central Banda"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("West Central Oromo", "gaz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gaz", "West Central Oromo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("West Coast Bajau", "bdr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bdr", "West Coast Bajau"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("West Damar", "drn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("drn", "West Damar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("West Goodenough", "ddi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ddi", "West Goodenough"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("West Kewa", "kew"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kew", "West Kewa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("West Lembata", "lmj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lmj", "West Lembata"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("West Makian", "mqs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mqs", "West Makian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("West Masela", "mss"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mss", "West Masela"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("West Tarangan", "txn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("txn", "West Tarangan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("West Uvean", "uve"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("uve", "West Uvean"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("West Yugur", "ybe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ybe", "West Yugur"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("West-Central Limba", "lia"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lia", "West-Central Limba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Western Abnaki", "abe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("abe", "Western Abnaki"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Western Apache", "apw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("apw", "Western Apache"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Western Arrarnta", "are"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("are", "Western Arrarnta"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Western Balochi", "bgn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bgn", "Western Balochi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Western Bolivian Guaraní", "gnw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gnw", "Western Bolivian Guaraní"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Western Bru", "brv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("brv", "Western Bru"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Western Bukidnon Manobo", "mbb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mbb", "Western Bukidnon Manobo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Western Canadian Inuktitut", "ikt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ikt", "Western Canadian Inuktitut"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Western Cham", "cja"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cja", "Western Cham"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Western Dani", "dnw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dnw", "Western Dani"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Western Fijian", "wyy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wyy", "Western Fijian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Western Frisian", "fry"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("fry", "Western Frisian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Western Gurung", "gvr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gvr", "Western Gurung"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Western Highland Chatino", "ctp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ctp", "Western Highland Chatino"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Western Highland Purepecha", "pua"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pua", "Western Highland Purepecha"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Western Huasteca Nahuatl", "nhw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nhw", "Western Huasteca Nahuatl"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Western Juxtlahuaca Mixtec", "jmx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jmx", "Western Juxtlahuaca Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Western Kanjobal", "knj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("knj", "Western Kanjobal"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Western Karaboro", "kza"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kza", "Western Karaboro"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Western Katu", "kuf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kuf", "Western Katu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Western Kayah", "kyu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kyu", "Western Kayah"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Western Keres", "kjq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kjq", "Western Keres"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Western Krahn", "krw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("krw", "Western Krahn"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Western Lalu", "ywl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ywl", "Western Lalu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Western Lawa", "lcp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lcp", "Western Lawa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Western Magar", "mrd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mrd", "Western Magar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Western Maninkakan", "mlq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mlq", "Western Maninkakan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Western Mari", "mrj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mrj", "Western Mari"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Western Mashan Hmong", "hmw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hmw", "Western Mashan Hmong"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Western Mashan Miao", "hmw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hmw", "Western Mashan Miao"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Western Meohang", "raf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("raf", "Western Meohang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Western Muria", "mut"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mut", "Western Muria"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Western Neo-Aramaic", "amw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("amw", "Western Neo-Aramaic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Western Niger Fulfulde", "fuh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("fuh", "Western Niger Fulfulde"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Western Ojibwa", "ojw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ojw", "Western Ojibwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Western Panjabi", "pnb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pnb", "Western Panjabi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Western Parbate Kham", "kjl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kjl", "Western Parbate Kham"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Western Penan", "pne"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pne", "Western Penan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Western Sisaala", "ssl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ssl", "Western Sisaala"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Western Subanon", "suc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("suc", "Western Subanon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Western Tamang", "tdg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tdg", "Western Tamang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Western Tawbuid", "twb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("twb", "Western Tawbuid"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Western Totonac", "tqt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tqt", "Western Totonac"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Western Tunebo", "tnb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tnb", "Western Tunebo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Western Xiangxi Miao", "mmr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mmr", "Western Xiangxi Miao"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Western Xwla Gbe", "xwl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xwl", "Western Xwla Gbe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Western Yiddish", "yih"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yih", "Western Yiddish"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Westphalien", "wep"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wep", "Westphalien"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wetamut", "wwo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wwo", "Wetamut"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wewaw", "wea"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wea", "Wewaw"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Weyto", "woy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("woy", "Weyto"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("White Gelao", "giw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("giw", "White Gelao"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("White Lachi", "lwh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("lwh", "White Lachi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Whitesands", "tnp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tnp", "Whitesands"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wiarumus", "tua"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tua", "Wiarumus"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wichita", "wic"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wic", "Wichita"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wichí Lhamtés Güisnay", "mzh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mzh", "Wichí Lhamtés Güisnay"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wichí Lhamtés Nocten", "mtp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mtp", "Wichí Lhamtés Nocten"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wichí Lhamtés Vejoz", "wlv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wlv", "Wichí Lhamtés Vejoz"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wik-Epa", "wie"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wie", "Wik-Epa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wik-Iiyanh", "wij"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wij", "Wik-Iiyanh"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wik-Keyangan", "wif"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wif", "Wik-Keyangan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wik-Me'anha", "wih"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wih", "Wik-Me'anha"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wik-Mungkan", "wim"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wim", "Wik-Mungkan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wik-Ngathana", "wig"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wig", "Wik-Ngathana"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wikalkan", "wik"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wik", "Wikalkan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wikngenchera", "wua"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wua", "Wikngenchera"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wilawila", "wil"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wil", "Wilawila"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wintu", "wit"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wit", "Wintu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Winyé", "kst"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kst", "Winyé"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wipi", "gdr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gdr", "Wipi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wiradhuri", "wrh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wrh", "Wiradhuri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wiraféd", "wir"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wir", "Wiraféd"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wirangu", "wiw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wiw", "Wirangu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wiru", "wiu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wiu", "Wiru"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wiyot", "wiy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wiy", "Wiyot"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Woccon", "xwc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xwc", "Woccon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wogamusin", "wog"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wog", "Wogamusin"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wogeo", "woc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("woc", "Wogeo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Woi", "wbw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wbw", "Woi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wojenaka", "jod"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jod", "Wojenaka"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wolaitta", "wal"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wal", "Wolaitta"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wolane", "wle"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wle", "Wolane"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wolani", "wod"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wod", "Wolani"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wolaytta", "wal"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wal", "Wolaytta"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Woleaian", "woe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("woe", "Woleaian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wolio", "wlo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wlo", "Wolio"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wolof", "wol"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wol", "Wolof"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wom (Nigeria)", "wom"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wom", "Wom (Nigeria)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wom (Papua New Guinea)", "wmo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wmo", "Wom (Papua New Guinea)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Womo", "wmx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wmx", "Womo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wongo", "won"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("won", "Wongo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Woods Cree", "cwd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("cwd", "Woods Cree"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Woria", "wor"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wor", "Woria"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Worimi", "kda"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kda", "Worimi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Worodougou", "jud"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jud", "Worodougou"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Worora", "unp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("unp", "Worora"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wotu", "wtw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wtw", "Wotu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Woun Meu", "noa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("noa", "Woun Meu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Written Oirat", "xwo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xwo", "Written Oirat"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wu Chinese", "wuu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wuu", "Wu Chinese"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wuding-Luquan Yi", "ywq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ywq", "Wuding-Luquan Yi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wudu", "wud"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wud", "Wudu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wuliwuli", "wlu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wlu", "Wuliwuli"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wulna", "wux"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wux", "Wulna"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wumboko", "bqm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bqm", "Wumboko"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wumbvu", "wum"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wum", "Wumbvu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wumeng Nasu", "ywu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ywu", "Wumeng Nasu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wunai Bunu", "bwn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bwn", "Wunai Bunu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wunambal", "wub"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wub", "Wunambal"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wurrugu", "wur"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wur", "Wurrugu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wusa Nasu", "yig"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yig", "Wusa Nasu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wushi", "bse"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bse", "Wushi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wusi", "wsi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wsi", "Wusi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wutung", "wut"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wut", "Wutung"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wutunhua", "wuh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wuh", "Wutunhua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wuvulu-Aua", "wuv"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wuv", "Wuvulu-Aua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wuzlam", "udl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("udl", "Wuzlam"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wyandot", "wya"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wya", "Wyandot"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wymysorys", "wym"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wym", "Wymysorys"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wára", "tci"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tci", "Wára"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wãpha", "juw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("juw", "Wãpha"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wè Northern", "wob"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wob", "Wè Northern"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wè Southern", "gxx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gxx", "Wè Southern"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Wè Western", "wec"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wec", "Wè Western"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Xaasongaxango", "kao"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kao", "Xaasongaxango"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Xadani Zapotec", "zax"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zax", "Xadani Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Xakriabá", "xkr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xkr", "Xakriabá"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Xamtanga", "xan"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xan", "Xamtanga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Xanaguía Zapotec", "ztg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ztg", "Xanaguía Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Xaragure", "axx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("axx", "Xaragure"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Xavánte", "xav"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xav", "Xavánte"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Xerénte", "xer"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xer", "Xerénte"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Xetá", "xet"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xet", "Xetá"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Xhosa", "xho"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xho", "Xhosa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Xiandao", "xia"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xia", "Xiandao"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Xiang Chinese", "hsn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("hsn", "Xiang Chinese"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Xibe", "sjo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sjo", "Xibe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Xicotepec De Juárez Totonac", "too"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("too", "Xicotepec De Juárez Totonac"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Xinca", "xin"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xin", "Xinca"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Xingú Asuriní", "asn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("asn", "Xingú Asuriní"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Xipaya", "xiy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xiy", "Xipaya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Xipináwa", "xip"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xip", "Xipináwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Xiri", "xii"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xii", "Xiri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Xiriâna", "xir"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xir", "Xiriâna"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Xishanba Lalo", "ywt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ywt", "Xishanba Lalo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Xokleng", "xok"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xok", "Xokleng"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Xukurú", "xoo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xoo", "Xukurú"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Xwela Gbe", "xwe"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xwe", "Xwela Gbe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Xârâcùù", "ane"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ane", "Xârâcùù"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yaaku", "muu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("muu", "Yaaku"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yabarana", "yar"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yar", "Yabarana"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yabaâna", "ybn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ybn", "Yabaâna"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yabem", "jae"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jae", "Yabem"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yaben", "ybm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ybm", "Yaben"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yabong", "ybo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ybo", "Yabong"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yace", "ekr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ekr", "Yace"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yaeyama", "rys"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rys", "Yaeyama"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yafi", "wfg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wfg", "Yafi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yagaria", "ygr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ygr", "Yagaria"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yagnobi", "yai"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yai", "Yagnobi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yagomi", "ygm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ygm", "Yagomi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yagua", "yad"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yad", "Yagua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yagwoia", "ygw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ygw", "Yagwoia"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yahadian", "ner"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ner", "Yahadian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yahang", "rhp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("rhp", "Yahang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yahuna", "ynu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ynu", "Yahuna"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yaka (Central African Republic)", "axk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("axk", "Yaka (Central African Republic)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yaka (Congo)", "iyx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("iyx", "Yaka (Congo)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yaka (Democratic Republic of Congo)", "yaf"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yaf", "Yaka (Democratic Republic of Congo)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yakaikeke", "ykk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ykk", "Yakaikeke"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yakama", "yak"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yak", "Yakama"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yakan", "yka"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yka", "Yakan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yakha", "ybh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ybh", "Yakha"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yakoma", "yky"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yky", "Yakoma"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yakut", "sah"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sah", "Yakut"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yala", "yba"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yba", "Yala"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yalahatan", "jal"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jal", "Yalahatan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yalakalore", "xyl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xyl", "Yalakalore"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yalarnnga", "ylr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ylr", "Yalarnnga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yale", "nce"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nce", "Yale"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yaleba", "ylb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ylb", "Yaleba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yalunka", "yal"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yal", "Yalunka"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yalálag Zapotec", "zpu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zpu", "Yalálag Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yamap", "ymp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ymp", "Yamap"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yamba", "yam"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yam", "Yamba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yambes", "ymb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ymb", "Yambes"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yambeta", "yat"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yat", "Yambeta"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yamdena", "jmd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jmd", "Yamdena"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yameo", "yme"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yme", "Yameo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yami", "tao"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tao", "Yami"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yaminahua", "yaa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yaa", "Yaminahua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yamna", "ymn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ymn", "Yamna"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yamongeri", "ymg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ymg", "Yamongeri"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yamphe", "yma"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yma", "Yamphe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yamphu", "ybi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ybi", "Yamphu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yan-nhangu", "jay"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jay", "Yan-nhangu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yana", "ynn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ynn", "Yana"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yanahuanca Pasco Quechua", "qur"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("qur", "Yanahuanca Pasco Quechua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yanda Dom Dogon", "dym"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dym", "Yanda Dom Dogon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yandruwandha", "ynd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ynd", "Yandruwandha"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yanesha'", "ame"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ame", "Yanesha'"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yang Zhuang", "zyg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zyg", "Yang Zhuang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yangben", "yav"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yav", "Yangben"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yangbye", "ybd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ybd", "Yangbye"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yangho", "ynh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ynh", "Yangho"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yangkam", "bsx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("bsx", "Yangkam"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yangman", "jng"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jng", "Yangman"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yango", "yng"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yng", "Yango"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yangulam", "ynl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ynl", "Yangulam"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yangum Dey", "yde"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yde", "Yangum Dey"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yangum Gel", "ygl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ygl", "Yangum Gel"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yangum Mon", "ymo"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ymo", "Yangum Mon"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yankunytjatjara", "kdd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kdd", "Yankunytjatjara"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yanomamö", "guu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("guu", "Yanomamö"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yanomámi", "wca"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wca", "Yanomámi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yansi", "yns"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yns", "Yansi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yanyuwa", "jao"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jao", "Yanyuwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yao", "yao"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yao", "Yao"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yaosakor Asmat", "asy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("asy", "Yaosakor Asmat"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yaouré", "yre"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yre", "Yaouré"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yapese", "yap"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yap", "Yapese"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yapunda", "yev"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yev", "Yapunda"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yaqay", "jaq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jaq", "Yaqay"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yaqui", "yaq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yaq", "Yaqui"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yarawata", "yrw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yrw", "Yarawata"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yareba", "yrb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yrb", "Yareba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yareni Zapotec", "zae"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zae", "Yareni Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yarsun", "yrs"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yrs", "Yarsun"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yarí", "yri"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yri", "Yarí"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yasa", "yko"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yko", "Yasa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yassic", "ysc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ysc", "Yassic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yatee Zapotec", "zty"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zty", "Yatee Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yatzachi Zapotec", "zav"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zav", "Yatzachi Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yau (Morobe Province)", "yuw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yuw", "Yau (Morobe Province)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yau (Sandaun Province)", "yyu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yyu", "Yau (Sandaun Province)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yaul", "yla"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yla", "Yaul"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yauma", "yax"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yax", "Yauma"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yaur", "jau"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jau", "Yaur"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yautepec Zapotec", "zpb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zpb", "Yautepec Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yauyos Quechua", "qux"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("qux", "Yauyos Quechua"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yavitero", "yvt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yvt", "Yavitero"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yawa", "yva"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yva", "Yawa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yawalapití", "yaw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yaw", "Yawalapití"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yawanawa", "ywn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ywn", "Yawanawa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yawarawarga", "yww"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yww", "Yawarawarga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yaweyuha", "yby"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yby", "Yaweyuha"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yawiyo", "ybx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ybx", "Yawiyo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yawuru", "ywr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ywr", "Yawuru"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yazgulyam", "yah"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yah", "Yazgulyam"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yecuatla Totonac", "tlc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tlc", "Yecuatla Totonac"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yei", "jei"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jei", "Yei"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yekhee", "ets"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ets", "Yekhee"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yekora", "ykr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ykr", "Yekora"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yela", "yel"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yel", "Yela"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yele", "yle"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yle", "Yele"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yelmek", "jel"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jel", "Yelmek"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yelogu", "ylg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ylg", "Yelogu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yemba", "ybb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ybb", "Yemba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yemsa", "jnj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jnj", "Yemsa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yendang", "yen"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yen", "Yendang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yeni", "yei"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yei", "Yeni"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yeniche", "yec"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yec", "Yeniche"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yerakai", "yra"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yra", "Yerakai"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yeretuar", "gop"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gop", "Yeretuar"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yerong", "yrn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yrn", "Yerong"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yerukula", "yeu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yeu", "Yerukula"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yeskwa", "yes"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yes", "Yeskwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yessan-Mayo", "yss"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yss", "Yessan-Mayo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yetfa", "yet"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yet", "Yetfa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yevanic", "yej"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yej", "Yevanic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yeyi", "yey"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yey", "Yeyi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yiddish", "yid"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yid", "Yiddish"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yiddish Sign Language", "yds"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yds", "Yiddish Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yidgha", "ydg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ydg", "Yidgha"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yidiny", "yii"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yii", "Yidiny"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yil", "yll"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yll", "Yil"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yimas", "yee"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yee", "Yimas"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yimchungru Naga", "yim"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yim", "Yimchungru Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yinbaw Karen", "kvu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kvu", "Yinbaw Karen"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yinchia", "yin"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yin", "Yinchia"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yindjibarndi", "yij"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yij", "Yindjibarndi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yindjilandji", "yil"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yil", "Yindjilandji"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yine", "pib"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pib", "Yine"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yinggarda", "yia"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yia", "Yinggarda"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yintale Karen", "kvy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kvy", "Yintale Karen"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yir Yoront", "yiy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yiy", "Yir Yoront"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yis", "yis"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yis", "Yis"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yiwom", "gek"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gek", "Yiwom"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yoba", "yob"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yob", "Yoba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yocoboué Dida", "gud"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gud", "Yocoboué Dida"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yogad", "yog"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yog", "Yogad"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yoidik", "ydk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ydk", "Yoidik"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yoke", "yki"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yki", "Yoke"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yokuts", "yok"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yok", "Yokuts"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yola", "yol"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yol", "Yola"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yoloxochitl Mixtec", "xty"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xty", "Yoloxochitl Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yom", "pil"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pil", "Yom"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yombe", "yom"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yom", "Yombe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yonaguni", "yoi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yoi", "Yonaguni"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yong", "yno"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yno", "Yong"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yongbei Zhuang", "zyb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zyb", "Yongbei Zhuang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yonggom", "yon"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yon", "Yonggom"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yongnan Zhuang", "zyn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zyn", "Yongnan Zhuang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yopno", "yut"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yut", "Yopno"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yora", "mts"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mts", "Yora"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yoron", "yox"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yox", "Yoron"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yoruba", "yor"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yor", "Yoruba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yos", "yos"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yos", "Yos"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yosondúa Mixtec", "mpm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mpm", "Yosondúa Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Youjiang Zhuang", "zyj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zyj", "Youjiang Zhuang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Youle Jinuo", "jiu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jiu", "Youle Jinuo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Younuo Bunu", "buh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("buh", "Younuo Bunu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yout Wam", "ytw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ytw", "Yout Wam"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yoy", "yoy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yoy", "Yoy"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yuaga", "nua"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nua", "Yuaga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yucatec Maya", "yua"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yua", "Yucatec Maya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yucatec Maya Sign Language", "msd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("msd", "Yucatec Maya Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yucateco", "yua"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yua", "Yucateco"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yuchi", "yuc"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yuc", "Yuchi"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yucuañe Mixtec", "mvg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mvg", "Yucuañe Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yucuna", "ycn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ycn", "Yucuna"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yue Chinese", "yue"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yue", "Yue Chinese"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yug", "yug"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yug", "Yug"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yugambal", "yub"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yub", "Yugambal"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yugh", "yuu"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yuu", "Yugh"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yugoslavian Sign Language", "ysl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ysl", "Yugoslavian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yuhup", "yab"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yab", "Yuhup"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yuki", "yuk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yuk", "Yuki"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yukpa", "yup"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yup", "Yukpa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yukuben", "ybl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ybl", "Yukuben"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yulu", "yul"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yul", "Yulu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yuqui", "yuq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yuq", "Yuqui"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yuracare", "yuz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yuz", "Yuracare"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yurok", "yur"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yur", "Yurok"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yurutí", "yui"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yui", "Yurutí"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yutanduchi Mixtec", "mab"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("mab", "Yutanduchi Mixtec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yuwana", "yau"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yau", "Yuwana"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Yámana", "yag"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yag", "Yámana"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Zaachila Zapotec", "ztx"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ztx", "Zaachila Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Zabana", "kji"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kji", "Zabana"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Zacatepec Chatino", "ctz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ctz", "Zacatepec Chatino"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Zacatlán-Ahuacatlán-Tepetzintla Nahuatl", "nhi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nhi", "Zacatlán-Ahuacatlán-Tepetzintla Nahuatl"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Zaghawa", "zag"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zag", "Zaghawa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Zaiwa", "atb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("atb", "Zaiwa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Zakhring", "zkr"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zkr", "Zakhring"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Zambian Sign Language", "zsl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zsl", "Zambian Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Zan Gula", "zna"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zna", "Zan Gula"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Zanaki", "zak"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zak", "Zanaki"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Zande (individual language)", "zne"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zne", "Zande (individual language)"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Zangskari", "zau"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zau", "Zangskari"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Zangwal", "zah"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zah", "Zangwal"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Zaniza Zapotec", "zpw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zpw", "Zaniza Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Zapotec", "zap"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zap", "Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Zaramo", "zaj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zaj", "Zaramo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Zari", "zaz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zaz", "Zari"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Zarma", "dje"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dje", "Zarma"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Zarphatic", "zrp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zrp", "Zarphatic"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Zauzou", "zal"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zal", "Zauzou"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Zay", "zwa"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zwa", "Zay"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Zayein Karen", "kxk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("kxk", "Zayein Karen"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Zayse-Zergulla", "zay"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zay", "Zayse-Zergulla"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Zaysete", "zay"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zay", "Zaysete"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Zaza", "zza"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zza", "Zaza"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Zazaki", "zza"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zza", "Zazaki"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Zazao", "jaj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jaj", "Zazao"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Zeem", "zua"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zua", "Zeem"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Zeeuws", "zea"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zea", "Zeeuws"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Zemba", "dhm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("dhm", "Zemba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Zeme Naga", "nzm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nzm", "Zeme Naga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Zemgalian", "xzm"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xzm", "Zemgalian"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Zenag", "zeg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zeg", "Zenag"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Zenaga", "zen"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zen", "Zenaga"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Zenzontepec Chatino", "czn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("czn", "Zenzontepec Chatino"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Zerenkel", "zrn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zrn", "Zerenkel"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Zhaba", "zhb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zhb", "Zhaba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Zhang-Zhung", "xzh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("xzh", "Zhang-Zhung"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Zhire", "zhi"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zhi", "Zhire"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Zhoa", "zhw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zhw", "Zhoa"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Zhuang", "zha"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zha", "Zhuang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Zia", "zia"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zia", "Zia"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Zigula", "ziw"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ziw", "Zigula"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Zimakani", "zik"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zik", "Zimakani"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Zimba", "zmb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zmb", "Zimba"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Zimbabwe Sign Language", "zib"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zib", "Zimbabwe Sign Language"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Zinza", "zin"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zin", "Zinza"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Zire", "sih"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("sih", "Zire"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ziriya", "zir"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zir", "Ziriya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Zizilivakan", "ziz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ziz", "Zizilivakan"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Zo'é", "pto"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("pto", "Zo'é"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Zokhuo", "yzk"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("yzk", "Zokhuo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Zoogocho Zapotec", "zpq"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zpq", "Zoogocho Zapotec"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Zoroastrian Dari", "gbz"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gbz", "Zoroastrian Dari"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Zorop", "wfg"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("wfg", "Zorop"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Zotung Chin", "czt"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("czt", "Zotung Chin"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Zou", "zom"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zom", "Zou"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Zulgo-Gemzek", "gnd"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("gnd", "Zulgo-Gemzek"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Zulu", "zul"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zul", "Zulu"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Zumaya", "zuy"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zuy", "Zumaya"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Zumbun", "jmb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("jmb", "Zumbun"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Zuni", "zun"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zun", "Zuni"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Zuojiang Zhuang", "zzj"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zzj", "Zuojiang Zhuang"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Zyphe", "zyp"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zyp", "Zyphe"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Záparo", "zro"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("zro", "Záparo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("finlandssvenskt teckenspråk", "fss"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("fss", "finlandssvenskt teckenspråk"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("suomenruotsalainen viittomakieli", "fss"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("fss", "suomenruotsalainen viittomakieli"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("tlhIngan-Hol", "tlh"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("tlh", "tlhIngan-Hol"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Àhàn", "ahn"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("ahn", "Àhàn"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Áncá", "acb"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("acb", "Áncá"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Äiwoo", "nfl"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("nfl", "Äiwoo"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Ömie", "aom"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("aom", "Ömie"); //$NON-NLS-1$ //$NON-NLS-2$

		langMap.put("Önge", "oon"); //$NON-NLS-1$ //$NON-NLS-2$
		langMap.put("oon", "Önge"); //$NON-NLS-1$ //$NON-NLS-2$
	}

}
